import * as i0 from '@angular/core';
import { EventEmitter, InjectionToken, signal, computed, Injectable, inject, DestroyRef, Inject, Input, Optional, Directive, Output, NgZone, ChangeDetectorRef, forwardRef, HostBinding, ElementRef, Pipe, NgModule } from '@angular/core';
import equal from 'fast-deep-equal';
import { toSignal, takeUntilDestroyed } from '@angular/core/rxjs-interop';
import * as i2 from '@fundamental-ngx/cdk/utils';
import { resizeObservable, RtlService, FDK_FOCUSABLE_ITEM_DIRECTIVE, FocusableItemDirective, isString, dfs, KeyUtil } from '@fundamental-ngx/cdk/utils';
import { skip, debounceTime, filter, map, distinctUntilChanged, takeUntil, take, startWith, share, tap } from 'rxjs/operators';
import { BehaviorSubject, Subject, Subscription, fromEvent, isObservable, Observable, filter as filter$1 } from 'rxjs';
import { ContentDensityMode } from '@fundamental-ngx/core/content-density';
import { TableCellDirective } from '@fundamental-ngx/core/table';
import * as i1 from '@angular/forms';
import { isDataSource, DataSourceDirective, FD_DATA_SOURCE_TRANSFORMER } from '@fundamental-ngx/cdk/data-source';
import { isSelectItem, isDataSource as isDataSource$1, isString as isString$1, isFunction } from '@fundamental-ngx/platform/shared';
import { get, cloneDeep, set } from 'lodash-es';
import { LEFT_ARROW, RIGHT_ARROW } from '@angular/cdk/keycodes';
import { DOCUMENT } from '@angular/common';

class Table {
    constructor() {
        /** Toolbar Sort Settings button click event */
        this.openTableSortSettings = new EventEmitter();
        /** Toolbar Filter Settings button click event */
        this.openTableFilterSettings = new EventEmitter();
        /** Toolbar Group Settings button click event */
        this.openTableGroupSettings = new EventEmitter();
        /** Toolbar Column Settings button click event */
        this.openTableColumnSettings = new EventEmitter();
        /** Event fired when table state changes. */
        this.tableColumnFilterChange = new EventEmitter();
    }
}

const FDP_TABLE_DRAGGABLE_DIRECTIVE = new InjectionToken('FdpTableDraggableDirective');
const FDP_TABLE_STATE_DIRECTIVE = new InjectionToken('FdpTableStateDirective');
const FDP_TABLE_VIRTUAL_SCROLL_DIRECTIVE = new InjectionToken('FdpTableVirtualScrollDirective');

const DEFAULT_TABLE_PAGE = {
    pageSize: 0,
    currentPage: 1
};
const DEFAULT_TABLE_SEARCH_INPUT = {
    text: '',
    category: ''
};
const DEFAULT_TABLE_STATE = {
    sortBy: [],
    filterBy: [],
    groupBy: [],
    page: DEFAULT_TABLE_PAGE,
    columns: [],
    columnKeys: [],
    freezeToColumn: null,
    freezeToEndColumn: null,
    searchInput: DEFAULT_TABLE_SEARCH_INPUT,
    scrollTopPosition: 0
};
const DEFAULT_HIGHLIGHTING_KEY = 'semantic';
const EDITABLE_ROW_SEMANTIC_STATE = 'information';
const SEMANTIC_HIGHLIGHTING_COLUMN_WIDTH = 6;
const SELECTION_COLUMN_WIDTH = new Map([
    [ContentDensityMode.COZY, 44],
    [ContentDensityMode.COMPACT, 32],
    [ContentDensityMode.CONDENSED, 32]
]);
const TABLE_COLUMN_MIN_WIDTH = 50;
const ROW_HEIGHT = new Map([
    [ContentDensityMode.COZY, 44],
    [ContentDensityMode.COMPACT, 32],
    [ContentDensityMode.CONDENSED, 24]
]);

class TableService {
    constructor() {
        /** Table state stream. */
        this.tableState$ = new BehaviorSubject(DEFAULT_TABLE_STATE);
        /** @hidden */
        this._semanticHighlighting$ = signal('');
        /** @hidden */
        this._isFilteringFromHeaderDisabled$ = signal(false);
        /** @hidden */
        this._isShownNavigationColumn$ = signal(false);
        /** @hidden */
        this._semanticHighlightingColumnWidth$ = signal(0);
        /** Visible columns stream. */
        this.visibleColumns$ = signal([]);
        /** Visible columns length. */
        this.visibleColumnsLength = computed(() => this.visibleColumns$().length);
        /** Popping columns stream. */
        this.poppingColumns$ = signal([]);
        /** Popping columns length. */
        this._poppingColumnsLength = 0;
        /** Table columns stream. */
        this.tableColumns$ = new BehaviorSubject([]);
        /** @hidden */
        this.tableStateChanges$ = this.tableState$.pipe(skip(1));
        /** Stream that emits when search state changes. */
        this.searchChange$ = new Subject();
        /** Stream that emits when sort state changes. */
        this.sortChange$ = new Subject();
        /** Stream that emits when filter state changes. */
        this.filterChange$ = new Subject();
        /** Stream that emits when group state changes. */
        this.groupChange$ = new Subject();
        /** Stream that emits when freeze state changes. */
        this.freezeChange$ = new Subject();
        /** Stream that emits when columns state changes. */
        this.columnsChange$ = new Subject();
        /** Stream that emits when page state changes. */
        this.pageChange$ = new Subject();
        /** Stream that emits when new fetch of data is needed. */
        this.needFetch$ = new Subject();
        /** @hidden */
        this.stateChange$ = new Subject();
        /** Stream that emits when loading state changes. */
        this.tableLoading$ = new BehaviorSubject(false);
        /** Listen for soft changes in table subcomponents (mostly table column) */
        this.markForCheck$ = new Subject();
        /** Listen for immediate changes in table subcomponents (mostly table column) */
        this.detectChanges$ = new Subject();
        /** Sort rules stream. */
        this.sortRules$ = signal(new Map());
        /**
         * Filter Rules Map stream. Where key is column key, and value is the associated filter rules.
         * Many filters can be applied to one column.
         */
        this.filterRules$ = signal(new Map());
        /**
         * Group Rules Map stream. Where key is column key and value is associated group rule
         */
        this.groupRulesSubject = new BehaviorSubject(new Map());
        /** Group Rules Map signal. */
        this.groupRules$ = toSignal(this.groupRulesSubject);
    }
    /** Get current state/settings of the Table. */
    getTableState() {
        return this.tableState$.getValue();
    }
    /** Set current state/settings of the Table. */
    setTableState(state) {
        this.tableState$.next(state);
    }
    /** Set current loading state of the Table. */
    setTableLoading(isLoading) {
        this.tableLoading$.next(isLoading);
    }
    /** Notify about changes in table subcomponents (mostly table column) and detect them on next detector check. */
    markForCheck() {
        this.markForCheck$.next();
    }
    /** Notify about changes in table subcomponents (mostly table column) and detect them immediately. */
    detectChanges() {
        this.detectChanges$.next();
    }
    /** Search */
    search(searchInput) {
        const prevState = this.getTableState();
        const state = { ...prevState, searchInput };
        this.setTableState(setCurrentPageToState(state, 1));
        const evt = { current: searchInput, previous: prevState.searchInput };
        this.needFetch$.next();
        this.stateChange$.next({ type: 'search', state: evt });
        this.searchChange$.next(evt);
    }
    /** Set new sort rules */
    setSort(sortRules) {
        const prevState = this.getTableState();
        const prevSortRules = (prevState && prevState.sortBy) || [];
        const newSortRules = sortRules ? [...sortRules] : [];
        const state = { ...prevState, sortBy: newSortRules };
        if (!equal(prevSortRules, state.sortBy)) {
            this.setTableState(setCurrentPageToState(state, 1));
            this.buildSortRulesMap();
            const evt = { current: state.sortBy, previous: prevSortRules };
            this.buildSortRulesMap();
            this.needFetch$.next();
            this.stateChange$.next({ type: 'sort', state: evt });
            this.sortChange$.next(evt);
        }
    }
    /** Add sort rules to the existing ones */
    addSort(sortRules) {
        const prevState = this.getTableState();
        const prevSortRules = (prevState && prevState.sortBy) || [];
        const newSortRules = [
            ...prevSortRules.filter((existing) => !sortRules.find(({ field }) => field === existing.field)),
            ...sortRules
        ];
        const state = { ...prevState, sortBy: newSortRules };
        if (!equal(prevSortRules, state.sortBy)) {
            this.setTableState(setCurrentPageToState(state, 1));
            const evt = { current: state.sortBy, previous: prevSortRules };
            this.buildSortRulesMap();
            this.needFetch$.next();
            this.stateChange$.next({ type: 'sort', state: evt });
            this.sortChange$.next(evt);
        }
    }
    /** Set new filter rules */
    setFilters(filterRules) {
        const prevState = this.getTableState();
        const prevFilterRules = (prevState && prevState.filterBy) || [];
        const newFilterRules = filterRules
            ? filterRules.map((rule) => ({
                ...rule,
                fieldName: this._getFieldName(rule.field, rule.fieldName)
            }))
            : [];
        const state = { ...prevState, filterBy: newFilterRules };
        if (!equal(prevFilterRules, state.filterBy)) {
            this.setTableState(setCurrentPageToState(state, 1));
            const evt = { current: state.filterBy, previous: prevFilterRules };
            this.buildFilterRulesMap();
            this.needFetch$.next();
            this.stateChange$.next({ type: 'filter', state: evt });
            this.filterChange$.next(evt);
        }
    }
    /** Add filter rules to the existing ones */
    addFilters(rulesToAdd) {
        const prevState = this.getTableState();
        const prevFilterRules = (prevState && prevState.filterBy) || [];
        const newFilterRules = [
            ...prevFilterRules.filter((existing) => !rulesToAdd.find(({ field }) => field === existing.field)),
            ...rulesToAdd.map((rule) => ({
                ...rule,
                fieldName: this._getFieldName(rule.field, rule.fieldName)
            }))
        ];
        const state = { ...prevState, filterBy: newFilterRules };
        if (!equal(prevFilterRules, state.filterBy)) {
            this.setTableState(setCurrentPageToState(state, 1));
            const evt = { current: state.filterBy, previous: prevFilterRules };
            this.buildFilterRulesMap();
            this.needFetch$.next();
            this.stateChange$.next({ type: 'filter', state: evt });
            this.filterChange$.next(evt);
        }
    }
    /** Removes filters for the provided fields */
    removeFilters(fields) {
        const prevState = this.getTableState();
        const prevFilterRules = (prevState && prevState.filterBy) || [];
        const newFilterRules = prevFilterRules.filter((existing) => !fields.includes(existing.field));
        const state = { ...prevState, filterBy: newFilterRules };
        this.setTableState(setCurrentPageToState(state, 1));
        const evt = { current: state.filterBy, previous: prevFilterRules };
        this.buildFilterRulesMap();
        this.needFetch$.next();
        this.stateChange$.next({ type: 'filter', state: evt });
        this.filterChange$.next(evt);
    }
    /** Set group rules */
    setGroups(groups) {
        const prevState = this.getTableState();
        const prevGroups = (prevState && prevState.groupBy) || [];
        const newGroups = groups ? [...groups] : [];
        const state = { ...prevState, groupBy: newGroups };
        if (!equal(prevGroups, state.groupBy)) {
            this.setTableState(state);
            const evt = { current: state.groupBy, previous: prevGroups };
            this.buildGroupRulesMap();
            this.stateChange$.next({ type: 'group', state: evt });
            this.groupChange$.next(evt);
        }
    }
    /** Add group rules to the existing ones */
    addGroups(groupsToAdd) {
        const prevState = this.getTableState();
        const prevGroups = (prevState && prevState.groupBy) || [];
        const newGroups = [
            ...prevGroups.filter((existing) => !groupsToAdd.find(({ field }) => field === existing.field)),
            ...groupsToAdd
        ];
        const state = { ...prevState, groupBy: newGroups };
        if (!equal(prevGroups, state.groupBy)) {
            this.setTableState(state);
            this.buildGroupRulesMap();
            const evt = { current: state.groupBy, previous: prevGroups };
            this.stateChange$.next({ type: 'group', state: evt });
            this.groupChange$.next(evt);
        }
    }
    /** Freeze table to column */
    freezeTo(columnName, end) {
        const prevState = this.getTableState();
        if (!end) {
            this.setTableState({ ...prevState, freezeToColumn: columnName });
        }
        else {
            this.setTableState({ ...prevState, freezeToEndColumn: columnName });
        }
        const evt = {
            current: columnName,
            previous: end ? prevState.freezeToEndColumn : prevState.freezeToColumn
        };
        this.stateChange$.next({ type: 'freeze', state: evt });
        this.freezeChange$.next(evt);
    }
    /** Set table columns */
    setColumns(columns, keys) {
        const prevState = this.getTableState();
        const prevColumns = (prevState && prevState.columns) || [];
        const newColumns = [...columns];
        const newColumnKeys = [...keys];
        const state = { ...prevState, columns: newColumns, columnKeys: newColumnKeys };
        if (!equal(prevColumns, state.columns)) {
            this.setTableState(state);
            const evt = { current: state.columns, previous: prevColumns };
            this.stateChange$.next({ type: 'columns', state: evt });
            this.columnsChange$.next(evt);
        }
    }
    /** Set page. */
    setCurrentPage(currentPage) {
        const prevState = this.getTableState();
        const prevPageState = prevState.page;
        if (prevPageState.currentPage === currentPage) {
            return;
        }
        const state = setCurrentPageToState(prevState, currentPage);
        this.setTableState(state);
        const evt = { current: state.page, previous: prevPageState };
        this.needFetch$.next();
        this.stateChange$.next({ type: 'page', state: evt });
        this.pageChange$.next(evt);
    }
    /** Set items per page. */
    setItemsPerPage(itemsPerPage, resetPageNumber = true) {
        const prevState = this.getTableState();
        const prevPageState = prevState.page;
        if (prevPageState.pageSize === itemsPerPage) {
            return;
        }
        let state = setPageSizeToState(prevState, itemsPerPage);
        if (resetPageNumber) {
            state = setCurrentPageToState(state, 1);
        }
        this.setTableState(state);
        const evt = { current: state.page, previous: prevPageState };
        this.needFetch$.next();
        this.stateChange$.next({ type: 'page', state: evt });
        this.pageChange$.next(evt);
    }
    /** Returns default table state. */
    getDefaultState() {
        const tableState = this.getTableState();
        return {
            columnKeys: tableState.columnKeys,
            sortBy: [],
            filterBy: [],
            groupBy: [],
            columns: tableState.columns,
            searchInput: {
                category: null,
                text: ''
            },
            freezeToColumn: null,
            freezeToEndColumn: null,
            page: {
                pageSize: 0,
                currentPage: 1
            },
            scrollTopPosition: 0
        };
    }
    /**
     * Shares visible columns array across listeners.
     * @param columns Visible columns to share.
     */
    setVisibleColumns(columns) {
        this.visibleColumns$.set(columns);
    }
    /**
     * Shares popping columns array across listeners.
     * @param columns Popping columns to share.
     */
    setPoppingColumns(columns) {
        this.poppingColumns$.set(columns);
        this._poppingColumnsLength = columns.length;
    }
    /**
     * Constructs grouping rules based on the table state.
     * @param state Table state.
     */
    buildGroupRulesMap(state = this.getTableState()) {
        const groupMap = new Map(state.groupBy.map((rule) => [rule.field, rule]));
        this.groupRulesSubject.next(groupMap);
    }
    /**
     * Constructs sorting rules based on the table state.
     * @param state Table state
     */
    buildSortRulesMap(state = this.getTableState()) {
        this.sortRules$.set(new Map(state.sortBy.filter((rule) => rule.field).map((rule) => [rule.field, rule])));
    }
    /**
     * Constructs filter rules based on the table state.
     * @param state Table state.
     */
    buildFilterRulesMap(state = this.getTableState()) {
        const filterRulesWithFieldNames = state.filterBy.map((rule) => ({
            ...rule,
            fieldName: this._getFieldName(rule.field, rule.fieldName)
        }));
        this.filterRules$.set(filterRulesWithFieldNames.reduce((hash, rule) => {
            const key = rule.field;
            if (!hash.has(key)) {
                hash.set(key, []);
            }
            hash.get(key)?.push(rule);
            return hash;
        }, new Map()));
        const evt = { current: filterRulesWithFieldNames, previous: state.filterBy };
        this.stateChange$.next({ type: 'filter', state: evt });
    }
    /** Reset all filter rules */
    resetFilters() {
        const prevState = this.getTableState();
        const prevFilterRules = (prevState && prevState.filterBy) || [];
        const newFilterRules = [];
        const state = { ...prevState, filterBy: newFilterRules };
        if (!equal(prevFilterRules, state.filterBy)) {
            this.setTableState(setCurrentPageToState(state, 1));
            const evt = { current: state.filterBy, previous: prevFilterRules };
            this.buildFilterRulesMap();
            this.needFetch$.next();
            this.stateChange$.next({ type: 'filter', state: evt });
            this.filterChange$.next(evt);
        }
    }
    /**
     * Constructs base table config.
     */
    constructTableMetadata() {
        this.buildGroupRulesMap();
        this.buildSortRulesMap();
        this.buildFilterRulesMap();
    }
    /** @hidden */
    _getFieldName(field, fieldName) {
        const column = this.tableColumns$.getValue().find((col) => col.key === field);
        return column ? column.name : fieldName ?? field;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableService, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableService }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableService, decorators: [{
            type: Injectable
        }] });
/** @hidden */
function setCurrentPageToState(state, currentPage) {
    const newPageState = { ...state.page, currentPage };
    return { ...state, page: newPageState };
}
/** @hidden */
function setPageSizeToState(state, pageSize) {
    const newPageState = { ...state.page, pageSize };
    return { ...state, page: newPageState };
}

class TableResponsiveService {
    /** @hidden */
    constructor(_table, _tableService, _elmRef) {
        this._table = _table;
        this._tableService = _tableService;
        this._elmRef = _elmRef;
        /** @hidden */
        this._responsiveBreakpoints = new Map();
        /** @hidden */
        this._destroyRef = inject(DestroyRef);
    }
    /**
     * Registers responsive column with it's breakpoint configuration.
     * @param column Table Column
     * @param breakpoints Breakpoint configuration.
     */
    registerResponsiveColumn(column, breakpoints) {
        this._responsiveBreakpoints.set(column, this._normalizeBreakpoints(breakpoints));
        this._listenToTableResize();
    }
    /**
     * @hidden
     * Transforms plain breakpoints into a list with minimal/maximal element width with it's visibility state.
     */
    _normalizeBreakpoints(breakpoints) {
        const sortedBreakpoints = Object.keys(breakpoints)
            .map((i) => parseInt(i, 10))
            .sort((a, b) => a - b);
        const newBreakpoints = sortedBreakpoints.reduce((result, value, index, array) => {
            result.push({
                min: value,
                max: index + 1 < array.length ? array[index + 1] - 1 : Infinity,
                visibility: breakpoints[value.toString()]
            });
            return result;
        }, []);
        return newBreakpoints;
    }
    /**
     * @hidden
     * Listens to the table element resize and calculates columns visibility based on the current breakpoint.
     */
    _listenToTableResize() {
        this._resizeSubscription?.unsubscribe();
        this._resizeSubscription = resizeObservable(this._elmRef.nativeElement)
            .pipe(debounceTime(20), takeUntilDestroyed(this._destroyRef))
            .subscribe(() => {
            this._processResponsiveColumns();
        });
    }
    /** @hidden */
    _processResponsiveColumns() {
        let shouldNotify = false;
        const { width } = this._elmRef.nativeElement.getBoundingClientRect();
        this._responsiveBreakpoints.forEach((breakpoints, item) => {
            const fittingBreakpoint = breakpoints.find((breakpoint) => breakpoint.min <= width && breakpoint.max >= width);
            if (fittingBreakpoint) {
                shouldNotify = shouldNotify || item.responsiveState !== fittingBreakpoint.visibility;
                item.responsiveState = fittingBreakpoint.visibility;
            }
        });
        // Notify only when there's actual visibility changes to any of the columns.
        if (!shouldNotify) {
            return;
        }
        const currentTableColumns = this._table.getVisibleTableColumns().map((column) => column.name);
        this._tableService.stateChange$.next({
            type: 'columns',
            state: { previous: currentTableColumns, current: currentTableColumns }
        });
        this._tableService.columnsChange$.next({ previous: currentTableColumns, current: currentTableColumns });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableResponsiveService, deps: [{ token: Table }, { token: TableService }, { token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableResponsiveService }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableResponsiveService, decorators: [{
            type: Injectable
        }], ctorParameters: () => [{ type: Table, decorators: [{
                    type: Inject,
                    args: [Table]
                }] }, { type: TableService }, { type: i0.ElementRef }] });

class TableColumn {
}

class PlatformTableColumnResponsiveDirective {
    /**
     * Responsive breakpoints configuration.
     * @param value where key is minimal table with and value is a column visibility type.
     */
    set breakpoints(value) {
        if (equal(value, this._breakpoints) || !this._column) {
            return;
        }
        this._breakpoints = value;
        this._responsiveTableService.registerResponsiveColumn(this._column, this._breakpoints);
    }
    get breakpoints() {
        return this._breakpoints;
    }
    /** @hidden */
    constructor(_responsiveTableService, _column) {
        this._responsiveTableService = _responsiveTableService;
        this._column = _column;
        /** @hidden */
        this._breakpoints = {};
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: PlatformTableColumnResponsiveDirective, deps: [{ token: TableResponsiveService }, { token: TableColumn, optional: true }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: PlatformTableColumnResponsiveDirective, isStandalone: true, selector: "[fdpTableColumnResponsive]", inputs: { breakpoints: ["fdpTableColumnResponsive", "breakpoints"] }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: PlatformTableColumnResponsiveDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[fdpTableColumnResponsive]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: TableResponsiveService }, { type: TableColumn, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [TableColumn]
                }] }], propDecorators: { breakpoints: [{
                type: Input,
                args: ['fdpTableColumnResponsive']
            }] } });

class TableCellHeaderPopoverDirective {
    /** @hidden */
    constructor(templateRef) {
        this.templateRef = templateRef;
    }
    /** @hidden */
    static ngTemplateContextGuard(directive, context) {
        return true;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableCellHeaderPopoverDirective, deps: [{ token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: TableCellHeaderPopoverDirective, isStandalone: true, selector: "[fdpTableCellHeaderPopover]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableCellHeaderPopoverDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[fdpTableCellHeaderPopover]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: i0.TemplateRef }] });

const TABLE_SCROLLABLE = new InjectionToken('Table Scrollable');
class TableScrollDispatcherService {
    constructor() {
        /** @hidden */
        this._scrollSubject = new Subject();
        /** @hidden */
        this._verticalScrollSubject = new Subject();
        /** @hidden */
        this._horizontalScrollSubject = new Subject();
        /** @hidden */
        this._scrollableSubscriptionsMap = new Map();
    }
    /** @hidden */
    register(scrollable) {
        if (this._scrollableSubscriptionsMap.has(scrollable)) {
            return;
        }
        const sub = new Subscription();
        sub.add(scrollable.getScrollStream().subscribe(() => this._scrollSubject.next(scrollable)));
        sub.add(scrollable.getHorizontalScrollStream().subscribe(() => this._horizontalScrollSubject.next(scrollable)));
        sub.add(scrollable.getVerticalScrollStream().subscribe(() => this._verticalScrollSubject.next(scrollable)));
        this._scrollableSubscriptionsMap.set(scrollable, sub);
    }
    /** @hidden */
    deregister(scrollable) {
        if (!this._scrollableSubscriptionsMap.has(scrollable)) {
            return;
        }
        this._scrollableSubscriptionsMap.get(scrollable)?.unsubscribe();
    }
    /** Scroll stream */
    scrolled() {
        return this._scrollSubject.asObservable();
    }
    /** Horizontal scroll stream */
    horizontallyScrolled() {
        return this._horizontalScrollSubject.asObservable();
    }
    /** Vertical scroll stream */
    verticallyScrolled() {
        return this._verticalScrollSubject.asObservable();
    }
    /** @hidden */
    ngOnDestroy() {
        Array.from(this._scrollableSubscriptionsMap.values()).forEach((subscription) => subscription.unsubscribe());
        this._scrollableSubscriptionsMap.clear();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableScrollDispatcherService, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableScrollDispatcherService }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableScrollDispatcherService, decorators: [{
            type: Injectable
        }] });

const TABLE_RESIZER_BORDER_WIDTH = 3;
/**
 * Service to handle all things related to column resizing:
 * - Setting resizer
 * - Processing resize
 * - Calculating the real columns width
 */
class TableColumnResizeService {
    /** Indicate if resizing process in progress. */
    get resizeInProgress() {
        return this._resizeInProgress;
    }
    /** Current column resizer position. */
    get resizerPosition() {
        return this._resizerPosition ?? 0;
    }
    /** Observable to notify to run CD */
    get markForCheck() {
        return this._markForCheck.asObservable();
    }
    /** table has fixed width if all of it's columns are fixed */
    get fixedWidth() {
        return this._fixedColumnsWidthMap.size === this._visibleColumnNames.length;
    }
    /** @hidden */
    constructor(_tableScrollDispatcherService, _rtlService) {
        this._tableScrollDispatcherService = _tableScrollDispatcherService;
        this._rtlService = _rtlService;
        /** Subject that emits new map of fixed columns and their respective width. */
        this.fixedColumsWidthChange = new BehaviorSubject(new Map());
        /** Resize progress stream. */
        this.resizeInProgress$ = new BehaviorSubject(false);
        /** Whether cell mock should be visible. */
        this.cellMockVisible$ = signal(false);
        /** Resizer position stream. */
        this.resizerPosition$ = new BehaviorSubject(0);
        /** @hidden */
        this._fixedColumnsWidthMap = new Map();
        /** @hidden */
        this._columnsCellMap = new Map();
        /** @hidden */
        this._visibleColumnNames = [];
        /** @hidden */
        this._visibleColumnLeftNeighbourMap = new Map();
        /** @hidden */
        this._visibleColumnRightNeighbourMap = new Map();
        /** @hidden */
        this._visibleColumnLeftOffsetPxMap = new Map();
        /** @hidden */
        this._visibleColumnRightOffsetPxMap = new Map();
        /** @hidden */
        this._startX = null;
        /** @hidden */
        this._clientStartX = null;
        /** @hidden */
        this._resizeInProgress = this.resizeInProgress$.value;
        /** @hidden */
        this._resizerPosition = null;
        /** @hidden */
        this._scrollLeft = 0;
        /** @hidden */
        this._markForCheck = new Subject();
        /** @hidden */
        this._destroyed = inject(DestroyRef);
        /** @hidden */
        this._resizerMoveSubscription = new Subscription();
        /** @hidden */
        this._rtl$ = computed(() => !!this._rtlService?.rtlSignal());
        this._tableScrollDispatcherService
            ?.horizontallyScrolled()
            .pipe(takeUntilDestroyed(this._destroyed))
            .subscribe((scrollable) => {
            this.resizerPosition$.next(0);
            this._scrollLeft = scrollable.getScrollLeft();
        });
    }
    /** @hidden */
    ngOnDestroy() {
        this._resizeInProgress = false;
        this.resizeInProgress$.next(false);
        this._resizerMoveSubscription.unsubscribe();
    }
    /** @hidden */
    setTableRef(ref) {
        this._tableRef = ref;
    }
    /** @hidden initializes service with data, trigger columns width calculation. */
    setColumnNames(visibleColumnNames) {
        this._visibleColumnLeftNeighbourMap.clear();
        this._visibleColumnRightNeighbourMap.clear();
        visibleColumnNames.forEach((column, index) => {
            this._visibleColumnLeftNeighbourMap.set(column, visibleColumnNames[index - 1]);
        });
        visibleColumnNames.reverse().forEach((column, index) => {
            this._visibleColumnRightNeighbourMap.set(column, visibleColumnNames[index - 1]);
        });
        this._visibleColumnNames = [...visibleColumnNames];
        this.updateFrozenColumnsWidth();
    }
    /** @hidden */
    updateFrozenColumnsWidth() {
        let allPreviousWidths = 0, allNextWidths = 0;
        for (const [column, prevColumn] of this._visibleColumnLeftNeighbourMap.entries()) {
            allPreviousWidths +=
                this._columnsCellMap.get(prevColumn)?.[0]?.nativeElement.getBoundingClientRect().width ?? 0;
            this._visibleColumnLeftOffsetPxMap.set(column, allPreviousWidths);
        }
        for (const [column, nextColumn] of this._visibleColumnRightNeighbourMap.entries()) {
            allNextWidths +=
                this._columnsCellMap.get(nextColumn)?.[0]?.nativeElement.getBoundingClientRect().width ?? 0;
            this._visibleColumnRightOffsetPxMap.set(column, allNextWidths);
        }
    }
    /** @hidden */
    updateFrozenColumnsWidthAfterResize(columnName, diffX) {
        let found = false;
        for (const [column, width] of this._visibleColumnLeftOffsetPxMap.entries()) {
            if (column === columnName) {
                found = true;
            }
            else if (found) {
                this._visibleColumnLeftOffsetPxMap.set(column, width + diffX);
            }
        }
    }
    /** Retrieves custom column value or returns `unset` */
    getColumnWidthStyle(columnName) {
        const calculatedWidth = this._fixedColumnsWidthMap.get(columnName);
        return calculatedWidth || 'unset';
    }
    /** Previous column name */
    getPreviousColumnName(columnName) {
        return this._visibleColumnLeftNeighbourMap.get(columnName);
    }
    /** Previous column name */
    getNextColumnName(columnName) {
        return this._visibleColumnRightNeighbourMap.get(columnName);
    }
    /** Overall previous columns width. Used to calculate offset for the absolute positioned cells. */
    getPrevColumnsWidth(columnName) {
        return this._visibleColumnLeftOffsetPxMap.get(columnName) ?? 0;
    }
    /** Overall next columns width. Used to calculate offset for the absolute positioned cells. */
    getNextColumnsWidth(columnName) {
        return this._visibleColumnRightOffsetPxMap.get(columnName) ?? 0;
    }
    /** Register column's cell to get its dimensions in further. */
    registerColumnCell(columnName, cellElRef) {
        const columnCells = this._columnsCellMap.get(columnName) || [];
        this._columnsCellMap.set(columnName, [cellElRef, ...columnCells]);
    }
    /** Unregister column's cell. */
    unregisterColumnCell(columnName, cellElRef) {
        const columnCells = this._columnsCellMap.get(columnName) || [];
        const elmIndex = columnCells.findIndex((e) => e.nativeElement === cellElRef.nativeElement);
        if (elmIndex === -1) {
            return;
        }
        columnCells.splice(elmIndex, 1);
        this._columnsCellMap.set(columnName, columnCells);
    }
    /** Register the value of column width that changed from  input */
    setCustomWidth(columnName, value) {
        this._fixedColumnsWidthMap.set(columnName, value);
        this.fixedColumsWidthChange.next(this._fixedColumnsWidthMap);
    }
    /** Set the appropriate column resizer position. */
    setInitialResizerPosition(resizerPosition, resizedColumn) {
        if (this.resizeInProgress) {
            return;
        }
        this._resizedColumn = resizedColumn;
        this._resizerPosition = null;
        this.resizerPosition$.next(this.resizerPosition);
        if (resizerPosition != null) {
            const scrollLeftOffset = this._scrollLeft * (this._rtl$() ? 1 : -1);
            this._resizerPosition = resizerPosition - TABLE_RESIZER_BORDER_WIDTH + scrollLeftOffset;
            this.resizerPosition$.next(this.resizerPosition);
        }
        this._markForCheck.next();
    }
    /** Hide the column resizer. */
    hideResizer() {
        if (this._resizeInProgress) {
            return;
        }
        this._resizerPosition = null;
        this.resizerPosition$.next(this.resizerPosition);
        this._markForCheck.next();
    }
    /** Handle start resizing. */
    startResize(event) {
        this._startX = this.resizerPosition;
        this._clientStartX = event.clientX;
        this._updateResizerPositionOnMouseMove();
        this._resizeInProgress = true;
        this.resizeInProgress$.next(true);
    }
    /** Handle end resizing. */
    finishResize(event) {
        this._resizerMoveSubscription.unsubscribe();
        this._resizeInProgress = false;
        this.resizeInProgress$.next(false);
        this._resizerPosition = null;
        this.resizerPosition$.next(this.resizerPosition);
        if (this._startX != null) {
            const clientStartX = this._clientStartX ?? 0;
            const diffX = this._rtl$() ? clientStartX - event.clientX : event.clientX - clientStartX;
            this._processResize(diffX);
        }
        this._startX = null;
        this._clientStartX = null;
        this._markForCheck.next();
    }
    /** Update columns width after resizing, prevent from having too small columns. */
    _processResize(diffX) {
        const resizedElement = this._columnsCellMap.get(this._resizedColumn)?.[0]?.nativeElement;
        const columnWidth = resizedElement?.getBoundingClientRect().width;
        if (!columnWidth || !resizedElement) {
            return;
        }
        if (diffX < 0 && columnWidth + diffX < this._tableRef.minimumColumnWidth) {
            diffX = this._tableRef.minimumColumnWidth - columnWidth;
        }
        if (!this.fixedWidth) {
            for (const [columnName, cells] of this._columnsCellMap.entries()) {
                if (cells.length) {
                    this._fixedColumnsWidthMap.set(columnName, cells[0].nativeElement.getBoundingClientRect().width + 'px');
                }
            }
            this.fixedColumsWidthChange.next(this._fixedColumnsWidthMap);
        }
        /**
         * In case "_resizedColumn" is freezable, make sure the overall width of freezable columns does not exceed the width of the table.
         * If it does, columns will be made unfrozen.
         */
        if (diffX > 0 &&
            (this._tableRef._freezableColumns.has(this._resizedColumn) ||
                this._tableRef._freezableEndColumns?.has(this._resizedColumn))) {
            let actualWidth = this.getPrevColumnsWidth(this._resizedColumn) + columnWidth;
            if (this._tableRef._freezableEndColumns?.has(this._resizedColumn)) {
                actualWidth = this.getNextColumnsWidth(this._resizedColumn) + columnWidth;
            }
            const newWidth = actualWidth + diffX;
            const maxWidth = this._tableRef.getMaxAllowedFreezableColumnsWidth();
            if (newWidth >= maxWidth) {
                this._tableRef._freezableColumns.forEach((column) => {
                    this._tableRef.unfreeze(column.toString());
                });
            }
        }
        const updatedWidth = columnWidth + diffX;
        this._fixedColumnsWidthMap.set(this._resizedColumn, updatedWidth + 'px');
        this.fixedColumsWidthChange.next(this._fixedColumnsWidthMap);
        this.updateFrozenColumnsWidthAfterResize(this._resizedColumn, diffX);
        const computedStyles = window.getComputedStyle(resizedElement);
        const padding = parseInt(computedStyles.paddingLeft, 10) + parseInt(computedStyles.paddingRight, 10);
        this._updateHeaderOverflowState(updatedWidth - padding);
        this._markForCheck.next();
    }
    /** Update column resizer position. */
    _updateResizerPositionOnMouseMove() {
        this._resizerMoveSubscription = fromEvent(document, 'mousemove')
            .pipe(debounceTime(10))
            .subscribe((event) => {
            const clientStartX = this._clientStartX ?? 0;
            const diffX = this._rtl$() ? clientStartX - event.clientX : event.clientX - clientStartX;
            this._resizerPosition = (this._startX ?? 0) + diffX;
            this.resizerPosition$.next(this.resizerPosition);
            this._markForCheck.next();
        });
    }
    /**
     * Check if the header text length goes over the column width. In such case applies text truncation with
     * ellipsis and make sure we also show a tooltip.
     *
     * @private
     */
    _updateHeaderOverflowState(updatedWidth) {
        const element = this._columnsCellMap.get(this._resizedColumn)?.find((c) => c?.nativeElement.tagName === 'TH')
            ?.nativeElement;
        const visibleColumn = this._tableRef.getVisibleTableColumns().find((c) => c.name === this._resizedColumn);
        const headerTextElem = element?.querySelector(`#${this._tableRef.id}-header-cell-${this._resizedColumn}`);
        if (!headerTextElem || !visibleColumn) {
            return;
        }
        visibleColumn.headerOverflows = headerTextElem.scrollWidth > updatedWidth;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableColumnResizeService, deps: [{ token: TableScrollDispatcherService }, { token: i2.RtlService, optional: true }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableColumnResizeService }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableColumnResizeService, decorators: [{
            type: Injectable
        }], ctorParameters: () => [{ type: TableScrollDispatcherService }, { type: i2.RtlService, decorators: [{
                    type: Optional
                }] }] });

const TABLE_CELL_RESIZABLE_THRESHOLD_PX = 4;
/**
 * Tracks mouse movement over the cell if the mouse pointer near the side of the cell, informs resize service.
 */
class PlatformTableCellResizableDirective extends TableCellDirective {
    constructor() {
        super(...arguments);
        /** @hidden */
        this._resizableSide = 'both';
        /** @hidden */
        this._rtl$ = computed(() => !!this._rtlService?.rtlSignal());
        /** @hidden */
        this._tableColumnResizeService = inject(TableColumnResizeService);
        /** @hidden */
        this._rtlService = inject(RtlService, {
            optional: true
        });
    }
    /** First column can be resized only by its end */
    set resizableSide(value) {
        if (!value) {
            return;
        }
        this._resizableSide = value;
    }
    /** @hidden */
    ngOnInit() {
        this._zone.runOutsideAngular(() => {
            fromEvent(this.elementRef.nativeElement, 'mousemove')
                .pipe(filter(() => this._tableColumnResizeService?.resizeInProgress !== true), debounceTime(5), map((event) => this._getResizer(event) || { resizerPosition: 0, resizedColumn: this.columnName }), distinctUntilChanged((prev, curr) => equal(prev, curr)), takeUntilDestroyed(this._destroyRef))
                .subscribe((data) => {
                this._tableColumnResizeService.setInitialResizerPosition(data.resizerPosition, data.resizedColumn);
            });
            fromEvent(this.elementRef.nativeElement, 'focus')
                .pipe(takeUntilDestroyed(this._destroyRef))
                .subscribe(() => {
                this._tableColumnResizeService.setInitialResizerPosition(0, this.columnName);
            });
        });
    }
    /** @hidden */
    ngAfterViewInit() {
        if (this.columnName == null) {
            return;
        }
        this._tableColumnResizeService?.registerColumnCell(this.columnName, this.elementRef);
    }
    /** @hidden */
    ngOnDestroy() {
        this._tableColumnResizeService?.unregisterColumnCell(this.columnName, this.elementRef);
    }
    /** @hidden */
    _getResizer(event) {
        const el = this.elementRef.nativeElement;
        const elPosition = el.getBoundingClientRect();
        // check if mouse is within 5px of cell border
        if (event.clientX > elPosition.x + elPosition.width - 5 || event.clientX < elPosition.x + 5) {
            let resizerPosition;
            let resizedColumn;
            const pointerOnLeft = this._rtl$()
                ? elPosition.right - event.clientX < TABLE_CELL_RESIZABLE_THRESHOLD_PX
                : event.clientX - elPosition.left < TABLE_CELL_RESIZABLE_THRESHOLD_PX;
            if (pointerOnLeft && this._resizableSide !== 'end') {
                resizerPosition = this._rtl$()
                    ? el.parentElement.offsetWidth - (el.offsetLeft + el.offsetWidth)
                    : el.offsetLeft;
                resizedColumn = this._tableColumnResizeService.getPreviousColumnName(this.columnName);
            }
            const pointerOnRight = this._rtl$()
                ? event.clientX - elPosition.left < TABLE_CELL_RESIZABLE_THRESHOLD_PX
                : elPosition.right - event.clientX < TABLE_CELL_RESIZABLE_THRESHOLD_PX;
            if (pointerOnRight && this._resizableSide !== 'start') {
                resizerPosition = this._rtl$()
                    ? el.parentElement.offsetWidth - el.offsetLeft
                    : el.offsetLeft + el.offsetWidth;
                resizedColumn = this.columnName;
            }
            if (!resizedColumn) {
                return null;
            }
            return { resizerPosition: resizerPosition, resizedColumn };
        }
        else {
            return null;
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: PlatformTableCellResizableDirective, deps: null, target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: PlatformTableCellResizableDirective, isStandalone: true, selector: "[fdpTableCellResizable]", inputs: { resizableSide: ["fdpTableCellResizable", "resizableSide"], columnName: "columnName" }, providers: [
            {
                provide: FDK_FOCUSABLE_ITEM_DIRECTIVE,
                useExisting: PlatformTableCellResizableDirective
            },
            {
                provide: FocusableItemDirective,
                useExisting: PlatformTableCellResizableDirective
            },
            {
                provide: TableCellDirective,
                useExisting: PlatformTableCellResizableDirective
            }
        ], usesInheritance: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: PlatformTableCellResizableDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[fdpTableCellResizable]',
                    standalone: true,
                    providers: [
                        {
                            provide: FDK_FOCUSABLE_ITEM_DIRECTIVE,
                            useExisting: PlatformTableCellResizableDirective
                        },
                        {
                            provide: FocusableItemDirective,
                            useExisting: PlatformTableCellResizableDirective
                        },
                        {
                            provide: TableCellDirective,
                            useExisting: PlatformTableCellResizableDirective
                        }
                    ]
                }]
        }], propDecorators: { resizableSide: [{
                type: Input,
                args: ['fdpTableCellResizable']
            }], columnName: [{
                type: Input
            }] } });

class EditableTableCell {
}

/** Column cell container. */
// eslint-disable-next-line @angular-eslint/directive-selector
// eslint-disable-next-line @angular-eslint/directive-class-suffix
class FdpTableCell {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: FdpTableCell, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: FdpTableCell, isStandalone: true, selector: "fdp-table-cell", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: FdpTableCell, decorators: [{
            type: Directive,
            args: [{ selector: 'fdp-table-cell', standalone: true }]
        }] });
/**
 * Cell definition for a platform table.
 * Captures the template of a column's data row cell as well as cell-specific properties.
 */
// eslint-disable-next-line @angular-eslint/directive-class-suffix
class FdpCellDef {
    /** @hidden */
    constructor(templateRef) {
        this.templateRef = templateRef;
    }
    /** @hidden */
    static ngTemplateContextGuard(dir, ctx) {
        return true;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: FdpCellDef, deps: [{ token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: FdpCellDef, isStandalone: true, selector: "[fdpCellDef]", inputs: { fdpCellDefAs: "fdpCellDefAs" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: FdpCellDef, decorators: [{
            type: Directive,
            args: [{ selector: '[fdpCellDef]', standalone: true }]
        }], ctorParameters: () => [{ type: i0.TemplateRef }], propDecorators: { fdpCellDefAs: [{
                type: Input
            }] } });
// eslint-disable-next-line @angular-eslint/directive-class-suffix
class FdpEditableCellDef {
    /** @hidden */
    constructor(templateRef) {
        this.templateRef = templateRef;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: FdpEditableCellDef, deps: [{ token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: FdpEditableCellDef, isStandalone: true, selector: "[fdpEditableCellDef]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: FdpEditableCellDef, decorators: [{
            type: Directive,
            args: [{ selector: '[fdpEditableCellDef]', standalone: true }]
        }], ctorParameters: () => [{ type: i0.TemplateRef }] });
class FdpEditableCellFormDirective {
    /** @hidden */
    constructor(form) {
        this.form = form;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: FdpEditableCellFormDirective, deps: [{ token: i1.NgForm }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: FdpEditableCellFormDirective, isStandalone: true, selector: "[fdpEditableCellForm]", providers: [{ provide: EditableTableCell, useExisting: FdpEditableCellFormDirective }], ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: FdpEditableCellFormDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[fdpEditableCellForm]',
                    standalone: true,
                    providers: [{ provide: EditableTableCell, useExisting: FdpEditableCellFormDirective }]
                }]
        }], ctorParameters: () => [{ type: i1.NgForm }] });

var FilterType;
(function (FilterType) {
    FilterType["INPUT"] = "input";
    FilterType["SINGLE"] = "single-select";
    FilterType["MULTI"] = "multi-select";
    FilterType["CATEGORY"] = "category";
    FilterType["CUSTOM"] = "custom";
})(FilterType || (FilterType = {}));
var FilterableColumnDataType;
(function (FilterableColumnDataType) {
    FilterableColumnDataType["STRING"] = "string";
    FilterableColumnDataType["NUMBER"] = "number";
    FilterableColumnDataType["BOOLEAN"] = "boolean";
    FilterableColumnDataType["DATE"] = "date";
})(FilterableColumnDataType || (FilterableColumnDataType = {}));

/** All possible strategies */
const FILTER_STRATEGY = {
    EQ: 'equalTo',
    CONTAINS: 'contains',
    BETWEEN: 'between',
    BEGINS_WITH: 'beginsWith',
    ENDS_WITH: 'endsWith',
    GT: 'greaterThan',
    GTE: 'greaterThanOrEqualTo',
    LT: 'lessThan',
    LTE: 'lessThanOrEqualTo',
    AFTER: 'after',
    ON_OR_AFTER: 'onOrAfter',
    BEFORE: 'before',
    BEFORE_OR_ON: 'beforeOrOn'
};
const FILTER_STRATEGY_LABEL = {
    equalTo: 'platformTable.P13FilterStrategyLabelEqualTo',
    contains: 'platformTable.P13FilterStrategyLabelContains',
    between: 'platformTable.P13FilterStrategyLabelBetween',
    beginsWith: 'platformTable.P13FilterStrategyLabelBeginsWith',
    endsWith: 'platformTable.P13FilterStrategyLabelEndsWith',
    greaterThan: 'platformTable.P13FilterStrategyLabelGreaterThan',
    greaterThanOrEqualTo: 'platformTable.P13FilterStrategyLabelGreaterThanOrEqualTo',
    lessThan: 'platformTable.P13FilterStrategyLabelLessThan',
    lessThanOrEqualTo: 'platformTable.P13FilterStrategyLabelLessThanOrEqualTo',
    after: 'platformTable.P13FilterStrategyLabelAfter',
    onOrAfter: 'platformTable.P13FilterStrategyLabelOnOrAfter',
    before: 'platformTable.P13FilterStrategyLabelBefore',
    beforeOrOn: 'platformTable.P13FilterStrategyLabelBeforeOrOn'
};
/** Date Filter Strategies */
const FILTER_DATE_STRATEGY = {
    EQ: 'equalTo',
    BETWEEN: 'between',
    AFTER: 'after',
    ON_OR_AFTER: 'onOrAfter',
    BEFORE: 'before',
    BEFORE_OR_ON: 'beforeOrOn'
};
const FILTER_DATE_STRATEGIES = Object.values(FILTER_DATE_STRATEGY);
/** Number Filter Strategies */
const FILTER_NUMBER_STRATEGY = {
    EQ: 'equalTo',
    BETWEEN: 'between',
    GT: 'greaterThan',
    GTE: 'greaterThanOrEqualTo',
    LT: 'lessThan',
    LTE: 'lessThanOrEqualTo'
};
const FILTER_NUMBER_STRATEGIES = Object.values(FILTER_NUMBER_STRATEGY);
/** String Filter Strategies */
const FILTER_STRING_STRATEGY = {
    CONTAINS: 'contains',
    EQ: 'equalTo',
    BEGINS_WITH: 'beginsWith',
    BETWEEN: 'between',
    ENDS_WITH: 'endsWith',
    GT: 'greaterThan',
    GTE: 'greaterThanOrEqualTo',
    LT: 'lessThan',
    LTE: 'lessThanOrEqualTo'
};
const FILTER_STRING_STRATEGIES = Object.values(FILTER_STRING_STRATEGY);
/** Boolean Filter Strategies */
const FILTER_BOOLEAN_STRATEGY = {
    EQ: 'equalTo'
};
const FILTER_BOOLEAN_STRATEGIES = Object.values(FILTER_BOOLEAN_STRATEGY);
/** Default Filter Strategies for undefined data type */
const FILTER_DEFAULT_STRATEGY = {
    EQ: 'equalTo'
};
const FILTER_DEFAULT_STRATEGIES = Object.values(FILTER_DEFAULT_STRATEGY);
// Get Possible strategies based on data type
const getFilterStrategiesBasedOnDataType = (dataType) => {
    switch (dataType) {
        case FilterableColumnDataType.STRING:
            return FILTER_STRING_STRATEGIES;
        case FilterableColumnDataType.NUMBER:
            return FILTER_NUMBER_STRATEGIES;
        case FilterableColumnDataType.DATE:
            return FILTER_DATE_STRATEGIES;
        case FilterableColumnDataType.BOOLEAN:
            return FILTER_BOOLEAN_STRATEGIES;
        default:
            return FILTER_DEFAULT_STRATEGIES;
    }
};

var ColumnAlign;
(function (ColumnAlign) {
    ColumnAlign["START"] = "start";
    ColumnAlign["CENTER"] = "center";
    ColumnAlign["END"] = "end";
})(ColumnAlign || (ColumnAlign = {}));

var TableRowType;
(function (TableRowType) {
    TableRowType["ITEM"] = "item";
    TableRowType["GROUP"] = "group";
    TableRowType["TREE"] = "tree";
})(TableRowType || (TableRowType = {}));

var SelectionMode;
(function (SelectionMode) {
    SelectionMode["SINGLE"] = "single";
    SelectionMode["MULTIPLE"] = "multiple";
    SelectionMode["NONE"] = "none";
})(SelectionMode || (SelectionMode = {}));

var SortDirection;
(function (SortDirection) {
    SortDirection["NONE"] = "none";
    SortDirection["ASC"] = "asc";
    SortDirection["DESC"] = "desc";
})(SortDirection || (SortDirection = {}));

const filterByString = (rows, filter) => {
    const filterValue = filter.value && filter.value.toLocaleLowerCase();
    const filterValue2 = (filter.value2 && filter.value2.toLocaleLowerCase()) || '';
    switch (filter.strategy) {
        case FILTER_STRING_STRATEGY.EQ:
            return rows.filter((r) => get(r.value, filter.field).toLocaleLowerCase() === filterValue);
        case FILTER_STRING_STRATEGY.GT:
            return rows.filter((r) => get(r.value, filter.field).toLocaleLowerCase() > filterValue);
        case FILTER_STRING_STRATEGY.GTE:
            return rows.filter((r) => get(r.value, filter.field).toLocaleLowerCase() >= filterValue);
        case FILTER_STRING_STRATEGY.LT:
            return rows.filter((r) => get(r.value, filter.field).toLocaleLowerCase() < filterValue);
        case FILTER_STRING_STRATEGY.LTE:
            return rows.filter((r) => get(r.value, filter.field).toLocaleLowerCase() <= filterValue);
        case FILTER_STRING_STRATEGY.BETWEEN:
            return rows.filter((r) => {
                const rowValue = get(r.value, filter.field).toLocaleLowerCase();
                return rowValue >= filterValue && rowValue <= filterValue2;
            });
        case FILTER_STRING_STRATEGY.BEGINS_WITH:
            return rows.filter((r) => get(r.value, filter.field).toLocaleLowerCase().startsWith(filterValue));
        case FILTER_STRING_STRATEGY.ENDS_WITH:
            return rows.filter((r) => get(r.value, filter.field).toLocaleLowerCase().endsWith(filterValue));
        case FILTER_STRING_STRATEGY.CONTAINS:
        default:
            return rows.filter((r) => {
                let rowValue = get(r.value, filter.field);
                if (typeof rowValue === 'number') {
                    rowValue = rowValue.toString();
                }
                return rowValue.toLocaleLowerCase().includes(filterValue);
            });
    }
};
const filterByNumber = (rows, filter) => {
    const filterValue = filter.value;
    const filterValue2 = filter.value2 || 0;
    switch (filter.strategy) {
        case FILTER_NUMBER_STRATEGY.EQ:
            return rows.filter((r) => get(r.value, filter.field) === filterValue);
        case FILTER_NUMBER_STRATEGY.GT:
            return rows.filter((r) => get(r.value, filter.field) > filterValue);
        case FILTER_NUMBER_STRATEGY.GTE:
            return rows.filter((r) => get(r.value, filter.field) >= filterValue);
        case FILTER_NUMBER_STRATEGY.LT:
            return rows.filter((r) => get(r.value, filter.field) < filterValue);
        case FILTER_NUMBER_STRATEGY.LTE:
            return rows.filter((r) => get(r.value, filter.field) <= filterValue);
        case FILTER_NUMBER_STRATEGY.BETWEEN:
            return rows.filter((r) => {
                const rowValue = get(r.value, filter.field);
                return rowValue >= filterValue && rowValue <= filterValue2;
            });
        default:
            return rows.filter((r) => get(r.value, filter.field).toString().includes(filterValue));
    }
};
const filterByDate = (rows, filter, adapter) => {
    const filterValue = filter.value;
    const filterValue2 = filter.value2;
    switch (filter.strategy) {
        case FILTER_DATE_STRATEGY.AFTER:
            return rows.filter((r) => adapter.compareDate(get(r.value, filter.field), filterValue) > 0);
        case FILTER_DATE_STRATEGY.ON_OR_AFTER:
            return rows.filter((r) => adapter.compareDate(get(r.value, filter.field), filterValue) >= 0);
        case FILTER_DATE_STRATEGY.BEFORE:
            return rows.filter((r) => adapter.compareDate(get(r.value, filter.field), filterValue) < 0);
        case FILTER_DATE_STRATEGY.BEFORE_OR_ON:
            return rows.filter((r) => adapter.compareDate(get(r.value, filter.field), filterValue) <= 0);
        case FILTER_DATE_STRATEGY.BETWEEN:
            return rows.filter((r) => adapter.isBetween(get(r.value, filter.field), filterValue, filterValue2));
        case FILTER_DATE_STRATEGY.EQ:
        default:
            return rows.filter((r) => adapter.dateTimesEqual(get(r.value, filter.field), filterValue));
    }
};
const filterByBoolean = (rows, filter) => rows.filter((r) => get(r.value, filter.field) === filter.value);
const getUniqueListValuesByKey = (list, key) => Array.from(list
    .reduce((map, item) => {
    map.delete(item[key]);
    map.set(item[key], item);
    return map;
}, new Map())
    .values());
const getScrollBarWidth = (document) => {
    const div = document.createElement('div');
    div.textContent = 'W';
    div.style.position = 'absolute';
    div.style.visibility = 'hidden';
    div.style.overflowY = 'scroll';
    document.body.appendChild(div);
    const scrollbarWidth = div.offsetWidth - div.clientWidth;
    document.body.removeChild(div);
    return scrollbarWidth;
};
/** @hidden */
function isCollectionFilter(item) {
    return item.type !== undefined && item.strategy !== undefined && item.value !== undefined;
}

class BaseTableDataProvider {
    constructor() {
        /** @hidden */
        this._destroy$ = new Subject();
    }
    /**
     * Method for setting external filtering conditions.
     * @param filterBy Set of column filters.
     * @param searchInput Search condition from search field.
     */
    setFilters(filterBy, searchInput) {
        this.filterBy = filterBy;
        this.searchInput = searchInput;
    }
    /**
     * Method for filtering the data.
     * @param items Array of data source items.
     * @param filters Set of column filters.
     * @returns Array of filtered items.
     */
    applyFiltering(items, filters) {
        items = items.filter((i) => this.getFilteringStrategy(i, filters));
        return items;
    }
    /** @hidden */
    getSelectItemValue(item) {
        return isSelectItem(item) ? item.value : item;
    }
    /**
     * Method which selects appropriate filtering strategy of the field depending on the filter type.
     * Developers can override this method to extend the filtering functionality.
     * @param item item of the data source.
     * @param filters Set of column filters.
     * @returns Whether or not item should be included in data array.
     */
    getFilteringStrategy(item, filters) {
        return filters
            .filter((condition) => condition.field)
            .every((filter) => {
            if (isCollectionFilter(filter)) {
                return this.collectionFilterStrategy(item, filter);
            }
            return this.collectionFilterGroupStrategy(item, filter);
        });
    }
    /**
     * Method which filters item depending on applied condition result.
     * Developers can override this method to extend its functionality
     * @param item Item to apply conditions to.
     * @param filter Column filter.
     * @returns {boolean} Whether this item should be present in filtered array of items.
     */
    collectionFilterStrategy(item, filter) {
        let result;
        switch (filter.type) {
            case 'boolean':
                result = this.filterBoolean(item, filter);
                break;
            case 'number':
                result = this.filterNumber(item, filter);
                break;
            case 'date': {
                result = this.filterDate(item, filter, this.dateTimeAdapter);
                break;
            }
            case 'string':
            default:
                result = Array.isArray(filter.value)
                    ? this.filterArray(item, filter)
                    : this.filterString(item, filter);
                break;
        }
        return result;
    }
    /**
     * Method which applies group filtering conditions for the item.
     * @param item Item to apply conditions to.
     * @param filter Filter group.
     * @returns {boolean} Whether this item should be present in filtered items array.
     */
    collectionFilterGroupStrategy(item, filter) {
        return filter.strategy === 'and'
            ? filter.filters.every((f) => this.collectionFilterStrategy(item, f))
            : filter.filters.some((f) => this.collectionFilterStrategy(item, f));
    }
    /**
     * Applies search of the search term for visible table columns.
     * Developers can override this method to extend the filtering functionality.
     * @param items data source items array.
     * @param tableState @see TableState Set of table parameters.
     * @returns filtered data source items array.
     */
    search(items, { searchInput, columnKeys }) {
        const searchText = searchInput?.text || '';
        const keysToSearchBy = columnKeys;
        if (searchText.trim() === '' || keysToSearchBy.length === 0) {
            return items;
        }
        items = items.filter((item) => {
            const valuesForSearch = keysToSearchBy.map((key) => get(item, key));
            return valuesForSearch
                .filter((value) => !!value)
                .map((value) => value.toString())
                .some((value) => value.toLocaleLowerCase().includes(searchText.toLocaleLowerCase()));
        });
        return items;
    }
    /**
     * String filtering strategy
     * @param item data source array item.
     * @param filter filter object
     * @returns whether or not item should be included in data source array.
     */
    filterString(item, filter) {
        const filterValue = filter.value && filter.value.toLocaleLowerCase();
        const filterValue2 = (filter.value2 && filter.value2.toLocaleLowerCase()) || '';
        let itemValue = get(item, filter.field);
        itemValue = itemValue ? itemValue.toLocaleLowerCase() : itemValue;
        let result;
        switch (filter.strategy) {
            case 'equalTo':
                result = itemValue === filterValue;
                break;
            case 'greaterThan':
                result = itemValue > filterValue;
                break;
            case 'greaterThanOrEqualTo':
                result = itemValue >= filterValue;
                break;
            case 'lessThan':
                result = itemValue < filterValue;
                break;
            case 'lessThanOrEqualTo':
                result = itemValue <= filterValue;
                break;
            case 'between':
                result = itemValue >= filterValue && itemValue <= filterValue2;
                break;
            case 'beginsWith':
                result = itemValue.startsWith(filterValue);
                break;
            case 'endsWith':
                result = itemValue.endsWith(filterValue);
                break;
            case 'contains':
            default:
                result = itemValue ? itemValue.includes(filterValue) : itemValue === filterValue;
        }
        return filter.exclude ? !result : result;
    }
    /**
     * Number filtering strategy
     * @param item data source array item.
     * @param filter filter object
     * @returns whether or not item should be included in data source array.
     */
    filterNumber(item, filter) {
        const filterValue = Number.parseFloat(filter.value);
        const filterValue2 = Number.parseFloat(filter.value2) || 0;
        const itemValue = Number.parseFloat(get(item, filter.field));
        let result;
        switch (filter.strategy) {
            case 'greaterThan':
                result = itemValue > filterValue;
                break;
            case 'greaterThanOrEqualTo':
                result = itemValue >= filterValue;
                break;
            case 'lessThan':
                result = itemValue < filterValue;
                break;
            case 'lessThanOrEqualTo':
                result = itemValue <= filterValue;
                break;
            case 'between':
                result = itemValue >= filterValue && itemValue <= filterValue2;
                break;
            case 'equalTo':
            default:
                result = itemValue === filterValue;
        }
        return filter.exclude ? !result : result;
    }
    /**
     * Date filtering strategy
     * @param item data source array item.
     * @param filter filter object
     * @param adapter
     * @returns whether or not item should be included in data source array.
     */
    filterDate(item, filter, adapter) {
        if (!adapter) {
            throw new Error('In order to filter date columns, please provide DateTime adapter in your TableDataProvider constructor.');
        }
        const filterValue = filter.value;
        const filterValue2 = filter.value2;
        const itemValue = get(item, filter.field);
        const diff = adapter.compareDate(itemValue, filterValue);
        let result;
        switch (filter.strategy) {
            case 'after':
                result = diff > 0;
                break;
            case 'onOrAfter':
                result = diff >= 0;
                break;
            case 'before':
                result = diff < 0;
                break;
            case 'beforeOrOn':
                result = diff <= 0;
                break;
            case 'between':
                result = adapter.isBetween(itemValue, filterValue, filterValue2);
                break;
            case 'equalTo':
            default:
                result = adapter.dateTimesEqual(itemValue, filterValue);
        }
        return filter.exclude ? !result : result;
    }
    /**
     * Boolean filtering strategy
     * @param item data source array item.
     * @param filter filter object
     * @returns whether or not item should be included in data source array.
     */
    filterBoolean(item, filter) {
        const filterValue = filter.value;
        const itemValue = get(item, filter.field);
        let result;
        switch (filter.strategy) {
            case 'equalTo':
            default:
                result = itemValue === filterValue;
        }
        return filter.exclude ? !result : result;
    }
    /**
     * Array filtering strategy
     * @param item data source array item.
     * @param filter filter object
     * @returns whether or not item should be included in data source array.
     */
    filterArray(item, filter) {
        const filterValues = filter.value.map((v) => this.getSelectItemValue(v));
        const itemValue = get(item, filter.field);
        let result;
        switch (filter.strategy) {
            case 'equalTo':
            default:
                result = filterValues.includes(itemValue);
        }
        return !filterValues.length || filter.exclude ? !result : result;
    }
    /** @hidden */
    unsubscribe() {
        this._destroy$.next();
        this._destroy$.complete();
    }
}
class TableDataProvider extends BaseTableDataProvider {
    /**
     * Method for retrieving the data.
     * @param tableState @see TableState Set of table parameters.
     * @returns Observable with data.
     */
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    fetch(tableState) {
        throw new Error('fetch method should be implemented by developers for custom data providers.');
    }
    /**
     * Method for applying filtering and retrieving the data.
     * @param state @see TableState Set of table parameters.
     * @returns Observable with filtered data.
     */
    fetchData(state) {
        return this.fetch(state).pipe(map((items) => {
            if (this.searchInput) {
                items = this.search(items, { ...state, ...{ searchInput: this.searchInput } });
            }
            if (this.filterBy) {
                items = this.applyFiltering(items, this.filterBy);
            }
            return items;
        }), takeUntil(this._destroy$));
    }
    /**
     * Method for getting all possible options for particular field.
     * Used for creating select options.
     * Developers can override this method to extend the filtering functionality.
     * @param field key of the data item.
     * @returns Observable with select items.
     */
    getFieldOptions(field) {
        return this.fetch().pipe(take(1), map((data) => {
            const options = data.map((item) => ({
                label: get(item, field),
                value: get(item, field)
            }));
            return options;
        }));
    }
}
class TableChildrenDataProvider extends BaseTableDataProvider {
    /**
     * Method for retrieving the data.
     * @param tableState @see TableState Set of table parameters.
     * @param parentRows
     * @returns Observable with data.
     */
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    fetch(tableState, parentRows) {
        throw new Error('fetch method should be implemented by developers for custom data providers.');
    }
    /**
     * Method for applying filtering and retrieving the data.
     * @param state @see TableState Set of table parameters.
     * @param tableRows
     * @returns Observable with filtered data.
     */
    fetchData(state, tableRows) {
        return this.fetch(state, tableRows).pipe(map((items) => {
            items.forEach((rowItems, row) => {
                if (this.searchInput) {
                    rowItems = this.search(rowItems, { ...state, ...{ searchInput: this.searchInput } });
                }
                if (this.filterBy) {
                    rowItems = this.applyFiltering(rowItems, this.filterBy);
                }
                if (rowItems.length === 0) {
                    items.delete(row);
                }
                else {
                    items.set(row, rowItems);
                }
            });
            return items;
        }), takeUntil(this._destroy$));
    }
}

class BaseTableDataSource {
    /** @hidden */
    constructor() {
        /** @hidden */
        this._dataChanges$ = new BehaviorSubject([]);
        /** @hidden */
        this._onDataRequested$ = new Subject();
        /** @hidden */
        this._onDataReceived$ = new Subject();
        /** @hidden */
        this._destroy$ = new Subject();
        this._dataLoading$ = new BehaviorSubject(this._dataLoading);
    }
    /**
     * Emitted when new data has been requested.
     * @returns Observable
     */
    get dataRequested() {
        return this._onDataRequested$.asObservable();
    }
    /**
     * Emitted when new data has been received.
     * @returns Observable
     */
    get dataReceived() {
        return this._onDataReceived$.asObservable();
    }
    /**
     * Emitted when loading state has been changed.
     * @returns Observable.
     */
    get dataLoading() {
        return this._dataLoading$.asObservable();
    }
    /**
     * Emits when the data from the provider has been changed.
     * @returns Observable of data source objects.
     */
    get dataChanges() {
        return this._dataChanges$.asObservable().pipe(takeUntil(this._destroy$));
    }
    /** @hidden */
    get isDataLoading() {
        return this._dataLoading;
    }
    /** @hidden */
    get data() {
        return this._dataChanges$.value;
    }
    /** @hidden */
    open() {
        return this._dataChanges$.asObservable();
    }
    /** @hidden */
    onDataRequested() {
        return this._onDataRequested$.asObservable().pipe(map(() => void 0));
    }
    /** @hidden */
    onDataReceived() {
        return this._onDataReceived$.asObservable().pipe(map(() => void 0));
    }
    /** @hidden */
    close() { }
    /**
     * Closes the stream
     */
    unsubscribe() {
        this._destroy$.next();
        this._destroy$.complete();
    }
}
class TableDataSource extends BaseTableDataSource {
    /**
     * @hidden
     */
    constructor(dataProvider) {
        super();
        this.dataProvider = dataProvider;
    }
    /**
     * Method for retrieving the data of the provided data source.
     * @param tableState @see TableState Set of table parameters.
     */
    fetch(tableState) {
        this._onDataRequested$.next(true);
        this._dataLoading$.next(true);
        this._dataLoading = true;
        this.dataProvider.fetchData(tableState).subscribe({
            next: (items) => {
                this._onDataReceived$.next(true);
                this._dataLoading = false;
                this._dataLoading$.next(false);
                this._dataChanges$.next(items);
            },
            error: (error) => {
                this._onDataReceived$.next(false);
                this._dataLoading = false;
                this._dataChanges$.error(error);
            }
        });
    }
}
class ChildTableDataSource extends BaseTableDataSource {
    /**
     * @hidden
     */
    constructor(dataProvider) {
        super();
        this.dataProvider = dataProvider;
    }
    /**
     * Method responsible for retrieving the items from the data provider.
     * Additionaly, it sets the loading state along with the dataLoading and dataChanges streams.
     * @param tableState
     * @param parentRows
     */
    fetch(tableState, parentRows) {
        this._onDataRequested$.next(true);
        const loadingState = parentRows?.map((row) => ({ row, loading: true })) || [];
        const finishedLoadingState = loadingState.map(({ row }) => ({ row, loading: false }));
        this._dataLoading$.next(loadingState);
        this._dataLoading = loadingState;
        this.dataProvider.fetchData(tableState, parentRows).subscribe({
            next: (items) => {
                this._onDataReceived$.next(true);
                this._dataLoading = finishedLoadingState;
                this._dataLoading$.next(finishedLoadingState);
                this._dataChanges$.next(items);
            },
            error: (error) => {
                this._onDataReceived$.next(false);
                this._dataLoading = finishedLoadingState;
                this._dataLoading$.next(finishedLoadingState);
                this._dataChanges$.error(error);
            }
        });
    }
}

/**
 * Table Data Provider based on an array.
 *
 * Used to convert array source to the TableDataProvider interface.
 *
 */
class ArrayTableDataProvider extends TableDataProvider {
    /** @hidden */
    constructor(items, dateTimeAdapter) {
        super();
        /** @hidden */
        this.items$ = new BehaviorSubject([]);
        this.items = items;
        this.totalItems = this.items.length;
        this.items$.next(this.items);
        this.dateTimeAdapter = dateTimeAdapter;
    }
    /**
     * Method for retrieving the data.
     * @param tableState @see TableState Set of table parameters.
     * @returns Observable with data.
     */
    fetch(state) {
        return this.items$.asObservable().pipe(map((items) => {
            if (state?.searchInput) {
                items = this.search(items, state);
            }
            if (state?.filterBy) {
                items = this.applyFiltering(items, state.filterBy);
            }
            this.totalItems = items.length;
            return items;
        }));
    }
}
class ArrayTableDataSource extends TableDataSource {
    /** @hidden */
    constructor(data, dateTimeAdapter) {
        super(new ArrayTableDataProvider(data, dateTimeAdapter));
    }
}

/**
 * Table Data Provider based on an Observable.
 *
 * Used to convert observable source to the TableDataProvider interface.
 *
 */
class ObservableTableDataProvider extends TableDataProvider {
    /** @hidden */
    constructor(items$, dateTimeAdapter) {
        super();
        this.items$ = items$;
        this.dateTimeAdapter = dateTimeAdapter;
    }
    /**
     * Method for retrieving the data.
     * @param tableState @see TableState Set of table parameters.
     * @returns Observable with data.
     */
    fetch(state) {
        return this.items$.pipe(map((items) => {
            this.items = items;
            if (state?.searchInput) {
                items = this.search(items, state);
            }
            if (state?.filterBy) {
                items = this.applyFiltering(items, state.filterBy);
            }
            this.totalItems = items.length;
            return items;
        }));
    }
}
class ObservableTableDataSource extends TableDataSource {
    /** @hidden */
    constructor(data, dateTimeAdapter) {
        super(new ObservableTableDataProvider(data, dateTimeAdapter));
    }
}

class TableDataSourceParser {
    /** @hidden */
    parse(source) {
        if (isDataSource(source)) {
            return source;
        }
        if (Array.isArray(source)) {
            return new ArrayTableDataSource(source);
        }
        if (isObservable(source)) {
            return new ObservableTableDataSource(source);
        }
        return undefined;
    }
}

class TableDataSourceDirective extends DataSourceDirective {
    constructor() {
        super(...arguments);
        /** Event emitted when child data source instance being changed. */
        this.childDataSourceChanged = new EventEmitter();
        /** Event emitted when data loading is started. */
        this.onDataRequested = new EventEmitter();
        /** Event emitted when data loading is finished. */
        this.onDataReceived = new EventEmitter();
        /** Total items count stream. */
        this.totalItems$ = signal(0);
        /** Items stream. */
        this.items$ = new BehaviorSubject([]);
        /** Child items stream. */
        this.childItems$ = new Subject();
        /** @hidden */
        this._firstLoadingDone = false;
        /** @hidden */
        this._internalLoadingState = false;
        /** @hidden */
        this._internalChildrenLoadingState = false;
        /** @hidden */
        this._childDsSubscription = new Subscription();
        /** @hidden */
        this.initialState = inject(FDP_TABLE_STATE_DIRECTIVE, {
            optional: true
        });
        /** @hidden */
        this._tableService = inject(TableService);
    }
    /**
     * Data source.
     * @param source
     */
    set childDataSource(source) {
        this._childDataSource = source;
        this._initializeChildDataSource();
        this.childDataSourceChanged.next();
    }
    get childDataSource() {
        return this._childDataSource;
    }
    /** @hidden */
    set totalItems(value) {
        this.totalItems$.set(value);
    }
    get totalItems() {
        return this.totalItems$();
    }
    /** @hidden */
    initializeDataSource() {
        this.dataSourceChanged.pipe(startWith(true), takeUntilDestroyed(this._destroyRef)).subscribe(() => {
            this._closeDataSource();
            this._listenToDataSource();
        });
    }
    /** @hidden */
    ngOnDestroy() {
        super.ngOnDestroy();
        this._tableDsSubscription?.unsubscribe();
    }
    /** @hidden */
    setTable(table) {
        this._table = table;
    }
    /**
     * @hidden
     * Initializes the child data source to fetch child rows.
     */
    _initializeChildDataSource() {
        if (isDataSource(this.childDataSource)) {
            this.childDataSource?.unsubscribe();
            this._childDsSubscription?.unsubscribe();
        }
        if (!this.childDataSource) {
            return;
        }
        this._childDsSubscription = new Subscription();
        this._childDsSubscription.add(this.childDataSource.onDataRequested().subscribe(() => {
            this._internalChildrenLoadingState = true;
            this._tableService.setTableLoading(this._table.loadingState);
        }));
        this._childDsSubscription.add(this.childDataSource.onDataReceived().subscribe(() => {
            this._internalChildrenLoadingState = false;
            this._tableService.setTableLoading(this._table.loadingState);
        }));
        this._childDsSubscription.add(this.childDataSource.dataLoading
            .pipe(filter((state) => !!state), takeUntilDestroyed(this._destroyRef))
            .subscribe((loadingStates) => {
            loadingStates.forEach((state) => state.row?.childItemsLoading$.next(state.loading));
        }));
        this._childDsSubscription.add(this.childDataSource.dataChanges.pipe(takeUntilDestroyed(this._destroyRef)).subscribe((data) => {
            this.childItems$.next(data);
        }));
    }
    /**
     * @hidden
     * This is a single point of data entry to the component. We don't want to set data on different
     * places. If any new data comes in, or you do a search, and you want to pass initial data
     * its here.
     */
    _listenToDataSource() {
        this._tableDsSubscription?.unsubscribe();
        this._tableDsSubscription = new Subscription();
        const dataSourceStream = this.dataSourceProvider;
        if (dataSourceStream === undefined) {
            throw new Error(`[TableDataSource] source did not match an Array, Observable, nor DataSource`);
        }
        this._tableDsSubscription.add(this.dataChanged$.subscribe((items) => {
            this.totalItems$.set(dataSourceStream.dataProvider.totalItems);
            this.items$.next(items);
        }));
        this._tableDsSubscription.add(dataSourceStream.onDataRequested().subscribe(() => {
            this.onDataRequested.emit();
            this._internalLoadingState = true;
            this._tableService.setTableLoading(this._table.loadingState);
        }));
        this._tableDsSubscription.add(dataSourceStream.onDataReceived().subscribe(() => {
            this.onDataReceived.emit();
            this._internalLoadingState = false;
            this._firstLoadingDone = true;
            this._tableService.setTableLoading(this._table.loadingState);
            // Restore normal pagination after the first fetch of data.
            if (this._table._loadPreviousPages) {
                const state = this._tableService.getTableState();
                this._tableService.setTableState({
                    ...state,
                    ...{
                        page: {
                            currentPage: this.initialState?.initialPage || state.page.currentPage,
                            pageSize: this._table.pageSize || state.page.pageSize
                        }
                    }
                });
                this._table._loadPreviousPages = false;
            }
        }));
        // initial data fetch
        dataSourceStream.fetch(this._tableService.getTableState());
        this._tableDataSource = dataSourceStream;
    }
    /** @hidden */
    _closeDataSource() {
        if (!this._tableDataSource) {
            return;
        }
        this._tableDataSource.close();
        this._tableDsSubscription?.unsubscribe();
        this._tableDsSubscription = null;
        this._table.clearTableRows();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableDataSourceDirective, deps: null, target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: TableDataSourceDirective, isStandalone: true, selector: "[fdpTableDataSource]", inputs: { childDataSource: "childDataSource" }, outputs: { childDataSourceChanged: "childDataSourceChanged", onDataRequested: "onDataRequested", onDataReceived: "onDataReceived" }, providers: [
            {
                provide: FD_DATA_SOURCE_TRANSFORMER,
                useClass: TableDataSourceParser
            }
        ], usesInheritance: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableDataSourceDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[fdpTableDataSource]',
                    standalone: true,
                    providers: [
                        {
                            provide: FD_DATA_SOURCE_TRANSFORMER,
                            useClass: TableDataSourceParser
                        }
                    ]
                }]
        }], propDecorators: { childDataSource: [{
                type: Input
            }], childDataSourceChanged: [{
                type: Output
            }], onDataRequested: [{
                type: Output
            }], onDataReceived: [{
                type: Output
            }] } });

/** @hidden */
function isTableRow(row) {
    return row && row.type !== undefined && row.value !== undefined && row.index >= 0 && row.state !== undefined;
}
class TableRowImpl {
    /** @hidden */
    set checked(value) {
        this._checkedSubject.next(value);
    }
    get checked() {
        return this._checkedSubject.value;
    }
    /** @hidden */
    set expanded(value) {
        this._expandedSubject.next(value);
        this.children.forEach((childRow) => {
            childRow.hidden = !value;
        });
    }
    get expanded() {
        return this._expandedSubject.value;
    }
    /** @hidden */
    constructor(row) {
        /** @hidden */
        this.stateChanged$ = new BehaviorSubject(false);
        /** @hidden */
        this.childItemsLoading$ = new BehaviorSubject(false);
        /** @hidden */
        this.forceFetch = false;
        /** @hidden */
        this._expandedSubject = new BehaviorSubject(false);
        /** @hidden */
        this._checkedSubject = new BehaviorSubject(false);
        this.checked$ = this._checkedSubject.asObservable().pipe(map(Boolean));
        this.expanded$ = this._expandedSubject.asObservable().pipe(map(Boolean));
        this.checked = row.checked || false;
        this.children = row.children || [];
        this.expandable = row.expandable || false;
        this.expanded = row.expanded || false;
        this.hidden = row.hidden || false;
        this.index = row.index || 0;
        this.level = row.level || 0;
        this.navigatable = row.navigatable || false;
        this.parent = row.parent || null;
        this.state = row.state || 'readonly';
        this.value = row.value;
        this.setRowType(row.type || TableRowType.ITEM);
    }
    /**
     * Sets the row type and checks whether the row is a tree.
     * @param type Row type.
     */
    setRowType(type) {
        this.type = type;
        this.isTree = this.type === TableRowType.TREE;
    }
}

/** @hidden */
function newTableRow(row) {
    if (!row.value) {
        throw new Error('Unexpected value. TableRow.value cannot be undefined.');
    }
    return new TableRowImpl({
        type: row.type || TableRowType.ITEM,
        checked: row.checked || false,
        index: row.index || 0,
        value: row.value,
        parent: row.parent || null,
        level: row.level || 0,
        expandable: row.expandable || false,
        expanded: row.expanded || false,
        hidden: row.hidden || false,
        navigatable: row.navigatable || false,
        state: row.state || 'readonly',
        children: row.children || []
    });
}
/** @hidden */
function getSelectableRows(rows, selectableKey) {
    return rows.filter((row) => isSelectableRow(row, selectableKey));
}
/** @hidden */
function isSelectableRow(row, selectableKey) {
    return (isItemRow(row) || isTreeRow(row)) && row.value[selectableKey] !== false;
}
/** @hidden */
function isItemRow(row) {
    return row.type === TableRowType.ITEM;
}
/** @hidden */
function isTreeRow(row) {
    return row.type === TableRowType.TREE;
}
/**
 * Since we dont work with the tree, we need to convert incoming tree to
 * flat format while maintaining original state.
 *
 * @hidden
 */
function convertTreeTableRowToFlatList(rows, rowNavigatable) {
    let flatList = [];
    for (const item of rows) {
        item.navigatable = isRowNavigatable(item, rowNavigatable);
        flatList.push(item);
        if (Array.isArray(item.children)) {
            item.children.forEach((c) => (c.hidden = !item.expanded));
            flatList = flatList.concat(convertTreeTableRowToFlatList(item.children, rowNavigatable));
        }
    }
    return flatList;
}
/** @hidden */
function isRowNavigatable(row, rowNavigatable) {
    if (!row) {
        return false;
    }
    /** If key of the row's item field passed */
    if (isString(rowNavigatable)) {
        return !!row[rowNavigatable];
    }
    return !!rowNavigatable;
}
/** @hidden */
function getFreezableColumns(columns, freezeColumnsTo) {
    const columnNames = new Map();
    if (!columns.length || !freezeColumnsTo) {
        return columnNames;
    }
    for (const column of columns) {
        if (!column.name) {
            continue;
        }
        // using columnNames.size as index of a column
        columnNames.set(column.name, columnNames.size);
        if (column.name === freezeColumnsTo) {
            return columnNames;
        }
    }
    return new Map();
}
/** @hidden */
function getFreezableEndColumns(columns, freezeEndColumnsTo) {
    const columnNames = new Map();
    if (!columns.length || !freezeEndColumnsTo) {
        return columnNames;
    }
    for (let i = columns.length - 1; i >= 0; i--) {
        // using columnNames.size as index of a column
        columnNames.set(columns[i].name, columnNames.size);
        if (columns[i].name === freezeEndColumnsTo) {
            return columnNames;
        }
    }
    return new Map();
}
/** @hidden */
function convertTreeObjectsToTableRows(source, selectionMode, tableRows, rowComparator, relationKey, hasChildrenKey, selectedKey, expandedStateKey, rowNavigatable, enableTristateMode = false) {
    const rows = [];
    const selectedRowsMap = getSelectionStatusByRowValue(source, selectionMode, tableRows, rowComparator);
    source.forEach((item, index) => {
        const hasChildren = Object.prototype.hasOwnProperty.call(item, relationKey) &&
            Array.isArray(item[relationKey]) &&
            item[relationKey].length;
        const row = isTableRow(item)
            ? item
            : newTableRow({
                type: hasChildren || item[hasChildrenKey] === true ? TableRowType.TREE : TableRowType.ITEM,
                checked: item[selectedKey] ?? !!selectedRowsMap.get(item),
                index,
                value: item
            });
        row.expanded =
            expandedStateKey && Object.prototype.hasOwnProperty.call(item, expandedStateKey)
                ? item[expandedStateKey]
                : false;
        row.navigatable = isRowNavigatable(item, rowNavigatable);
        rows.push(row);
        if (hasChildren) {
            const children = convertTreeObjectsToTableRows(item[relationKey], selectionMode, tableRows, rowComparator, relationKey, hasChildrenKey, selectedKey, expandedStateKey, rowNavigatable, enableTristateMode);
            children.forEach((c) => {
                c.parent = c.parent || row;
                c.level = c.parent.level + 1;
                c.hidden = !c.parent.expanded;
            });
            row.children.push(...children);
            rows.push(...children);
        }
        if (row.level === 0 && row.isTree) {
            dfs([row], (r) => {
                const children = r.children || [];
                const selectedChildren = [];
                const indeterminateChildren = [];
                children.forEach((child) => {
                    if (child.checked === true) {
                        selectedChildren.push(child);
                    }
                    else if (child.checked === null) {
                        indeterminateChildren.push(child);
                    }
                });
                const selectedAll = selectedChildren.length === children.length && children.length > 0;
                const selectedSome = selectedChildren.length > 0;
                if (r.checked) {
                    applySelectionToChildren(rows, r, [], []);
                    return;
                }
                if (!enableTristateMode) {
                    return;
                }
                if (selectedAll) {
                    r.checked = true;
                    return;
                }
                if (selectedSome || indeterminateChildren.length > 0) {
                    r.checked = null;
                    return;
                }
            });
        }
    });
    return rows;
}
/**
 * @hidden
 * Runs `rowComparator` function against checked rows and compares them with the new `source`
 * If matched, creates an association between the source item and checked status of corresponding row.
 *
 * @returns `Map` object with the `checked` status for particular source item
 */
function getSelectionStatusByRowValue(source, selectionMode, tableRows, rowComparator) {
    const rowMap = new Map();
    if ((selectionMode === SelectionMode.SINGLE || selectionMode === SelectionMode.MULTIPLE) &&
        typeof rowComparator === 'function') {
        const checkedRows = tableRows.filter((r) => r.checked);
        checkedRows.forEach((row) => {
            const found = source.find((e) => rowComparator(row.value, e));
            if (found) {
                rowMap.set(found, row.checked);
            }
        });
    }
    return rowMap;
}
/** @hidden */
function applySelectionToChildren(allRows, row, addedRows, removedRows) {
    const allChildren = findRowChildren(row, allRows);
    allChildren.forEach((r) => {
        r.checked = row.checked;
        r.checked ? addedRows.push(r) : removedRows.push(r);
    });
}
/** @hidden */
function findRowChildren(row, allRows) {
    const rowsLength = allRows.length;
    /**
     * Since we are dealing with a flat list
     * it means all children go next right after the expandable row
     * until the next row has a mutual parent
     */
    let index = allRows.indexOf(row);
    const parents = getRowParents(row);
    const children = [];
    while (index++ < rowsLength) {
        const nextRow = allRows[index];
        if (!nextRow?.parent || parents.includes(nextRow.parent)) {
            break;
        }
        children.push(nextRow);
    }
    return children;
}
/**
 * Get row parents path where the first is the direct parent
 * @param row Row which parents we need to find
 * @param untilParent Parent to stop a search on. Default is "null" that means look up until the root
 * @returns parents list [direct parent, ..., ancestor]
 * @hidden
 */
function getRowParents(row, untilParent = null) {
    untilParent = untilParent || null; // to avoid "undefined"
    const parents = [];
    let parent = row.parent || null; // empty parent should be coerced to "null" do not get into infinite loop
    while (parent && parent !== untilParent) {
        parents.push(parent);
        parent = parent.parent || null;
    }
    return parents;
}
/** @hidden */
function toDataStream(source) {
    if (isDataSource$1(source)) {
        return source;
    }
    if (Array.isArray(source)) {
        return new ArrayTableDataSource(source);
    }
    if (isObservable(source)) {
        return new ObservableTableDataSource(source);
    }
    return undefined;
}
/**
 * @hidden
 * Creates an empty column skeleton object.
 * @returns Column model.
 */
function buildNewRowSkeleton(editableRowSkeleton, columns) {
    if (editableRowSkeleton) {
        return cloneDeep(editableRowSkeleton);
    }
    let newRow = {};
    columns.forEach((column) => {
        newRow = set(newRow, column.key, undefined);
    });
    return newRow;
}
/** @hidden */
function isGroupRow(row) {
    return row.type === TableRowType.GROUP;
}
/** @hidden */
function sortTreeLikeGroupedRows(groupedRows, groupRulesMap) {
    if (!groupedRows[0] || !isGroupRow(groupedRows[0])) {
        return groupedRows;
    }
    const treeLikeRows = groupedRows;
    const firstRow = treeLikeRows[0];
    treeLikeRows.forEach((group) => {
        if (group._children) {
            group._children = sortTreeLikeGroupedRows(group._children, groupRulesMap);
        }
    });
    const groupRule = groupRulesMap.get(firstRow.value.field);
    if (!groupRule || groupRule.direction === SortDirection.NONE) {
        return treeLikeRows;
    }
    const direction = groupRule.direction;
    const directionMultiplier = direction === SortDirection.ASC ? 1 : direction === SortDirection.DESC ? -1 : 0;
    return treeLikeRows.slice().sort((a, b) => {
        const aValue = a.value.value;
        const bValue = b.value.value;
        const aNumber = Number.parseFloat(aValue);
        const bNumber = Number.parseFloat(bValue);
        if (!Number.isNaN(aNumber) && !Number.isNaN(bNumber)) {
            return (aNumber - bNumber) * directionMultiplier;
        }
        return (aValue > bValue ? 1 : -1) * directionMultiplier;
    });
}
/** @hidden */
function convertTreeLikeToFlatList(treeLikeList) {
    let flatList = [];
    for (const item of treeLikeList) {
        flatList.push(item);
        if (Array.isArray(item._children)) {
            flatList = flatList.concat(convertTreeLikeToFlatList(item._children));
            delete item._children;
        }
    }
    return flatList;
}
/**
 * @hidden
 * Group table rows and return tree like rows list.
 * It's intended to be called recursively.
 * @param rules group rules to group by
 * @param rows source table rows
 * @param parent row parent
 * @param level level of nesting
 */
function createGroupedTableRowsTree(rules, rows, parent = null, level = 0) {
    rules = [...rules];
    if (!rules.length) {
        // no rules mean that it's the level of source items
        return rows.map((row) => {
            row.parent = parent;
            row.level = level;
            row.hidden = !!parent && !parent.expanded;
            return row;
        });
    }
    // Retrieve first group rule
    const rule = rules.shift();
    // Build map of unique values for a given group rule
    const valuesHash = rows
        .filter((r) => r.state !== 'editable')
        .reduce((hash, row) => {
        const modelValue = get(row.value, rule.field);
        if (!hash.has(modelValue)) {
            hash.set(modelValue, []);
        }
        hash.get(modelValue)?.push(row);
        return hash;
    }, new Map());
    // Build table rows tree
    let groupedTableRows = [];
    if (rows.some((r) => r.state === 'editable')) {
        groupedTableRows = rows
            .filter((r) => r.state === 'editable')
            .map((row) => {
            row.parent = parent;
            row.level = -1;
            row.hidden = !!parent && !parent.expanded;
            return row;
        });
    }
    for (const [value, values] of Array.from(valuesHash)) {
        const filteredRows = rows.filter((_item) => values.includes(_item));
        if (filteredRows.length === 0) {
            continue;
        }
        const groupTableRow = newTableRow({
            type: TableRowType.GROUP,
            checked: false,
            index: 0,
            value: { field: rule.field, value, count: 0 },
            parent,
            level,
            expandable: true,
            expanded: true,
            hidden: !!parent && !parent.expanded
        });
        // Ads group's children rows
        groupTableRow._children = createGroupedTableRowsTree(rules, filteredRows, groupTableRow, level + 1);
        groupTableRow.value.count = groupTableRow._children?.length;
        groupedTableRows.push(groupTableRow);
    }
    return groupedTableRows;
}
/** @hidden */
function convertObjectsToTableRows(source, addedItems, selectedKey, rowNavigatable, selectionMode, rows, rowComparator) {
    const rowItem = source[0];
    if (isTableRow(rowItem)) {
        return source;
    }
    const selectedRowsMap = getSelectionStatusByRowValue(source, selectionMode, rows, rowComparator);
    return source.map((item, index) => {
        const isNewItem = addedItems.includes(item);
        const row = newTableRow({
            type: TableRowType.ITEM,
            checked: item[selectedKey] ?? !!selectedRowsMap.get(item),
            index,
            value: item
        });
        row.navigatable = isRowNavigatable(item, rowNavigatable);
        row.state = isNewItem ? 'editable' : 'readonly';
        return row;
    });
}
/** @hidden */
function isTreeRowFirstCell(cellIndex, row, event) {
    return (cellIndex === 0 && isTreeRow(row)) || (!!event && !row.isTree);
}

class TableColumnFreezeEvent {
    /**
     * Table column freeze event
     * @param source Table component
     * @param current Current freezeTo column
     * @param previous Previous freezeTo column
     */
    constructor(source, current, previous) {
        this.source = source;
        this.current = current;
        this.previous = previous;
    }
}

class ColumnsChange {
    /**
     * Columns change event
     * @param current Current columns
     * @param previous Previous columns
     */
    constructor(current, previous) {
        this.current = current;
        this.previous = previous;
    }
}
class TableColumnsChangeEvent extends ColumnsChange {
    /**
     * Table columns change event
     * @param source Table component
     * @param current Current columns
     * @param previous Previous columns
     */
    constructor(source, current, previous) {
        super(current, previous);
        this.source = source;
    }
}

class TableDraggable {
}
class TableInitialState {
}
class TableVirtualScroll {
}

class TableFilterChangeEvent {
    /**
     * Table filter change event
     * @param source Table component
     * @param current Current filters
     * @param previous Previous filters
     */
    constructor(source, current, previous) {
        this.source = source;
        this.current = current;
        this.previous = previous;
    }
}

class TableGroupChangeEvent {
    /**
     * Table group change event
     * @param source Table component
     * @param current Current groups
     * @param previous Previous groups
     */
    constructor(source, current, previous) {
        this.source = source;
        this.current = current;
        this.previous = previous;
    }
}

class PageChange {
    /**
     * Page change event
     * @param current Current page
     * @param previous Previous page
     */
    constructor(current, previous) {
        this.current = current;
        this.previous = previous;
    }
}
class TablePageChangeEvent extends PageChange {
    /**
     * Table page change event
     * @param source Table component
     * @param current Current page
     * @param previous Previous page
     */
    constructor(source, current, previous) {
        super(current, previous);
        this.source = source;
    }
}

class SaveRowsEvent {
    /**
     * Table save rows event
     * @param done Callback function. Call it when newly added items has been saved and new data source fetch is needed.
     * @param items Array of newly created items.
     */
    constructor(done, items) {
        this.done = done;
        this.items = items;
    }
}

class TableSearchChangeEvent {
    /**
     * Table search change event
     * @param source Table component
     * @param current Current search (fields, directions)
     * @param previous Previous search (fields, directions)
     */
    constructor(source, current, previous) {
        this.source = source;
        this.current = current;
        this.previous = previous;
    }
}

class TableSelectionChangeEvent {
}
class TableRowSelectionChangeEvent extends TableSelectionChangeEvent {
}

class TableSortChangeEvent {
    /**
     * Table sort change event
     * @param source Table component
     * @param current Current sort (fields, directions)
     * @param previous Previous sort (fields, directions)
     */
    constructor(source, current, previous) {
        this.source = source;
        this.current = current;
        this.previous = previous;
    }
}

class TableCellActivateEvent {
    /**
     * Table cell activate event
     * @param columnIndex Index of the clicked column
     * @param row Row that was activated
     */
    constructor(columnIndex, row) {
        this.columnIndex = columnIndex;
        this.row = row;
    }
}

/** Common properties for table dialogs */

class TableRowActivateEvent {
    /**
     * Table row activate event
     * @param index Index of the activated row
     * @param ctrlKey Whether control/command key was pressed during the mouse/keyboard event
     * @param row Row that was activated
     */
    constructor(index, ctrlKey, row) {
        this.index = index;
        this.ctrlKey = ctrlKey;
        this.row = row;
    }
}

class TableRowToggleOpenStateEvent {
    /**
     * Table row toggle open state event
     * @param index Index of the row
     * @param row Row that was toggled
     * @param expanded Expanded state of the row
     */
    constructor(index, row, expanded) {
        this.index = index;
        this.row = row;
        this.expanded = expanded;
    }
}

class TableRowsRearrangeEvent {
    /**
     * Table rows rearrange event
     * @param row Row that was rearranged
     * @param dropRow Row where the dragged item has been dropped
     * @param previousIndex Previous index of the row
     * @param newIndex New index of the row
     * @param insertAt Whether dragged the row was inserted before or after the dropped row
     * @param mode Whether the dropped element shifts other items or is grouped with them
     * @param rows All rows of the table in the state they exist on the datasource
     */
    constructor(row, dropRow, previousIndex, newIndex, insertAt, mode, rows) {
        this.row = row;
        this.dropRow = dropRow;
        this.previousIndex = previousIndex;
        this.newIndex = newIndex;
        this.insertAt = insertAt;
        this.mode = mode;
        this.rows = rows;
    }
}

class TableDraggableDirective extends TableDraggable {
    constructor() {
        super(...arguments);
        /** Whether to allow for row reordering on tree tables via drag and drop. */
        this.enableRowReordering = true;
        /** Whether tree mode is enabled. */
        this.isTreeTable = false;
        /**
         * Row drop mode.
         */
        this.dropMode = 'auto';
        /** Event fired when tree rows rearranged through drag & drop. Consider that rows rearranged with their children rows. */
        this.rowsRearrange = new EventEmitter();
        /** @hidden */
        this.dragDropInProgress = false;
        /** @hidden */
        this._ngZone = inject(NgZone);
        /** @hidden */
        this._cdr = inject(ChangeDetectorRef, {
            host: true
        });
    }
    /** @hidden */
    get _rowsDraggable() {
        return this.isTreeTable && this.enableRowReordering;
    }
    /** Sets table reference. */
    setTable(table) {
        this._table = table;
    }
    /**
     * Initiates drag&drop sequence.
     */
    dragDropStart() {
        this.dragDropInProgress = true;
    }
    /** Method called when dnd performed with the keyboard. */
    dragRowFromKeyboard(dir, event, currentRowIndex, mode) {
        if (!this._rowsDraggable) {
            return;
        }
        event.preventDefault();
        let replacedIndex;
        dir === 'up' ? (replacedIndex = currentRowIndex - 1) : (replacedIndex = currentRowIndex + 1);
        if (this._table._tableRowsVisible[replacedIndex]) {
            const dragDropEvent = {
                items: this._table._tableRowsVisible,
                draggedItemIndex: currentRowIndex,
                replacedItemIndex: replacedIndex,
                insertAt: dir === 'down' ? 'after' : 'before',
                mode
            };
            this.dragDropItemDrop(dragDropEvent);
            setTimeout(() => {
                event.target.focus();
            });
        }
    }
    /** Method called when drag&drop event being cancelled. */
    dropCancelled() {
        /** After timeout to make click event handled first */
        this._ngZone.runOutsideAngular(() => {
            setTimeout(() => (this.dragDropInProgress = false));
        });
    }
    /** Method called when dragged item being dropped. */
    dragDropItemDrop(event) {
        /** After timeout to make click event handled first */
        this._ngZone.runOutsideAngular(() => {
            setTimeout(() => (this.dragDropInProgress = false));
        });
        this._onZoneFree(() => {
            if (this.isTreeTable && event.draggedItemIndex !== event.replacedItemIndex) {
                event.draggedItemIndex = this._table._tableCurrentlyRenderedRowsPlaceholder[event.draggedItemIndex];
                event.replacedItemIndex = this._table._tableCurrentlyRenderedRowsPlaceholder[event.replacedItemIndex];
                const dragRow = this._table._tableRows.find((row) => row === this._table._tableRowsVisible[event.draggedItemIndex]);
                const dropRow = this._table._tableRows.find((row) => row === this._table._tableRowsVisible[event.replacedItemIndex]);
                if (!dragRow || !dropRow || this._isDroppedInsideItself(dropRow, dragRow)) {
                    return;
                }
                this._dragDropUpdateDragParentRowAttributes(dragRow);
                this._dragDropRearrangeTreeRows(dragRow, dropRow, event);
                this._dragDropUpdateDropRowAttributes(dragRow, dropRow, event.mode);
                if (!dropRow.expanded && event.mode === 'group') {
                    this._table.toggleExpandableTableRow(dropRow, true);
                }
                else {
                    this._table.onTableRowsChanged();
                }
                this._cdr.detectChanges();
                this._emitRowsRearrangeEvent(dragRow, dropRow, event);
                this._table.refreshDndList();
            }
        });
    }
    /**
     * @hidden
     * Create table rows rearrange event
     */
    _emitRowsRearrangeEvent(row, dropRow, event) {
        const rows = [];
        this._table._tableRows.forEach((r, index) => {
            r.index = index;
            rows.push(r.value);
        });
        this.rowsRearrange.emit(new TableRowsRearrangeEvent(row.value, dropRow.value, event.draggedItemIndex, event.replacedItemIndex, event.insertAt, event.mode, rows));
    }
    /** @hidden */
    _isDroppedInsideItself(dropRow, dragRow) {
        const dropRowParents = getRowParents(dropRow);
        return !!dropRowParents.find((row) => row === dragRow);
    }
    /** @hidden */
    _dragDropUpdateDragParentRowAttributes(dragRow) {
        const parentRow = dragRow.parent;
        if (!parentRow) {
            return;
        }
        const parentRowChildren = findRowChildren(parentRow, this._table._tableRows);
        const dragRowChildren = findRowChildren(dragRow, this._table._tableRows);
        if (parentRowChildren.length - (dragRowChildren.length + 1) === 0) {
            parentRow.setRowType(TableRowType.ITEM);
        }
    }
    /** @hidden */
    _dragDropUpdateDropRowAttributes(dragRow, dropRow, mode) {
        if (dragRow.parent) {
            // Remove child row from previous parent row.
            dragRow.parent.children.splice(dragRow.parent.children.indexOf(dragRow), 1);
        }
        dragRow.level = dropRow.level + (mode === 'group' ? 1 : 0);
        if (mode === 'group') {
            dragRow.parent = dropRow;
            if (!dropRow.isTree) {
                dropRow.setRowType(TableRowType.TREE);
            }
            dropRow.children.push(dragRow);
            dropRow.lastChild = dragRow;
        }
        else {
            dragRow.parent = dropRow.parent;
            dropRow.parent?.children.push(dragRow);
            if (dropRow.parent) {
                dropRow.parent.lastChild = dragRow;
            }
        }
        const children = findRowChildren(dragRow, this._table._tableRows);
        children.forEach((row) => {
            row.level = getRowParents(row).length;
        });
    }
    /** @hidden */
    _dragDropRearrangeTreeRows(dragRow, dropRow, event) {
        if (event.mode === 'shift') {
            this._handleShiftDropAction(dragRow, dropRow, event);
        }
        else {
            this._handleReplaceDropAction(dragRow, dropRow);
        }
    }
    /** @hidden */
    _getNewDragDropRowsPosition(dragRow, dropRow) {
        const allRows = this._table._tableRows;
        const dragRowIndex = allRows.findIndex((row) => row === dragRow);
        const dragRowChildren = findRowChildren(dragRow, this._table._tableRows);
        const rowsToMove = allRows.splice(dragRowIndex, dragRowChildren.length + 1);
        const dropRowIndex = allRows.findIndex((row) => row === dropRow);
        const dropRowChildren = findRowChildren(dropRow, this._table._tableRows);
        const dropRowItemsLength = dropRowChildren.length + 1;
        const rowsAfterDropRow = allRows.splice(dropRowIndex + dropRowItemsLength, allRows.length + dropRowItemsLength);
        const dropRowItems = allRows.splice(dropRowIndex, dropRowItemsLength);
        return {
            allRows,
            rowsToMove,
            rowsAfterDropRow,
            dropRowItems
        };
    }
    /** @hidden */
    _handleShiftDropAction(dragRow, dropRow, event) {
        const { allRows, rowsToMove, rowsAfterDropRow, dropRowItems } = this._getNewDragDropRowsPosition(dragRow, dropRow);
        this._table._tableRows = [
            ...allRows,
            ...(event.insertAt === 'after' ? dropRowItems : []),
            ...rowsToMove,
            ...(event.insertAt === 'after' ? [] : dropRowItems),
            ...rowsAfterDropRow
        ];
    }
    /** @hidden */
    _handleReplaceDropAction(dragRow, dropRow) {
        const { allRows, rowsToMove, rowsAfterDropRow, dropRowItems } = this._getNewDragDropRowsPosition(dragRow, dropRow);
        this._table._tableRows = [...allRows, ...dropRowItems, ...rowsToMove, ...rowsAfterDropRow];
    }
    /** @hidden */
    _onZoneFree(callback) {
        this._ngZone.onMicrotaskEmpty.pipe(take(1)).subscribe(() => {
            callback();
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableDraggableDirective, deps: null, target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: TableDraggableDirective, isStandalone: true, selector: "fdp-table[isTreeTable], fdp-table[enableRowReordering]", inputs: { enableRowReordering: "enableRowReordering", isTreeTable: "isTreeTable", dropMode: "dropMode", dropPredicate: "dropPredicate", dragoverPredicate: "dragoverPredicate" }, outputs: { rowsRearrange: "rowsRearrange" }, host: { properties: { "class.fd-table--tree": "this.isTreeTable" } }, providers: [
            {
                provide: FDP_TABLE_DRAGGABLE_DIRECTIVE,
                useExisting: forwardRef(() => TableDraggableDirective)
            }
        ], usesInheritance: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableDraggableDirective, decorators: [{
            type: Directive,
            args: [{
                    // eslint-disable-next-line @angular-eslint/directive-selector
                    selector: 'fdp-table[isTreeTable], fdp-table[enableRowReordering]',
                    standalone: true,
                    providers: [
                        {
                            provide: FDP_TABLE_DRAGGABLE_DIRECTIVE,
                            useExisting: forwardRef(() => TableDraggableDirective)
                        }
                    ]
                }]
        }], propDecorators: { enableRowReordering: [{
                type: Input
            }], isTreeTable: [{
                type: HostBinding,
                args: ['class.fd-table--tree']
            }, {
                type: Input
            }], dropMode: [{
                type: Input
            }], dropPredicate: [{
                type: Input
            }], dragoverPredicate: [{
                type: Input
            }], rowsRearrange: [{
                type: Output
            }] } });

class TableRowService {
    /** @hidden */
    constructor() {
        /** Editable cells map. */
        this.editableCells = new Map();
        /** @hidden */
        this.childRowsAdded$ = new Subject();
        /** Stream to load child items for a particular rows. */
        this.loadChildRows$ = new Subject();
        /** @hidden */
        this._toggleRowSubject = new Subject();
        /** @hidden */
        this._scrollToOverlappedCellSubject = new Subject();
        /** @hidden */
        this._cellClickedSubject = new Subject();
        /** @hidden */
        this._cellActivateSubject = new Subject();
        /** @hidden */
        this._cellFocusedSubject = new Subject();
        /** @hidden */
        this._toggleAllSelectableRowsSubject = new Subject();
        this.toggleAllSelectableRows$ = this._toggleAllSelectableRowsSubject.asObservable();
        this.cellClicked$ = this._cellClickedSubject.asObservable();
        this.scrollToOverlappedCell$ = this._scrollToOverlappedCellSubject.asObservable();
        this.cellFocused$ = this._cellFocusedSubject.asObservable();
        this.cellActivate$ = this._cellActivateSubject.asObservable();
        this.toggleRow$ = this._toggleRowSubject.asObservable();
    }
    /** `toggleRow$` stream trigger. */
    toggleRow(evt) {
        this._toggleRowSubject.next(evt);
    }
    /** `scrollToOverlappedCell$` stream trigger. */
    scrollToOverlappedCell() {
        this._scrollToOverlappedCellSubject.next();
    }
    /** `cellClicked$` stream trigger. */
    cellClicked(evt) {
        this._cellClickedSubject.next(evt);
    }
    /** @hidden */
    cellActivate(columnIndex, row) {
        this._cellActivateSubject.next({ columnIndex, row: row.value });
    }
    /** `cellFocused$` stream trigger. */
    cellFocused(evt) {
        this._cellFocusedSubject.next(evt);
    }
    /** `toggleAllSelectableRows$` stream trigger. */
    toggleAllSelectableRows(selectAll) {
        this._toggleAllSelectableRowsSubject.next(selectAll);
    }
    /** Groups the table rows. */
    groupTableRows(sourceRows, groups, groupRulesMap) {
        const groupRules = Array.from(groups);
        if (!groupRules.length) {
            /**
             * In case if previously we had groups with collapsed items
             * but now we don't have it we need to reset row.hidden flag
             * in order to avoid empty table after groups settings removing
             */
            sourceRows.forEach((row) => {
                row.hidden = false;
            });
            return sourceRows;
        }
        // Build tree like groups
        const treeLikeGroupedRows = createGroupedTableRowsTree(groupRules, sourceRows);
        // Sort tree like groups
        const sortedTreeLikeGroupedRows = sortTreeLikeGroupedRows(treeLikeGroupedRows, groupRulesMap);
        // Convert tree like list to a flat list
        return convertTreeLikeToFlatList(sortedTreeLikeGroupedRows);
    }
    /** Sets editable cells for particular row. */
    updateEditableCells(row, cells) {
        this.editableCells.set(row, cells);
    }
    /** Removes editable cells of particular row. */
    removeEditableCells(row) {
        this.editableCells.delete(row);
    }
    /** Triggers stream to load child items for a particular rows. */
    loadChildRows(rows) {
        const expandableRows = [];
        if (Array.isArray(rows)) {
            expandableRows.push(...rows.filter((r) => !r.childItemsLoading$.value));
        }
        else {
            if (rows.childItemsLoading$.value) {
                return;
            }
            expandableRows.push(rows);
        }
        this.loadChildRows$.next(expandableRows);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableRowService, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableRowService }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableRowService, decorators: [{
            type: Injectable
        }], ctorParameters: () => [] });

class TableHeaderResizerDirective {
    constructor() {
        /** @hidden */
        this._destroyRef = inject(DestroyRef);
        /** @hidden */
        this._rtl = inject(RtlService, {
            optional: true
        });
        /** @hidden */
        this._elmRef = inject(ElementRef);
        /** @hidden */
        this._tableColumnResizeService = inject(TableColumnResizeService);
        /** @hidden */
        this._tableRowService = inject(TableRowService);
        /** @hidden */
        this._zone = inject(NgZone);
        /** @hidden */
        this._document = inject(DOCUMENT);
    }
    /** @hidden */
    get _headerCellFocused() {
        return this._document.activeElement?.tagName.toLowerCase() === 'th';
    }
    /** @hidden */
    ngOnInit() {
        this._zone.runOutsideAngular(() => {
            fromEvent(this._elmRef.nativeElement, 'keydown')
                .pipe(takeUntilDestroyed(this._destroyRef))
                .subscribe((event) => {
                if ((KeyUtil.isKeyCode(event, LEFT_ARROW) ||
                    (this._rtl?.rtl.value && KeyUtil.isKeyCode(event, RIGHT_ARROW))) &&
                    event.shiftKey &&
                    this._headerCellFocused) {
                    this._tableColumnResizeService._processResize(-32);
                    event.preventDefault();
                    event.stopImmediatePropagation();
                }
                else if ((KeyUtil.isKeyCode(event, RIGHT_ARROW) ||
                    (this._rtl?.rtl.value && KeyUtil.isKeyCode(event, LEFT_ARROW))) &&
                    event.shiftKey &&
                    this._headerCellFocused) {
                    this._tableColumnResizeService._processResize(32);
                    event.preventDefault();
                    event.stopImmediatePropagation();
                }
            });
            fromEvent(this._elmRef.nativeElement, 'focusin')
                .pipe(takeUntilDestroyed(this._destroyRef))
                .subscribe(() => {
                if (!this._focusinTimerId) {
                    return;
                }
                clearTimeout(this._focusinTimerId);
                this._focusinTimerId = null;
            });
            fromEvent(this._elmRef.nativeElement, 'focusout')
                .pipe(takeUntilDestroyed(this._destroyRef))
                .subscribe(() => {
                this._focusinTimerId = setTimeout(() => {
                    this.focusedCellPosition = null;
                });
            });
        });
        this._tableRowService.cellFocused$.subscribe((evt) => {
            this._onCellFocused(evt);
        });
    }
    /** @hidden */
    _onCellFocused(position) {
        this.focusedCellPosition = { rowIndex: position.rowIndex, colIndex: position.colIndex };
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableHeaderResizerDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: TableHeaderResizerDirective, isStandalone: true, selector: "[fdpTableHeaderResizer]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableHeaderResizerDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[fdpTableHeaderResizer]',
                    standalone: true
                }]
        }] });

/** Column header container. */
// eslint-disable-next-line @angular-eslint/directive-selector
// eslint-disable-next-line @angular-eslint/directive-class-suffix
class FdpTableHeader {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: FdpTableHeader, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: FdpTableHeader, isStandalone: true, selector: "fdp-table-header", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: FdpTableHeader, decorators: [{
            type: Directive,
            args: [{ selector: 'fdp-table-header', standalone: true }]
        }] });
/**
 * Header cell definition for a platform table.
 * Captures the template of a column's data row cell as well as cell-specific properties.
 */
// eslint-disable-next-line @angular-eslint/directive-class-suffix
class FdpHeaderCellDef {
    /** @hidden */
    constructor(templateRef) {
        this.templateRef = templateRef;
    }
    /** @hidden */
    static ngTemplateContextGuard(dir, ctx) {
        return true;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: FdpHeaderCellDef, deps: [{ token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: FdpHeaderCellDef, isStandalone: true, selector: "[fdpHeaderCellDef]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: FdpHeaderCellDef, decorators: [{
            type: Directive,
            args: [{ selector: '[fdpHeaderCellDef]', standalone: true }]
        }], ctorParameters: () => [{ type: i0.TemplateRef }] });

class TableInitialStateDirective extends TableInitialState {
    constructor() {
        super(...arguments);
        /** Initial sort options. */
        this.initialSortBy = [];
        /** Initial filter options. */
        this.initialFilterBy = [];
        /** Initial group options. */
        this.initialGroupBy = [];
        /** Initial page. */
        this.initialPage = 1;
        /** @hidden */
        this._tableService = inject(TableService);
    }
    /**
     * Initial state of table.
     */
    set state(value) {
        this.setTableState(value || DEFAULT_TABLE_STATE);
    }
    get state() {
        return this.getTableState();
    }
    /** @hidden */
    setTable(table) {
        this._table = table;
    }
    /** @hidden */
    setInitialState() {
        const prevState = this.getTableState();
        const columns = this._table.getTableColumns();
        const page = prevState.page;
        const visibleColumns = this.initialVisibleColumns ||
            (prevState.columns.length ? prevState.columns : columns.filter((c) => c.visible).map(({ name }) => name));
        const directiveInitialPage = this.initialPage ?? 1;
        this._table._loadPreviousPages =
            this._table.pageScrolling && this._table.loadPagesBefore && directiveInitialPage > 1;
        const initialPage = this._table._loadPreviousPages
            ? 1
            : directiveInitialPage < page.currentPage
                ? page.currentPage
                : directiveInitialPage;
        const initialPageSize = (this._table._loadPreviousPages ? directiveInitialPage : initialPage) *
            (this._table.pageSize || page.pageSize);
        this.setTableState({
            ...prevState,
            columns: visibleColumns,
            columnKeys: columns.filter((c) => visibleColumns.includes(c.name)).map((c) => c.key),
            sortBy: this.initialSortBy || prevState.sortBy,
            filterBy: this.initialFilterBy || prevState.filterBy,
            groupBy: this.initialGroupBy || prevState.groupBy,
            freezeToColumn: this._table.freezeColumnsTo || prevState.freezeToColumn,
            freezeToEndColumn: this._table.freezeEndColumnsTo || prevState.freezeToEndColumn,
            page: {
                currentPage: initialPage,
                pageSize: initialPageSize
            }
        });
    }
    /** Get current state/settings of the Table. */
    getTableState() {
        return this._tableService.getTableState();
    }
    /** Set current state/settings of the Table. */
    setTableState(state) {
        this._tableService.setTableState(state);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableInitialStateDirective, deps: null, target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: TableInitialStateDirective, isStandalone: true, selector: "[fdpTableInitialState],\n    fdp-table", inputs: { initialVisibleColumns: "initialVisibleColumns", initialSortBy: "initialSortBy", initialFilterBy: "initialFilterBy", initialGroupBy: "initialGroupBy", initialPage: "initialPage", state: "state" }, providers: [
            {
                provide: FDP_TABLE_STATE_DIRECTIVE,
                useExisting: TableInitialStateDirective
            }
        ], usesInheritance: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableInitialStateDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: `[fdpTableInitialState],
    fdp-table`,
                    standalone: true,
                    providers: [
                        {
                            provide: FDP_TABLE_STATE_DIRECTIVE,
                            useExisting: TableInitialStateDirective
                        }
                    ]
                }]
        }], propDecorators: { initialVisibleColumns: [{
                type: Input
            }], initialSortBy: [{
                type: Input
            }], initialFilterBy: [{
                type: Input
            }], initialGroupBy: [{
                type: Input
            }], initialPage: [{
                type: Input
            }], state: [{
                type: Input
            }] } });

/**
 * Table Scrollable.
 * That directive should be assigned to table scrollable area.
 * It registers itself in table scroll dispatcher
 * and notify once element scrolled.
 *
 * For internal usage.
 */
class TableScrollableDirective {
    /** @hidden */
    constructor(elementRef, scrollDispatcher, ngZone) {
        this.elementRef = elementRef;
        this.scrollDispatcher = scrollDispatcher;
        this.ngZone = ngZone;
        /** @hidden */
        this._skipEvent = false;
        /** @hidden */
        this._prevScrollTop = 0;
        /** @hidden */
        this._prevScrollLeft = 0;
        /** @hidden */
        this._destroyRef = inject(DestroyRef);
        /** @hidden */
        this._document = inject(DOCUMENT);
        /** Scroll events stream */
        this._elementScrollStream = new Observable((observer) => {
            const subscription = this.ngZone.runOutsideAngular(() => fromEvent(this.elementRef.nativeElement, 'scroll').subscribe(observer));
            return () => subscription.unsubscribe();
        }).pipe(filter(() => {
            if (this._skipEvent) {
                this._skipEvent = false;
                return false;
            }
            return true;
        }), takeUntilDestroyed(this._destroyRef), share());
        /** Vertical scroll stream */
        this._elementVerticalScrollStream = this._elementScrollStream.pipe(filter(() => this._prevScrollTop !== this.getScrollTop()), tap(() => (this._prevScrollTop = this.getScrollTop())));
        /** Horizontal scroll stream */
        this._elementHorizontalScrollStream = this._elementScrollStream.pipe(filter(() => this._prevScrollLeft !== this.getScrollLeft()), tap(() => (this._prevScrollLeft = this.getScrollLeft())));
    }
    /** @hidden */
    ngOnInit() {
        this.scrollDispatcher.register(this);
    }
    /** @hidden */
    ngOnDestroy() {
        this.scrollDispatcher.deregister(this);
    }
    /** Returns observable that emits when a scroll event is fired on the host element. */
    getScrollStream() {
        return this._elementScrollStream;
    }
    /** Returns observable that emits when a vertical scroll event is happened on the host element. */
    getVerticalScrollStream() {
        return this._elementVerticalScrollStream;
    }
    /** Returns observable that emits when a horizontal scroll event is happened on the host element. */
    getHorizontalScrollStream() {
        return this._elementHorizontalScrollStream;
    }
    /** Returns scrollTop position of the host element. */
    getScrollTop() {
        return this.elementRef.nativeElement?.scrollTop;
    }
    /** Returns scrollLeft position of the host element. */
    getScrollLeft() {
        return this.elementRef.nativeElement?.scrollLeft;
    }
    /** Set scrollTop position of the host element. */
    setScrollTop(scrollTop, emitEvent = true) {
        if (!this.elementRef.nativeElement || this.elementRef.nativeElement.scrollTop === scrollTop) {
            return;
        }
        this._skipEvent = !emitEvent;
        this.elementRef.nativeElement.scrollTop = scrollTop;
    }
    /** Set scrollLeft position of the host element. */
    setScrollLeft(scrollLeft, emitEvent = true) {
        if (!this.elementRef.nativeElement || this.elementRef.nativeElement.scrollLeft === scrollLeft) {
            return;
        }
        this._skipEvent = !emitEvent;
        this.elementRef.nativeElement.scrollLeft = scrollLeft;
    }
    /** Scrolls to overlapped cell. */
    scrollToOverlappedCell(rtl, freezableColumnsSize, freezableEndColumnSize) {
        const tableScrollableEl = this.elementRef.nativeElement;
        const isRtl = rtl;
        if ((freezableColumnsSize || freezableEndColumnSize) &&
            tableScrollableEl.scrollWidth > tableScrollableEl.clientWidth) {
            const activeEl = this._document.activeElement;
            if (activeEl &&
                !(activeEl.classList.contains('fd-table__cell--fixed') ||
                    activeEl.classList.contains('fd-table__cell--fixed-end'))) {
                if (freezableColumnsSize && !freezableEndColumnSize) {
                    activeEl.scrollIntoView({ block: 'nearest', inline: 'end' });
                }
                else if (!freezableColumnsSize && freezableEndColumnSize) {
                    activeEl.scrollIntoView({ block: 'nearest', inline: 'center' });
                }
                else if (freezableColumnsSize && freezableEndColumnSize) {
                    // check to see if another element obstructs the active element
                    const activeElmRect = activeEl.getBoundingClientRect();
                    const activeElLeft = activeElmRect.left;
                    const activeElTop = activeElmRect.top;
                    const activeElWidth = activeElmRect.width;
                    const topElementFromLeft = this._document.elementFromPoint(activeElLeft, activeElTop);
                    // if the activeEl is overlapped
                    if (topElementFromLeft &&
                        !activeEl.isSameNode(topElementFromLeft) &&
                        topElementFromLeft.classList.contains('fd-table__cell--fixed-end')) {
                        const topElementX = topElementFromLeft.getBoundingClientRect().left;
                        const leftVal = isRtl
                            ? (activeElLeft + activeElWidth - topElementX) * -1
                            : activeElLeft + activeElWidth - topElementX;
                        tableScrollableEl.scrollBy({ top: 0, left: leftVal });
                    }
                    else if (topElementFromLeft &&
                        !activeEl.isSameNode(topElementFromLeft) &&
                        topElementFromLeft.classList.contains('fd-table__cell--fixed')) {
                        const topElementX = topElementFromLeft.getBoundingClientRect().right;
                        const leftVal = isRtl
                            ? (activeElLeft - activeElWidth - topElementX) * -1
                            : activeElLeft - activeElWidth - topElementX;
                        tableScrollableEl.scrollBy({ top: 0, left: leftVal });
                    }
                }
            }
        }
    }
    /** Set Scroll Position during component initialization. */
    initializeScrollTop(scrollTop) {
        this.ngZone.runOutsideAngular(() => {
            setTimeout(() => {
                this.setScrollTop(scrollTop, false);
            });
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableScrollableDirective, deps: [{ token: i0.ElementRef }, { token: TableScrollDispatcherService }, { token: i0.NgZone }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: TableScrollableDirective, isStandalone: true, selector: "[fdpTableScrollable]", providers: [{ provide: TABLE_SCROLLABLE, useExisting: forwardRef(() => TableScrollableDirective) }], exportAs: ["tableScrollable"], ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableScrollableDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[fdpTableScrollable]',
                    exportAs: 'tableScrollable',
                    standalone: true,
                    providers: [{ provide: TABLE_SCROLLABLE, useExisting: forwardRef(() => TableScrollableDirective) }]
                }]
        }], ctorParameters: () => [{ type: i0.ElementRef }, { type: TableScrollDispatcherService }, { type: i0.NgZone }] });

// eslint-disable-next-line @angular-eslint/directive-class-suffix
class FdpViewSettingsFilterCustomDef {
    /** @hidden */
    constructor(templateRef) {
        this.templateRef = templateRef;
    }
    /** @hidden */
    static ngTemplateContextGuard(dir, ctx) {
        return true;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: FdpViewSettingsFilterCustomDef, deps: [{ token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: FdpViewSettingsFilterCustomDef, isStandalone: true, selector: "[fdpViewSettingsFilterCustomDef]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: FdpViewSettingsFilterCustomDef, decorators: [{
            type: Directive,
            args: [{ selector: '[fdpViewSettingsFilterCustomDef]', standalone: true }]
        }], ctorParameters: () => [{ type: i0.TemplateRef }] });

class TableVirtualScrollDirective extends TableVirtualScroll {
    /** @hidden */
    constructor(_tableService) {
        super();
        this._tableService = _tableService;
        /** Whether to show only visible rows in matter of performance
         * false by default, when true setting bodyHeight and rowHeight is required.
         */
        this.virtualScroll = false;
        /** Cache size for the virtualScroll, default is 40 in each direction */
        this.renderAhead = 40;
        /**
         * Minimum height of the row, required for the virtualScroll,
         * default is 44px in cozy, 32px in compact and 24px in condensed (set automatically)
         */
        this.rowHeight = ROW_HEIGHT.get(ContentDensityMode.COZY);
        /** @hidden */
        this.virtualScrollTotalHeight = 0;
        /** @hidden */
        this.virtualScrollTransform$ = new BehaviorSubject(0);
        /** @hidden */
        this._virtualScrollCache = { startNodeIndex: -1, visibleNodeCount: -1, totalNodeCount: -1 };
        /** @hidden */
        this._tableScrollDispatcher = inject(TableScrollDispatcherService);
        /** @hidden */
        this._destroyRef = inject(DestroyRef);
    }
    /** @hidden */
    ngOnChanges(changes) {
        if (!this._table.tableScrollable) {
            return;
        }
        if (this.virtualScroll && (changes['rowHeight'] || changes['virtualScroll'] || changes['renderAhead'])) {
            this.calculateVirtualScrollRows();
        }
    }
    /** Sets table reference. */
    setTable(table) {
        this._table = table;
    }
    /**
     * Calculates rows to be rendered in the table.
     */
    calculateVirtualScrollRows() {
        if (!this.virtualScroll || !this.bodyHeight) {
            return;
        }
        let rowHeight = this.rowHeight + 1;
        if (this._tableService.poppingColumns$().length > 0) {
            rowHeight =
                rowHeight +
                    this._table.tableContainer.nativeElement.getElementsByClassName('fd-table__row--secondary')[0]
                        .clientHeight;
        }
        const rowsVisible = this._table._tableRowsVisible;
        const currentlyRenderedRows = this._table.getCurrentlyRenderedRows();
        const totalNodeCount = rowsVisible.length;
        const scrollTop = this._table.tableScrollable.getScrollTop();
        let startNodeIndex = Math.floor(scrollTop / rowHeight) - this.renderAhead;
        startNodeIndex = Math.max(0, startNodeIndex);
        let visibleNodeCount = Math.ceil(this._table.tableContainer.nativeElement.clientHeight / rowHeight) + 2 * this.renderAhead;
        visibleNodeCount = Math.min(totalNodeCount - startNodeIndex, visibleNodeCount);
        this.virtualScrollTransform$.next(startNodeIndex * rowHeight);
        // Simple caching to avoid unnecessary re-renderings
        const isCached = startNodeIndex === this._virtualScrollCache.startNodeIndex &&
            visibleNodeCount === this._virtualScrollCache.visibleNodeCount &&
            totalNodeCount === this._virtualScrollCache.totalNodeCount &&
            // On rows change, even if the total number of rows is the same, the row object will be different
            startNodeIndex === currentlyRenderedRows[0];
        if (isCached) {
            return;
        }
        this._virtualScrollCache = { startNodeIndex, visibleNodeCount, totalNodeCount };
        this.virtualScrollTotalHeight = totalNodeCount * rowHeight - visibleNodeCount * rowHeight;
        this._table.setCurrentlyRenderedRows(startNodeIndex, rowsVisible.slice(startNodeIndex, startNodeIndex + visibleNodeCount).length);
    }
    /**
     * Initialises scroll listener.
     */
    listenOnVirtualScroll() {
        this._tableScrollDispatcher
            .verticallyScrolled()
            .pipe(filter$1(() => this.virtualScroll && !!this.bodyHeight), takeUntilDestroyed(this._destroyRef))
            .subscribe((scrollable) => {
            this.calculateVirtualScrollRows();
            this._table.setTableState({
                ...this._table.getTableState(),
                scrollTopPosition: scrollable.getScrollTop()
            });
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableVirtualScrollDirective, deps: [{ token: TableService }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: TableVirtualScrollDirective, isStandalone: true, selector: "fdp-table[virtualScroll]", inputs: { virtualScroll: "virtualScroll", renderAhead: "renderAhead", bodyHeight: "bodyHeight", rowHeight: "rowHeight", secondaryRowHeight: "secondaryRowHeight" }, providers: [
            {
                provide: FDP_TABLE_VIRTUAL_SCROLL_DIRECTIVE,
                useExisting: TableVirtualScrollDirective
            }
        ], usesInheritance: true, usesOnChanges: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableVirtualScrollDirective, decorators: [{
            type: Directive,
            args: [{
                    // eslint-disable-next-line @angular-eslint/directive-selector
                    selector: 'fdp-table[virtualScroll]',
                    standalone: true,
                    providers: [
                        {
                            provide: FDP_TABLE_VIRTUAL_SCROLL_DIRECTIVE,
                            useExisting: TableVirtualScrollDirective
                        }
                    ]
                }]
        }], ctorParameters: () => [{ type: TableService }], propDecorators: { virtualScroll: [{
                type: Input
            }], renderAhead: [{
                type: Input
            }], bodyHeight: [{
                type: Input
            }], rowHeight: [{
                type: Input
            }], secondaryRowHeight: [{
                type: Input
            }] } });

class TableCellStylesPipe {
    /** @hidden */
    transform(column, isRtl, semanticHighlightingColumnWidth, selectionColumnWidth, isFrozenColumn, isFrozenEndColumn, prevColumnWidthPx, columnWidth, nextColumnWidthPx, noBorders, noBordersY, noBordersX) {
        const styles = {};
        if (isFrozenColumn) {
            const key = isRtl ? 'right.px' : 'left.px';
            styles[key] =
                (semanticHighlightingColumnWidth || 0) + (selectionColumnWidth || 0) + (prevColumnWidthPx || 0);
        }
        if (isFrozenEndColumn) {
            const key = isRtl ? 'left.px' : 'right.px';
            styles[key] = (nextColumnWidthPx || 0).toString();
        }
        if (noBordersY) {
            styles['border-top'] = 'none';
            styles['border-bottom'] = 'none';
        }
        if (noBordersX) {
            styles['border-left'] = 'none';
            styles['border-right'] = 'none';
        }
        if (noBorders) {
            styles['border'] = 'none';
        }
        else {
            styles['min-width'] = columnWidth;
            styles['max-width'] = columnWidth;
            styles['width'] = columnWidth;
        }
        // The "start" value does align left when you are using a LTR browser.
        // In RTL browsers, the "start" value aligns right.
        // Since we want to dynamically apply alignment only depending on the service value, we are using "left"/"right" as values instead
        switch (column.align) {
            case ColumnAlign.START:
                styles['text-align'] = isRtl ? 'right' : 'left';
                break;
            case ColumnAlign.END:
                styles['text-align'] = isRtl ? 'left' : 'right';
                break;
            default:
                styles['text-align'] = 'center';
                break;
        }
        return styles;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableCellStylesPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe }); }
    static { this.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "19.0.7", ngImport: i0, type: TableCellStylesPipe, isStandalone: true, name: "tableCellStyles" }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableCellStylesPipe, decorators: [{
            type: Pipe,
            args: [{ name: 'tableCellStyles', standalone: true }]
        }] });

class ColumnResizableSidePipe {
    /** @hidden */
    transform(columnIndex, visibleColumnsLength, navigationColumnShown) {
        if (columnIndex === 0) {
            return 'end';
        }
        const isLastColumn = columnIndex === visibleColumnsLength - 1;
        if (isLastColumn && navigationColumnShown) {
            return 'start';
        }
        return 'both';
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: ColumnResizableSidePipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe }); }
    static { this.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "19.0.7", ngImport: i0, type: ColumnResizableSidePipe, isStandalone: true, name: "columnResizableSide" }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: ColumnResizableSidePipe, decorators: [{
            type: Pipe,
            args: [{ name: 'columnResizableSide', standalone: true }]
        }] });

const ARIA_SORT = {
    none: 'none',
    asc: 'ascending',
    desc: 'descending'
};
class TableColumnSortingDirectionPipe {
    /** @hidden */
    transform(sorting) {
        if (!sorting) {
            return 'none';
        }
        return ARIA_SORT[sorting.direction];
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableColumnSortingDirectionPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe }); }
    static { this.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "19.0.7", ngImport: i0, type: TableColumnSortingDirectionPipe, isStandalone: true, name: "fdpTableColumnSortingDirection" }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableColumnSortingDirectionPipe, decorators: [{
            type: Pipe,
            args: [{
                    pure: true,
                    standalone: true,
                    name: 'fdpTableColumnSortingDirection'
                }]
        }] });

class RowClassesPipe {
    /** @hidden */
    transform(row, rowsClass) {
        const treeRowClass = row.type === TableRowType.TREE ? 'fdp-table__row--tree' : '';
        const rowClasses = this._getRowCustomCssClasses(row, rowsClass);
        return rowClasses.concat(' ', treeRowClass).trim();
    }
    /** @hidden */
    _getRowCustomCssClasses(row, rowsClass) {
        if (!rowsClass) {
            return '';
        }
        let rowClasses = '';
        if (isString$1(rowsClass)) {
            rowClasses = rowsClass;
        }
        if (isFunction(rowsClass)) {
            rowClasses = rowsClass(row.value) || '';
        }
        return rowClasses.trim();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: RowClassesPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe }); }
    static { this.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "19.0.7", ngImport: i0, type: RowClassesPipe, isStandalone: true, name: "rowClasses" }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: RowClassesPipe, decorators: [{
            type: Pipe,
            args: [{ name: 'rowClasses', standalone: true }]
        }] });

class SelectionCellStylesPipe {
    /** @hidden */
    transform(contentDensity, rtl, semanticHighlightingColumnWidth) {
        if (!contentDensity) {
            contentDensity = ContentDensityMode.COZY;
        }
        const rtlKey = rtl ? 'right' : 'left';
        const selectionColumnWidth = SELECTION_COLUMN_WIDTH.get(contentDensity) + 'px';
        return {
            [rtlKey]: (semanticHighlightingColumnWidth || 0) + 'px',
            'min-width': selectionColumnWidth,
            'max-width': selectionColumnWidth
        };
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: SelectionCellStylesPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe }); }
    static { this.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "19.0.7", ngImport: i0, type: SelectionCellStylesPipe, isStandalone: true, name: "selectionCellStyles" }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: SelectionCellStylesPipe, decorators: [{
            type: Pipe,
            args: [{ name: 'selectionCellStyles', standalone: true }]
        }] });

const importExports = [
    TableInitialStateDirective,
    TableDraggableDirective,
    TableVirtualScrollDirective,
    PlatformTableColumnResponsiveDirective,
    FdpTableCell,
    TableCellHeaderPopoverDirective,
    PlatformTableCellResizableDirective,
    TableDataSourceDirective,
    TableDraggableDirective,
    FdpTableHeader,
    TableHeaderResizerDirective,
    TableInitialStateDirective,
    TableScrollableDirective,
    FdpCellDef,
    FdpEditableCellDef,
    FdpEditableCellFormDirective,
    FdpViewSettingsFilterCustomDef,
    TableVirtualScrollDirective,
    FdpHeaderCellDef,
    SelectionCellStylesPipe,
    TableCellStylesPipe,
    ColumnResizableSidePipe,
    RowClassesPipe
];
class TableHelpersModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableHelpersModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "19.0.7", ngImport: i0, type: TableHelpersModule, imports: [TableInitialStateDirective,
            TableDraggableDirective,
            TableVirtualScrollDirective,
            PlatformTableColumnResponsiveDirective,
            FdpTableCell,
            TableCellHeaderPopoverDirective,
            PlatformTableCellResizableDirective,
            TableDataSourceDirective,
            TableDraggableDirective,
            FdpTableHeader,
            TableHeaderResizerDirective,
            TableInitialStateDirective,
            TableScrollableDirective,
            FdpCellDef,
            FdpEditableCellDef,
            FdpEditableCellFormDirective,
            FdpViewSettingsFilterCustomDef,
            TableVirtualScrollDirective,
            FdpHeaderCellDef,
            SelectionCellStylesPipe,
            TableCellStylesPipe,
            ColumnResizableSidePipe,
            RowClassesPipe], exports: [TableInitialStateDirective,
            TableDraggableDirective,
            TableVirtualScrollDirective,
            PlatformTableColumnResponsiveDirective,
            FdpTableCell,
            TableCellHeaderPopoverDirective,
            PlatformTableCellResizableDirective,
            TableDataSourceDirective,
            TableDraggableDirective,
            FdpTableHeader,
            TableHeaderResizerDirective,
            TableInitialStateDirective,
            TableScrollableDirective,
            FdpCellDef,
            FdpEditableCellDef,
            FdpEditableCellFormDirective,
            FdpViewSettingsFilterCustomDef,
            TableVirtualScrollDirective,
            FdpHeaderCellDef,
            SelectionCellStylesPipe,
            TableCellStylesPipe,
            ColumnResizableSidePipe,
            RowClassesPipe] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableHelpersModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TableHelpersModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [...importExports],
                    exports: [...importExports]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { ARIA_SORT, ArrayTableDataProvider, ArrayTableDataSource, BaseTableDataProvider, BaseTableDataSource, ChildTableDataSource, ColumnAlign, ColumnResizableSidePipe, ColumnsChange, DEFAULT_HIGHLIGHTING_KEY, DEFAULT_TABLE_PAGE, DEFAULT_TABLE_SEARCH_INPUT, DEFAULT_TABLE_STATE, EDITABLE_ROW_SEMANTIC_STATE, EditableTableCell, FDP_TABLE_DRAGGABLE_DIRECTIVE, FDP_TABLE_STATE_DIRECTIVE, FDP_TABLE_VIRTUAL_SCROLL_DIRECTIVE, FILTER_BOOLEAN_STRATEGIES, FILTER_BOOLEAN_STRATEGY, FILTER_DATE_STRATEGIES, FILTER_DATE_STRATEGY, FILTER_DEFAULT_STRATEGIES, FILTER_DEFAULT_STRATEGY, FILTER_NUMBER_STRATEGIES, FILTER_NUMBER_STRATEGY, FILTER_STRATEGY, FILTER_STRATEGY_LABEL, FILTER_STRING_STRATEGIES, FILTER_STRING_STRATEGY, FdpCellDef, FdpEditableCellDef, FdpEditableCellFormDirective, FdpHeaderCellDef, FdpTableCell, FdpTableHeader, FdpViewSettingsFilterCustomDef, FilterType, FilterableColumnDataType, ObservableTableDataProvider, ObservableTableDataSource, PageChange, PlatformTableCellResizableDirective, PlatformTableColumnResponsiveDirective, ROW_HEIGHT, RowClassesPipe, SELECTION_COLUMN_WIDTH, SEMANTIC_HIGHLIGHTING_COLUMN_WIDTH, SaveRowsEvent, SelectionCellStylesPipe, SelectionMode, SortDirection, TABLE_CELL_RESIZABLE_THRESHOLD_PX, TABLE_COLUMN_MIN_WIDTH, TABLE_RESIZER_BORDER_WIDTH, TABLE_SCROLLABLE, Table, TableCellActivateEvent, TableCellHeaderPopoverDirective, TableCellStylesPipe, TableChildrenDataProvider, TableColumn, TableColumnFreezeEvent, TableColumnResizeService, TableColumnSortingDirectionPipe, TableColumnsChangeEvent, TableDataProvider, TableDataSource, TableDataSourceDirective, TableDataSourceParser, TableDraggable, TableDraggableDirective, TableFilterChangeEvent, TableGroupChangeEvent, TableHeaderResizerDirective, TableHelpersModule, TableInitialState, TableInitialStateDirective, TablePageChangeEvent, TableResponsiveService, TableRowActivateEvent, TableRowImpl, TableRowSelectionChangeEvent, TableRowService, TableRowToggleOpenStateEvent, TableRowType, TableRowsRearrangeEvent, TableScrollDispatcherService, TableScrollableDirective, TableSearchChangeEvent, TableSelectionChangeEvent, TableService, TableSortChangeEvent, TableVirtualScroll, TableVirtualScrollDirective, applySelectionToChildren, buildNewRowSkeleton, convertObjectsToTableRows, convertTreeLikeToFlatList, convertTreeObjectsToTableRows, convertTreeTableRowToFlatList, createGroupedTableRowsTree, filterByBoolean, filterByDate, filterByNumber, filterByString, findRowChildren, getFilterStrategiesBasedOnDataType, getFreezableColumns, getFreezableEndColumns, getRowParents, getScrollBarWidth, getSelectableRows, getSelectionStatusByRowValue, getUniqueListValuesByKey, isCollectionFilter, isGroupRow, isItemRow, isRowNavigatable, isSelectableRow, isTableRow, isTreeRow, isTreeRowFirstCell, newTableRow, sortTreeLikeGroupedRows, toDataStream };
//# sourceMappingURL=fundamental-ngx-platform-table-helpers.mjs.map
