import * as i0 from '@angular/core';
import { Injectable, EventEmitter, inject, DestroyRef, ChangeDetectorRef, Output, Input, Directive, ViewChildren, ChangeDetectionStrategy, ViewEncapsulation, Component, ViewChild, TemplateRef, ContentChild, NgModule } from '@angular/core';
import { take, map, debounceTime, finalize, filter } from 'rxjs/operators';
import { takeUntilDestroyed } from '@angular/core/rxjs-interop';
import { ContentDensityMode, ContentDensityDirective } from '@fundamental-ngx/core/content-density';
import { cloneDeep, uniq, mergeWith, concat } from 'lodash-es';
import { Subject, BehaviorSubject, of, firstValueFrom } from 'rxjs';
import * as i4 from '@fundamental-ngx/cdk/utils';
import { isFunction, selectStrategy, InitialFocusDirective, RepeatDirective } from '@fundamental-ngx/cdk/utils';
import * as i2 from '@fundamental-ngx/platform/form';
import { FormGeneratorComponent, FormGeneratorService, FORM_GENERATOR_ITEM_CONFIG, FORM_GENERATOR_CONFIG, PlatformFormGeneratorModule } from '@fundamental-ngx/platform/form';
import { CdkScrollable } from '@angular/cdk/overlay';
import { NgTemplateOutlet } from '@angular/common';
import * as i2$2 from '@fundamental-ngx/core/bar';
import { BarModule } from '@fundamental-ngx/core/bar';
import { ButtonComponent } from '@fundamental-ngx/core/button';
import * as i1$1 from '@fundamental-ngx/core/dialog';
import { DialogBodyComponent, DialogHeaderComponent, DialogComponent } from '@fundamental-ngx/core/dialog';
import * as i3 from '@fundamental-ngx/core/message-box';
import { MessageBoxService, MessageBoxModule } from '@fundamental-ngx/core/message-box';
import { ScrollbarDirective } from '@fundamental-ngx/core/scrollbar';
import { TitleComponent } from '@fundamental-ngx/core/title';
import * as i2$1 from '@fundamental-ngx/core/wizard';
import { WizardModule } from '@fundamental-ngx/core/wizard';
import { isFunction as isFunction$1 } from '@fundamental-ngx/platform/shared';
import { FormLabelComponent } from '@fundamental-ngx/core/form';
import * as i1 from '@fundamental-ngx/core/layout-grid';
import { LayoutGridModule } from '@fundamental-ngx/core/layout-grid';
import { FdTranslatePipe } from '@fundamental-ngx/i18n';
import { LinkComponent } from '@fundamental-ngx/platform/link';
import { SkeletonComponent } from '@fundamental-ngx/core/skeleton';

var WizardGeneratorRefreshStrategy;
(function (WizardGeneratorRefreshStrategy) {
    WizardGeneratorRefreshStrategy["REFRESH_STEP_VISIBILITY"] = "refreshStepVisibility";
    WizardGeneratorRefreshStrategy["REVALIDATE_STEP_FORMS"] = "revalidateStepForms";
    WizardGeneratorRefreshStrategy["REFRESH_FORM_VISIBILITY"] = "refreshFormVisibility";
})(WizardGeneratorRefreshStrategy || (WizardGeneratorRefreshStrategy = {}));
/**
 * @description Helper service to keep all transformations and data in one place.
 */
class WizardGeneratorService {
    /** @hidden */
    constructor(_formGeneratorService) {
        this._formGeneratorService = _formGeneratorService;
        /**
         * @description Visible steps components
         */
        this.stepsComponents = new Map();
        /**
         * @description Object with steps that are dependencies for another steps.
         */
        this.dependencySteps = {};
        /**
         * @description Whether to append the step to the wizard. If false, each step will be displayed on a different page.
         * Default is true.
         */
        this.appendToWizard = true;
        /** @hidden */
        this._shouldRedirectToSummary = false;
        /**
         * @description Steps that are visible to the user.
         */
        this.visibleWizardSteps = [];
        /** @hidden */
        this._visibleWizardSteps$ = new Subject();
        /** @hidden */
        this._appendToWizard$ = new BehaviorSubject(this.appendToWizard);
        /** @hidden */
        this._stepsComponents$ = new Subject();
        /** @hidden */
        this._stepsOrderChanged$ = new Subject();
        /** @hidden */
        this._nextStepIndex$ = new BehaviorSubject(1);
        /** @hidden */
        this._submittedFormRawValues = {};
        /** @hidden */
        this._wizardStepIds = [];
    }
    /**
     * @description Returns current step ID in Wizard.
     * @returns {string | undefined} Current step ID.
     */
    getCurrentStepId() {
        const currentStepId = this.visibleWizardSteps?.find((i) => i.status === 'current');
        return currentStepId?.id;
    }
    /**
     * @description Returns current step index.
     * @returns {number} Index of the current step in array of Wizard steps.
     */
    getCurrentStepIndex() {
        const currentStepId = this.getCurrentStepId();
        return this.getStepIndex(currentStepId);
    }
    /**
     * Searches for the index of the step based on it's ID
     * @param stepId Step ID
     * @returns Index in the array of steps for defined step ID.
     */
    getStepIndex(stepId) {
        const stepIndex = this.visibleWizardSteps?.findIndex((i) => i.id === stepId);
        return stepIndex > -1 ? stepIndex : 0;
    }
    /**
     * @description Sends command to submit all forms inside the step.
     * @param stepId Step ID for which forms need's to be submitted.
     * @param skipIfUntouched Skip validation if form haven't been touched.
     * @returns {Observable<WizardStepSubmittedForms>} Observable, which will emit
     * when all visible forms in step are submitted.
     */
    submitStepForms(stepId, skipIfUntouched = false) {
        if (!stepId) {
            return of(null);
        }
        const step = this.stepsComponents.get(stepId);
        if (!step) {
            return of(null);
        }
        return step.submitForms(skipIfUntouched).pipe(take(1), map((result) => {
            if (result) {
                this._submittedFormRawValues[stepId] = result;
            }
            return result;
        }));
    }
    /**
     * @description Triggers validation for current step forms.
     * @returns {Observable<boolean>} Observable with validation result.
     */
    validateStepForms(skipIfUntouched = false) {
        const currentStepId = this.getCurrentStepId();
        if (!currentStepId) {
            return of(true);
        }
        return this.submitStepForms(currentStepId, skipIfUntouched).pipe(map((result) => result === null || Object.values(result).every((r) => r.success)));
    }
    /**
     * @description Sets new array of visible steps for Wizard.
     * @param steps steps that needs to be shown in Wizard.
     */
    setVisibleSteps(steps) {
        this.visibleWizardSteps = steps;
        this._visibleWizardSteps$.next(steps);
        this.setWizardStepIds(steps.map((s) => s.id));
    }
    /**
     * @returns {Observable<PreparedWizardGeneratorItem[]>} Observable, which will emit every time
     * when array of visible step items has been changed.
     */
    getVisibleSteps() {
        return this._visibleWizardSteps$.asObservable();
    }
    /**
     * @description Runs initial transformation for passed Wizard steps.
     * @param items All Wizard steps.
     * @returns {Promise<PreparedWizardGeneratorItem[]>} Array of transformed Wizard steps.
     */
    async prepareWizardItems(items) {
        let newItems = await Promise.all(items.map(async (i, index) => {
            const item = { ...i };
            item.status = item.status || 'upcoming';
            item.title = await this._getFormItemPropertyValue(item, index, 'title');
            item.name = await this._getFormItemPropertyValue(item, index, 'name');
            return item;
        }));
        // Summary step must be the last one in array
        const summaryStepIndex = newItems.findIndex((i) => i.summary === true);
        if (summaryStepIndex !== -1 && summaryStepIndex !== newItems.length - 1) {
            newItems.splice(newItems.length - 1, 0, newItems.splice(summaryStepIndex, 1)[0]);
        }
        // If no current step found, set first as current.
        if (newItems.findIndex((i) => i.status === 'current') === -1) {
            newItems[0].status = 'current';
        }
        newItems = this._setBranchingSteps(newItems);
        this._buildDependencyMap(newItems);
        this.items = newItems;
        await this.refreshStepVisibility();
        return newItems;
    }
    /**
     * @description Runs conditional function for each step to define if step should be shown.
     */
    async refreshStepVisibility() {
        const formValue = await this.getWizardFormValue();
        const visibleStepIds = {};
        for (const item of this.items) {
            if (!isFunction(item.when)) {
                visibleStepIds[item.id] = true;
                continue;
            }
            const obj = item.when(this._getCompletedStepIds(), formValue, this._formGeneratorService.forms);
            visibleStepIds[item.id] = await this._getFunctionValue(obj);
        }
        this.setVisibleSteps(this.items.filter((item) => visibleStepIds[item.id] === true));
    }
    /**
     * Checks whether current form group should be visible.
     * @param formGroup Form Group.
     */
    async refreshFormVisibility(formGroup) {
        const formValue = await this.getWizardFormValue();
        const obj = formGroup.when?.(this._getCompletedStepIds(), formValue, this._formGeneratorService.forms) ?? true;
        return await this._getFunctionValue(obj);
    }
    /**
     * @description Clears array of Wizard steps components.
     */
    clearWizardStepComponents() {
        this.stepsComponents.clear();
        this._stepsComponents$.next(this.stepsComponents);
    }
    /**
     * @returns {Observable<StepsComponents>} Observable, which will emit every time
     * when Set visible steps components has been changed.
     */
    trackStepsComponents() {
        return this._stepsComponents$.asObservable();
    }
    /**
     * @description Adds new component to the set of Wizard steps components.
     * @param component Wizard Step component to add.
     * @param key Wizard Step ID.
     */
    addWizardStepComponent(component, key) {
        this.stepsComponents.set(key, component);
        this._stepsComponents$.next(this.stepsComponents);
    }
    /**
     * @description Removes Component from the set of Wizard steps components.
     * @param key
     */
    removeWizardStepComponent(key) {
        this.stepsComponents.delete(key);
        this._stepsComponents$.next(this.stepsComponents);
    }
    /**
     * @param step Step ID which includes dependency fields.
     * @returns Object with forms and their fields that are dependencies for other steps.
     */
    getStepDependencyFields(step) {
        return this.dependencySteps[step];
    }
    /**
     * Notifies step components to revalidate inner forms.
     * @param stepIds Step ID's which needs to be revalidated.
     */
    notifyStepsToRevalidateForms(stepIds) {
        stepIds.forEach((stepId) => this.stepsComponents.get(stepId)?.updateFormsState());
    }
    /**
     * Notifies step components to refresh form groups visibility.
     * @param stepIds Step ID's which needs to be checked.
     */
    async refreshFormsVisibility(stepIds) {
        for (const stepId of stepIds) {
            await this.stepsComponents.get(stepId)?.refreshFormsVisibility();
        }
    }
    /**
     * @param formatted Flag defining whether form value should be formatted by form control
     * transformers, or return raw value
     * @returns {WizardGeneratorFormsValue} Wizard form value
     */
    async getWizardFormValue(formatted = false) {
        const wizardFormValue = {};
        if (!this.visibleWizardSteps) {
            return wizardFormValue;
        }
        for (const item of this.visibleWizardSteps.filter((s) => !s.summary)) {
            wizardFormValue[item.id] = {};
            const component = this.stepsComponents.get(item.id);
            if (!component) {
                continue;
            }
            const forms = component.getVisibleForms();
            for (const form of item?.formGroups ?? []) {
                if (!forms[form.id]) {
                    continue;
                }
                wizardFormValue[item.id][form.id] = formatted
                    ? await this._formGeneratorService.getFormValue(forms[form.id]?.form)
                    : this._formGeneratorService._getFormValueWithoutUngrouped(cloneDeep(forms[form.id]?.form.value));
            }
        }
        return wizardFormValue;
    }
    /**
     *
     * @param stepId Step ID to check.
     * @returns {Boolean} indicator if step was previously validated and submitted.
     */
    stepSubmitted(stepId) {
        return this._submittedFormRawValues[stepId] !== undefined;
    }
    /**
     *
     * @returns {Boolean} if steps are untouched, will return true, if yes - false
     */
    isStepsUntouched() {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        return [...this.stepsComponents].every(([_, component]) => component.formGenerators.toArray().every((item) => !item.form.touched));
    }
    /**
     * Reverts wizard to defined step.
     * @param stepId Step ID to edit
     */
    editStep(stepId) {
        const stepIndex = this.visibleWizardSteps.findIndex((s) => s.id === stepId);
        this.visibleWizardSteps = this.visibleWizardSteps.map((step, index) => {
            step.status = index === stepIndex ? 'current' : index < stepIndex ? 'completed' : 'upcoming';
            return step;
        });
        const summaryStepIndex = this.visibleWizardSteps.findIndex((s) => s.summary);
        this._shouldRedirectToSummary = true;
        this._appendToWizard$.next(false);
        this.setVisibleSteps(this.visibleWizardSteps);
        this.setNextStepIndex(summaryStepIndex);
    }
    /**
     * Set's current set of step ID's
     * @param ids step ID's
     */
    setWizardStepIds(ids) {
        if (!this._wizardStepIds.every((stepId, index) => ids[index] === stepId)) {
            const firstChangedStepIndex = this._wizardStepIds.findIndex((stepId, index) => ids[index] !== stepId);
            this._nextStepIndex$.next(firstChangedStepIndex);
            this._stepsOrderChanged$.next(firstChangedStepIndex);
        }
        this._wizardStepIds = ids;
    }
    /**
     * Set next step index.
     * @param index Next step index.
     */
    setNextStepIndex(index) {
        this._nextStepIndex$.next(index);
    }
    /**
     * Returns observable of next step index which emits when index is changed.
     */
    trackNextStepIndex() {
        return this._nextStepIndex$.asObservable();
    }
    /**
     * Returns observable which emits when steps order has been changed.
     */
    trackStepsOrder() {
        return this._stepsOrderChanged$.asObservable();
    }
    /**
     * Stores original 'appendToWizard' input property.
     * @param value
     */
    setOriginalAppendToWizardState(value) {
        this.appendToWizard = value;
        this._appendToWizard$.next(this.appendToWizard);
    }
    /**
     * Resets modified 'appendToWizard' property to original one.
     */
    restoreAppendToWizardState() {
        this._appendToWizard$.next(this.appendToWizard);
    }
    /**
     * Returns observable which emits when 'appendToWizard' property has been changed.
     */
    trackAppendToWizardState() {
        return this._appendToWizard$.asObservable();
    }
    /**
     * @hidden
     * @param items
     * @param index
     * @param key
     * @returns
     */
    async _getFormItemPropertyValue(items, index, key) {
        let value = items[key];
        if (typeof value === 'function') {
            const obj = value(index);
            value = await this._getFunctionValue(obj);
        }
        return value;
    }
    /**
     * @hidden
     * @description Returns value from Promise-like, Observable-like, simple function or just some object.
     * @param obj
     * @returns
     */
    async _getFunctionValue(obj) {
        const strategy = selectStrategy(obj);
        let result;
        await strategy.createSubscription(obj, (value) => {
            result = value;
        });
        if (isFunction(result)) {
            await this._getFunctionValue(result);
        }
        return result;
    }
    /**
     * @hidden
     * @description Sets `branching` property for the steps that will create new branches.
     * @param items
     * @returns
     */
    _setBranchingSteps(items) {
        const branchingItems = items
            .filter((i) => i.dependencyFields)
            .reduce((stepIds, item) => {
            if (item.dependencyFields) {
                stepIds = [...stepIds, ...Object.keys(item.dependencyFields)];
            }
            return stepIds;
        }, []);
        if (branchingItems.length > 0) {
            items = items.map((item) => {
                item.branching = item.branching || branchingItems.includes(item.id);
                return item;
            });
        }
        return items;
    }
    /** @hidden */
    _normalizeDependencyStep(dependency, strategy, dependentStep) {
        const newDependency = {};
        Object.entries(dependency).forEach(([key, value]) => {
            newDependency[key] = {};
            value.forEach((v) => {
                newDependency[key][v] = {
                    [strategy]: dependentStep ? [dependentStep] : []
                };
            });
        });
        return newDependency;
    }
    /** @hidden */
    _buildDependencyMap(items) {
        this.dependencySteps = {};
        const mergeArrays = (objValue, srcValue) => {
            if (Array.isArray(objValue) && !objValue.includes(srcValue)) {
                return uniq(objValue.concat(srcValue));
            }
        };
        const buildDependencySteps = (dependencyFields, strategy, stepId) => {
            for (const [id, forms] of Object.entries(dependencyFields)) {
                this.dependencySteps[id] = mergeWith(this.dependencySteps[id] || {}, this._normalizeDependencyStep(forms, strategy, stepId), mergeArrays);
            }
        };
        items
            .filter((s) => s.formGroups?.length)
            .forEach((step) => {
            if (step.dependencyFields) {
                buildDependencySteps(step.dependencyFields, WizardGeneratorRefreshStrategy.REFRESH_STEP_VISIBILITY);
            }
            const dependentForms = step.formGroups?.filter((form) => form.dependencyFields);
            dependentForms?.forEach((form) => {
                buildDependencySteps(form.dependencyFields, WizardGeneratorRefreshStrategy.REFRESH_FORM_VISIBILITY, step.id);
            });
            const stepFields = concat(...[...(step.formGroups ?? [])].map((item) => item.formItems.filter((f) => f.dependencyFields)));
            stepFields.forEach((formItem) => {
                buildDependencySteps(formItem.dependencyFields, WizardGeneratorRefreshStrategy.REVALIDATE_STEP_FORMS, step.id);
            });
        });
    }
    /** @hidden */
    _getCompletedStepIds() {
        return this.items.filter((i) => i.status === 'completed').map((i) => i.id);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardGeneratorService, deps: [{ token: i2.FormGeneratorService }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardGeneratorService }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardGeneratorService, decorators: [{
            type: Injectable
        }], ctorParameters: () => [{ type: i2.FormGeneratorService }] });

/**
 * @description Default button labels.
 */
const DEFAULT_WIZARD_NAVIGATION_BUTTONS = {
    goBack: {
        label: 'Previous Step',
        contentDensity: ContentDensityMode.COMPACT,
        type: 'ghost'
    },
    goNext: {
        label: 'Next Step',
        contentDensity: ContentDensityMode.COMPACT,
        type: 'emphasized'
    },
    cancel: {
        label: 'Cancel',
        contentDensity: ContentDensityMode.COMPACT,
        type: 'ghost'
    },
    finish: {
        label: 'Finish',
        contentDensity: ContentDensityMode.COMPACT,
        type: 'emphasized'
    },
    review: {
        label: 'Review',
        contentDensity: ContentDensityMode.COMPACT,
        type: 'emphasized'
    }
};
/**
 * @description Base Wizard Generator component with necessary inputs and methods.
 */
class BaseWizardGenerator {
    /**
     * @description Button labels to be used in Wizard navigation
     */
    set navigationButtonLabels(value) {
        this._navigationButtonLabels = Object.assign({}, DEFAULT_WIZARD_NAVIGATION_BUTTONS, value);
    }
    get navigationButtonLabels() {
        return this._navigationButtonLabels;
    }
    /**
     * @description Array of Wizard Steps.
     */
    set items(items) {
        this.wizardCreated = false;
        this._wizardGeneratorService.clearWizardStepComponents();
        if (!items) {
            return;
        }
        this._wizardGeneratorService.prepareWizardItems(items).then((newItems) => {
            this._items = newItems;
            this._visibleItems = newItems;
            this.wizardCreated = true;
            this._cd.detectChanges();
        });
    }
    get items() {
        return this._items;
    }
    /**
     * Whether to append the step to the wizard. If false, each step will be displayed on a different page.
     * Default is true.
     */
    set appendToWizard(value) {
        this._wizardGeneratorService.setOriginalAppendToWizardState(value);
    }
    get appendToWizard() {
        return this._appendToWizard;
    }
    /**
     * @description Is current step is summary step.
     */
    get isSummaryStep() {
        const currentIndex = this._wizardGeneratorService.getCurrentStepIndex();
        return this.visibleItems[currentIndex]?.summary === true;
    }
    /**
     * @description Array of visible Wizard Steps.
     */
    set visibleItems(items) {
        this._visibleItems = items;
    }
    get visibleItems() {
        return this._visibleItems || this._items;
    }
    /**
     * @description Whether or not Wizard is currently on the first step
     */
    get isFirstStep() {
        return this.visibleItems[0]?.status === 'current';
    }
    /**
     * @description Whether or not Wizard is currently on the last step
     */
    get isLastStep() {
        const lastStep = this.visibleItems[this.visibleItems.length - 1];
        return lastStep?.status === 'current' && !lastStep.branching;
    }
    /**
     * Whether current step is a branching step.
     */
    get isBranchingStep() {
        const currentIndex = this._wizardGeneratorService.getCurrentStepIndex();
        return this._visibleItems[currentIndex]?.branching === true;
    }
    /**
     * Whether the next step is a Summary step.
     */
    get isNextStepSummary() {
        const nextStep = this.visibleItems[this._nextStepIndex];
        return nextStep?.summary === true;
    }
    /** Whether the current step is completed */
    get isCurrentStepCompleted() {
        const currentIndex = this._wizardGeneratorService.getCurrentStepIndex();
        return this._visibleItems[currentIndex]?.completed === true;
    }
    /** @hidden */
    constructor() {
        /**
         * @description Whether or not apply responsive paddings styling.
         */
        this.responsivePaddings = true;
        /** If navigation buttons should be visible. */
        this.navigationButtons = true;
        /**
         * @description Boolean flag indicating whether to display Summary step in Wizard progress bar.
         */
        this.displaySummaryStep = false;
        /** Whether all form items should have identical layout provided for form group. */
        this.unifiedLayout = true;
        /**
         * @description Emits wizard value when it's completed.
         */
        this.wizardFinished = new EventEmitter();
        /**
         * Flag indicating that wizard is ready to be shown
         */
        this.wizardCreated = false;
        /**
         * @hidden
         */
        this._visibleItems = [];
        /** @hidden */
        this._stepsOrderChanged = false;
        /**
         * @hidden
         */
        this._navigationButtonLabels = DEFAULT_WIZARD_NAVIGATION_BUTTONS;
        /**
         * @hidden
         */
        this._destroyRef = inject(DestroyRef);
        /** @hidden */
        this._wizardGeneratorService = inject(WizardGeneratorService);
        /** @hidden */
        this._cd = inject(ChangeDetectorRef);
        /**
         * @hidden
         */
        this._items = [];
        /** @hidden */
        this._appendToWizard = false;
        /**
         * @hidden
         */
        this._formPending = false;
        this._wizardGeneratorService
            .getVisibleSteps()
            .pipe(debounceTime(10), takeUntilDestroyed(this._destroyRef))
            .subscribe((visibleSteps) => {
            this.visibleItems = visibleSteps;
            this._cd.detectChanges();
        });
        this._wizardGeneratorService
            .trackStepsComponents()
            .pipe(debounceTime(10), takeUntilDestroyed(this._destroyRef))
            .subscribe(async (stepsComponents) => {
            if (stepsComponents.size === this.items?.length) {
                await this._setVisibleSteps();
            }
        });
        this._wizardGeneratorService
            .trackAppendToWizardState()
            .pipe(takeUntilDestroyed(this._destroyRef))
            .subscribe((value) => {
            this._appendToWizard = value;
        });
        this._wizardGeneratorService
            .trackStepsOrder()
            .pipe(takeUntilDestroyed(this._destroyRef))
            .subscribe((newNextStep) => {
            this._nextStepIndex = newNextStep;
            this._stepsOrderChanged = true;
        });
        this._wizardGeneratorService
            .trackNextStepIndex()
            .pipe(takeUntilDestroyed(this._destroyRef))
            .subscribe((index) => {
            this._nextStepIndex = index;
        });
    }
    /**
     * @description Reverts Wizard progress one step back.
     */
    goBack() {
        const currentStepIndex = this._wizardGeneratorService.getCurrentStepIndex();
        const previousStepIndex = currentStepIndex - 1;
        if (this.visibleItems[previousStepIndex] === undefined) {
            return;
        }
        this._stepsOrderChanged = false;
        this.visibleItems[currentStepIndex].status = 'upcoming';
        this.visibleItems[previousStepIndex].status = 'current';
        this._wizardGeneratorService.setVisibleSteps(this.visibleItems);
        this._wizardGeneratorService.setNextStepIndex(currentStepIndex);
        this._cd.detectChanges();
    }
    /**
     * @description Progresses the Wizard one step further.
     */
    async goNext() {
        if (this._formPending) {
            return;
        }
        const currentStepIndex = this._wizardGeneratorService.getCurrentStepIndex();
        const nextStepIndex = this._nextStepIndex;
        this._wizardGeneratorService
            .validateStepForms()
            .pipe(takeUntilDestroyed(this._destroyRef))
            .subscribe(async (result) => {
            if (!result) {
                return;
            }
            await this._setVisibleSteps();
            const steps = this.visibleItems;
            if (steps[nextStepIndex] === undefined) {
                return;
            }
            this._stepsOrderChanged = false;
            steps[currentStepIndex].status = 'completed';
            steps[currentStepIndex].completed = true;
            steps[nextStepIndex].status = 'current';
            this._wizardGeneratorService.setVisibleSteps(steps);
            this._wizardGeneratorService.setNextStepIndex(nextStepIndex + 1);
            this._cd.detectChanges();
        });
    }
    /**
     * @description Submits step forms.
     * @param currentStepIndex current step index.
     * @returns {Observable<WizardStepSubmittedForms | null>} Observable with form values.
     */
    submitStepForms(currentStepIndex) {
        this._formPending = true;
        return this._wizardGeneratorService.submitStepForms(currentStepIndex).pipe(finalize(() => {
            this._formPending = false;
        }));
    }
    /**
     * @description Completes the wizard and emits it's formatted value.
     */
    async finish() {
        if (this.isSummaryStep) {
            const wizardResult = await this._wizardGeneratorService.getWizardFormValue(true);
            this.wizardFinished.emit(wizardResult);
            return;
        }
        this._wizardGeneratorService
            .validateStepForms()
            .pipe(takeUntilDestroyed(this._destroyRef))
            .subscribe(async (result) => {
            if (!result) {
                return;
            }
            const wizardResult = await this._wizardGeneratorService.getWizardFormValue(true);
            this.wizardFinished.emit(wizardResult);
        });
    }
    /**
     * @description Callback function to update steps status.
     * @param stepId Step ID to be updated.
     * @param status New step status.
     */
    async stepStatusChanged(stepId, status) {
        const stepIndex = this._visibleItems.findIndex((s) => s.id === stepId);
        this._visibleItems[stepIndex].status = status;
        await this._wizardGeneratorService.refreshStepVisibility();
    }
    /**
     * @hidden
     */
    async _setVisibleSteps() {
        await this._wizardGeneratorService.refreshStepVisibility();
        this.wizardCreated = true;
        this._cd.detectChanges();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: BaseWizardGenerator, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: BaseWizardGenerator, isStandalone: true, inputs: { responsivePaddings: "responsivePaddings", navigationButtonLabels: "navigationButtonLabels", items: "items", appendToWizard: "appendToWizard", navigationButtons: "navigationButtons", contentHeight: "contentHeight", displaySummaryStep: "displaySummaryStep", unifiedLayout: "unifiedLayout" }, outputs: { wizardFinished: "wizardFinished" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: BaseWizardGenerator, decorators: [{
            type: Directive
        }], ctorParameters: () => [], propDecorators: { responsivePaddings: [{
                type: Input
            }], navigationButtonLabels: [{
                type: Input
            }], items: [{
                type: Input
            }], appendToWizard: [{
                type: Input
            }], navigationButtons: [{
                type: Input
            }], contentHeight: [{
                type: Input
            }], displaySummaryStep: [{
                type: Input
            }], unifiedLayout: [{
                type: Input
            }], wizardFinished: [{
                type: Output
            }] } });

class WizardGeneratorStepComponent {
    /** @hidden */
    constructor(_wizardGeneratorService, _formGeneratorService) {
        this._wizardGeneratorService = _wizardGeneratorService;
        this._formGeneratorService = _formGeneratorService;
        /** Whether all form items should have identical layout provided for form group. */
        this.unifiedLayout = true;
        /**
         * @description Emits when all forms in the step has been submited
         */
        this.formsSubmitted = new EventEmitter();
        /**
         * @description Emits when all forms in the step has been created
         */
        this.formsCreated = new EventEmitter();
        /**
         * @hidden
         */
        this._submittedForms = {};
        /**
         * @hidden
         */
        this._formSubmitted$ = new Subject();
        /**
         * @hidden
         */
        this._forms = {};
        /**
         * @hidden
         * @private
         */
        this._hiddenFormGroupValues = {};
        /**
         * @hidden
         * An RxJS Subject that will kill the data stream upon component’s destruction (for unsubscribing)
         */
        this._destroyRef = inject(DestroyRef);
    }
    /**
     * @hidden
     */
    async ngOnInit() {
        await this.refreshFormsVisibility();
    }
    /**
     * @hidden
     */
    ngOnDestroy() {
        this._wizardGeneratorService.removeWizardStepComponent(this.item.id);
    }
    /** @hidden */
    async ngOnChanges(changes) {
        if (changes.stepStatus && changes.stepStatus.currentValue === 'current' && this.formGenerators?.length > 0) {
            await this.updateFormsState();
        }
    }
    /**
     * @description Adds created form instance to the object of the forms.
     * @param form Form instance
     * @param key Form ID from @see item
     */
    async onFormCreated(form, key) {
        this._forms[key] = {
            title: this.item.formGroups?.find((g) => g.id === key)?.title ?? '',
            form
        };
        await this._wizardGeneratorService.refreshStepVisibility();
        if (Object.keys(this._forms).length === this.item.formGroups?.length) {
            await this.addComponentToParent();
            this.formsCreated.emit(this._forms);
        }
        if (this._hiddenFormGroupValues[key]) {
            form?.patchValue(this._hiddenFormGroupValues[key]);
        }
        this._trackDependencyFieldsChanges(form, key);
    }
    /**
     * @description Adds component to the service for further usage.
     */
    async addComponentToParent() {
        this._wizardGeneratorService.addWizardStepComponent(this, this.item.id);
    }
    /**
     * @description Callback function when form has been submitted.
     * @param result Form submission result.
     * @param index Form index in the step.
     */
    onFormSubmitted(result, index) {
        this._submittedForms[index] = result;
        if (Object.keys(this._submittedForms).length === this.formGenerators.length) {
            // Last form submission. Send data back to the parent component.
            this._formSubmitted$.next(this._submittedForms);
        }
    }
    /**
     * Returns visible forms in the step.
     */
    getVisibleForms() {
        return Object.keys(this._forms).reduce((acc, key) => {
            if (this._visibleFormGroupIds[key] || this._visibleFormGroupIds[key] === undefined) {
                acc[key] = this._forms[key];
            }
            return acc;
        }, {});
    }
    /**
     * @description Clears previously submitted form values and re-submit them again.
     * @returns Subject, which will return forms value.
     */
    submitForms(skipIfUntouched = false) {
        if (skipIfUntouched && !this._wizardGeneratorService.stepSubmitted(this.item.id)) {
            setTimeout(() => {
                this._formSubmitted$.next(null);
            });
            return this._formSubmitted$;
        }
        // Clear previously submitted form values and re-submit them again.
        this._submittedForms = {};
        setTimeout(() => {
            this.formGenerators.forEach((formComponent) => {
                formComponent.submit();
            });
        });
        return this._formSubmitted$;
    }
    /**
     * Triggers form value revalidation and checks if field in form should be visible.
     */
    async updateFormsState() {
        if (this.formGenerators) {
            for (const formGenerator of this.formGenerators.toArray()) {
                await formGenerator.refreshStepsVisibility();
            }
        }
    }
    /**
     * Refreshes step form group visibility.
     */
    async refreshFormsVisibility() {
        this._visibleFormGroupIds = {};
        if (!this.item.formGroups) {
            return;
        }
        for (const formGroup of this.item.formGroups.filter((fg) => isFunction$1(fg.when))) {
            const result = await this._wizardGeneratorService.refreshFormVisibility(formGroup);
            if (result === this._visibleFormGroupIds[formGroup.id]) {
                return;
            }
            const form = this._forms[formGroup.id]?.form;
            // Store form value so it could be restored later.
            if (!result) {
                this._hiddenFormGroupValues[formGroup.id] = form?.value;
            }
            this._visibleFormGroupIds[formGroup.id] = result;
        }
    }
    /**
     * @hidden
     * @description Keeps track on dependency fields for other steps and refreshes the view when they changed.
     * @param form
     * @param key
     */
    _trackDependencyFieldsChanges(form, key) {
        this._trackedFields = this._wizardGeneratorService.getStepDependencyFields(this.item.id);
        if (this._trackedFields && this._trackedFields[key]) {
            Object.entries(this._trackedFields[key]).forEach(([fieldId, strategies]) => {
                const control = this._formGeneratorService.getFormControl(form, fieldId);
                const refreshSteps = strategies[WizardGeneratorRefreshStrategy.REFRESH_STEP_VISIBILITY] !== undefined;
                const revalidateForms = strategies[WizardGeneratorRefreshStrategy.REVALIDATE_STEP_FORMS];
                const refreshFormVisibility = strategies[WizardGeneratorRefreshStrategy.REFRESH_FORM_VISIBILITY];
                control?.valueChanges
                    .pipe(debounceTime(50), takeUntilDestroyed(this._destroyRef))
                    .subscribe(async () => {
                    if (refreshSteps) {
                        await this._wizardGeneratorService.refreshStepVisibility();
                    }
                    if (revalidateForms?.length) {
                        this._wizardGeneratorService.notifyStepsToRevalidateForms(revalidateForms);
                    }
                    if (refreshFormVisibility?.length) {
                        await this._wizardGeneratorService.refreshFormsVisibility(refreshFormVisibility);
                    }
                });
            });
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardGeneratorStepComponent, deps: [{ token: WizardGeneratorService }, { token: i2.FormGeneratorService }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.0.7", type: WizardGeneratorStepComponent, isStandalone: true, selector: "fdp-wizard-generator-step", inputs: { item: "item", stepStatus: "stepStatus", unifiedLayout: "unifiedLayout" }, outputs: { formsSubmitted: "formsSubmitted", formsCreated: "formsCreated" }, viewQueries: [{ propertyName: "formGenerators", predicate: FormGeneratorComponent, descendants: true }], usesOnChanges: true, ngImport: i0, template: "@if (item.title) {\n    <h3 fd-title>{{ item.title }}</h3>\n}\n@for (form of item.formGroups; track form; let i = $index) {\n    @if (_visibleFormGroupIds[form.id] !== false) {\n        <fdp-form-generator\n            [mainTitle]=\"form.title\"\n            [columnLayout]=\"form.columnLayout\"\n            [noAdditionalContent]=\"true\"\n            [formName]=\"form.id\"\n            [formItems]=\"$any(form.formItems)\"\n            [hint]=\"form.guiOptions?.hint!\"\n            [unifiedLayout]=\"unifiedLayout\"\n            (formCreated)=\"onFormCreated($event, form.id)\"\n            (formSubmittedStatus)=\"onFormSubmitted($event, form.id)\"\n        ></fdp-form-generator>\n    }\n}\n", dependencies: [{ kind: "component", type: TitleComponent, selector: "[fd-title], [fdTitle]", inputs: ["headerSize", "wrap"], exportAs: ["fd-title"] }, { kind: "component", type: FormGeneratorComponent, selector: "fdp-form-generator", inputs: ["formName", "formItems", "mainTitle", "hideMainTItle", "hint", "columnLayout", "labelColumnLayout", "fieldColumnLayout", "gapColumnLayout", "unifiedLayout", "inlineColumnLayout", "noAdditionalContent"], outputs: ["formCreated", "formSubmitted", "formSubmittedStatus"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardGeneratorStepComponent, decorators: [{
            type: Component,
            args: [{ selector: 'fdp-wizard-generator-step', encapsulation: ViewEncapsulation.None, changeDetection: ChangeDetectionStrategy.OnPush, imports: [TitleComponent, FormGeneratorComponent], template: "@if (item.title) {\n    <h3 fd-title>{{ item.title }}</h3>\n}\n@for (form of item.formGroups; track form; let i = $index) {\n    @if (_visibleFormGroupIds[form.id] !== false) {\n        <fdp-form-generator\n            [mainTitle]=\"form.title\"\n            [columnLayout]=\"form.columnLayout\"\n            [noAdditionalContent]=\"true\"\n            [formName]=\"form.id\"\n            [formItems]=\"$any(form.formItems)\"\n            [hint]=\"form.guiOptions?.hint!\"\n            [unifiedLayout]=\"unifiedLayout\"\n            (formCreated)=\"onFormCreated($event, form.id)\"\n            (formSubmittedStatus)=\"onFormSubmitted($event, form.id)\"\n        ></fdp-form-generator>\n    }\n}\n" }]
        }], ctorParameters: () => [{ type: WizardGeneratorService }, { type: i2.FormGeneratorService }], propDecorators: { formGenerators: [{
                type: ViewChildren,
                args: [FormGeneratorComponent]
            }], item: [{
                type: Input
            }], stepStatus: [{
                type: Input
            }], unifiedLayout: [{
                type: Input
            }], formsSubmitted: [{
                type: Output
            }], formsCreated: [{
                type: Output
            }] } });

class WizardSummarySectionComponent {
    constructor() {
        /**
         * Emits when step edit button clicked.
         */
        this.editStep = new EventEmitter();
    }
    /**
     * @hidden
     * @description Sets current step on wizard.
     * @param event Mouse click event to prevent.
     */
    _editStep(event) {
        event.preventDefault();
        this.editStep.emit(this.step.id);
    }
    /** @hidden */
    _trackFn(_, form) {
        return form.id;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardSummarySectionComponent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.0.7", type: WizardSummarySectionComponent, isStandalone: true, selector: "fdp-wizard-summary-section", inputs: { step: "step" }, outputs: { editStep: "editStep" }, ngImport: i0, template: "@for (group of step.forms; track _trackFn($index, group)) {\n    <h4 fd-title [headerSize]=\"4\">{{ group.title }}</h4>\n    <fd-layout-grid [style.margin-bottom.rem]=\"2\">\n        @for (item of group.items; track item; let index = $index) {\n            <div fdLayoutGridRow>\n                <div [fdLayoutGridCol]=\"4\" [colMd]=\"2\" [colLg]=\"2\" [colXl]=\"2\">\n                    <div>\n                        <label fd-form-label [allowWrap]=\"true\" [alignLabelEnd]=\"true\"> {{ item.label }}: </label>\n                    </div>\n                </div>\n                <div [fdLayoutGridCol]=\"4\" [colMd]=\"5\" [colLg]=\"5\" [colXl]=\"5\">\n                    <div>\n                        <label fd-form-label [allowWrap]=\"true\">\n                            {{ item.value || '-' }}\n                        </label>\n                    </div>\n                </div>\n                @if (index === group.items.length - 1) {\n                    <div [fdLayoutGridCol]=\"1\" [colMd]=\"5\" [colLg]=\"5\" [colXl]=\"5\">\n                        <fdp-link href=\"#\" (click)=\"_editStep($event)\">\n                            {{ 'platformWizardGenerator.summarySectionEditStep' | fdTranslate }}\n                        </fdp-link>\n                    </div>\n                }\n            </div>\n        }\n    </fd-layout-grid>\n}\n", dependencies: [{ kind: "component", type: TitleComponent, selector: "[fd-title], [fdTitle]", inputs: ["headerSize", "wrap"], exportAs: ["fd-title"] }, { kind: "ngmodule", type: LayoutGridModule }, { kind: "component", type: i1.LayoutGridComponent, selector: "fd-layout-grid, [fdLayoutGrid]", inputs: ["class", "noGap", "noHorizontalGap", "noVerticalGap"] }, { kind: "directive", type: i1.LayoutGridColDirective, selector: "[fd-layout-grid-col], [fdLayoutGridCol]", inputs: ["fdLayoutGridCol", "colGrow", "colMd", "colLg", "colXl", "colOffset", "colOffsetMd", "colOffsetLg", "colOffsetXl", "class"] }, { kind: "directive", type: i1.LayoutGridRowDirective, selector: "[fd-layout-grid-row], [fdLayoutGridRow]" }, { kind: "component", type: FormLabelComponent, selector: "[fd-form-label]", inputs: ["required", "colon", "alignLabelEnd", "inlineHelpContent", "inlineHelpGlyph", "inlineHelpTriggers", "inlineHelpBodyPlacement", "inlineHelpPlacement", "allowWrap", "inlineHelpLabel", "id"] }, { kind: "component", type: LinkComponent, selector: "fdp-link", inputs: ["href", "target", "linkType", "type", "inverted", "title", "hreflang", "download", "media", "rel", "truncate"], outputs: ["click"] }, { kind: "pipe", type: FdTranslatePipe, name: "fdTranslate" }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardSummarySectionComponent, decorators: [{
            type: Component,
            args: [{ selector: 'fdp-wizard-summary-section', changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, imports: [TitleComponent, LayoutGridModule, FormLabelComponent, LinkComponent, FdTranslatePipe], template: "@for (group of step.forms; track _trackFn($index, group)) {\n    <h4 fd-title [headerSize]=\"4\">{{ group.title }}</h4>\n    <fd-layout-grid [style.margin-bottom.rem]=\"2\">\n        @for (item of group.items; track item; let index = $index) {\n            <div fdLayoutGridRow>\n                <div [fdLayoutGridCol]=\"4\" [colMd]=\"2\" [colLg]=\"2\" [colXl]=\"2\">\n                    <div>\n                        <label fd-form-label [allowWrap]=\"true\" [alignLabelEnd]=\"true\"> {{ item.label }}: </label>\n                    </div>\n                </div>\n                <div [fdLayoutGridCol]=\"4\" [colMd]=\"5\" [colLg]=\"5\" [colXl]=\"5\">\n                    <div>\n                        <label fd-form-label [allowWrap]=\"true\">\n                            {{ item.value || '-' }}\n                        </label>\n                    </div>\n                </div>\n                @if (index === group.items.length - 1) {\n                    <div [fdLayoutGridCol]=\"1\" [colMd]=\"5\" [colLg]=\"5\" [colXl]=\"5\">\n                        <fdp-link href=\"#\" (click)=\"_editStep($event)\">\n                            {{ 'platformWizardGenerator.summarySectionEditStep' | fdTranslate }}\n                        </fdp-link>\n                    </div>\n                }\n            </div>\n        }\n    </fd-layout-grid>\n}\n" }]
        }], propDecorators: { step: [{
                type: Input
            }], editStep: [{
                type: Output
            }] } });

class WizardSummaryStepComponent {
    /**
     * @description Summary step status.
     */
    set status(status) {
        if (status === 'current') {
            this._wizardGeneratorService.getWizardFormValue(true).then((forms) => {
                this._submittedForms = forms;
                this._wizardSteps = this._wizardGeneratorService.items.filter((i) => this._submittedForms[i.id] !== undefined);
                this._formatWizardValue().then(() => {
                    this._shouldRender = true;
                    this._cd.detectChanges();
                });
            });
        }
        else {
            this._shouldRender = false;
        }
        this._cd.detectChanges();
    }
    /** @hidden */
    constructor(_wizardGeneratorService, _formGeneratorService, _cd) {
        this._wizardGeneratorService = _wizardGeneratorService;
        this._formGeneratorService = _formGeneratorService;
        this._cd = _cd;
        /**
         * Formatted wizard value with steps and their forms.
         */
        this.formattedWizardValue = [];
        /**
         * @hidden
         */
        this._shouldRender = false;
        /** @hidden */
        this._editStepFn = (stepId) => this.editStep(stepId);
    }
    /**
     * Opens defined step for editing.
     * @param stepId Defined step ID.
     */
    editStep(stepId) {
        this._wizardGeneratorService.editStep(stepId);
    }
    /** @hidden */
    _trackFn(_, step) {
        return step.id;
    }
    /** @hidden */
    async _formatWizardValue() {
        this.formattedWizardValue = [];
        for (const step of this._wizardSteps) {
            const component = this._wizardGeneratorService.stepsComponents.get(step.id);
            const componentForms = component?.getVisibleForms();
            if (!componentForms) {
                continue;
            }
            const formattedStepValue = await this._formatStepValue(componentForms, step);
            this.formattedWizardValue.push({
                id: step.id,
                name: step.name,
                title: step.title,
                forms: formattedStepValue
            });
        }
    }
    /** @hidden */
    async _formatStepValue(componentForms, step) {
        const formattedStepValue = [];
        for (const formGroup of step.formGroups ?? []) {
            const formId = formGroup.id;
            const form = componentForms[formId];
            // Form might be skipped due to conditional rendering
            if (!form) {
                continue;
            }
            const formattedFormValue = await this._formGeneratorService.getFormValue(form.form, true);
            const formattedForm = {
                title: form.title,
                id: formId,
                items: Object.keys(this._submittedForms[step.id][formId]).map((key) => {
                    const formItem = this._formGeneratorService.getFormControl(form.form, key).formItem;
                    return {
                        label: formItem.message,
                        value: formattedFormValue[key]
                    };
                })
            };
            formattedStepValue.push(formattedForm);
        }
        return formattedStepValue;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardSummaryStepComponent, deps: [{ token: WizardGeneratorService }, { token: i2.FormGeneratorService }, { token: i0.ChangeDetectorRef }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.0.7", type: WizardSummaryStepComponent, isStandalone: true, selector: "fdp-wizard-summary-step", inputs: { status: "status", customSummaryStepTemplate: "customSummaryStepTemplate" }, ngImport: i0, template: "@if (_shouldRender) {\n    <ng-template #defaultsummaryStepTemplate>\n        @for (step of formattedWizardValue; track _trackFn($index, step)) {\n            <fdp-wizard-summary-section [step]=\"step\" (editStep)=\"editStep($event)\"></fdp-wizard-summary-section>\n        }\n    </ng-template>\n    <ng-template\n        [ngTemplateOutlet]=\"customSummaryStepTemplate || defaultsummaryStepTemplate\"\n        [ngTemplateOutletContext]=\"{ steps: formattedWizardValue, editStep: _editStepFn }\"\n    ></ng-template>\n}\n", dependencies: [{ kind: "component", type: WizardSummarySectionComponent, selector: "fdp-wizard-summary-section", inputs: ["step"], outputs: ["editStep"] }, { kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardSummaryStepComponent, decorators: [{
            type: Component,
            args: [{ selector: 'fdp-wizard-summary-step', changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, imports: [WizardSummarySectionComponent, NgTemplateOutlet], template: "@if (_shouldRender) {\n    <ng-template #defaultsummaryStepTemplate>\n        @for (step of formattedWizardValue; track _trackFn($index, step)) {\n            <fdp-wizard-summary-section [step]=\"step\" (editStep)=\"editStep($event)\"></fdp-wizard-summary-section>\n        }\n    </ng-template>\n    <ng-template\n        [ngTemplateOutlet]=\"customSummaryStepTemplate || defaultsummaryStepTemplate\"\n        [ngTemplateOutletContext]=\"{ steps: formattedWizardValue, editStep: _editStepFn }\"\n    ></ng-template>\n}\n" }]
        }], ctorParameters: () => [{ type: WizardGeneratorService }, { type: i2.FormGeneratorService }, { type: i0.ChangeDetectorRef }], propDecorators: { status: [{
                type: Input
            }], customSummaryStepTemplate: [{
                type: Input
            }] } });

class WizardBodyComponent {
    /**
     * @description Array of visible Wizard Steps.
     */
    set visibleItems(items) {
        this._visibleItems = items;
    }
    get visibleItems() {
        return this._visibleItems || this._wizardGeneratorService.items;
    }
    /** @hidden */
    constructor(_wizardGeneratorService, _cd) {
        this._wizardGeneratorService = _wizardGeneratorService;
        this._cd = _cd;
        /**
         * @description Whether or not apply responsive paddings styling.
         */
        this.responsivePaddings = true;
        /**
         * @description Whether or not to append the step to the wizard. If false, each step will be displayed on a different page.
         * Default is true.
         */
        this.appendToWizard = true;
        /**
         * @description Whether or not Wizard is currently on the first step
         */
        this.isFirstStep = false;
        /**
         * @description Whether Wizard is currently on the last step.
         */
        this.isLastStep = false;
        /**
         * Whether next step is a Summary step.
         */
        this.isNextStepSummary = false;
        /**
         * @description Boolean flag indicating whether to display Summary step in Wizard progress bar.
         */
        this.displaySummaryStep = false;
        /**
         * @description Is current step is summary step.
         */
        this.isSummaryStep = false;
        /** If navigation buttons should be visible. */
        this.navigationButtons = true;
        /** Whether or not all form items should have identical layout provided for form group. */
        this.unifiedLayout = true;
        /** Whether current step is a branching step. */
        this.isBranchingStep = false;
        /** Whether current step is completed */
        this.isCurrentStepCompleted = false;
        /** Whether order of steps was changed  */
        this.stepsOrderChanged = false;
        /**
         * @description Emits when some step status has been changed.
         */
        this.statusChange = new EventEmitter();
        /**
         * @description Emits when "Go next" button has been clicked
         */
        this.goNext = new EventEmitter();
        /**
         * @description Emits when "Finish" button has been clicked
         */
        // eslint-disable-next-line @angular-eslint/no-output-native
        this.finish = new EventEmitter();
        /**
         * @hidden
         * An RxJS Subject that will kill the data stream upon component’s destruction (for unsubscribing)
         */
        this._destroyRef = inject(DestroyRef);
        /**
         * @hidden
         */
        this._goNextFn = () => this.goNext.emit();
        /**
         * @hidden
         */
        this._finishFn = () => this.finish.emit();
    }
    /**
     * @description Custom step status change validator function.
     * @param index Current step index.
     * @returns {Promise<boolean>} If this step status can be changed.
     */
    stepClickValidatorFn(index) {
        return async () => {
            const currentStepIndex = this._wizardGeneratorService.getCurrentStepIndex();
            const isSummaryStep = this._visibleItems[currentStepIndex]?.summary === true;
            if (index === currentStepIndex || isSummaryStep) {
                return true;
            }
            if (this._visibleItems.slice(0, index).find((i) => !i.completed && !i.summary)) {
                return false;
            }
            return await firstValueFrom(this._wizardGeneratorService.validateStepForms(true));
        };
    }
    /**
     * @hidden
     */
    ngOnInit() {
        this._wizardGeneratorService
            .getVisibleSteps()
            .pipe(debounceTime(50), takeUntilDestroyed(this._destroyRef))
            .subscribe((visibleSteps) => {
            this._visibleItems = visibleSteps;
            this._cd.detectChanges();
        });
    }
    /**
     * @description Callback function for status change.
     * @param id step ID.
     * @param status new step status.
     */
    stepStatusChanged(id, status) {
        this.statusChange.emit({
            id,
            status
        });
    }
    /** @hidden */
    stepClicked(stepId) {
        const stepIndex = this._wizardGeneratorService.getStepIndex(stepId);
        this._wizardGeneratorService.setNextStepIndex(stepIndex + 1);
    }
    /**
     * @hidden
     * @param _index
     * @param item
     * @returns
     */
    _trackFn(_index, item) {
        return item.id;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardBodyComponent, deps: [{ token: WizardGeneratorService }, { token: i0.ChangeDetectorRef }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.0.7", type: WizardBodyComponent, isStandalone: true, selector: "fdp-wizard-body", inputs: { responsivePaddings: "responsivePaddings", navigationButtonLabels: "navigationButtonLabels", appendToWizard: "appendToWizard", contentHeight: "contentHeight", isFirstStep: "isFirstStep", isLastStep: "isLastStep", isNextStepSummary: "isNextStepSummary", displaySummaryStep: "displaySummaryStep", goNextButtonTemplate: "goNextButtonTemplate", finishButtonTemplate: "finishButtonTemplate", customSummaryStepTemplate: "customSummaryStepTemplate", reviewButtonTemplate: "reviewButtonTemplate", isSummaryStep: "isSummaryStep", navigationButtons: "navigationButtons", unifiedLayout: "unifiedLayout", isBranchingStep: "isBranchingStep", isCurrentStepCompleted: "isCurrentStepCompleted", stepsOrderChanged: "stepsOrderChanged" }, outputs: { statusChange: "statusChange", goNext: "goNext", finish: "finish" }, ngImport: i0, template: "@if (visibleItems) {\n    <fd-wizard\n        [displaySummaryStep]=\"displaySummaryStep\"\n        [responsivePaddings]=\"responsivePaddings\"\n        [appendToWizard]=\"appendToWizard\"\n        [contentHeight]=\"contentHeight\"\n    >\n        <fd-wizard-navigation>\n            <ul fd-wizard-progress-bar>\n                @for (step of visibleItems; track _trackFn(i, step); let i = $index) {\n                    <li\n                        fd-wizard-step\n                        [label]=\"step.name\"\n                        [status]=\"step.status\"\n                        [optionalText]=\"step.optionalText\"\n                        [branching]=\"!!step.branching\"\n                        [stepClickValidator]=\"stepClickValidatorFn(i)\"\n                        [isSummary]=\"!!step.summary\"\n                        (statusChange)=\"stepStatusChanged(step.id, $event)\"\n                        (stepClicked)=\"stepClicked(step.id)\"\n                    >\n                        <fd-wizard-step-indicator [glyph]=\"step.icon || null\">\n                            @if (!step.icon) {\n                                {{ i + 1 }}\n                            }\n                        </fd-wizard-step-indicator>\n                        <fd-wizard-content>\n                            @if (!step.summary) {\n                                <fdp-wizard-generator-step\n                                    [unifiedLayout]=\"unifiedLayout\"\n                                    [item]=\"step\"\n                                    [stepStatus]=\"step.status\"\n                                ></fdp-wizard-generator-step>\n                            } @else {\n                                <fdp-wizard-summary-step\n                                    [customSummaryStepTemplate]=\"customSummaryStepTemplate\"\n                                    [status]=\"step.status\"\n                                ></fdp-wizard-summary-step>\n                            }\n                            @if (navigationButtons) {\n                                <fd-wizard-next-step [canHide]=\"appendToWizard\">\n                                    @if (!isLastStep) {\n                                        <ng-template #defaultGoNextButtonTemplate>\n                                            <button\n                                                fd-button\n                                                [fdContentDensity]=\"navigationButtonLabels.goNext.contentDensity\"\n                                                [fdType]=\"navigationButtonLabels.goNext.type\"\n                                                [label]=\"navigationButtonLabels.goNext.label\"\n                                                (click)=\"goNext.emit()\"\n                                            ></button>\n                                        </ng-template>\n                                        <ng-template #defaultReviewButtonTemplate>\n                                            <button\n                                                fd-button\n                                                [fdContentDensity]=\"navigationButtonLabels.review.contentDensity\"\n                                                [fdType]=\"navigationButtonLabels.review.type\"\n                                                [label]=\"navigationButtonLabels.review.label\"\n                                                (click)=\"goNext.emit()\"\n                                            ></button>\n                                        </ng-template>\n                                        @if (\n                                            (isBranchingStep && (!isCurrentStepCompleted || stepsOrderChanged)) ||\n                                            !isNextStepSummary\n                                        ) {\n                                            <ng-template\n                                                [ngTemplateOutlet]=\"goNextButtonTemplate || defaultGoNextButtonTemplate\"\n                                                [ngTemplateOutletContext]=\"{ goNext: _goNextFn }\"\n                                            ></ng-template>\n                                        } @else {\n                                            <ng-template\n                                                [ngTemplateOutlet]=\"reviewButtonTemplate || defaultReviewButtonTemplate\"\n                                                [ngTemplateOutletContext]=\"{ goNext: _goNextFn }\"\n                                            ></ng-template>\n                                        }\n                                    } @else {\n                                        <ng-template #defaultFinishButtonTemplate>\n                                            <button\n                                                fd-button\n                                                [fdContentDensity]=\"navigationButtonLabels.finish.contentDensity\"\n                                                [fdType]=\"navigationButtonLabels.finish.type\"\n                                                [label]=\"navigationButtonLabels.finish.label\"\n                                                (click)=\"finish.emit()\"\n                                            ></button>\n                                        </ng-template>\n                                        <ng-template\n                                            [ngTemplateOutlet]=\"finishButtonTemplate || defaultFinishButtonTemplate\"\n                                            [ngTemplateOutletContext]=\"{ finish: _finishFn }\"\n                                        ></ng-template>\n                                    }\n                                </fd-wizard-next-step>\n                            }\n                        </fd-wizard-content>\n                    </li>\n                }\n            </ul>\n        </fd-wizard-navigation>\n    </fd-wizard>\n}\n", dependencies: [{ kind: "ngmodule", type: WizardModule }, { kind: "component", type: i2$1.WizardComponent, selector: "fd-wizard", inputs: ["appendToWizard", "contentHeight", "responsivePaddings", "displaySummaryStep", "ariaLabel"] }, { kind: "component", type: i2$1.WizardNavigationComponent, selector: "fd-wizard-navigation", inputs: ["ariaLabel"] }, { kind: "directive", type: i2$1.WizardProgressBarDirective, selector: "[fd-wizard-progress-bar]", inputs: ["size"] }, { kind: "component", type: i2$1.WizardStepComponent, selector: "[fd-wizard-step]", inputs: ["ariaLabel", "ariaRoleDecription", "status", "branching", "label", "optionalText", "isSummary", "stepClickValidator"], outputs: ["statusChange", "stepClicked", "stepIndicatorItemClicked"] }, { kind: "component", type: i2$1.WizardStepIndicatorComponent, selector: "fd-wizard-step-indicator", inputs: ["glyph", "glyphFont"], outputs: ["stepIndicatorItemClicked"] }, { kind: "component", type: i2$1.WizardContentComponent, selector: "fd-wizard-content", inputs: ["contentBackground", "size"] }, { kind: "component", type: i2$1.WizardNextStepComponent, selector: "fd-wizard-next-step", inputs: ["floating", "canHide"] }, { kind: "component", type: WizardGeneratorStepComponent, selector: "fdp-wizard-generator-step", inputs: ["item", "stepStatus", "unifiedLayout"], outputs: ["formsSubmitted", "formsCreated"] }, { kind: "component", type: WizardSummaryStepComponent, selector: "fdp-wizard-summary-step", inputs: ["status", "customSummaryStepTemplate"] }, { kind: "component", type: ButtonComponent, selector: "button[fd-button], a[fd-button], span[fd-button]", inputs: ["class"], exportAs: ["fd-button"] }, { kind: "directive", type: ContentDensityDirective, selector: "[fdContentDensity]:not([fdCompact]):not([fdCondensed]):not([fdCozy]),                [fdCompact]:not([fdContentDensity]):not([fdCondensed]):not([fdCozy]),                [fdCondensed]:not([fdContentDensity]):not([fdCompact]):not([fdCozy]),                [fdCozy]:not([fdContentDensity]):not([fdCompact]):not([fdCondensed])", inputs: ["fdContentDensity", "fdCompact", "fdCondensed", "fdCozy"], exportAs: ["fdContentDensity"] }, { kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardBodyComponent, decorators: [{
            type: Component,
            args: [{ selector: 'fdp-wizard-body', encapsulation: ViewEncapsulation.None, changeDetection: ChangeDetectionStrategy.OnPush, imports: [
                        WizardModule,
                        WizardGeneratorStepComponent,
                        WizardSummaryStepComponent,
                        ButtonComponent,
                        ContentDensityDirective,
                        NgTemplateOutlet
                    ], template: "@if (visibleItems) {\n    <fd-wizard\n        [displaySummaryStep]=\"displaySummaryStep\"\n        [responsivePaddings]=\"responsivePaddings\"\n        [appendToWizard]=\"appendToWizard\"\n        [contentHeight]=\"contentHeight\"\n    >\n        <fd-wizard-navigation>\n            <ul fd-wizard-progress-bar>\n                @for (step of visibleItems; track _trackFn(i, step); let i = $index) {\n                    <li\n                        fd-wizard-step\n                        [label]=\"step.name\"\n                        [status]=\"step.status\"\n                        [optionalText]=\"step.optionalText\"\n                        [branching]=\"!!step.branching\"\n                        [stepClickValidator]=\"stepClickValidatorFn(i)\"\n                        [isSummary]=\"!!step.summary\"\n                        (statusChange)=\"stepStatusChanged(step.id, $event)\"\n                        (stepClicked)=\"stepClicked(step.id)\"\n                    >\n                        <fd-wizard-step-indicator [glyph]=\"step.icon || null\">\n                            @if (!step.icon) {\n                                {{ i + 1 }}\n                            }\n                        </fd-wizard-step-indicator>\n                        <fd-wizard-content>\n                            @if (!step.summary) {\n                                <fdp-wizard-generator-step\n                                    [unifiedLayout]=\"unifiedLayout\"\n                                    [item]=\"step\"\n                                    [stepStatus]=\"step.status\"\n                                ></fdp-wizard-generator-step>\n                            } @else {\n                                <fdp-wizard-summary-step\n                                    [customSummaryStepTemplate]=\"customSummaryStepTemplate\"\n                                    [status]=\"step.status\"\n                                ></fdp-wizard-summary-step>\n                            }\n                            @if (navigationButtons) {\n                                <fd-wizard-next-step [canHide]=\"appendToWizard\">\n                                    @if (!isLastStep) {\n                                        <ng-template #defaultGoNextButtonTemplate>\n                                            <button\n                                                fd-button\n                                                [fdContentDensity]=\"navigationButtonLabels.goNext.contentDensity\"\n                                                [fdType]=\"navigationButtonLabels.goNext.type\"\n                                                [label]=\"navigationButtonLabels.goNext.label\"\n                                                (click)=\"goNext.emit()\"\n                                            ></button>\n                                        </ng-template>\n                                        <ng-template #defaultReviewButtonTemplate>\n                                            <button\n                                                fd-button\n                                                [fdContentDensity]=\"navigationButtonLabels.review.contentDensity\"\n                                                [fdType]=\"navigationButtonLabels.review.type\"\n                                                [label]=\"navigationButtonLabels.review.label\"\n                                                (click)=\"goNext.emit()\"\n                                            ></button>\n                                        </ng-template>\n                                        @if (\n                                            (isBranchingStep && (!isCurrentStepCompleted || stepsOrderChanged)) ||\n                                            !isNextStepSummary\n                                        ) {\n                                            <ng-template\n                                                [ngTemplateOutlet]=\"goNextButtonTemplate || defaultGoNextButtonTemplate\"\n                                                [ngTemplateOutletContext]=\"{ goNext: _goNextFn }\"\n                                            ></ng-template>\n                                        } @else {\n                                            <ng-template\n                                                [ngTemplateOutlet]=\"reviewButtonTemplate || defaultReviewButtonTemplate\"\n                                                [ngTemplateOutletContext]=\"{ goNext: _goNextFn }\"\n                                            ></ng-template>\n                                        }\n                                    } @else {\n                                        <ng-template #defaultFinishButtonTemplate>\n                                            <button\n                                                fd-button\n                                                [fdContentDensity]=\"navigationButtonLabels.finish.contentDensity\"\n                                                [fdType]=\"navigationButtonLabels.finish.type\"\n                                                [label]=\"navigationButtonLabels.finish.label\"\n                                                (click)=\"finish.emit()\"\n                                            ></button>\n                                        </ng-template>\n                                        <ng-template\n                                            [ngTemplateOutlet]=\"finishButtonTemplate || defaultFinishButtonTemplate\"\n                                            [ngTemplateOutletContext]=\"{ finish: _finishFn }\"\n                                        ></ng-template>\n                                    }\n                                </fd-wizard-next-step>\n                            }\n                        </fd-wizard-content>\n                    </li>\n                }\n            </ul>\n        </fd-wizard-navigation>\n    </fd-wizard>\n}\n" }]
        }], ctorParameters: () => [{ type: WizardGeneratorService }, { type: i0.ChangeDetectorRef }], propDecorators: { responsivePaddings: [{
                type: Input
            }], navigationButtonLabels: [{
                type: Input
            }], appendToWizard: [{
                type: Input
            }], contentHeight: [{
                type: Input
            }], isFirstStep: [{
                type: Input
            }], isLastStep: [{
                type: Input
            }], isNextStepSummary: [{
                type: Input
            }], displaySummaryStep: [{
                type: Input
            }], goNextButtonTemplate: [{
                type: Input
            }], finishButtonTemplate: [{
                type: Input
            }], customSummaryStepTemplate: [{
                type: Input
            }], reviewButtonTemplate: [{
                type: Input
            }], isSummaryStep: [{
                type: Input
            }], navigationButtons: [{
                type: Input
            }], unifiedLayout: [{
                type: Input
            }], isBranchingStep: [{
                type: Input
            }], isCurrentStepCompleted: [{
                type: Input
            }], stepsOrderChanged: [{
                type: Input
            }], statusChange: [{
                type: Output
            }], goNext: [{
                type: Output
            }], finish: [{
                type: Output
            }] } });

class DialogWizardGeneratorComponent extends BaseWizardGenerator {
    /** @hidden */
    constructor(_dialogRef) {
        super();
        this._dialogRef = _dialogRef;
        /**
         * @description Confirmation message when wizard dialog close button has been clicked.
         */
        this.confirmationDialogText = 'Are you sure you want to discard your progress?';
        /**
         * @description Confirm button text for close confirmation dialog.
         */
        this.confirmationDialogCloseText = 'Confirm';
        /**
         * @description Cancel button text for close confirmation dialog.
         */
        this.confirmationDialogCancelText = 'Cancel';
        /** @hidden */
        this._messageBoxService = inject(MessageBoxService);
        /**
         * @hidden
         */
        this._goBackFn = () => this.goBack();
        /**
         * @hidden
         */
        this._goNextFn = () => this.goNext();
        /**
         * @hidden
         */
        this._finishFn = () => this.finish();
        /**
         * @hidden
         */
        this._cancelFn = () => this.cancel();
        this.items = this._dialogRef.data.items;
        this.title = this._dialogRef.data.title;
        this.appendToWizard = this._dialogRef.data.appendToWizard;
        this.navigationButtonLabels = this._dialogRef.data.navigationButtonLabels ?? {};
        this.contentHeight = this._dialogRef.data.contentHeight;
        this.responsivePaddings = this._dialogRef.data.responsivePaddings;
        this.goNextButtonTemplate = this._dialogRef.data.goNextButtonTemplate;
        this.goBackButtonTemplate = this._dialogRef.data.goBackButtonTemplate;
        this.finishButtonTemplate = this._dialogRef.data.finishButtonTemplate;
        this.cancelButtonTemplate = this._dialogRef.data.cancelButtonTemplate;
        this.confirmationDialogTemplate = this._dialogRef.data.confirmationDialogTemplate;
        this.displaySummaryStep = this._dialogRef.data.displaySummaryStep || false;
        this.summaryStepTemplate = this._dialogRef.data.summaryStepTemplate;
        this.reviewButtonTemplate = this._dialogRef.data.reviewButtonTemplate;
        if (this._dialogRef.data.unifiedLayout !== undefined) {
            this.unifiedLayout = this._dialogRef.data.unifiedLayout;
        }
        if (this._dialogRef.data.navigationButtons !== undefined) {
            this.navigationButtons = this._dialogRef.data.navigationButtons;
        }
        if (this._dialogRef.data.confirmationDialogText) {
            this.confirmationDialogText = this._dialogRef.data.confirmationDialogText;
        }
        if (this._dialogRef.data.confirmationDialogCancelText) {
            this.confirmationDialogCancelText = this._dialogRef.data.confirmationDialogCancelText;
        }
        if (this._dialogRef.data.confirmationDialogCloseText) {
            this.confirmationDialogCloseText = this._dialogRef.data.confirmationDialogCloseText;
        }
    }
    /**
     * @description Cancels Wizard progress and closes the dialog.
     */
    cancel() {
        if (this._wizardGeneratorService.isStepsUntouched()) {
            this._dialogRef.dismiss();
            return;
        }
        const template = this.confirmationDialogTemplate || this.defaultConfirmationDialogTemplate;
        const messageBoxRef = this._messageBoxService.open(template, { responsivePadding: true });
        messageBoxRef.afterClosed
            .pipe(filter((result) => result), takeUntilDestroyed(this._destroyRef))
            .subscribe(() => {
            this._dialogRef.dismiss();
        });
    }
    /**
     * @description Completes Wizard process, emits Wizard result and closes the dialog.
     */
    async finish() {
        if (this.isSummaryStep) {
            const wizardResult = await this._wizardGeneratorService.getWizardFormValue(true);
            this._dialogRef.close(wizardResult);
            return;
        }
        const currentStepId = this._wizardGeneratorService.getCurrentStepId();
        this.submitStepForms(currentStepId)
            .pipe(takeUntilDestroyed(this._destroyRef))
            .subscribe(async (result) => {
            if (result && Object.values(result).some((r) => !r.success)) {
                return;
            }
            const wizardResult = await this._wizardGeneratorService.getWizardFormValue(true);
            this._dialogRef.close(wizardResult);
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: DialogWizardGeneratorComponent, deps: [{ token: i1$1.DialogRef }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.0.7", type: DialogWizardGeneratorComponent, isStandalone: true, selector: "fdp-dialog-wizard-generator", providers: [WizardGeneratorService, FormGeneratorService], viewQueries: [{ propertyName: "defaultConfirmationDialogTemplate", first: true, predicate: ["defaultConfirmationDialogTemplate"], descendants: true }], usesInheritance: true, ngImport: i0, template: "<fd-dialog>\n    <fd-dialog-header>\n        <h2 fd-title [headerSize]=\"title.size\">{{ title.text }}</h2>\n    </fd-dialog-header>\n    <fd-dialog-body>\n        <fdp-wizard-body\n            [navigationButtons]=\"false\"\n            [displaySummaryStep]=\"displaySummaryStep\"\n            [responsivePaddings]=\"responsivePaddings\"\n            [navigationButtonLabels]=\"_navigationButtonLabels\"\n            [customSummaryStepTemplate]=\"summaryStepTemplate\"\n            [reviewButtonTemplate]=\"reviewButtonTemplate\"\n            [hidden]=\"!wizardCreated\"\n            [appendToWizard]=\"appendToWizard\"\n            [unifiedLayout]=\"unifiedLayout\"\n            [contentHeight]=\"contentHeight\"\n            (statusChange)=\"stepStatusChanged($event.id, $event.status)\"\n        ></fdp-wizard-body>\n        @if (wizardCreated) {\n            <div fd-bar barDesign=\"floating-footer\" [class.fd-bar--responsive-paddings]=\"responsivePaddings\">\n                <div fd-bar-right>\n                    @if (!isFirstStep) {\n                        <ng-template #defaultGoBackButtonTemplate>\n                            <button\n                                fd-button\n                                [fdContentDensity]=\"_navigationButtonLabels.goBack.contentDensity\"\n                                [fdType]=\"_navigationButtonLabels.goBack.type\"\n                                [label]=\"_navigationButtonLabels.goBack.label\"\n                                (click)=\"goBack()\"\n                            ></button>\n                        </ng-template>\n                        <ng-template\n                            [ngTemplateOutlet]=\"goBackButtonTemplate || defaultGoBackButtonTemplate\"\n                            [ngTemplateOutletContext]=\"{ goBack: _goBackFn }\"\n                        ></ng-template>\n                    }\n                    @if (!isLastStep) {\n                        <ng-template #defaultGoNextButtonTemplate>\n                            <button\n                                fd-button\n                                [fdContentDensity]=\"_navigationButtonLabels.goNext.contentDensity\"\n                                [fdType]=\"_navigationButtonLabels.goNext.type\"\n                                [label]=\"_navigationButtonLabels.goNext.label\"\n                                (click)=\"goNext()\"\n                            ></button>\n                        </ng-template>\n                        <ng-template #defaultReviewButtonTemplate>\n                            <button\n                                fd-button\n                                [fdContentDensity]=\"_navigationButtonLabels.review.contentDensity\"\n                                [fdType]=\"_navigationButtonLabels.review.type\"\n                                [label]=\"_navigationButtonLabels.review.label\"\n                                (click)=\"goNext()\"\n                            ></button>\n                        </ng-template>\n                        @if (\n                            (isBranchingStep && (!isCurrentStepCompleted || _stepsOrderChanged)) || !isNextStepSummary\n                        ) {\n                            <ng-template\n                                [ngTemplateOutlet]=\"goNextButtonTemplate || defaultGoNextButtonTemplate\"\n                                [ngTemplateOutletContext]=\"{ goNext: _goNextFn }\"\n                            ></ng-template>\n                        } @else {\n                            <ng-template\n                                [ngTemplateOutlet]=\"reviewButtonTemplate || defaultReviewButtonTemplate\"\n                                [ngTemplateOutletContext]=\"{ goNext: _goNextFn }\"\n                            ></ng-template>\n                        }\n                    }\n                    @if (isLastStep) {\n                        <ng-template #defaultFinishButtonTemplate>\n                            <button\n                                fd-button\n                                [fdContentDensity]=\"_navigationButtonLabels.finish.contentDensity\"\n                                [fdType]=\"_navigationButtonLabels.finish.type\"\n                                [label]=\"_navigationButtonLabels.finish.label\"\n                                (click)=\"finish()\"\n                            ></button>\n                        </ng-template>\n                        <ng-template\n                            [ngTemplateOutlet]=\"finishButtonTemplate || defaultFinishButtonTemplate\"\n                            [ngTemplateOutletContext]=\"{ finish: _finishFn }\"\n                        ></ng-template>\n                    }\n                    <ng-template #defaultCancelButtonTemplate>\n                        <button\n                            fd-button\n                            [fdContentDensity]=\"_navigationButtonLabels.cancel.contentDensity\"\n                            [fdType]=\"_navigationButtonLabels.cancel.type\"\n                            [label]=\"_navigationButtonLabels.cancel.label\"\n                            (click)=\"cancel()\"\n                        ></button>\n                    </ng-template>\n                    <ng-template\n                        [ngTemplateOutlet]=\"cancelButtonTemplate || defaultCancelButtonTemplate\"\n                        [ngTemplateOutletContext]=\"{ cancel: _cancelFn }\"\n                    ></ng-template>\n                </div>\n            </div>\n        }\n    </fd-dialog-body>\n</fd-dialog>\n<ng-template\n    [fdMessageBoxTemplate]\n    let-messageBox\n    let-messageBoxConfig=\"messageBoxConfig\"\n    #defaultConfirmationDialogTemplate\n>\n    <fd-message-box [messageBoxConfig]=\"messageBoxConfig\" [messageBoxRef]=\"messageBox\">\n        <fd-message-box-body>\n            {{ confirmationDialogText }}\n        </fd-message-box-body>\n        <fd-message-box-footer>\n            <fd-button-bar\n                fdType=\"emphasized\"\n                [label]=\"confirmationDialogCloseText\"\n                type=\"submit\"\n                (click)=\"messageBox.close(true)\"\n            >\n            </fd-button-bar>\n            <fd-button-bar fdkInitialFocus [label]=\"confirmationDialogCancelText\" (click)=\"messageBox.dismiss()\">\n            </fd-button-bar>\n        </fd-message-box-footer>\n    </fd-message-box>\n</ng-template>\n", dependencies: [{ kind: "component", type: TitleComponent, selector: "[fd-title], [fdTitle]", inputs: ["headerSize", "wrap"], exportAs: ["fd-title"] }, { kind: "component", type: WizardBodyComponent, selector: "fdp-wizard-body", inputs: ["responsivePaddings", "navigationButtonLabels", "appendToWizard", "contentHeight", "isFirstStep", "isLastStep", "isNextStepSummary", "displaySummaryStep", "goNextButtonTemplate", "finishButtonTemplate", "customSummaryStepTemplate", "reviewButtonTemplate", "isSummaryStep", "navigationButtons", "unifiedLayout", "isBranchingStep", "isCurrentStepCompleted", "stepsOrderChanged"], outputs: ["statusChange", "goNext", "finish"] }, { kind: "ngmodule", type: BarModule }, { kind: "component", type: i2$2.BarComponent, selector: "[fd-bar]", inputs: ["class", "barDesign", "inPage", "inHomePage", "size"] }, { kind: "directive", type: i2$2.BarRightDirective, selector: "[fd-bar-right]" }, { kind: "component", type: i2$2.ButtonBarComponent, selector: "fd-button-bar", inputs: ["fullWidth", "fdType", "title", "ariaLabelledby", "id"] }, { kind: "component", type: ButtonComponent, selector: "button[fd-button], a[fd-button], span[fd-button]", inputs: ["class"], exportAs: ["fd-button"] }, { kind: "directive", type: ContentDensityDirective, selector: "[fdContentDensity]:not([fdCompact]):not([fdCondensed]):not([fdCozy]),                [fdCompact]:not([fdContentDensity]):not([fdCondensed]):not([fdCozy]),                [fdCondensed]:not([fdContentDensity]):not([fdCompact]):not([fdCozy]),                [fdCozy]:not([fdContentDensity]):not([fdCompact]):not([fdCondensed])", inputs: ["fdContentDensity", "fdCompact", "fdCondensed", "fdCozy"], exportAs: ["fdContentDensity"] }, { kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }, { kind: "ngmodule", type: MessageBoxModule }, { kind: "component", type: i3.MessageBoxFooterComponent, selector: "fd-message-box-footer" }, { kind: "component", type: i3.MessageBoxBodyComponent, selector: "fd-message-box-body" }, { kind: "component", type: i3.MessageBoxComponent, selector: "fd-message-box", inputs: ["class", "messageBoxRef", "messageBoxConfig"] }, { kind: "directive", type: i3.MessageBoxTemplateDirective, selector: "[fdMessageBoxTemplate]", inputs: ["fdMessageBoxTemplateConfig"] }, { kind: "directive", type: i4.InitialFocusDirective, selector: "[fdkInitialFocus]", inputs: ["fdkInitialFocus", "enabled", "focusLastElement"] }, { kind: "component", type: DialogBodyComponent, selector: "fd-dialog-body", inputs: ["disablePaddings"] }, { kind: "component", type: DialogHeaderComponent, selector: "fd-dialog-header" }, { kind: "component", type: DialogComponent, selector: "fd-dialog", inputs: ["class", "dialogRef", "dialogConfig"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: DialogWizardGeneratorComponent, decorators: [{
            type: Component,
            args: [{ selector: 'fdp-dialog-wizard-generator', encapsulation: ViewEncapsulation.None, changeDetection: ChangeDetectionStrategy.OnPush, providers: [WizardGeneratorService, FormGeneratorService], imports: [
                        TitleComponent,
                        CdkScrollable,
                        ScrollbarDirective,
                        WizardBodyComponent,
                        BarModule,
                        ButtonComponent,
                        ContentDensityDirective,
                        NgTemplateOutlet,
                        MessageBoxModule,
                        InitialFocusDirective,
                        DialogBodyComponent,
                        DialogHeaderComponent,
                        DialogComponent
                    ], template: "<fd-dialog>\n    <fd-dialog-header>\n        <h2 fd-title [headerSize]=\"title.size\">{{ title.text }}</h2>\n    </fd-dialog-header>\n    <fd-dialog-body>\n        <fdp-wizard-body\n            [navigationButtons]=\"false\"\n            [displaySummaryStep]=\"displaySummaryStep\"\n            [responsivePaddings]=\"responsivePaddings\"\n            [navigationButtonLabels]=\"_navigationButtonLabels\"\n            [customSummaryStepTemplate]=\"summaryStepTemplate\"\n            [reviewButtonTemplate]=\"reviewButtonTemplate\"\n            [hidden]=\"!wizardCreated\"\n            [appendToWizard]=\"appendToWizard\"\n            [unifiedLayout]=\"unifiedLayout\"\n            [contentHeight]=\"contentHeight\"\n            (statusChange)=\"stepStatusChanged($event.id, $event.status)\"\n        ></fdp-wizard-body>\n        @if (wizardCreated) {\n            <div fd-bar barDesign=\"floating-footer\" [class.fd-bar--responsive-paddings]=\"responsivePaddings\">\n                <div fd-bar-right>\n                    @if (!isFirstStep) {\n                        <ng-template #defaultGoBackButtonTemplate>\n                            <button\n                                fd-button\n                                [fdContentDensity]=\"_navigationButtonLabels.goBack.contentDensity\"\n                                [fdType]=\"_navigationButtonLabels.goBack.type\"\n                                [label]=\"_navigationButtonLabels.goBack.label\"\n                                (click)=\"goBack()\"\n                            ></button>\n                        </ng-template>\n                        <ng-template\n                            [ngTemplateOutlet]=\"goBackButtonTemplate || defaultGoBackButtonTemplate\"\n                            [ngTemplateOutletContext]=\"{ goBack: _goBackFn }\"\n                        ></ng-template>\n                    }\n                    @if (!isLastStep) {\n                        <ng-template #defaultGoNextButtonTemplate>\n                            <button\n                                fd-button\n                                [fdContentDensity]=\"_navigationButtonLabels.goNext.contentDensity\"\n                                [fdType]=\"_navigationButtonLabels.goNext.type\"\n                                [label]=\"_navigationButtonLabels.goNext.label\"\n                                (click)=\"goNext()\"\n                            ></button>\n                        </ng-template>\n                        <ng-template #defaultReviewButtonTemplate>\n                            <button\n                                fd-button\n                                [fdContentDensity]=\"_navigationButtonLabels.review.contentDensity\"\n                                [fdType]=\"_navigationButtonLabels.review.type\"\n                                [label]=\"_navigationButtonLabels.review.label\"\n                                (click)=\"goNext()\"\n                            ></button>\n                        </ng-template>\n                        @if (\n                            (isBranchingStep && (!isCurrentStepCompleted || _stepsOrderChanged)) || !isNextStepSummary\n                        ) {\n                            <ng-template\n                                [ngTemplateOutlet]=\"goNextButtonTemplate || defaultGoNextButtonTemplate\"\n                                [ngTemplateOutletContext]=\"{ goNext: _goNextFn }\"\n                            ></ng-template>\n                        } @else {\n                            <ng-template\n                                [ngTemplateOutlet]=\"reviewButtonTemplate || defaultReviewButtonTemplate\"\n                                [ngTemplateOutletContext]=\"{ goNext: _goNextFn }\"\n                            ></ng-template>\n                        }\n                    }\n                    @if (isLastStep) {\n                        <ng-template #defaultFinishButtonTemplate>\n                            <button\n                                fd-button\n                                [fdContentDensity]=\"_navigationButtonLabels.finish.contentDensity\"\n                                [fdType]=\"_navigationButtonLabels.finish.type\"\n                                [label]=\"_navigationButtonLabels.finish.label\"\n                                (click)=\"finish()\"\n                            ></button>\n                        </ng-template>\n                        <ng-template\n                            [ngTemplateOutlet]=\"finishButtonTemplate || defaultFinishButtonTemplate\"\n                            [ngTemplateOutletContext]=\"{ finish: _finishFn }\"\n                        ></ng-template>\n                    }\n                    <ng-template #defaultCancelButtonTemplate>\n                        <button\n                            fd-button\n                            [fdContentDensity]=\"_navigationButtonLabels.cancel.contentDensity\"\n                            [fdType]=\"_navigationButtonLabels.cancel.type\"\n                            [label]=\"_navigationButtonLabels.cancel.label\"\n                            (click)=\"cancel()\"\n                        ></button>\n                    </ng-template>\n                    <ng-template\n                        [ngTemplateOutlet]=\"cancelButtonTemplate || defaultCancelButtonTemplate\"\n                        [ngTemplateOutletContext]=\"{ cancel: _cancelFn }\"\n                    ></ng-template>\n                </div>\n            </div>\n        }\n    </fd-dialog-body>\n</fd-dialog>\n<ng-template\n    [fdMessageBoxTemplate]\n    let-messageBox\n    let-messageBoxConfig=\"messageBoxConfig\"\n    #defaultConfirmationDialogTemplate\n>\n    <fd-message-box [messageBoxConfig]=\"messageBoxConfig\" [messageBoxRef]=\"messageBox\">\n        <fd-message-box-body>\n            {{ confirmationDialogText }}\n        </fd-message-box-body>\n        <fd-message-box-footer>\n            <fd-button-bar\n                fdType=\"emphasized\"\n                [label]=\"confirmationDialogCloseText\"\n                type=\"submit\"\n                (click)=\"messageBox.close(true)\"\n            >\n            </fd-button-bar>\n            <fd-button-bar fdkInitialFocus [label]=\"confirmationDialogCancelText\" (click)=\"messageBox.dismiss()\">\n            </fd-button-bar>\n        </fd-message-box-footer>\n    </fd-message-box>\n</ng-template>\n" }]
        }], ctorParameters: () => [{ type: i1$1.DialogRef }], propDecorators: { defaultConfirmationDialogTemplate: [{
                type: ViewChild,
                args: ['defaultConfirmationDialogTemplate']
            }] } });

class WizardGeneratorFinishButtonDirective {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardGeneratorFinishButtonDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: WizardGeneratorFinishButtonDirective, isStandalone: true, selector: "[fdpWizardGeneratorFinishButton], [fdp-wizard-generator-finish-button]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardGeneratorFinishButtonDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[fdpWizardGeneratorFinishButton], [fdp-wizard-generator-finish-button]',
                    standalone: true
                }]
        }] });

class WizardGeneratorGoNextButtonDirective {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardGeneratorGoNextButtonDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: WizardGeneratorGoNextButtonDirective, isStandalone: true, selector: "[fdpWizardGeneratorGoNextButton], [fdp-wizard-generator-go-next-button]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardGeneratorGoNextButtonDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[fdpWizardGeneratorGoNextButton], [fdp-wizard-generator-go-next-button]',
                    standalone: true
                }]
        }] });

class WizardGeneratorReviewButtonDirective {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardGeneratorReviewButtonDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: WizardGeneratorReviewButtonDirective, isStandalone: true, selector: "[fdpWizardGeneratorReviewButton], [fdp-wizard-generator-review-button]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardGeneratorReviewButtonDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[fdpWizardGeneratorReviewButton], [fdp-wizard-generator-review-button]',
                    standalone: true
                }]
        }] });

class WizardGeneratorSummaryStepDirective {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardGeneratorSummaryStepDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: WizardGeneratorSummaryStepDirective, isStandalone: true, selector: "[fdpWizardGeneratorSummaryStep], [fdp-wizard-generator-summary-step]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardGeneratorSummaryStepDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[fdpWizardGeneratorSummaryStep], [fdp-wizard-generator-summary-step]',
                    standalone: true
                }]
        }] });

class WizardGeneratorComponent extends BaseWizardGenerator {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardGeneratorComponent, deps: null, target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.0.7", type: WizardGeneratorComponent, isStandalone: true, selector: "fdp-wizard-generator", providers: [WizardGeneratorService, FormGeneratorService], queries: [{ propertyName: "finishButtonTemplate", first: true, predicate: WizardGeneratorFinishButtonDirective, descendants: true, read: TemplateRef }, { propertyName: "goNextButtonTemplate", first: true, predicate: WizardGeneratorGoNextButtonDirective, descendants: true, read: TemplateRef }, { propertyName: "customSummaryStepTemplate", first: true, predicate: WizardGeneratorSummaryStepDirective, descendants: true, read: TemplateRef }, { propertyName: "reviewButtonTemplate", first: true, predicate: WizardGeneratorReviewButtonDirective, descendants: true, read: TemplateRef }], usesInheritance: true, ngImport: i0, template: "@if (wizardCreated) {\n    <fdp-wizard-body\n        [navigationButtonLabels]=\"_navigationButtonLabels\"\n        [goNextButtonTemplate]=\"goNextButtonTemplate\"\n        [finishButtonTemplate]=\"finishButtonTemplate\"\n        [customSummaryStepTemplate]=\"customSummaryStepTemplate\"\n        [reviewButtonTemplate]=\"reviewButtonTemplate\"\n        [responsivePaddings]=\"responsivePaddings\"\n        [hidden]=\"!wizardCreated\"\n        [appendToWizard]=\"appendToWizard\"\n        [contentHeight]=\"contentHeight\"\n        [isFirstStep]=\"isFirstStep\"\n        [isLastStep]=\"isLastStep\"\n        [isNextStepSummary]=\"isNextStepSummary\"\n        [isSummaryStep]=\"isSummaryStep\"\n        [displaySummaryStep]=\"displaySummaryStep\"\n        [navigationButtons]=\"navigationButtons\"\n        [unifiedLayout]=\"unifiedLayout\"\n        [isBranchingStep]=\"isBranchingStep\"\n        [isCurrentStepCompleted]=\"isCurrentStepCompleted\"\n        [stepsOrderChanged]=\"_stepsOrderChanged\"\n        (statusChange)=\"stepStatusChanged($event.id, $event.status)\"\n        (goNext)=\"goNext()\"\n        (finish)=\"finish()\"\n    ></fdp-wizard-body>\n} @else {\n    <fd-wizard>\n        <fd-wizard-navigation>\n            <ul fd-wizard-progress-bar size=\"lg\" role=\"list\" aria-label=\"Wizard Steps\">\n                <li *fdkRepeat=\"3; let i = index\" fd-wizard-step [status]=\"i === 0 ? 'current' : 'upcoming'\">\n                    <fd-skeleton type=\"rectangle\" width=\"32px\" height=\"32px\"></fd-skeleton>\n                    <fd-wizard-content size=\"lg\">\n                        <fd-skeleton type=\"text\" width=\"60%\"></fd-skeleton>\n                    </fd-wizard-content>\n                </li>\n            </ul>\n        </fd-wizard-navigation>\n    </fd-wizard>\n}\n<!-- The Wizard -->\n", dependencies: [{ kind: "component", type: WizardBodyComponent, selector: "fdp-wizard-body", inputs: ["responsivePaddings", "navigationButtonLabels", "appendToWizard", "contentHeight", "isFirstStep", "isLastStep", "isNextStepSummary", "displaySummaryStep", "goNextButtonTemplate", "finishButtonTemplate", "customSummaryStepTemplate", "reviewButtonTemplate", "isSummaryStep", "navigationButtons", "unifiedLayout", "isBranchingStep", "isCurrentStepCompleted", "stepsOrderChanged"], outputs: ["statusChange", "goNext", "finish"] }, { kind: "ngmodule", type: WizardModule }, { kind: "component", type: i2$1.WizardComponent, selector: "fd-wizard", inputs: ["appendToWizard", "contentHeight", "responsivePaddings", "displaySummaryStep", "ariaLabel"] }, { kind: "component", type: i2$1.WizardNavigationComponent, selector: "fd-wizard-navigation", inputs: ["ariaLabel"] }, { kind: "directive", type: i2$1.WizardProgressBarDirective, selector: "[fd-wizard-progress-bar]", inputs: ["size"] }, { kind: "component", type: i2$1.WizardStepComponent, selector: "[fd-wizard-step]", inputs: ["ariaLabel", "ariaRoleDecription", "status", "branching", "label", "optionalText", "isSummary", "stepClickValidator"], outputs: ["statusChange", "stepClicked", "stepIndicatorItemClicked"] }, { kind: "component", type: i2$1.WizardContentComponent, selector: "fd-wizard-content", inputs: ["contentBackground", "size"] }, { kind: "directive", type: RepeatDirective, selector: "[fdkRepeat]", inputs: ["fdkRepeat"] }, { kind: "component", type: SkeletonComponent, selector: "fd-skeleton", inputs: ["animated", "type", "textLines", "width", "height"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardGeneratorComponent, decorators: [{
            type: Component,
            args: [{ selector: 'fdp-wizard-generator', encapsulation: ViewEncapsulation.None, changeDetection: ChangeDetectionStrategy.OnPush, providers: [WizardGeneratorService, FormGeneratorService], imports: [WizardBodyComponent, WizardModule, RepeatDirective, SkeletonComponent], template: "@if (wizardCreated) {\n    <fdp-wizard-body\n        [navigationButtonLabels]=\"_navigationButtonLabels\"\n        [goNextButtonTemplate]=\"goNextButtonTemplate\"\n        [finishButtonTemplate]=\"finishButtonTemplate\"\n        [customSummaryStepTemplate]=\"customSummaryStepTemplate\"\n        [reviewButtonTemplate]=\"reviewButtonTemplate\"\n        [responsivePaddings]=\"responsivePaddings\"\n        [hidden]=\"!wizardCreated\"\n        [appendToWizard]=\"appendToWizard\"\n        [contentHeight]=\"contentHeight\"\n        [isFirstStep]=\"isFirstStep\"\n        [isLastStep]=\"isLastStep\"\n        [isNextStepSummary]=\"isNextStepSummary\"\n        [isSummaryStep]=\"isSummaryStep\"\n        [displaySummaryStep]=\"displaySummaryStep\"\n        [navigationButtons]=\"navigationButtons\"\n        [unifiedLayout]=\"unifiedLayout\"\n        [isBranchingStep]=\"isBranchingStep\"\n        [isCurrentStepCompleted]=\"isCurrentStepCompleted\"\n        [stepsOrderChanged]=\"_stepsOrderChanged\"\n        (statusChange)=\"stepStatusChanged($event.id, $event.status)\"\n        (goNext)=\"goNext()\"\n        (finish)=\"finish()\"\n    ></fdp-wizard-body>\n} @else {\n    <fd-wizard>\n        <fd-wizard-navigation>\n            <ul fd-wizard-progress-bar size=\"lg\" role=\"list\" aria-label=\"Wizard Steps\">\n                <li *fdkRepeat=\"3; let i = index\" fd-wizard-step [status]=\"i === 0 ? 'current' : 'upcoming'\">\n                    <fd-skeleton type=\"rectangle\" width=\"32px\" height=\"32px\"></fd-skeleton>\n                    <fd-wizard-content size=\"lg\">\n                        <fd-skeleton type=\"text\" width=\"60%\"></fd-skeleton>\n                    </fd-wizard-content>\n                </li>\n            </ul>\n        </fd-wizard-navigation>\n    </fd-wizard>\n}\n<!-- The Wizard -->\n" }]
        }], propDecorators: { finishButtonTemplate: [{
                type: ContentChild,
                args: [WizardGeneratorFinishButtonDirective, { read: TemplateRef }]
            }], goNextButtonTemplate: [{
                type: ContentChild,
                args: [WizardGeneratorGoNextButtonDirective, { read: TemplateRef }]
            }], customSummaryStepTemplate: [{
                type: ContentChild,
                args: [WizardGeneratorSummaryStepDirective, { read: TemplateRef }]
            }], reviewButtonTemplate: [{
                type: ContentChild,
                args: [WizardGeneratorReviewButtonDirective, { read: TemplateRef }]
            }] } });

/**
 * @description Helper service to launch Wizard in dialog with minimal user input.
 */
class WizardDialogGeneratorService {
    /** @hidden */
    constructor(_dialogService) {
        this._dialogService = _dialogService;
    }
    /**
     * @description Opens dialog with wizard component inside.
     * @param config Dialog options with wizard config.
     * @returns DialogRef instance.
     */
    open(config) {
        const dialogRef = this._dialogService.open(DialogWizardGeneratorComponent, config);
        return dialogRef;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardDialogGeneratorService, deps: [{ token: i1$1.DialogService }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardDialogGeneratorService }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: WizardDialogGeneratorService, decorators: [{
            type: Injectable
        }], ctorParameters: () => [{ type: i1$1.DialogService }] });

/**
 * Adds Wizard Generator functionality to your application.
 *
 * Can be imported in two ways:
 * * Plain PlatformWizardGeneratorModule with default configuration
 * * With `withConfig()` method which allows passing custom default configuration.
 */
class PlatformWizardGeneratorModule {
    /**
     * Allows configuring module on a global level with custom configuration.
     * @param config User's custom configuration.
     */
    static withConfig(config) {
        return {
            ngModule: PlatformWizardGeneratorModule,
            providers: [
                {
                    provide: FORM_GENERATOR_ITEM_CONFIG,
                    useValue: config.itemConfig
                },
                {
                    provide: FORM_GENERATOR_CONFIG,
                    useValue: config.formConfig
                }
            ]
        };
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: PlatformWizardGeneratorModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "19.0.7", ngImport: i0, type: PlatformWizardGeneratorModule, imports: [PlatformFormGeneratorModule,
            WizardGeneratorComponent,
            WizardGeneratorStepComponent,
            WizardBodyComponent,
            DialogWizardGeneratorComponent,
            WizardSummaryStepComponent,
            WizardSummarySectionComponent,
            WizardGeneratorGoNextButtonDirective,
            WizardGeneratorFinishButtonDirective,
            WizardGeneratorSummaryStepDirective,
            WizardGeneratorReviewButtonDirective], exports: [WizardGeneratorComponent,
            WizardGeneratorStepComponent,
            WizardBodyComponent,
            DialogWizardGeneratorComponent,
            WizardSummaryStepComponent,
            WizardSummarySectionComponent,
            WizardGeneratorGoNextButtonDirective,
            WizardGeneratorFinishButtonDirective,
            WizardGeneratorSummaryStepDirective,
            WizardGeneratorReviewButtonDirective] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: PlatformWizardGeneratorModule, providers: [WizardDialogGeneratorService], imports: [PlatformFormGeneratorModule,
            WizardGeneratorComponent,
            WizardGeneratorStepComponent,
            WizardBodyComponent,
            DialogWizardGeneratorComponent,
            WizardSummaryStepComponent,
            WizardSummarySectionComponent] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: PlatformWizardGeneratorModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [
                        PlatformFormGeneratorModule,
                        WizardGeneratorComponent,
                        WizardGeneratorStepComponent,
                        WizardBodyComponent,
                        DialogWizardGeneratorComponent,
                        WizardSummaryStepComponent,
                        WizardSummarySectionComponent,
                        WizardGeneratorGoNextButtonDirective,
                        WizardGeneratorFinishButtonDirective,
                        WizardGeneratorSummaryStepDirective,
                        WizardGeneratorReviewButtonDirective
                    ],
                    exports: [
                        WizardGeneratorComponent,
                        WizardGeneratorStepComponent,
                        WizardBodyComponent,
                        DialogWizardGeneratorComponent,
                        WizardSummaryStepComponent,
                        WizardSummarySectionComponent,
                        WizardGeneratorGoNextButtonDirective,
                        WizardGeneratorFinishButtonDirective,
                        WizardGeneratorSummaryStepDirective,
                        WizardGeneratorReviewButtonDirective
                    ],
                    providers: [WizardDialogGeneratorService]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { BaseWizardGenerator, DEFAULT_WIZARD_NAVIGATION_BUTTONS, DialogWizardGeneratorComponent, PlatformWizardGeneratorModule, WizardBodyComponent, WizardDialogGeneratorService, WizardGeneratorComponent, WizardGeneratorFinishButtonDirective, WizardGeneratorGoNextButtonDirective, WizardGeneratorRefreshStrategy, WizardGeneratorReviewButtonDirective, WizardGeneratorService, WizardGeneratorStepComponent, WizardGeneratorSummaryStepDirective, WizardSummarySectionComponent, WizardSummaryStepComponent };
//# sourceMappingURL=fundamental-ngx-platform-wizard-generator.mjs.map
