import { ElementRef, EventEmitter } from '@angular/core';
import { FormStates } from '@fundamental-ngx/cdk/forms';
import { Nullable } from '@fundamental-ngx/cdk/utils';
import { CalendarType, CalendarYearGrid, DateRange, DaysOfWeek, FdCalendarView } from '@fundamental-ngx/core/calendar';
import { DatePickerComponent as FdDatePickerComponent } from '@fundamental-ngx/core/date-picker';
import { MobileModeConfig } from '@fundamental-ngx/core/mobile-mode';
import { Placement, SpecialDayRule } from '@fundamental-ngx/core/shared';
import { BaseInput } from '@fundamental-ngx/platform/shared';
import * as i0 from "@angular/core";
/**
 * The Platform date picker component is a wrapper around fd-date-picker using platform form.
 * It is able to provide form field functionalities as well along with fd-date-picker functions.
 *
 * <fdp-form-group #ffg [formGroup]="datePickerForm">
 *      <fdp-form-field #ffl1 id="birthday">
 *          <fdp-date-picker name="birthday" type="single" [formControl]="ffl1.formControl"></fdp-date-picker>
 *      </fdp-form-field>
 * </fdp-form-group>
 *
 */
export declare class PlatformDatePickerComponent<D> extends BaseInput {
    /**
     * date-picker value set as controller value
     */
    set value(value: D | Array<D> | DateRange<D> | Array<DateRange<D>>);
    get value(): DateRange<D> | D;
    /** below code taken from core/date-picker */
    /** The type of calendar, 'single' for single date selection or 'range' for a range of dates. */
    type: CalendarType;
    /** The currently selected date model in single mode. */
    selectedDate: Nullable<D>;
    /** The currently selected date model in multiple mode. */
    selectedMultipleDates: Array<D>;
    /** The currently selected date model with start and end in range mode. */
    selectedRangeDate: DateRange<D>;
    /** The currently selected date model with multiple ranges. */
    selectedMultipleDateRanges: Array<DateRange<D>>;
    /** The day of the week the calendar should start on. 1 represents Sunday, 2 is Monday, 3 is Tuesday, and so on. */
    startingDayOfWeek: DaysOfWeek;
    /**
     * Whether user wants to mark day cells on hover.
     * Works only on range mode, when start date is selected on Day View.
     */
    rangeHoverEffect: boolean;
    /** Whether to validate the date picker input. */
    useValidation: boolean;
    /** Enables Today-Selection-Button if true */
    showTodayButton: boolean;
    /** Label for Today-Selection-Button */
    todayButtonLabel: string;
    /** Whether a null input is considered valid. */
    allowNull: boolean;
    /** Actually shown active view one of 'day' | 'month' | 'year' in calendar component*/
    activeView: FdCalendarView;
    /**
     *  The placement of the popover. It can be one of: top, top-start, top-end, bottom,
     *  bottom-start, bottom-end, right, right-start, right-end, left, left-start, left-end.
     */
    placement: Placement;
    /** The element to which the popover should be appended. */
    appendTo: ElementRef;
    /** Defines if date picker should be closed after date choose */
    closeOnDateChoose: boolean;
    /**
     *  The state of the form control - applies css classes.
     *  Can be `success`, `error`, `warning`, `information` or blank for default.
     */
    set state(state: FormStates);
    get state(): FormStates;
    /**
     * Whether AddOn Button should be focusable
     * @default true
     */
    buttonFocusable: boolean;
    /**
     * Special days mark, it can be used by passing array of object with
     * Special day number, list 1-20 [class:`fd-calendar__item--legend-{{number}}`] is available there:
     * https://sap.github.io/fundamental-styles/components/calendar.html calendar special days section
     * Rule accepts method with FdDate object as a parameter. ex:
     * `rule: (fdDate: FdDate) => fdDate.getDay() === 1`, which will mark all sundays as special day.
     */
    specialDaysRules: SpecialDayRule<D>[];
    /**
     * Object to customize year grid,
     * Row, Columns and method to display year can be modified
     */
    yearGrid: CalendarYearGrid;
    /**
     * Object to customize aggregated year grid,
     * Row, Columns and method to display year can be modified
     */
    aggregatedYearGrid: CalendarYearGrid;
    /**
     * Whether user wants to mark sunday/saturday with `fd-calendar__item--weekend` class
     */
    markWeekends: boolean;
    /**
     * Whether user wants to show week numbers next to days
     */
    showWeekNumbers: boolean;
    /**
     * Whether the user wants to select multiple days or multiple range dates.
     * If `displayWeekNumbers` is true, the user can click on the week number to mark the related row.
     * The user can click on week days to mark the related column.
     * Note: Clickable selection for week row or column does not work for range selections.
     */
    allowMultipleSelection: boolean;
    /** Whether the date picker is open. Can be used through two-way binding. */
    isOpen: boolean;
    /** Should date picker be inlined. */
    inline: boolean;
    /**
     * Whether to recalculate value from the input as user types or on blur.
     * By default, updates the value as user types.
     * @default false
     */
    processInputOnBlur: boolean;
    /**
     * Whether to prevent page scrolling when focusing date picker input field after calendar has been closed.
     */
    preventScrollOnFocus: boolean;
    /** Whether date picker should rendered in mobile mode. */
    mobile: boolean;
    /** Mobile mode configuration. */
    mobileConfig: MobileModeConfig;
    /** Whether calendar is used inside mobile in landscape mode, it also adds close button on right side */
    mobileLandscape: boolean;
    /** Whether calendar is used inside mobile in portrait mode */
    mobilePortrait: boolean;
    /** Event emitted when the state of the isOpen property changes. */
    readonly isOpenChange: EventEmitter<boolean>;
    /** Event thrown every time selected date in single mode is changed */
    readonly selectedDateChange: EventEmitter<Nullable<D>>;
    /** Event thrown every time the selected dates in multiple mode are changed. */
    readonly selectedMultipleDatesChange: EventEmitter<Array<D>>;
    /** Event thrown every time selected first or last date in range mode is changed */
    readonly selectedRangeDateChange: EventEmitter<DateRange<D>>;
    /** Event thrown every time the first or last date in multiple range mode is changed. */
    readonly selectedMultipleDateRangesChange: EventEmitter<Array<DateRange<D>>>;
    /** Event thrown every time calendar active view is changed */
    readonly activeViewChange: EventEmitter<FdCalendarView>;
    /**
     * @hidden core date-picker as child
     */
    fdDatePickerComponent: FdDatePickerComponent<D>;
    /** @hidden */
    private _datePickerValid;
    /** @hidden */
    private readonly _dateTimeAdapter;
    /** @hidden */
    private readonly _dateTimeFormats;
    /** @hidden */
    constructor();
    /**
     * Function used to disable certain dates in the calendar.
     */
    disableFunction: (value: D) => boolean;
    /**
     * Function used to disable certain dates in the calendar for the range start selection.
     */
    disableRangeStartFunction: (value: D) => boolean;
    /**
     * Function used to disable certain dates in the calendar for the range end selection.
     */
    disableRangeEndFunction: (value: D) => boolean;
    /** @hidden */
    writeValue(value: D | Array<D> | DateRange<D> | Array<DateRange<D>> | null): void;
    /**
     * validates date on date change.
     * @param value D | DateRange<D>
     */
    handleDateChange(value: D | Array<D> | DateRange<D> | Array<DateRange<D>>): void;
    /**
     * Method that handles changes when popover is opened or closed.
     */
    handleOpenChange: (open: boolean) => void;
    /** @hidden */
    handleSelectedDateChange: (fdDate: Nullable<D>) => void;
    /** @hidden */
    handleSelectedMultipleDatesChange: (fdDate: Array<D>) => void;
    /** @hidden */
    handleSelectedRangeDateChange: (fdRangeDate: DateRange<D>) => void;
    /** @hidden */
    handleSelectedMultipleDateRangesChange: (fdRangeDate: Array<DateRange<D>>) => void;
    /** @hidden */
    handleActiveViewChange: (fdCalendarView: FdCalendarView) => void;
    static ɵfac: i0.ɵɵFactoryDeclaration<PlatformDatePickerComponent<any>, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<PlatformDatePickerComponent<any>, "fdp-date-picker", never, { "value": { "alias": "value"; "required": false; }; "type": { "alias": "type"; "required": false; }; "selectedDate": { "alias": "selectedDate"; "required": false; }; "selectedMultipleDates": { "alias": "selectedMultipleDates"; "required": false; }; "selectedRangeDate": { "alias": "selectedRangeDate"; "required": false; }; "selectedMultipleDateRanges": { "alias": "selectedMultipleDateRanges"; "required": false; }; "startingDayOfWeek": { "alias": "startingDayOfWeek"; "required": false; }; "rangeHoverEffect": { "alias": "rangeHoverEffect"; "required": false; }; "useValidation": { "alias": "useValidation"; "required": false; }; "showTodayButton": { "alias": "showTodayButton"; "required": false; }; "todayButtonLabel": { "alias": "todayButtonLabel"; "required": false; }; "allowNull": { "alias": "allowNull"; "required": false; }; "activeView": { "alias": "activeView"; "required": false; }; "placement": { "alias": "placement"; "required": false; }; "appendTo": { "alias": "appendTo"; "required": false; }; "closeOnDateChoose": { "alias": "closeOnDateChoose"; "required": false; }; "state": { "alias": "state"; "required": false; }; "buttonFocusable": { "alias": "buttonFocusable"; "required": false; }; "specialDaysRules": { "alias": "specialDaysRules"; "required": false; }; "yearGrid": { "alias": "yearGrid"; "required": false; }; "aggregatedYearGrid": { "alias": "aggregatedYearGrid"; "required": false; }; "markWeekends": { "alias": "markWeekends"; "required": false; }; "showWeekNumbers": { "alias": "showWeekNumbers"; "required": false; }; "allowMultipleSelection": { "alias": "allowMultipleSelection"; "required": false; }; "isOpen": { "alias": "isOpen"; "required": false; }; "inline": { "alias": "inline"; "required": false; }; "processInputOnBlur": { "alias": "processInputOnBlur"; "required": false; }; "preventScrollOnFocus": { "alias": "preventScrollOnFocus"; "required": false; }; "mobile": { "alias": "mobile"; "required": false; }; "mobileConfig": { "alias": "mobileConfig"; "required": false; }; "mobileLandscape": { "alias": "mobileLandscape"; "required": false; }; "mobilePortrait": { "alias": "mobilePortrait"; "required": false; }; "disableFunction": { "alias": "disableFunction"; "required": false; }; "disableRangeStartFunction": { "alias": "disableRangeStartFunction"; "required": false; }; "disableRangeEndFunction": { "alias": "disableRangeEndFunction"; "required": false; }; }, { "isOpenChange": "isOpenChange"; "selectedDateChange": "selectedDateChange"; "selectedMultipleDatesChange": "selectedMultipleDatesChange"; "selectedRangeDateChange": "selectedRangeDateChange"; "selectedMultipleDateRangesChange": "selectedMultipleDateRangesChange"; "activeViewChange": "activeViewChange"; }, never, ["*"], true, never>;
}
/** @hidden */
export declare function createMissingDateImplementationError(provider: string): Error;
