import { Type } from '@angular/core';
import { FormBuilder } from '@angular/forms';
import { BaseDynamicFormGeneratorControl } from './base-dynamic-form-generator-control';
import { DynamicFormControlGroup, DynamicFormGroupControl } from './dynamic-form-control';
import { FormGeneratorComponentsAccessorService } from './form-generator-components-accessor.service';
import { DynamicFormGroup } from './interfaces/dynamic-form-group';
import { DynamicFormFieldItem, DynamicFormItem, DynamicFormItemMap, DynamicFormValue } from './interfaces/dynamic-form-item';
import { FormComponentDefinition } from './interfaces/form-component-definition';
import { FormGeneratorConfig } from './interfaces/form-generator-module-config';
import * as i0 from "@angular/core";
/**
 * @description Form generator service
 */
export declare class FormGeneratorService {
    private _fb;
    private _componentsAccessor;
    private _defaultItemConfig;
    private _providedConfig;
    /**
     * Map of generated forms to access later.
     */
    readonly forms: Map<string, DynamicFormGroup>;
    /**
     * Predefined validation error messages.
     */
    validationErrorHints: {
        required: string;
        validator: string;
    };
    /** @hidden */
    private readonly _config;
    /**
     * @hidden
     * An RxJS Subject that will kill the data stream upon component’s destruction (for unsubscribing)
     */
    private readonly _destroyRef;
    /** @hidden */
    constructor(_fb: FormBuilder, _componentsAccessor: FormGeneratorComponentsAccessorService, _defaultItemConfig: Partial<DynamicFormFieldItem>, _providedConfig: Partial<FormGeneratorConfig>);
    /**
     * @description Generates `FormGroup` class with the list of `DynamicFormControl` control classes
     * defined in `formItems` argument.
     * @param formName Form name.
     * @param formItems the list of form items which used for form control generation.
     * @returns `FormGroup` class with the list of `DynamicFormControl` control classes.
     */
    generateForm(formName: string, formItems: Map<string, DynamicFormItemMap>): Promise<DynamicFormGroup>;
    /**
     * Adds new control to a form.
     * @param control Control to add
     * @param form Form of the control to be added.
     * @param path Group path.
     */
    addControl(control: DynamicFormItem, form: DynamicFormGroup, path?: string[]): Promise<void>;
    /**
     * Removes the control with given name from the form.
     * @param controlName
     * @param path
     */
    removeControl(controlName: string, form: DynamicFormGroup, path?: string[]): void;
    /**
     * @description Returns form value with applied filters and transformers.
     * @param form Form generator form
     * @param renderValue Whether or not form value will be used for rendering purposes.
     * @returns form value object.
     */
    getFormValue(form: DynamicFormGroup | DynamicFormControlGroup, renderValue?: boolean): Promise<DynamicFormValue>;
    /**
     * @description Adds custom component to the list of available components for the form generator items.
     * @param component Angular component.
     * @param types types of the form item.
     */
    addComponent(component: Type<BaseDynamicFormGeneratorControl>, types: string[]): boolean;
    /**
     * @description Returns best-matched component based on provided `type` argument
     * @param type form item type
     * @returns @see FormComponentDefinition Component definition for the form item
     */
    getComponentDefinitionByType(type: string): FormComponentDefinition | undefined;
    /**
     * @returns Object of available errors, where key is an error type,
     * and it's value is a display text.
     */
    getValidationErrorHints(key: string): string;
    /**
     * @description Adds custom error hint defined by user.
     * @param type error type from validator.
     * @param value display text.
     */
    addValidationErrorHint(type: string, value: string): void;
    /**
     * @description Checks if some items of the form should be hidden,
     * or visible based on `when` method result.
     * @param form
     * @returns `Set` where key is item name, and boolean value if field needs to be shown.
     */
    checkVisibleFormItems(form: DynamicFormGroup): Promise<{
        [key: string]: boolean;
    }>;
    /**
     * Method for searching a form control inside form group. Can also search inside ungrouped form group.
     * @param form Form group.
     * @param controlNames Name of the form control.
     * @returns Found form control.
     */
    getFormControl(form: DynamicFormGroup, ...controlNames: string[]): DynamicFormGroupControl;
    /** Resets the form to it's initial state. */
    resetForm(form: DynamicFormGroup): void;
    /** @hidden */
    _getFormValueWithoutUngrouped(value: any): any;
    /** @hidden */
    private _assignFormItemValidators;
    /** @hidden */
    private _generateDynamicFormItem;
    /** @hidden */
    private _checkFormControlsVisibility;
    /**
     * @hidden
     * @param formItem
     * @param form
     * @param key
     * @returns
     */
    private _getFormItemPropertyValue;
    /** @hidden */
    private _getFormItemProperties;
    /**
     * @hidden
     * @param formItem
     * @param form
     * @returns
     */
    private _getFormItemChoices;
    /**
     * @hidden
     * @description Returns value from Promise-like, Observable-like, simple function or just some object.
     * @param obj
     * @returns
     */
    private _getFunctionValue;
    /** @hidden */
    private _formatPasswordValue;
    /** @hidden */
    private _getMergedFormFieldItemConfig;
    /** @hidden */
    private _transformFormItems;
    /** @hidden */
    private _triggerFieldsOnchange;
    /** @hidden */
    private _cloneOldFormGroup;
    static ɵfac: i0.ɵɵFactoryDeclaration<FormGeneratorService, [null, null, { optional: true; skipSelf: true; }, { optional: true; skipSelf: true; }]>;
    static ɵprov: i0.ɵɵInjectableDeclaration<FormGeneratorService>;
}
