import { ChangeDetectorRef, EventEmitter, OnChanges, OnDestroy, QueryList, SimpleChanges } from '@angular/core';
import { FormControlStatus, FormGroupDirective, NgForm } from '@angular/forms';
import { Nullable } from '@fundamental-ngx/cdk/utils';
import { ColumnLayout, FieldHintOptions, HintInput, HintOptions, PlatformFormFieldControl } from '@fundamental-ngx/platform/shared';
import { BehaviorSubject, Observable, Subject } from 'rxjs';
import { FormGeneratorFieldComponent } from '../form-generator-field/form-generator-field.component';
import { DynamicFormGroupControl } from '../dynamic-form-control';
import { FormGeneratorService } from '../form-generator.service';
import { DynamicFormGroup } from '../interfaces/dynamic-form-group';
import { DynamicFormItem, DynamicFormItemValidationObject, DynamicFormValue } from '../interfaces/dynamic-form-item';
import * as i0 from "@angular/core";
export type FormGeneratorAcceptableItems = DynamicFormItem[] | Observable<DynamicFormItem[]>;
export declare const FDP_FORM_IGNORED_STATUSES: FormControlStatus[];
export interface SubmitFormEventResult {
    /**
     * @description Indicator if validation has been passed.
     */
    success: boolean;
    /**
     * @description Formatted form value.
     */
    value: DynamicFormValue | null;
}
/**
 * @description Form Generator component represents a high-level component
 * which includes @see FormGroup inside.
 */
export declare class FormGeneratorComponent implements OnDestroy, OnChanges {
    private _fgService;
    private _cd;
    /** @description Unique form name */
    formName: string;
    /**
     * @description List of @see DynamicFormItem representing the list of items
     * to be rendered in the form.
     */
    set formItems(formItems: FormGeneratorAcceptableItems);
    get formItems(): FormGeneratorAcceptableItems;
    /**
     * @description Form main title.
     */
    mainTitle: Nullable<string>;
    /**
     * Whether to hide main title. Default is false.
     */
    hideMainTItle: boolean;
    /**
     * @description
     * Hint for the main title
     */
    hint: HintInput;
    /**
     * @description Specify the column layout in the format `XLn-Ln-Mn-Sn`
     * where n is the number of columns and can be different for each size.
     * eg: XL2-L2-M2-S1 would create 2-column layouts for XL, L,
     * and M sizes and single-column layout for S size.
     */
    columnLayout: Nullable<string>;
    /**
     * Defines label's column layout.
     */
    labelColumnLayout: ColumnLayout;
    /**
     * Defines field's column layout.
     */
    fieldColumnLayout: ColumnLayout;
    /**
     * Defines gap column layout.
     */
    gapColumnLayout: ColumnLayout;
    /** Whether all form items should have identical layout provided for form group. */
    unifiedLayout: boolean;
    /** Inline column layout */
    inlineColumnLayout: ColumnLayout;
    /**
     * @hidden
     * Removes extra empty row.
     * TODO: remove after #7533 has been fixed.
     */
    noAdditionalContent: boolean;
    /**
     * @description Event which notifies parent component that the form has been successfully created
     * and all controls are in place.
     */
    formCreated: EventEmitter<DynamicFormGroup>;
    /**
     * @description Event which notifies parent component that the form has been successfully validated
     * and submitted. Contains form
     */
    formSubmitted: EventEmitter<DynamicFormValue>;
    /**
     * @description Event which notifies parent component that the form was submitted.
     */
    formSubmittedStatus: EventEmitter<SubmitFormEventResult>;
    /**
     * @description Represents the form instance. @see NgForm
     */
    ngForm: NgForm;
    /** Inner form group directive. */
    formGroup: FormGroupDirective;
    /** Inner form fields */
    fields: QueryList<FormGeneratorFieldComponent>;
    /** @hidden */
    doCheck$: Subject<void>;
    /** @hidden */
    ngSubmit$: Subject<void>;
    /** Array of form field controls. */
    get formFields(): PlatformFormFieldControl[];
    /**
     * @description Dynamically generated form. @see FormGeneratorService
     */
    form: DynamicFormGroup;
    /**
     * @description List of the form controls.
     */
    formControlItems: DynamicFormGroupControl[];
    /**
     * Set of flags representing if particular form item should be visible to the user.
     */
    shouldShowFields: {
        [key: string]: boolean;
    };
    /**
     * Flag representing that form is loading
     */
    formLoading: boolean;
    /**
     * Stream of form loading state.
     */
    loading$: BehaviorSubject<boolean>;
    /**
     * Stream of last form submission status.
     */
    formSubmittedStatus$: Subject<SubmitFormEventResult>;
    /**
     * Stream of last form submission value.
     */
    formValue$: BehaviorSubject<DynamicFormValue>;
    /**
     * @hidden
     * To differentiate between first loading when skeletons be shown and subsequent loadings when busy indicator be shown
     */
    _firstLoadingDone: boolean;
    /**
     * @hidden
     */
    hintOptions: HintOptions;
    /** @hidden */
    _errorModels: {
        type: string;
        value: any;
    }[];
    /**
     * @hidden
     */
    private _formItems;
    /**
     * @hidden
     */
    private _formValueSubscription;
    /** @hidden */
    private _refresh$;
    /**
     * @hidden
     * An RxJS Subject that will kill the data stream upon component’s destruction (for unsubscribing)
     */
    private readonly _destroyRef;
    /** @hidden */
    private readonly _defaultHintOptions;
    /** @hidden */
    private _ngSubmitSubscription;
    /** @hidden */
    private _mappedFormitems;
    /** @hidden */
    constructor(_fgService: FormGeneratorService, _cd: ChangeDetectorRef, _providedHintOptions: FieldHintOptions);
    /** @hidden */
    ngOnChanges(changes: SimpleChanges): void;
    /**
     * @hidden
     */
    ngOnDestroy(): void;
    /**
     * Refreshes form items visibility with 'when' condition.
     */
    refreshStepsVisibility(): Promise<void>;
    /**
     * @hidden
     */
    _onSubmit(): Promise<void>;
    /**
     * Adds new control to an existing form.
     * @param control Control configuration to add.
     * @param path Path of the control.
     */
    addControl(control: DynamicFormItem, path?: string[]): Promise<void>;
    /**
     * Removes the control with the given name by the given path.
     * @param name Name of the control.
     * @param path Path of the control.
     */
    removeControl(name: string, path: string[]): Promise<void>;
    /**
     *
     * @hidden
     */
    _trackFn(index: number, value: DynamicFormGroupControl): string;
    /** @hidden */
    _groupTrackFn(index: number, value: DynamicFormGroupControl): string;
    /**
     *
     * @description Programmatically submit form.
     * This method also calls validation for the form items.
     */
    submit(): void;
    /** @hidden */
    _isAdvancedError(error: any): error is DynamicFormItemValidationObject;
    /** @hidden */
    _errorsTrackBy(_: number, error: {
        type: string;
        value: any;
    }): string;
    /**
     * @hidden
     */
    private _generateForm;
    /** @hidden */
    private _getOrderedControls;
    /** @hidden */
    private _getErrors;
    /** @hidden */
    private _listenToSubmit;
    /** @hidden */
    private _onFormItemsChange;
    /** @hidden */
    private _applyRank;
    /** @hidden */
    private _addControlToFormItems;
    /** @hidden */
    private _removeControlFromItems;
    static ɵfac: i0.ɵɵFactoryDeclaration<FormGeneratorComponent, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<FormGeneratorComponent, "fdp-form-generator", never, { "formName": { "alias": "formName"; "required": false; }; "formItems": { "alias": "formItems"; "required": false; }; "mainTitle": { "alias": "mainTitle"; "required": false; }; "hideMainTItle": { "alias": "hideMainTItle"; "required": false; }; "hint": { "alias": "hint"; "required": false; }; "columnLayout": { "alias": "columnLayout"; "required": false; }; "labelColumnLayout": { "alias": "labelColumnLayout"; "required": false; }; "fieldColumnLayout": { "alias": "fieldColumnLayout"; "required": false; }; "gapColumnLayout": { "alias": "gapColumnLayout"; "required": false; }; "unifiedLayout": { "alias": "unifiedLayout"; "required": false; }; "inlineColumnLayout": { "alias": "inlineColumnLayout"; "required": false; }; "noAdditionalContent": { "alias": "noAdditionalContent"; "required": false; }; }, { "formCreated": "formCreated"; "formSubmitted": "formSubmitted"; "formSubmittedStatus": "formSubmittedStatus"; }, never, ["*"], true, never>;
}
