import { AfterContentInit, AfterViewInit, ChangeDetectorRef, ElementRef, EventEmitter, OnChanges, OnDestroy, OnInit, QueryList, Signal, SimpleChanges, TemplateRef } from '@angular/core';
import { FormControl, ValidatorFn } from '@angular/forms';
import { FormFieldControl, FormStates } from '@fundamental-ngx/cdk/forms';
import { Subject } from 'rxjs';
import { Nullable } from '@fundamental-ngx/cdk/utils';
import { Column, ColumnLayout, FieldHintInput, FieldHintOptions, FormError, FormFieldErrorDirectiveContext, FormFieldGroup, FormGroupContainer, HintContent, PlatformFormField, PlatformFormFieldControl, ResponsiveBreakPointConfig, ResponsiveBreakpointsService } from '@fundamental-ngx/platform/shared';
import { HintLayoutConfig } from '../fdp-form.tokens';
import { FormFieldLayoutService } from '../services/form-field-layout.service';
import * as i0 from "@angular/core";
import * as i1 from "@fundamental-ngx/platform/shared";
/**
 * Form Field represent actual row and aggregates common behavior for the input field such as
 * error, label or hint.
 * Hint is also responsible for listening for FieldControl changes and trigger necessary
 * change detection
 *
 */
export declare class FormFieldComponent implements PlatformFormField, AfterContentInit, AfterViewInit, OnDestroy, OnInit, OnChanges {
    private _cd;
    readonly formFieldGroup: FormFieldGroup;
    readonly _defaultResponsiveBreakPointConfig: ResponsiveBreakPointConfig;
    readonly _responsiveBreakpointsService: ResponsiveBreakpointsService;
    private _hintLayoutConfig;
    /** Form field label */
    label: string;
    /**
     * Suffix for ids of control elements inside this form field
     */
    id: string;
    /** Hint to be placed next to label */
    hint: Nullable<FieldHintInput>;
    /**
     * Indicates when form field label should not be displayed
     */
    noLabelLayout: boolean;
    /**
     * The list of validators applied to the form field.
     */
    validators: Array<ValidatorFn>;
    /**
     * Rank is used for ordering.
     * First lower number, then - higher
     */
    rank: number;
    /**
     * Placeholder for the field
     */
    placeholder: string;
    /** Form Container column it belongs to */
    column: number;
    /** object for placing field in column in each screen layout */
    set columnLayout(layout: ColumnLayout | undefined);
    get columnLayout(): ColumnLayout;
    /**
     * Defines label's column layout.
     */
    set labelColumnLayout(value: ColumnLayout | undefined);
    get labelColumnLayout(): ColumnLayout;
    /**
     * Defines field's column layout.
     */
    set fieldColumnLayout(value: ColumnLayout | undefined);
    get fieldColumnLayout(): ColumnLayout;
    /**
     * Defines gap column layout.
     */
    set gapColumnLayout(value: ColumnLayout | undefined);
    get gapColumnLayout(): ColumnLayout;
    /**
     * Translations template reference.
     * This is in most of the cases set from parent container (form-group)
     */
    i18Strings: TemplateRef<any>;
    /** Set when form field is a mandatory one. */
    required: boolean;
    /**
     * Indicates if field is editable
     */
    set editable(value: boolean);
    get editable(): boolean;
    /**
     * Form field custom width in columns must be between 1 - 12
     */
    columns: Column;
    /**
     * marks field as disabled. used in reactive form approach.
     */
    disabled: boolean;
    /** Whether label text should be appended with colon. */
    colon: boolean;
    /**
     * Form Group Container to bind the Form-Field to.
     * This will override default value injected by constructor
     */
    formGroupContainer: FormGroupContainer;
    /** Emits whenever formFieldControl's state changes */
    onChange: EventEmitter<string>;
    /** Emits whenever column layout is changed */
    onColumnChange: EventEmitter<boolean>;
    /**
     * @hidden
     * Form field template reference
     */
    renderer: TemplateRef<any>;
    /** @hidden */
    formFieldExtras?: ElementRef<HTMLElement>;
    /** @hidden */
    labelCol?: ElementRef<HTMLDivElement>;
    /** @hidden */
    inputMessageGroup: ElementRef<HTMLElement>;
    /** @hidden */
    innerErrorRenderers: TemplateRef<any>;
    /** @hidden */
    private _errorDirectiveQuery;
    /** @hidden */
    private readonly _inputMessageGroupCmp;
    /** Combined Error directives from field itself and parent form container. */
    get errorDirectives(): FormError[];
    /** @hidden */
    isHorizontal$: Signal<boolean | undefined>;
    /**
     * Child FormFieldControl
     */
    control: PlatformFormFieldControl | null;
    /** @hidden */
    _labelColumnLayoutClass: string;
    /** @hidden */
    _fieldColumnLayoutClass: string;
    /** @hidden */
    _gapColumnLayoutClass: string;
    /**
     * hint and hint placement coerced to the FieldHintOptions
     */
    hintOptions: FieldHintOptions;
    /** @hidden */
    _errorDirectives: FormError[];
    /** Grouped errors. */
    groupedErrors: FormFieldErrorDirectiveContext[];
    /** Event emited when errors object being changed. */
    errorsChange$: Subject<void>;
    /**
     * @hidden
     * Optional FormControl
     */
    formControl: FormControl;
    /** @hidden */
    get _groupHost(): FormGroupContainer | FormFieldGroup;
    /**
     * Will be updated during onChanges and resize, resulting correct placement of the
     * hint respecting passed configs and given breakpoint of screen.
     */
    hintTarget?: string;
    /** @hidden */
    protected _editable: boolean;
    /** @hidden */
    private _isColumnLayoutEnabled;
    /** @hidden column number for different screen sizes */
    private _xlColumnNumber;
    /** @hidden */
    private _lgColumnNumber;
    /** @hidden */
    private _mdColumnNumber;
    /** @hidden */
    private _sColumnNumber;
    /** @hidden */
    private _columnLayout;
    /** @hidden */
    private readonly _responsiveBreakPointConfig;
    /** @hidden */
    private _labelColumnLayout;
    /** @hidden */
    private _fieldColumnLayout;
    /** @hidden */
    private _gapColumnLayout;
    /** @hidden */
    private _formGroupErrorDirectives;
    /** @hidden */
    private _formGroupErrorDirectivesSubscription;
    /** @hidden */
    private _errorDirectivesCdr;
    /** @hidden whether label and control are vertically aligned */
    private get _isHorizontalAlignment();
    /**
     * @hidden
     * Sum of extra heights inside form control and formFieldExtras.
     * Label will be shifted by this number in order to be properly aligned with the control
     */
    get _controlExtrasHeightPx(): number | null;
    /** @hidden */
    private _labelColumnLayout$;
    /** @hidden */
    private _fieldColumnLayout$;
    /** @hidden */
    private _gapColumnLayout$;
    /** @hidden */
    private _needsInlineHelpPlaceSubscription?;
    /** @hidden */
    private _breakPointObserver;
    /** @hidden */
    private _formFieldLayoutService;
    /** @hidden */
    private readonly _defaultHintOptions;
    /** @hidden */
    private readonly _destroyRef;
    /** @hidden */
    constructor(_cd: ChangeDetectorRef, formGroupContainer: FormGroupContainer, formFieldGroup: FormFieldGroup, _defaultResponsiveBreakPointConfig: ResponsiveBreakPointConfig, _responsiveBreakpointsService: ResponsiveBreakpointsService, _providedHintOptions: FieldHintOptions, _hintLayoutConfig: HintLayoutConfig, _selfFormFieldLayoutService: FormFieldLayoutService, _parentFormFieldLayoutService: FormFieldLayoutService);
    /**
     * Sets initial values for label, field and gap columns
     */
    setDefaultColumnLayout(): void;
    /** @hidden */
    listenToInlineHelpPlaceRequirementChanges(getSource: () => any): void;
    /** @hidden */
    ngOnInit(): void;
    /** @hidden */
    ngOnChanges(changes: SimpleChanges): void;
    /** @hidden */
    ngAfterContentInit(): void;
    /** @hidden */
    ngAfterViewInit(): void;
    /** @hidden */
    ngOnDestroy(): void;
    /** @hidden */
    hasErrors(): boolean;
    /**
     * Register underlying form control
     * @param formFieldControl
     */
    registerFormFieldControl(formFieldControl: FormFieldControl): void;
    /**
     * Unregister underlying form control
     * @param formFieldControl
     */
    unregisterFormFieldControl(formFieldControl: FormFieldControl): void;
    /**
     * Groups
     */
    groupErrors(): void;
    /**
     * Gets prioritized control state based on the error types it has.
     */
    getPriorityState(): FormStates;
    /**
     * Sets error directives.
     */
    setErrorDirectives(directives: QueryList<FormError>): void;
    /**
     * Returns whether content of the provided hint is a string.
     * @hidden
     */
    isStringHint(hintOptions: HintContent): hintOptions is string;
    /**
     * @hidden
     * Listens to form message component changes and passes its instance to the form control component.
     */
    private _listenToFormMessage;
    /** @hidden */
    private _validateErrorHandler;
    /** @hidden */
    private _hasValidators;
    /**
     * @hidden
     * Add FormField to FormGroup
     */
    private _addToFormGroup;
    /**
     * @hidden
     * Remove FormField from FormGroup
     */
    private _removeFromFormGroup;
    /**
     * @hidden
     * Add FormControl from FormGroup
     */
    private _addControlToFormGroup;
    /**
     * @hidden
     * Remove FormControl from FormGroup
     */
    private _removeControlFromFormGroup;
    /**
     * @hidden
     * Need to be able to set these properties on every level.
     *  - Global FormGroup Level as well each field
     *
     *  Todo: use more elegant way to set these properties.
     */
    private _updateControlProperties;
    /** @hidden */
    private _setLayout;
    /** @hidden */
    private _updateLayout;
    /** @hidden */
    private _updateHintOptions;
    /** @hidden */
    private _assignErrorDirectives;
    static ɵfac: i0.ɵɵFactoryDeclaration<FormFieldComponent, [null, { optional: true; }, { optional: true; }, { optional: true; }, null, null, null, { self: true; }, { optional: true; skipSelf: true; }]>;
    static ɵcmp: i0.ɵɵComponentDeclaration<FormFieldComponent, "fdp-form-field", never, { "label": { "alias": "label"; "required": false; }; "id": { "alias": "id"; "required": false; }; "hint": { "alias": "hint"; "required": false; }; "noLabelLayout": { "alias": "noLabelLayout"; "required": false; }; "validators": { "alias": "validators"; "required": false; }; "rank": { "alias": "rank"; "required": false; }; "placeholder": { "alias": "placeholder"; "required": false; }; "column": { "alias": "column"; "required": false; }; "columnLayout": { "alias": "columnLayout"; "required": false; }; "labelColumnLayout": { "alias": "labelColumnLayout"; "required": false; }; "fieldColumnLayout": { "alias": "fieldColumnLayout"; "required": false; }; "gapColumnLayout": { "alias": "gapColumnLayout"; "required": false; }; "i18Strings": { "alias": "i18Strings"; "required": false; }; "required": { "alias": "required"; "required": false; }; "editable": { "alias": "editable"; "required": false; }; "columns": { "alias": "columns"; "required": false; }; "disabled": { "alias": "disabled"; "required": false; }; "colon": { "alias": "colon"; "required": false; }; "formGroupContainer": { "alias": "formGroupContainer"; "required": false; }; }, { "onChange": "onChange"; "onColumnChange": "onColumnChange"; }, ["formFieldExtras", "_errorDirectiveQuery"], ["*"], true, never>;
    static ngAcceptInputType_rank: string | number;
    static ngAcceptInputType_required: unknown;
    static ngAcceptInputType_editable: unknown;
    static ngAcceptInputType_columns: i1.Column | `${i1.Column}`;
}
