/**
 * @license
 * F. Kolar
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *
 */
import { AfterContentInit, AfterViewInit, DestroyRef, EventEmitter, OnChanges, OnDestroy, OnInit, Provider, QueryList, SimpleChanges, TemplateRef } from '@angular/core';
import { AbstractControl, FormGroup, NgForm } from '@angular/forms';
import { FormField } from '@fundamental-ngx/cdk/forms';
import { Nullable } from '@fundamental-ngx/cdk/utils';
import { ContentDensityObserver } from '@fundamental-ngx/core/content-density';
import { ColumnLayout, FormFieldGroup, FormGroupContainer, HintInput, HintOptions, PlatformFormField } from '@fundamental-ngx/platform/shared';
import { Field, FieldColumn, FieldGroup } from '../models/field.model';
import * as i0 from "@angular/core";
export declare const formGroupProvider: Provider;
type FormGroupField = (FormField | FormFieldGroup) & {
    hintOptions?: HintOptions;
};
/**
 *
 * FormGroup represent high order container aggregating FormFields and ability to distribute these
 * fields into columns. It mainly hides implementation details that we need to deal with every
 * time we are building a form. We have input fields , error managements, different states,
 * hints and many more.
 *
 * Well behaved form that has information side by side in multiple
 * columns needs to also know how to merge properly.
 *
 * Just to get the idea about the structure without actual input components inside
 *
 * ```html
 * <fdp-form-group [hintPlacement]="'inline'" columnLayout="XL2-L2-M1-S1">
 *    <fdp-form-field [id]="'Field A'" [rank]="10" column="1">
 *    </fdp-form-field>
 *
 *    <fdp-form-field [id]="'Field B'" [rank]="30" column="1">
 *    </fdp-form-field>
 *
 *    <fdp-form-field [id]="'Field C'"  [rank]="20" column="1">
 *    </fdp-form-field>
 *
 *    <fdp-form-field [id]="Field D"  [rank]="20" column="2">
 *    </fdp-form-field>
 *
 *
 *    <fdp-form-field [id]="'Field E'" [rank]="30" column="2">
 *    </fdp-form-field>
 * </fdp-form-group>

 *
 * ````
 *
 *
 * Which can be translated like this
 * [Field A] - First column, rank 10
 * [Field B] - First column, rank 30
 * [Field C] - First column, rank 20
 * [Field D] - Second column, rank 20
 * [Field E] - Second column, rank 30
 *
 * What we expect that it creates following form with two column layout:
 * [Field A]      [Field D]
 * [Field C]      [Field E]
 * [Field B]
 *
 * You can see all is sorted accordingly by rank (C goes before B).
 *
 * When you move into smaller device and all is merged into one columns the naive solution would be:
 *
 * [Field A]
 * [Field C]
 * [Field B]
 * [Field D]
 * [Field E]
 *
 * The fields needs to restructure according to their original positioning. When Field D was on the
 * top why we should move it to the bottom? The more correct solution should be following.
 *
 * [Field A]
 * [Field D]
 * [Field C]
 * [Field E]
 * [Field B]
 *
 * Fields nicely merge together. they don't wrap.
 *
 *
 * Besides this layout support it also wraps form functionality and it can work with FormGroup.
 * You can also decide if you want your form with <form> element or without it.
 *
 *
 */
export declare class FormGroupComponent implements FormGroupContainer, OnInit, AfterContentInit, AfterViewInit, OnDestroy, OnChanges {
    /** Id for the form group element */
    id: string;
    /** Name property to be set on a form. Will be used if `useForm` is set to true */
    name: string;
    /** Indicates if group is editable */
    editable: boolean;
    /** Indicates when labels should not be displayed */
    noLabelLayout: boolean;
    /** User's custom classes */
    class: string;
    /**
     * Form group hint options
     */
    hint: HintInput;
    /**
     * Defines column layout for inline items.
     */
    set inlineColumnLayout(value: ColumnLayout);
    get inlineColumnLayout(): ColumnLayout;
    /**
     * Defines label's column layout.
     */
    labelColumnLayout: ColumnLayout;
    /**
     * Defines field's column layout.
     */
    fieldColumnLayout: ColumnLayout;
    /**
     * Defines gap column layout.
     */
    gapColumnLayout: ColumnLayout;
    /** Angular FormGroup where all underlying controls will be attached to. */
    formGroup: FormGroup;
    /** This is rather simple for now just to have some Section Title if needed */
    topTitle: string;
    /** Form's main title. */
    mainTitle: Nullable<string>;
    /** Whether to hide form's main title. Default is false. */
    hideMainTItle: boolean;
    /**
     * Convenient way to initialize internal FormControls from object
     */
    object: any;
    /**
     * Translations template reference.
     *
     * This is just here to support several ways to pass translation.
     *
     * One way is to provide ng-template #i18n inside the form-group tag and the other one
     * some global one as binding
     */
    i18Strings: TemplateRef<any>;
    /** Whether to wrap all the provided content in a `<form>` */
    useForm: boolean;
    /**
     * Specify the column layout in the format `XLn-Ln-Mn-Sn` where n is the number of columns and can be different for each size.
     * eg: XL2-L2-M2-S1 would create 2-column layouts for XL, L, and M sizes and single-column layout for S size.
     */
    columnLayout: Nullable<string>;
    /** Whether all form items should have identical layout provided for form group */
    unifiedLayout: boolean;
    /**
     * onSubmit event
     */
    onSubmit: EventEmitter<any>;
    /**
     * Native form element. Available with `[useForm]="true"` input property.
     */
    nativeForm: NgForm;
    /** @hidden */
    i18Template: TemplateRef<any>;
    /**
     * @hidden
     *
     * Keep track of added form fields children in correct order.
     *
     * Since "descendants: true" includes fields of nested form group as well.
     *
     * We are forced to use "ContentChildren" so FormField children can be used
     * with optional rendering (ngIf) safely.
     *
     * Form fields within the formGroup is driven by multi-zone layout support. We need to be
     * able to add number of FormFields, and based on given configuration (zone, rank) render them
     * under correct zone  (top, bottom, left, right).
     *
     * We want to make sure that we don't include content and then try to somehow position it as it
     * would lead to the UI where user can see elementing moving as you try to position it.
     *
     */
    protected formGroupChildren: QueryList<FormGroupField>;
    /** @hidden */
    private _errorDirectives;
    /** @hidden */
    xlColumnsNumber: number;
    /** @hidden */
    lgColumnsNumber: number;
    /** @hidden */
    mdColumnsNumber: number;
    /**
     * @hidden
     * User responsive layout
     */
    xlCol: string;
    /**
     * @hidden
     * Packed fields which should be rendered
     */
    formRows: {
        [key: number]: FieldColumn | FieldGroup;
    };
    /** @hidden */
    _hintOptions: Nullable<HintOptions>;
    /** @hidden */
    readonly contentDensityObserver: ContentDensityObserver;
    /** @hidden */
    _inlineColumnLayoutClass: string;
    /** @hidden */
    protected _destroyRef: DestroyRef;
    /** @hidden */
    private _inlineColumnLayout;
    /**
     * @hidden
     *
     * List of direct FdpFormGroup's children.
     *
     * Since "formGroupChildren" uses "{ descendants: true }" option that means
     * "formGroupChildren" will keep track of nested FdpFormGroup fields as well.
     * This list relies on the injection mechanism so nested FdpFormGroup's fields/fieldGroups
     * will be omitted
     */
    private _formGroupDirectChildren;
    /** @hidden */
    private readonly _subscriptions;
    /** @hidden */
    private readonly _cd;
    /** @hidden */
    private readonly _elementRef;
    /** @hidden */
    private readonly _formContainer;
    /** @hidden */
    private readonly _defaultHintOptions;
    /** @hidden */
    constructor();
    /** @hidden */
    ngOnChanges(changes: SimpleChanges): void;
    /** @hidden */
    ngOnInit(): void;
    /** @hidden */
    ngAfterContentInit(): void;
    /** @hidden */
    ngAfterViewInit(): void;
    /** @hidden */
    ngOnDestroy(): void;
    /** @hidden */
    addFormField(formField: PlatformFormField): void;
    /** @hidden */
    removeFormField(formField: PlatformFormField): void;
    /** @hidden */
    addFormFieldGroup(formFieldGroup: FormFieldGroup): void;
    /** @hidden */
    removeFormFieldGroup(formFieldGroup: FormFieldGroup): void;
    /** @hidden */
    addFormControl(name: string, control: AbstractControl): void;
    /** @hidden */
    removeFormControl(name: string): void;
    /** @hidden */
    trackByFn(index: number): number;
    /** @hidden */
    trackByFieldName(index: number, field: Field): string | undefined;
    /** @hidden */
    buildComponentCssClass(): string[];
    /** @hidden used for template side Type correction */
    $fieldGroup: (f: any) => FieldGroup;
    /** @hidden */
    private _listenToFormGroupChildren;
    /** @hidden */
    private _listenFormFieldColumnChange;
    /**
     * @hidden
     * Assign a fields or field group to specified columns with rank.
     * if `columnLayoutType` is given a fields without column property will set on last column.
     * Otherwise the fields set 1 column.
     */
    private _updateFieldByColumn;
    /** @hidden Validate column number */
    private _validateFieldColumn;
    /** @hidden */
    private _updateFormFieldsProperties;
    /**
     * @hidden
     * Pass some global properties to each field. Even formGroup cna be inject directly inside form
     * field we are using here a setter method to initialize the
     *
     */
    private _updateFormFieldProperties;
    /**
     * @hidden
     * if `columnLayoutType` is given, set those column layouts appropriately. Otherwise, a layout will set on 1 column
     */
    private _setUserLayout;
    /** @hidden */
    private _getColumnNumbers;
    /** @hidden */
    private _addDirectFormGroupChild;
    /** @hidden */
    private _removeDirectFormGroupChild;
    /**
     * @hidden
     * Get direct form group children in the original order
     */
    private _getFormGroupChildren;
    /** @hidden */
    private _updateInlineColumnLayout;
    /**
     * @hidden
     * have to trigger change detection here in order to re-check contents
     * inside "FormField.renderer" template every time component's dimensions changed
     */
    private _trackFormGroupResize;
    static ɵfac: i0.ɵɵFactoryDeclaration<FormGroupComponent, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<FormGroupComponent, "fdp-form-group", never, { "id": { "alias": "id"; "required": false; }; "name": { "alias": "name"; "required": false; }; "editable": { "alias": "editable"; "required": false; }; "noLabelLayout": { "alias": "noLabelLayout"; "required": false; }; "class": { "alias": "class"; "required": false; }; "hint": { "alias": "hint"; "required": false; }; "inlineColumnLayout": { "alias": "inlineColumnLayout"; "required": false; }; "labelColumnLayout": { "alias": "labelColumnLayout"; "required": false; }; "fieldColumnLayout": { "alias": "fieldColumnLayout"; "required": false; }; "gapColumnLayout": { "alias": "gapColumnLayout"; "required": false; }; "formGroup": { "alias": "formGroup"; "required": false; }; "topTitle": { "alias": "topTitle"; "required": false; }; "mainTitle": { "alias": "mainTitle"; "required": false; }; "hideMainTItle": { "alias": "hideMainTItle"; "required": false; }; "object": { "alias": "object"; "required": false; }; "i18Strings": { "alias": "i18Strings"; "required": false; }; "useForm": { "alias": "useForm"; "required": false; }; "columnLayout": { "alias": "columnLayout"; "required": false; }; "unifiedLayout": { "alias": "unifiedLayout"; "required": false; }; }, { "onSubmit": "onSubmit"; }, ["i18Template", "formGroupChildren", "_errorDirectives"], ["fdp-form-group"], true, never>;
    static ngAcceptInputType_useForm: unknown;
}
export {};
