import { EventEmitter, OnInit } from '@angular/core';
import { Nullable } from '@fundamental-ngx/cdk/utils';
import { BaseInput } from '@fundamental-ngx/platform/shared';
import * as i0 from "@angular/core";
/** Change event object emitted by Platform Step Input component */
export declare class StepInputChangeEvent<T extends StepInputComponent = StepInputComponent, K = number> {
    /** The source Step Input of the event. */
    source: T;
    /** The new value of a control. */
    payload: K;
    /**
     * Step input selection change event
     * @param source Step input component
     * @param payload Selected value
     */
    constructor(
    /** The source Step Input of the event. */
    source: T, 
    /** The new value of a control. */
    payload: K);
}
export declare enum StepInputAlign {
    Left = "left",
    Center = "center",
    Right = "right"
}
export type StepInputStepFunctionAction = 'increase' | 'decrease';
export type StepInputStepFunction = (value: number, action: StepInputStepFunctionAction) => number;
/**
 * StepInputComponent is a base abstract class that should be used
 * to create type specific StepInput components such as number, money, unitOfMeasure
 * This holds base Step Input functionality that can be abstracted
 */
export declare abstract class StepInputComponent extends BaseInput implements OnInit {
    /** Create valueChange event */
    abstract createChangeEvent(value: number): StepInputChangeEvent;
    /** Format value for view presentation */
    abstract formatValue(value: number | null): string;
    /** Format value for "in focus" mode */
    abstract formatValueInFocusMode(value: number): string;
    /** Parse value entered "in focus" mode */
    abstract parseValueInFocusMode(value: string): number | null;
    /** Sets input value */
    set value(value: Nullable<number>);
    get value(): Nullable<number>;
    /** Sets minimum value boundary */
    set min(min: number);
    get min(): number;
    /** Sets maximum value boundary */
    set max(max: number);
    get max(): number;
    /** Sets input step value */
    set step(step: number);
    get step(): number;
    /** Custom function to calculate step dynamically */
    set stepFn(stepFn: StepInputStepFunction);
    /** Number of digits after the decimal point */
    set precision(precision: number);
    get precision(): number;
    /** Sets largeStep multiplier */
    largerStep: number;
    /** Hides -/+ icons and shows labels */
    showLabels: boolean;
    /** Horizontally aligns value inside input */
    set align(align: StepInputAlign | null);
    get align(): StepInputAlign | null;
    /**
     * ARIA label for increment button
     */
    incrementLabel: string;
    /**
     * ARIA label for decrement button
     */
    decrementLabel: string;
    /** Emits new value when control value has changed */
    valueChange: EventEmitter<StepInputChangeEvent<StepInputComponent, number>>;
    /** @hidden */
    lastEmittedValue: number;
    /** @hidden */
    canIncrement: boolean;
    /** @hidden */
    canDecrement: boolean;
    /** @hidden
     * Indicates if control has an error
     */
    isErrorState: boolean;
    /** @hidden */
    get canChangeValue(): boolean;
    /** @hidden */
    readonly _align$: import("@angular/core").WritableSignal<StepInputAlign | null>;
    /** @hidden */
    readonly _textAlign$: import("@angular/core").Signal<StepInputAlign | null>;
    /** @hidden */
    private _max;
    /** @hidden */
    private _min;
    /** @hidden */
    private _step;
    /** @hidden */
    private _stepFn;
    /** @hidden */
    private _precision;
    /** @hidden */
    private _pendingEnteredValue;
    /** @hidden */
    private get _currentValue();
    /** @hidden */
    private readonly _renderer;
    /** @hidden */
    private readonly _rtlService;
    /** @hidden */
    ngOnInit(): void;
    /** @hidden
     * Override writeValue method to keep input view value up to date
     */
    writeValue(value: number): void;
    /** Increase value */
    increase(step?: number): void;
    /** Decrease value */
    decrease(step?: number): void;
    /** Increase value by large step */
    largeStepIncrease(): void;
    /** Decrease value by large step */
    largeStepDecrease(): void;
    /** @hidden
     * catch value during entering from view.
     */
    onEnterValue(value: string): void;
    /** @hidden
     * Commit entered value from view.
     */
    commitEnteredValue(): void;
    /** @hidden
     * Indicates when input gets focused
     */
    onFocus(): void;
    /** @hidden
     * Indicates when input loses focus
     */
    onBlur(): void;
    /** @hidden */
    private _listenToFormErrorState;
    /** @hidden */
    private _emitChangedValue;
    /** @hidden */
    private _getStepValueForIncrease;
    /** @hidden */
    private _getStepValueForDecrease;
    /** @hidden
     * Get step value based either on "stepFn" or "step"
     */
    private _getStepValue;
    /** @hidden */
    private _updateViewValue;
    /** @hidden */
    private _formatValue;
    /** @hidden */
    private _renderValue;
    /** @hidden */
    private _getInputNativeElement;
    /** @hidden */
    private _getValueSelection;
    /** @hidden */
    private _setValueSelection;
    /** @hidden */
    private _calculateCanIncrement;
    /** @hidden */
    private _calculateCanDecrement;
    /** @hidden */
    private _calculateCanDecrementIncrement;
    /** @hidden */
    private _validateValueByLimits;
    /** @hidden */
    private _checkAndThrowErrorIfStepDoesntMatchPrecision;
    /** @hidden */
    private _setPendingEnteredValue;
    /** @hidden */
    private _resetPendingEnteredValue;
    static ɵfac: i0.ɵɵFactoryDeclaration<StepInputComponent, never>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<StepInputComponent, never, never, { "value": { "alias": "value"; "required": false; }; "min": { "alias": "min"; "required": false; }; "max": { "alias": "max"; "required": false; }; "step": { "alias": "step"; "required": false; }; "stepFn": { "alias": "stepFn"; "required": false; }; "precision": { "alias": "precision"; "required": false; }; "largerStep": { "alias": "largerStep"; "required": false; }; "showLabels": { "alias": "showLabels"; "required": false; }; "align": { "alias": "align"; "required": false; }; "incrementLabel": { "alias": "incrementLabel"; "required": false; }; "decrementLabel": { "alias": "decrementLabel"; "required": false; }; }, { "valueChange": "valueChange"; }, never, never, true, never>;
}
