import { AfterViewInit, ChangeDetectorRef, ElementRef, EventEmitter, NgZone, OnChanges, OnDestroy, OnInit, QueryList, SimpleChanges } from '@angular/core';
import { Nullable, OverflowListDirective } from '@fundamental-ngx/cdk/utils';
import { IconTabBarItem } from '../interfaces/icon-tab-bar-item.interface';
import { TabDestinyMode } from '../types';
import { IconTabBarPopoverBase } from './popovers/icon-tab-bar-popover-base.class';
import { TextTypePopoverComponent } from './popovers/text-type-popover/text-type-popover.component';
import * as i0 from "@angular/core";
export declare abstract class IconTabBarBase implements OnInit, OnChanges, AfterViewInit, OnDestroy {
    /**
     * @hidden
     * List of tab elements, that can receive focus.
     * There's a separate mapping function to get actual html element: `_getTabUIElementFocusable`
     */
    abstract _tabUIElements: QueryList<unknown>;
    /** @hidden */
    abstract _tabBarPopover: IconTabBarPopoverBase;
    /**
     * @description A tab bar configuration that stores the state of each tab. Based on this configuration, a tab bar is representing.
     */
    set tabs(value: IconTabBarItem[]);
    /** @hidden */
    get tabs(): IconTabBarItem[];
    /**
     * @description Flag representing rtl mode
     */
    isRtl: boolean;
    /**
     * @description densityMode setter triggers tabs to re-calculation overflowed tabs
     */
    set densityMode(value: TabDestinyMode);
    /** Currently selected tab Unique ID. */
    selectedUid: string | undefined;
    /**
     * @description Emits when some tab is selected.
     */
    selected: EventEmitter<IconTabBarItem>;
    /** Emits when selected tab Unique ID changes. */
    selectedUidChange: EventEmitter<string | undefined>;
    /**
     * @description Reference to OverflowListDirective
     */
    overflowDirective: OverflowListDirective;
    /** Header element reference. */
    headerElement: ElementRef<HTMLElement>;
    /** @hidden */
    _extraTabs$: import("@angular/core").WritableSignal<IconTabBarItem[]>;
    /** @hidden */
    _lastVisibleTabIndex: number;
    /** @hidden */
    _extraItems$: import("@angular/core").WritableSignal<boolean>;
    /** @hidden Heading level of the tab. */
    tabHeadingLevel: import("@angular/core").InputSignal<number | null>;
    /**
     * @description Associations for colors of the tabs.
     * If any of the color associations provided, they'll be read by screenreader instead of the actual color
     */
    colorAssociations: import("@angular/core").InputSignal<Partial<Record<import("../types").SemanticColor, string>> | undefined>;
    /** @hidden */
    readonly _inDynamicPage: boolean;
    /** @hidden */
    protected readonly _cd: ChangeDetectorRef;
    /** @Hidden */
    protected readonly _ngZone: NgZone;
    /** @hidden */
    private _tabs;
    /** @hidden */
    private _densityMode;
    /** @hidden */
    private _destroyRef;
    /** @hidden */
    private _destroyed;
    /** @hidden */
    ngOnChanges(changes: SimpleChanges): void;
    /** @hidden */
    ngOnInit(): void;
    /** @hidden */
    ngAfterViewInit(): void;
    /** @hidden */
    ngOnDestroy(): void;
    /**
     * @hidden
     * @param selectedItem
     * @param event
     */
    _selectItem(selectedItem: IconTabBarItem, event?: Event): void;
    /** @hidden */
    _keyDownHandler(event: KeyboardEvent, tab: IconTabBarItem | undefined, currentIndex: number, popover?: TextTypePopoverComponent): void;
    /** @hidden focuses next tab */
    _focusNextItem(currentIndex: number): void;
    /** @hidden focuses previous tab */
    _focusPreviousItem(currentIndex: number): void;
    /**
     * @hidden
     * @param tabIndex index of the tab to apply focus
     * @param focusLast whether to focus first or last item in the popover
     */
    _focusItem(tabIndex: number, focusLast?: boolean): void;
    /**
     * @hidden
     * @param selectedItem
     * @description select extra item inside popover
     */
    _selectExtraItem(selectedItem: IconTabBarItem): void;
    /**
     * @hidden
     * @param extraItems
     * @description recalculate _nextSteps and _prevSteps array if we have extra items
     */
    _recalculateVisibleItems(extraItems: number): void;
    /**
     * @hidden
     * Mapping function to resolve UI tab element to it's focusable part/descendant
     */
    protected _getTabUIElementFocusable(tabUIElement: unknown): Nullable<HTMLElement>;
    /**
     * @hidden
     * @description trigger recalculation items, need to do it asynchronously after dom was rerendered
     */
    protected _triggerRecalculationVisibleItems(): void;
    /**
     * @hidden
     * @description initialize state of tabs
     */
    protected _initTabs(): void;
    /**
     * @hidden
     * Resets and hides extra tabs.
     * @param tabs - The list of tab items.
     * @param extraTabs - The list of extra tab items.
     */
    private _resetAndHideExtraTabs;
    /**
     * @hidden
     * Handles changes to the selected UID.
     */
    private _handleSelectedUidChange;
    /**
     * @hidden
     * Calculates the index of the last visible tab item.
     * @param totalTabs - The total number of tabs.
     * @param extraItems - The number of extra items.
     * @param selected - The currently selected tab item (optional).
     * @returns The index of the last visible tab item.
     */
    private _getLastVisibleTabIndex;
    /**
     * @hidden
     * Resets the visibility and CSS classes of a tab item.
     * @param tab - The tab item to reset.
     */
    private _resetTab;
    /**
     * @hidden
     * Hides extra tabs and updates the extraTabs array with the hidden tabs.
     * @param tabs - The array of all tab items.
     * @param extraTabs - The array to store the hidden tab items.
     */
    private _hideExtraTabs;
    /**
     * @hidden
     * Recursively finds the active tab item in a list of tabs.
     * @param tabs - The array of tab items.
     * @returns The active tab item, or undefined if no active tab is found.
     */
    private _findActiveTabId;
    static ɵfac: i0.ɵɵFactoryDeclaration<IconTabBarBase, never>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<IconTabBarBase, never, never, { "tabs": { "alias": "tabs"; "required": false; }; "isRtl": { "alias": "isRtl"; "required": false; }; "densityMode": { "alias": "densityMode"; "required": false; }; "selectedUid": { "alias": "selectedUid"; "required": false; }; "tabHeadingLevel": { "alias": "tabHeadingLevel"; "required": false; "isSignal": true; }; "colorAssociations": { "alias": "colorAssociations"; "required": false; "isSignal": true; }; }, { "selected": "selected"; "selectedUidChange": "selectedUidChange"; }, never, never, true, never>;
}
