import { AfterViewChecked, AfterViewInit, ElementRef, EventEmitter, OnDestroy, TemplateRef } from '@angular/core';
import { ColorAccent, Nullable, Size } from '@fundamental-ngx/cdk/utils';
import { CheckboxComponent } from '@fundamental-ngx/core/checkbox';
import { IconComponent } from '@fundamental-ngx/core/icon';
import { ListUnreadIndicator } from '@fundamental-ngx/core/list';
import { RadioButtonComponent } from '@fundamental-ngx/core/radio';
import { BaseComponent } from '@fundamental-ngx/platform/shared';
import { Observable } from 'rxjs';
import { ListConfig } from './list.config';
import { ListType, SelectionType } from './models/list';
import * as i0 from "@angular/core";
export declare const IS_ACTIVE_CLASS = "is-active";
export type StatusType = 'negative' | 'critical' | 'positive' | 'informative';
export declare class ListIconConfig {
    /** The icon name to display. See the icon page for the list of icons
     * here: https://sap.github.io/fundamental-ngx/icon
     * */
    glyph: IconComponent['glyph'];
    /** user's custom classes */
    class: IconComponent['class'];
    /**
     * The icon font
     * Options include: 'SAP-icons', 'BusinessSuiteInAppSymbols' and 'SAP-icons-TNT'
     */
    font: IconComponent['font'];
    /** Aria-label for Icon. */
    ariaLabel: IconComponent['ariaLabel'];
}
export declare class ListAvatarConfig {
    /** @hidden */
    class: string;
    /** @hidden */
    ariaLabel: Nullable<string>;
    /** @hidden */
    ariaLabelledby: Nullable<string>;
    /** @hidden */
    label: Nullable<string>;
    /** @hidden */
    size: Size;
    /** @hidden */
    glyph: Nullable<string>;
    /** @hidden */
    zoomGlyph: Nullable<string>;
    /** @hidden */
    circle: boolean;
    /** @hidden */
    transparent: boolean;
    /** @hidden */
    contain: boolean;
    /** @hidden */
    placeholder: boolean;
    /** @hidden */
    tile: boolean;
    /** @hidden */
    border: boolean;
    /** @hidden */
    colorAccent: Nullable<ColorAccent>;
    /** @hidden */
    random: boolean;
    /** @hidden */
    clickable: boolean;
    /** @hidden */
    image: Nullable<string>;
    /** @hidden */
    alterIcon: Nullable<string>;
    /** @hidden */
    backupImage: Nullable<string>;
}
/**
 * Base interface for a list variant definition.
 * Captures a list item template definition.
 */
export interface ItemDef {
    templateRef: TemplateRef<any>;
}
export interface ListAdvancedDescription {
    text: string;
    ariaLabel?: string;
    title?: string;
}
export type ListDescription = string | ListAdvancedDescription;
export declare enum LIST_ITEM_TYPE {
    ITEM = "item",
    GROUP = "group",
    FOOTER = "footer",
    HEADER = "header"
}
export declare class ModifyItemEvent {
    /** List Item component */
    source: BaseListItem;
    /** Action */
    action: 'delete' | 'edit';
}
export declare class ListItemDef implements ItemDef {
    templateRef: TemplateRef<any>;
    /** @hidden */
    constructor(/** @docs-private */ templateRef: TemplateRef<any>);
    static ɵfac: i0.ɵɵFactoryDeclaration<ListItemDef, never>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<ListItemDef, "[fdpItemDef]", never, {}, {}, never, never, true, never>;
}
/**
 * This class contains common properties used across list Item components.
 * this can be extended to reduce the code duplication across list Item components.
 */
export declare class BaseListItem extends BaseComponent implements AfterViewInit, AfterViewChecked, OnDestroy {
    itemEl: ElementRef<HTMLElement>;
    protected _listConfig: ListConfig;
    private readonly _list;
    /** ID of the list item */
    id: string;
    /** define level of item for screen reader */
    ariaLevel: number;
    /** define position of item for screen reader */
    ariaPosinset: number;
    /** Avatar component properties. @see AvatarComponent for more details */
    set avatar(value: Nullable<Partial<ListAvatarConfig> | string>);
    get avatar(): ListAvatarConfig;
    /** attribute to hold counter value */
    counter?: string;
    /** Description of the title */
    description: Nullable<ListDescription>;
    /** To invert the status of secondary text */
    inverted: boolean;
    /** Whether there is no data inside list item */
    noDataText: string;
    /**
     * The type of the secondary text.fd-list__byline-right--*
     * Can be one of *positive*, *negative*, *informative*, *critical*, *neutral
     */
    statusType?: StatusType;
    /** attribute to hold secondary text*/
    secondary: Nullable<ListDescription>;
    /**
     * Enabling this flag causes forcing secondary item directive to not wrap text,
     * instead of wrapping there will be text truncation
     */
    secondaryWrap?: boolean;
    /** Tooltip text to show when focused for more than timeout value*/
    title?: string;
    /** Used for placeing navigation Link */
    link: Nullable<string>;
    /**
     * Enabling this flag causes forcing title directive to not wrap text,
     * instead of wrapping there will be text truncation
     */
    titleWrap?: boolean;
    /**
     * List item icon configuration.
     */
    set icon(value: Partial<ListIconConfig>);
    get icon(): ListIconConfig;
    /**
     * sets the list item in bold text
     * which respresents unread data
     */
    unRead: boolean;
    /**
     * Whether to display unread notification circle.
     */
    get displayUnreadNotification(): boolean;
    /** radio button value */
    value: string;
    /** By default selection mode is 'none' */
    set selectionMode(value: SelectionType);
    get selectionMode(): SelectionType;
    /** By default selection mode is 'active' */
    listType: ListType;
    /**
     * Whether Navigation mode is included to list component
     * for all the items
     */
    navigated: boolean;
    /**
     * Whether Navigation mode is included to list component
     * only a subset of the list items are navigable
     * you should indicate those by displaying a navigation arrow
     */
    navigationIndicator: boolean;
    /** role */
    role: Nullable<string>;
    /**
     * event emitter for selected item
     * seprate PR for custom event
     */
    itemSelected: EventEmitter<ModifyItemEvent>;
    /**
     * Event emitted for selected item through the checkbox.
     */
    itemCheckboxSelected: EventEmitter<ModifyItemEvent>;
    /** Event sent when delete, details or any other action buttons are clicked */
    buttonClicked: EventEmitter<ModifyItemEvent>;
    /** @hidden */
    listItem: ElementRef<HTMLLIElement>;
    /** Access child element, for checking link content*/
    anchor: ElementRef<HTMLLinkElement>;
    /** Access edit button*/
    edit: ElementRef<HTMLButtonElement>;
    /** Access delete button*/
    delete: ElementRef<HTMLButtonElement>;
    /** @hidden */
    checkboxComponent: CheckboxComponent;
    /** @hidden */
    radioButtonComponent: RadioButtonComponent;
    /** @hidden */
    _type: LIST_ITEM_TYPE;
    /**
     * @hidden
     * Whether By line mode is included to list component, by which
     * list item will accomdate the data in 2 column
     */
    _hasByLine: boolean;
    /**
     * @hidden
     * Whether listitem has row level selection enabled
     */
    set rowSelection(value: boolean);
    get rowSelection(): boolean;
    /**
     * @hidden
     * Whether listitem is selected binded to template
     */
    _selected: boolean;
    /** @hidden */
    get _selectedAttr(): boolean | null;
    /**
     * @hidden
     * get the focused element for key manager
     */
    _focused: boolean;
    /** @hidden */
    get _listItemRole(): string;
    /** @hidden */
    ariaSetSize: Observable<number>;
    /** @hidden */
    private _rowSelection;
    /** @hidden */
    private _defaultRole;
    /** @hidden */
    private _listComponent;
    /** @hidden */
    private _avatarConfig;
    /** @hidden */
    private _iconConfig;
    /**
     * @hidden
     * radio button selected value binded to template
     */
    private _selectionValue;
    /** @hidden */
    private _selectionMode;
    /** @hidden */
    constructor(itemEl: ElementRef<HTMLElement>, _listConfig: ListConfig, _list: ListUnreadIndicator);
    /** Selection value */
    set selectionValue(value: Nullable<string>);
    get selectionValue(): Nullable<string>;
    /**
     * @hidden
     * On item click event will be emitted
     */
    _onItemClick(): void;
    /** @hidden */
    _handleKeyboardEvent(event: KeyboardEvent): void;
    /**
     * @hidden
     * Handler for mouse events
     */
    _onClick(event: MouseEvent): void;
    /**
     * @hidden
     * helps to avoid multi rows active class with navigation
     */
    _onBlur(event: KeyboardEvent): void;
    /** @hidden */
    ngOnDestroy(): void;
    /** @hidden */
    ngAfterViewInit(): void;
    /**
     * @hidden
     * To detect changes from parent-listbox to list item
     * for example single, multi option selection those details
     * will be deducted in list item
     */
    ngAfterViewChecked(): void;
    /**
     * @hidden
     * Created focus on list item on mouseclick,
     * Up,down arrow press
     */
    focus(): void;
    /**
     * Programmatically set selected state of the list item.
     * @param selected Whether the list item is selected
     */
    setSelected(selected: boolean): void;
    /**
     * @hidden
     * on keydown append active styles on actionable item
     */
    _onKeyDown(event: KeyboardEvent): void;
    /**
     * @hidden
     * on keyup remove active styles from actionable item
     */
    _onKeyUp(event: KeyboardEvent): void;
    /** @hidden */
    _onCheckboxModelChange(): void;
    /**
     * @hidden
     * Handles action button click
     */
    _onActionButtonClick(action: 'delete' | 'edit'): void;
    /**
     * @hidden
     * Handles action button click on key press
     */
    _onKeyButtonClick(event: KeyboardEvent, action: 'delete' | 'edit'): void;
    /** @hidden */
    _isAdvancedText(text: ListDescription): text is ListAdvancedDescription;
    /** @hidden */
    private _onKeyboardClick;
    /** @hidden */
    private _setAttrRole;
    static ɵfac: i0.ɵɵFactoryDeclaration<BaseListItem, [null, null, { optional: true; }]>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<BaseListItem, never, never, { "id": { "alias": "id"; "required": false; }; "ariaLevel": { "alias": "ariaLevel"; "required": false; }; "ariaPosinset": { "alias": "ariaPosinset"; "required": false; }; "avatar": { "alias": "avatar"; "required": false; }; "counter": { "alias": "counter"; "required": false; }; "description": { "alias": "description"; "required": false; }; "inverted": { "alias": "inverted"; "required": false; }; "noDataText": { "alias": "noDataText"; "required": false; }; "statusType": { "alias": "statusType"; "required": false; }; "secondary": { "alias": "secondary"; "required": false; }; "secondaryWrap": { "alias": "secondaryWrap"; "required": false; }; "title": { "alias": "title"; "required": false; }; "link": { "alias": "link"; "required": false; }; "titleWrap": { "alias": "titleWrap"; "required": false; }; "icon": { "alias": "icon"; "required": false; }; "unRead": { "alias": "unRead"; "required": false; }; "value": { "alias": "value"; "required": false; }; "selectionMode": { "alias": "selectionMode"; "required": false; }; "listType": { "alias": "listType"; "required": false; }; "navigated": { "alias": "navigated"; "required": false; }; "navigationIndicator": { "alias": "navigationIndicator"; "required": false; }; "role": { "alias": "role"; "required": false; }; "selectionValue": { "alias": "selectionValue"; "required": false; }; }, { "itemSelected": "itemSelected"; "itemCheckboxSelected": "itemCheckboxSelected"; "buttonClicked": "buttonClicked"; }, never, never, true, never>;
}
