/**
 *
 * When working with large set of data using components such as:
 *  - Datatables
 *  - InputSearch
 *  - ComboBox (Autocomplete)
 *  - Tree
 *  - List
 *  - etc..
 *  and for these we need to unify the way we access a data. Currently we leave most of the complex
 *  data access or data manipulation on application developer which leads to different
 *  implementation approaches and allot of code duplication.
 *
 *  It is common practice when working with different data input sources we try to apply some
 *  Design patterns e.g. Adapter Pattern that provides us with guidelines how different interfaces
 *  can talk to each other and from here we can derive another pattern so called Datasource pattern
 *  which is high-level architectural pattern to abstract data and internal resources
 *  access (DB calls, Rest calls, domain objects) with a mininal amount a code.
 *
 * What we want to achieve is to define abstract layer where each specific resource type needs to
 * implement and move all the related logic into reusable DataSources. (TableDataSource,
 * ComboBoxDataSource, ... )
 *
 * Observable<T[]> is a reactive stream, that changes its content based on what it pushed
 * into it using .next().
 *
 * Each type of data component such Autocomplete, DataTable, List,.. needs to provide its own
 * specific implementation to handle specific use-case.
 *
 * ```
 *  export abstract class DataTableDataSource<T> implements DataSource<T> {
 *    data: T[];
 *    filterTerm: string;
 *    paginator: Paginator | null
 *
 *    filterPredicate: ((data: T, filter: string) => boolean;
 *
 *    sortData: ((data: T[], sort: Sort) => T[]);
 *
 *    //
 *    create (t: T): T;
 *    update (t: T): T;
 *    remove (t: T): T;
 *
 *  }
 * ```
 *
 * After we define a format we can start creating concrete implementations based on the usage or
 * resource .e.g:
 *
 *
 * ```
 * export abstract class RestAutoCompleteDataSource<T> extend DataChooserDataSource <T> {
 *   // We can use a registry that is able to map a Entity Type = Endpoint
 *   // have identical way how to fetch , search ,...
 * }
 * ```
 *
 * Probably when working with data only locally, it can be:
 *
 * ```
 * export abstract class LocalArrayAutoCompleteDataSource<T> extend DataChooserDataSource <T> {
 * }
 * ```
 *
 *
 */
import { InjectionToken } from '@angular/core';
import { BehaviorSubject, Observable, Subject } from 'rxjs';
export declare enum MatchingStrategy {
    STARTS_WITH_PER_TERM = "starts with per term",
    STARTS_WITH = "starts with",
    CONTAINS = "contains"
}
export declare const DATA_PROVIDERS: InjectionToken<Map<string, DataProvider<any>>>;
export interface DataSource<T, P = T[], L = boolean> {
    isDataLoading: L;
    open(): Observable<P>;
    close(): void;
    onDataRequested(): Observable<void>;
    onDataReceived(): Observable<void>;
}
export type MatchBy = (item: any) => any;
export interface MatchingBy {
    firstBy: MatchBy;
    secondaryBy?: MatchBy;
}
/** Matching Strategy: StartsWithPerTerm - Reqexp */
export declare function getMatchingStrategyStartsWithPerTermReqexp(value: string): RegExp;
/** @hidden */
export declare function isDataSource<T = any>(value: any): value is DataSource<T>;
export type ProviderParams = ReadonlyMap<string, any>;
/**
 * Provider is a data driver that can access data and retrieve them. It knows how to get 1
 * or more records, maybe do paging and some other things.
 *
 */
export declare abstract class DataProvider<T> {
    abstract fetch(params: ProviderParams, start?: number, end?: number): Observable<T[]>;
    /** @hidden */
    protected _keyPath: string;
    /** @hidden */
    protected _matchingStrategy: MatchingStrategy;
    /** @hidden */
    protected _matchingBy: MatchingBy | null;
    /** @hidden */
    getTotalItems?(): Observable<number>;
    /**
     * Tells if this DataProvider supports INSERT, REMOVE
     *
     */
    canCRUD(): boolean;
    /** @hidden */
    getOne(params: ProviderParams): Observable<T>;
    /** @hidden */
    insert(payload: any, params?: ProviderParams): Observable<T>;
    /** @hidden */
    remove(params: ProviderParams): Observable<boolean>;
    /** @hidden */
    update(payload: any, params?: ProviderParams): Observable<T>;
    /** @hidden */
    setLookupKey(key: string): void;
    /** @hidden */
    setMatchingBy(matchingBy: MatchingBy): void;
    /** @hidden */
    setMatchingStrategy(strategy: MatchingStrategy): void;
}
export declare class ComboBoxDataSource<T> implements DataSource<T> {
    dataProvider: DataProvider<any>;
    /** @hidden */
    static readonly MaxLimit = 5;
    /** @hidden */
    limitless: boolean;
    /** @hidden */
    protected readonly _dataChanges: BehaviorSubject<T[]>;
    /** @hidden */
    protected readonly _onDataRequested$: Subject<void>;
    /** @hidden */
    protected readonly _onDataReceived$: Subject<void>;
    /** @hidden */
    protected readonly _onDestroy$: Subject<void>;
    /** @hidden */
    protected _dataLoading: boolean;
    /** @hidden */
    get isDataLoading(): boolean;
    /** @hidden */
    constructor(dataProvider: DataProvider<any>);
    /** @hidden */
    match(predicate?: string | Map<string, string>, start?: number, end?: number): void;
    /** @hidden */
    open(): Observable<T[]>;
    /** @hidden */
    close(): void;
    /** @hidden */
    onDataRequested(): Observable<void>;
    /** @hidden */
    onDataReceived(): Observable<void>;
}
export declare class MultiComboBoxDataSource<T> extends ComboBoxDataSource<T> {
    dataProvider: DataProvider<any>;
    /** @hidden */
    constructor(dataProvider: DataProvider<any>);
}
export declare class SearchFieldDataSource<T> extends ComboBoxDataSource<T> {
    dataProvider: DataProvider<any>;
    /** @hidden */
    constructor(dataProvider: DataProvider<any>);
}
export declare class ListDataSource<T> extends ComboBoxDataSource<T> {
    dataProvider: DataProvider<any>;
    /** @hidden */
    limitless: boolean;
    /** @hidden */
    constructor(dataProvider: DataProvider<any>);
}
export declare class MultiInputDataSource<T> extends ComboBoxDataSource<T> {
    dataProvider: DataProvider<any>;
    /** @hidden */
    constructor(dataProvider: DataProvider<any>);
}
export declare class ApprovalFlowUserDataSource<T> extends ComboBoxDataSource<T> {
    dataProvider: DataProvider<T>;
    /** @hidden */
    constructor(dataProvider: DataProvider<T>);
}
export declare class ApprovalFlowTeamDataSource<T> extends ComboBoxDataSource<T> {
    dataProvider: DataProvider<T>;
    /** @hidden */
    constructor(dataProvider: DataProvider<T>);
}
