import { BooleanInput } from '@angular/cdk/coercion';
import { AfterViewInit, DestroyRef, DoCheck, ElementRef, InjectionToken, OnDestroy, OnInit, TemplateRef } from '@angular/core';
import { ControlContainer, ControlValueAccessor, NgControl, NgForm } from '@angular/forms';
import { HasElementRef, Nullable } from '@fundamental-ngx/cdk/utils';
import { Observable, Subject } from 'rxjs';
import { FormFieldAdvancedStateMessage, FormStates } from '@fundamental-ngx/cdk/forms';
import { FormInputMessageGroupComponent } from '@fundamental-ngx/core/form';
import { TriggerConfig } from '@fundamental-ngx/core/popover';
import { BaseComponent } from '../base';
import { PlatformFormField, PlatformFormFieldControl } from './form-field';
import * as i0 from "@angular/core";
export declare const FDP_DO_CHECK: InjectionToken<Observable<void>>;
export declare const FDP_FORM_SUBMIT: InjectionToken<Observable<void>>;
/**
 * All form components share the same information (value, name, placeholder,.. ) as well as
 * the same behavior given by ControlValueAccessor.
 *
 * Even this is not ideal solution there is no other way then use inheritance to reuse some of the
 * common logic. It should be possible to use some kind of compositions with Proxies but something
 * similar that exists in Aspect Oriented Programing.
 *
 * Usually try to fire stateChange only for things that can change dynamically in runtime. We don't expect
 * that e.g. placeholder will change after component is created
 */
export declare abstract class BaseInput extends BaseComponent implements PlatformFormFieldControl, ControlValueAccessor, OnInit, DoCheck, AfterViewInit, OnDestroy, HasElementRef {
    /** Input placeholder */
    placeholder: string;
    /**
     * To allow user to determine what event he wants to trigger the messages to show
     * Accepts any [HTML DOM Events](https://www.w3schools.com/jsref/dom_obj_event.asp).
     */
    triggers: (string | TriggerConfig)[];
    /**
     *  The state of the form control - applies css classes.
     *  Can be 'success', 'error', 'warning', 'default', 'information'.
     *
     * @default 'default'
     */
    set state(state: FormStates | undefined);
    get state(): FormStates;
    /** Holds the message with respect to state */
    stateMessage: Nullable<string>;
    /** Advanced form message configuration. Used for components that can render form messages in its template. */
    stateMessageConfig: Nullable<TemplateRef<any>>;
    /** Whether the input is disabled */
    set disabled(value: boolean);
    get disabled(): boolean;
    /**
     * readOnly Value to Mark component read only
     */
    readonly: boolean;
    /**
     * Tell the component if we are in editing mode.
     */
    set editable(value: boolean);
    get editable(): boolean;
    /** Form control validation event strategy. */
    validationStrategy: ('touched' | 'dirty' | 'submitted')[];
    /**
     * Reference to internal Input element
     */
    protected _elementRef: ElementRef;
    /**
     * need to make  these value accessor as abstract to be implemented by subclasses. Having them
     * in superclass have issue getting reference to them with Object.getOwnPropertyDescripton
     * which we need to programmatically wraps components set/get value
     *
     */
    abstract get value(): any;
    abstract set value(value: any);
    /** @hidden */
    formMessage: FormInputMessageGroupComponent;
    /**
     * See @FormFieldControl
     */
    focused: boolean;
    /** set when input field is mandatory form field */
    required: boolean;
    /** @hidden */
    innerErrorsTemplate?: TemplateRef<any>;
    /** Whether control has errors */
    get controlInvalid(): boolean;
    /** @hidden */
    advancedStateMessage: Nullable<FormFieldAdvancedStateMessage>;
    /**
     * See @FormFieldControl
     */
    readonly stateChanges: Subject<any>;
    /** @hidden */
    readonly formField: PlatformFormField | null;
    /** @hidden */
    readonly _doCheck$: Observable<void> | null;
    /** Element reference. */
    readonly elementRef: ElementRef<any>;
    /** NgControl reference. */
    readonly ngControl: NgControl | null;
    /** Control container reference. */
    readonly controlContainer: ControlContainer | null;
    /** Form reference. */
    readonly ngForm: NgForm | null;
    /**
     * @hidden
     * The state of the form control - applies css classes.
     * Can be `success`, `error`, `warning`, `information` or 'default'
     */
    protected _state: FormStates | undefined;
    /** @hidden */
    protected defaultId: string;
    /** @hidden */
    protected _value: any;
    /** @hidden */
    protected _editable: boolean;
    /** @hidden */
    protected _destroyed: DestroyRef;
    /** @hidden */
    private readonly _externalSubmit;
    /** @hidden */
    private _externalFormSubmitted;
    /**
     * @hidden
     */
    private _controlInvalid;
    /** @hidden */
    constructor();
    /** @hidden */
    onChange: (value: any) => void;
    /** @hidden */
    onTouched: () => void;
    /** @hidden */
    ngOnInit(): void;
    /**
     * Re-validate and emit event to parent container on every CD cycle as they are some errors
     * that we can't subscribe to.
     */
    ngDoCheck(): void;
    /** @hidden */
    ngAfterViewInit(): void;
    /** @hidden */
    registerOnChange(fn: (_: any) => void): void;
    /** @hidden */
    registerOnTouched(fn: () => void): void;
    /** @hidden */
    ngOnDestroy(): void;
    /** @hidden */
    setDisabledState(isDisabled: BooleanInput): void;
    /**
     * Each sub class must override this method as inheritance does not work
     */
    writeValue(value: any): void;
    /**
     *
     * Keeps track of element focus
     */
    _onFocusChanged(isFocused: boolean): void;
    /**
     * Handles even when we click on parent container which is the FormField Wrapping this
     * control
     */
    onContainerClick(event: MouseEvent): void;
    /**
     * In most of the cases when working with input element directly you should be just find to assing
     * variable to this element
     *
     * ```
     * <input #inputElementRef fd-form-control ...>
     * ```
     *
     * and this default behavior used. For other cases implement focus.
     *
     */
    focus(event?: MouseEvent): void;
    /**
     *  Need re-validates errors on every CD iteration to make sure we are also
     *  covering non-control errors, errors that happens outside of this control
     */
    protected updateErrorState(): void;
    /**
     * Used to change the value of a control.
     * @param value the value to be applied
     * @param emitOnChange whether to emit "onChange" event.
     * Should be "false", if the change is made programmatically (internally) by the control, "true" otherwise
     */
    protected setValue(value: any, emitOnChange?: boolean): void;
    /** @hidden */
    protected getValue(): any;
    static ɵfac: i0.ɵɵFactoryDeclaration<BaseInput, never>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<BaseInput, never, never, { "placeholder": { "alias": "placeholder"; "required": false; }; "triggers": { "alias": "triggers"; "required": false; }; "state": { "alias": "state"; "required": false; }; "stateMessage": { "alias": "stateMessage"; "required": false; }; "stateMessageConfig": { "alias": "stateMessageConfig"; "required": false; }; "disabled": { "alias": "disabled"; "required": false; }; "readonly": { "alias": "readonly"; "required": false; }; "editable": { "alias": "editable"; "required": false; }; "validationStrategy": { "alias": "validationStrategy"; "required": false; }; }, {}, never, never, true, never>;
    static ngAcceptInputType_editable: unknown;
}
