import { DatetimeAdapter, FdDate } from '@fundamental-ngx/core/datetime';
import { SearchInput } from '@fundamental-ngx/platform/search-field';
import { SelectItem } from '@fundamental-ngx/platform/shared';
import { Observable, Subject } from 'rxjs';
import { CollectionBooleanFilter, CollectionDateFilter, CollectionFilter, CollectionFilterAndGroup, CollectionFilterGroup, CollectionNumberFilter, CollectionSelectFilter, CollectionStringFilter } from '../interfaces/collection-filter.interface';
import { TableState } from '../interfaces/table-state.interface';
import { TableRow } from '../models';
export declare abstract class BaseTableDataProvider<T, P = T[]> {
    abstract fetch(state?: TableState, parentRows?: TableRow<T>[]): Observable<P>;
    abstract fetchData(state: TableState, parentRows?: TableRow<T>[]): Observable<P>;
    /** Total items count. */
    totalItems: number;
    /** Array of items. */
    items: P;
    /** Date time adapter for date field filtering. */
    dateTimeAdapter?: DatetimeAdapter<any>;
    /** Additional set of filters provided by outside component. */
    protected filterBy: CollectionFilterAndGroup[];
    /** Additional search query provided by outside component. */
    protected searchInput?: SearchInput;
    /** @hidden */
    protected _destroy$: Subject<void>;
    /**
     * Method for setting external filtering conditions.
     * @param filterBy Set of column filters.
     * @param searchInput Search condition from search field.
     */
    setFilters(filterBy: CollectionFilterAndGroup[], searchInput?: SearchInput): void;
    /**
     * Method for filtering the data.
     * @param items Array of data source items.
     * @param filters Set of column filters.
     * @returns Array of filtered items.
     */
    applyFiltering(items: T[], filters: CollectionFilterAndGroup[]): T[];
    /** @hidden */
    getSelectItemValue(item: any): any;
    /**
     * Method which selects appropriate filtering strategy of the field depending on the filter type.
     * Developers can override this method to extend the filtering functionality.
     * @param item item of the data source.
     * @param filters Set of column filters.
     * @returns Whether or not item should be included in data array.
     */
    getFilteringStrategy(item: T, filters: CollectionFilterAndGroup[]): boolean;
    /**
     * Method which filters item depending on applied condition result.
     * Developers can override this method to extend its functionality
     * @param item Item to apply conditions to.
     * @param filter Column filter.
     * @returns {boolean} Whether this item should be present in filtered array of items.
     */
    collectionFilterStrategy(item: T, filter: CollectionFilter): boolean;
    /**
     * Method which applies group filtering conditions for the item.
     * @param item Item to apply conditions to.
     * @param filter Filter group.
     * @returns {boolean} Whether this item should be present in filtered items array.
     */
    collectionFilterGroupStrategy(item: T, filter: CollectionFilterGroup): boolean;
    /**
     * Applies search of the search term for visible table columns.
     * Developers can override this method to extend the filtering functionality.
     * @param items data source items array.
     * @param tableState @see TableState Set of table parameters.
     * @returns filtered data source items array.
     */
    search(items: T[], { searchInput, columnKeys }: TableState): T[];
    /**
     * String filtering strategy
     * @param item data source array item.
     * @param filter filter object
     * @returns whether or not item should be included in data source array.
     */
    protected filterString(item: T, filter: CollectionStringFilter): boolean;
    /**
     * Number filtering strategy
     * @param item data source array item.
     * @param filter filter object
     * @returns whether or not item should be included in data source array.
     */
    protected filterNumber(item: T, filter: CollectionNumberFilter): boolean;
    /**
     * Date filtering strategy
     * @param item data source array item.
     * @param filter filter object
     * @param adapter
     * @returns whether or not item should be included in data source array.
     */
    protected filterDate<D = FdDate>(item: T, filter: CollectionDateFilter, adapter?: DatetimeAdapter<D>): boolean;
    /**
     * Boolean filtering strategy
     * @param item data source array item.
     * @param filter filter object
     * @returns whether or not item should be included in data source array.
     */
    protected filterBoolean(item: T, filter: CollectionBooleanFilter): boolean;
    /**
     * Array filtering strategy
     * @param item data source array item.
     * @param filter filter object
     * @returns whether or not item should be included in data source array.
     */
    protected filterArray(item: T, filter: CollectionSelectFilter): boolean;
    /** @hidden */
    protected unsubscribe(): void;
}
export declare class TableDataProvider<T> extends BaseTableDataProvider<T> {
    /**
     * Method for retrieving the data.
     * @param tableState @see TableState Set of table parameters.
     * @returns Observable with data.
     */
    fetch(tableState?: TableState): Observable<T[]>;
    /**
     * Method for applying filtering and retrieving the data.
     * @param state @see TableState Set of table parameters.
     * @returns Observable with filtered data.
     */
    fetchData(state: TableState): Observable<T[]>;
    /**
     * Method for getting all possible options for particular field.
     * Used for creating select options.
     * Developers can override this method to extend the filtering functionality.
     * @param field key of the data item.
     * @returns Observable with select items.
     */
    getFieldOptions(field: string): Observable<SelectItem[]>;
}
export declare abstract class TableChildrenDataProvider<T> extends BaseTableDataProvider<T, Map<TableRow<T>, T[]>> {
    /**
     * Method for retrieving the amount of child itemf or a defined row.
     * @param row
     * @param state
     */
    abstract rowChildrenCount(row: TableRow<T>, state: TableState): Observable<number>;
    /**
     * Method for retrieving the data.
     * @param tableState @see TableState Set of table parameters.
     * @param parentRows
     * @returns Observable with data.
     */
    fetch(tableState?: TableState, parentRows?: TableRow<T>[]): Observable<Map<TableRow<T>, T[]>>;
    /**
     * Method for applying filtering and retrieving the data.
     * @param state @see TableState Set of table parameters.
     * @param tableRows
     * @returns Observable with filtered data.
     */
    fetchData(state: TableState, tableRows?: TableRow<T>[]): Observable<Map<TableRow<T>, T[]>>;
}
