import { BehaviorSubject, Observable } from 'rxjs';
import { TableDataSource } from '../domain';
import { TableRowType } from '../enums';
export type TableRowState = 'editable' | 'readonly';
export interface TableRowKeyboardDrag {
    direction: 'up' | 'down';
    event: KeyboardEvent;
    mode: 'group' | 'shift';
}
/**
 * Table row entity
 * Used to represent table row in the template
 */
export interface TableRow<T = any> {
    /**
     * Row semantic type
     */
    type: TableRowType.ITEM | TableRowType.GROUP | TableRowType.TREE;
    /**
     * Indicates if row is selected. It also supports intermediate state
     */
    checked: boolean | null;
    checked$?: Observable<TableRow['checked']>;
    /**
     * Index of a "value" in data source list
     */
    index: number;
    /**
     * Data model it represents
     */
    readonly value: T;
    /**
     * Reference to a parent if any
     */
    parent: TableRow | null;
    /**
     * Nesting level
     */
    level: number;
    /**
     * Expandable
     */
    expandable: boolean;
    /**
     * Expanded/Collapsed
     */
    expanded: boolean;
    expanded$?: Observable<TableRow['expanded']>;
    childItems$: Observable<T[]>;
    childItemsLoading$: BehaviorSubject<boolean>;
    stateChanged$: Observable<boolean>;
    /**
     * If item should be hidden. Used to skip rendering
     */
    hidden: boolean;
    /**
     * If the row is navigatable
     */
    navigatable: boolean;
    /**
     * Row state: readonly or editable.
     */
    state: TableRowState;
    /**
     * Children table rows.
     */
    children: TableRow<T>[];
    isTree: boolean;
    setRowType: (type: TableRowType) => void;
    lastChild?: TableRow<T>;
    /** Flag indicating that all child rows should be fetched again. */
    forceFetch: boolean;
}
export type TableRowClass<T = any> = string | ((row: T) => string);
/** @hidden */
export declare function isTableRow<T = any>(row: TableRow<T>): row is TableRow<T>;
export declare class TableRowImpl<T> implements TableRow<T> {
    /** @hidden */
    set checked(value: boolean | null);
    get checked(): boolean | null;
    /** @hidden */
    readonly checked$: Observable<boolean>;
    /** @hidden */
    readonly childItems$: Observable<T[]>;
    /** @hidden */
    readonly stateChanged$: BehaviorSubject<boolean>;
    /** @hidden */
    readonly childItemsLoading$: BehaviorSubject<boolean>;
    /** @hidden */
    forceFetch: boolean;
    /** @hidden */
    children: TableRow<T>[];
    /** @hidden */
    expandable: boolean;
    /** @hidden */
    set expanded(value: boolean);
    get expanded(): boolean;
    /** @hidden */
    readonly expanded$: Observable<boolean>;
    /** @hidden */
    hidden: boolean;
    /** @hidden */
    index: number;
    /** @hidden */
    level: number;
    /** @hidden */
    navigatable: boolean;
    /** @hidden */
    parent: TableRow | null;
    /** @hidden */
    state: TableRowState;
    /** @hidden */
    type: TableRowType;
    /** @hidden */
    readonly value: T;
    /** @hidden */
    childDataSource: TableDataSource<T> | undefined;
    /** @hidden */
    isTree: boolean;
    /** @hidden */
    lastChild?: TableRow<T>;
    /** @hidden */
    private readonly _expandedSubject;
    /** @hidden */
    private readonly _checkedSubject;
    /** @hidden */
    constructor(row: Partial<TableRow>);
    /**
     * Sets the row type and checks whether the row is a tree.
     * @param type Row type.
     */
    setRowType(type: TableRowType): void;
}
