import { SearchInput } from '@fundamental-ngx/platform/search-field';
import { BehaviorSubject, Subject } from 'rxjs';
import { CollectionFilter, CollectionGroup, CollectionSort, TableState } from '../interfaces';
import { ColumnsChange, FilterChange, FreezeChange, GroupChange, PageChange, SearchChange, SortChange } from '../models';
import { TableColumn } from '../table-column';
import * as i0 from "@angular/core";
export type TableStateChange = TableStateProperty<'sort', SortChange> | TableStateProperty<'page', PageChange> | TableStateProperty<'filter', FilterChange> | TableStateProperty<'group', GroupChange> | TableStateProperty<'freeze', FreezeChange> | TableStateProperty<'columns', ColumnsChange> | TableStateProperty<'search', SearchChange>;
export type TableStateProperty<T = 'sort' | 'page' | 'filter' | 'group' | 'freeze' | 'columns' | 'search', P = SortChange | PageChange | FilterChange | GroupChange | FreezeChange | ColumnsChange> = {
    type: T;
    state: P;
};
export declare class TableService {
    /** Table state stream. */
    readonly tableState$: BehaviorSubject<Readonly<TableState>>;
    /** @hidden */
    readonly _semanticHighlighting$: import("@angular/core").WritableSignal<string>;
    /** @hidden */
    readonly _isFilteringFromHeaderDisabled$: import("@angular/core").WritableSignal<boolean>;
    /** @hidden */
    readonly _isShownNavigationColumn$: import("@angular/core").WritableSignal<boolean>;
    /** @hidden */
    readonly _semanticHighlightingColumnWidth$: import("@angular/core").WritableSignal<number>;
    /** Visible columns stream. */
    readonly visibleColumns$: import("@angular/core").WritableSignal<TableColumn[]>;
    /** Visible columns length. */
    visibleColumnsLength: import("@angular/core").Signal<number>;
    /** Popping columns stream. */
    readonly poppingColumns$: import("@angular/core").WritableSignal<TableColumn[]>;
    /** Popping columns length. */
    _poppingColumnsLength: number;
    /** Table columns stream. */
    readonly tableColumns$: BehaviorSubject<TableColumn[]>;
    /** @hidden */
    readonly tableStateChanges$: import("rxjs").Observable<Readonly<TableState>>;
    /** Stream that emits when search state changes. */
    readonly searchChange$: Subject<SearchChange>;
    /** Stream that emits when sort state changes. */
    readonly sortChange$: Subject<SortChange>;
    /** Stream that emits when filter state changes. */
    readonly filterChange$: Subject<FilterChange>;
    /** Stream that emits when group state changes. */
    readonly groupChange$: Subject<GroupChange>;
    /** Stream that emits when freeze state changes. */
    readonly freezeChange$: Subject<FreezeChange>;
    /** Stream that emits when columns state changes. */
    readonly columnsChange$: Subject<ColumnsChange>;
    /** Stream that emits when page state changes. */
    readonly pageChange$: Subject<PageChange>;
    /** Stream that emits when new fetch of data is needed. */
    readonly needFetch$: Subject<void>;
    /** @hidden */
    readonly stateChange$: Subject<TableStateChange>;
    /** Stream that emits when loading state changes. */
    readonly tableLoading$: BehaviorSubject<boolean>;
    /** Listen for soft changes in table subcomponents (mostly table column) */
    readonly markForCheck$: Subject<void>;
    /** Listen for immediate changes in table subcomponents (mostly table column) */
    readonly detectChanges$: Subject<void>;
    /** Sort rules stream. */
    readonly sortRules$: import("@angular/core").WritableSignal<Map<string, CollectionSort>>;
    /**
     * Filter Rules Map stream. Where key is column key, and value is the associated filter rules.
     * Many filters can be applied to one column.
     */
    readonly filterRules$: import("@angular/core").WritableSignal<Map<string, CollectionFilter[]>>;
    /**
     * Group Rules Map stream. Where key is column key and value is associated group rule
     */
    readonly groupRulesSubject: BehaviorSubject<Map<string, CollectionGroup>>;
    /** Group Rules Map signal. */
    readonly groupRules$: import("@angular/core").Signal<Map<string, CollectionGroup> | undefined>;
    /** Get current state/settings of the Table. */
    getTableState(): TableState;
    /** Set current state/settings of the Table. */
    setTableState(state: TableState): void;
    /** Set current loading state of the Table. */
    setTableLoading(isLoading: boolean): void;
    /** Notify about changes in table subcomponents (mostly table column) and detect them on next detector check. */
    markForCheck(): void;
    /** Notify about changes in table subcomponents (mostly table column) and detect them immediately. */
    detectChanges(): void;
    /** Search */
    search(searchInput: SearchInput): void;
    /** Set new sort rules */
    setSort(sortRules: CollectionSort[] | undefined): void;
    /** Add sort rules to the existing ones */
    addSort(sortRules: CollectionSort[]): void;
    /** Set new filter rules */
    setFilters(filterRules: CollectionFilter[] | undefined): void;
    /** Add filter rules to the existing ones */
    addFilters(rulesToAdd: CollectionFilter[]): void;
    /** Removes filters for the provided fields */
    removeFilters(fields: string[]): void;
    /** Set group rules */
    setGroups(groups: CollectionGroup[] | undefined): void;
    /** Add group rules to the existing ones */
    addGroups(groupsToAdd: CollectionGroup[]): void;
    /** Freeze table to column */
    freezeTo(columnName: string | null, end?: boolean): void;
    /** Set table columns */
    setColumns(columns: string[], keys: string[]): void;
    /** Set page. */
    setCurrentPage(currentPage: number): void;
    /** Set items per page. */
    setItemsPerPage(itemsPerPage: number, resetPageNumber?: boolean): void;
    /** Returns default table state. */
    getDefaultState(): TableState;
    /**
     * Shares visible columns array across listeners.
     * @param columns Visible columns to share.
     */
    setVisibleColumns(columns: TableColumn[]): void;
    /**
     * Shares popping columns array across listeners.
     * @param columns Popping columns to share.
     */
    setPoppingColumns(columns: TableColumn[]): void;
    /**
     * Constructs grouping rules based on the table state.
     * @param state Table state.
     */
    buildGroupRulesMap(state?: TableState): void;
    /**
     * Constructs sorting rules based on the table state.
     * @param state Table state
     */
    buildSortRulesMap(state?: TableState): void;
    /**
     * Constructs filter rules based on the table state.
     * @param state Table state.
     */
    buildFilterRulesMap(state?: TableState): void;
    /** Reset all filter rules */
    resetFilters(): void;
    /**
     * Constructs base table config.
     */
    constructTableMetadata(): void;
    /** @hidden */
    private _getFieldName;
    static ɵfac: i0.ɵɵFactoryDeclaration<TableService, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<TableService>;
}
