import { OnChanges, OnInit, SimpleChanges, TemplateRef } from '@angular/core';
import { Nullable } from '@fundamental-ngx/cdk/utils';
import { ColumnAlignValue, FdpCellDef, FdpColumnResponsiveState, FdpEditableCellDef, FdpHeaderCellDef, FilterableColumnDataType, TableCellHeaderPopoverDirective, TableColumn, TableColumnResizeService, TableService } from '@fundamental-ngx/platform/table-helpers';
import { BehaviorSubject } from 'rxjs';
import * as i0 from "@angular/core";
/**
 * The component that represents a table column.
 * ```html
 * <fdp-column
 *  name="name"
 *  key="name"
 *  label="Name"
 *  align="start">
 * </fdp-column>
 *
 * <fdp-column
 *  name="price"
 *  key="price.value">
 *  <fdp-table-header *fdpHeaderCellDef>Price</fdp-table-header>
 *  <fdp-table-cell *fdpCellDef="let item">
 *   {{item.price.value}} {{item.price.currency}}
 *  </fdp-table-cell>
 * </fdp-column>
 * ```
 * */
export declare class TableColumnComponent extends TableColumn implements OnInit, OnChanges {
    private readonly _tableColumnResizeService;
    private readonly _tableService?;
    /** Column unique identifier. */
    name: string;
    /** Column data accessor. */
    key: string;
    /** Column header label. */
    set label(value: string);
    get label(): string;
    /** Cell text alignment. */
    align: ColumnAlignValue;
    /** Toggles sort feature for the column. */
    sortable: boolean;
    /** Toggles filter feature for the column. */
    filterable: boolean;
    /** Initial visibility state of the column. Default `true`. */
    visible: boolean;
    /**
     * Data type the column represents. Default is 'string'
     * @type { 'string' | 'number' | 'date' | 'boolean' }
     */
    dataType: FilterableColumnDataType;
    /**
     * Optional array of available filter options.
     * Providing values to this input will cause the filter to change from a text-type input to a select-type input.
     * */
    filterSelectOptions: string[];
    /** Toggles grouping feature for the column. */
    groupable: boolean;
    /** Toggles freeze feature for the column. */
    freezable: boolean;
    /** Toggles freeze feature for the column. */
    endFreezable: boolean;
    /** Whether the table cell inside table header should be non-interactive. */
    nonInteractive: boolean;
    /** Width of the column cells. */
    set width(value: string);
    get width(): string;
    /** Whether the text should wrap, when text is too long for 1 line */
    noWrap: boolean;
    /** Whether to apply fd-table-text (text-shadow) to the cell content, if disabled noWrap has no effect. */
    applyText: boolean;
    /** Column role attribute. */
    role: 'cell' | 'rowheader' | 'gridcell';
    /** Column cell template */
    columnCellTemplate: Nullable<TemplateRef<any>>;
    /** Label change event */
    labelValueChanges$: BehaviorSubject<string>;
    /** Editable column cell template. */
    editableColumnCellTemplate: Nullable<TemplateRef<any>>;
    /** Column header template */
    headerCellTemplate: Nullable<TemplateRef<any>>;
    /** Column header popover template. */
    headerCellPopoverTemplate: Nullable<TemplateRef<any>>;
    /** Stores information for the header cell if the ellipsis are visible after the column resize */
    headerOverflows: boolean;
    /** @hidden */
    _freezed: boolean;
    /** @hidden */
    _endFreezed: boolean;
    /** @hidden */
    set fdpCellDef(fdpCellDef: Nullable<FdpCellDef>);
    /** Editable column cell template definition. */
    set fdpEditableCellDef(fdpEditableCellDef: Nullable<FdpEditableCellDef>);
    /** @hidden */
    set fdpHeaderCellDef(fdpHeaderCellDef: Nullable<FdpHeaderCellDef>);
    /** @hidden */
    set fdpHeaderCellPopover(popover: Nullable<TableCellHeaderPopoverDirective>);
    /** Responsive state of the column. */
    responsiveState: FdpColumnResponsiveState;
    /** @hidden */
    private _width;
    /** @hidden */
    constructor(_tableColumnResizeService: TableColumnResizeService, _tableService?: TableService | undefined);
    /** @hidden */
    ngOnInit(): void;
    /** Table won't know about column properties update so notify about it manually
     * @hidden */
    ngOnChanges(changes: SimpleChanges): void;
    /** @hidden */
    private _validateNameOption;
    static ɵfac: i0.ɵɵFactoryDeclaration<TableColumnComponent, [null, { optional: true; host: true; }]>;
    static ɵcmp: i0.ɵɵComponentDeclaration<TableColumnComponent, "fdp-column", never, { "name": { "alias": "name"; "required": false; }; "key": { "alias": "key"; "required": false; }; "label": { "alias": "label"; "required": false; }; "align": { "alias": "align"; "required": false; }; "sortable": { "alias": "sortable"; "required": false; }; "filterable": { "alias": "filterable"; "required": false; }; "visible": { "alias": "visible"; "required": false; }; "dataType": { "alias": "dataType"; "required": false; }; "filterSelectOptions": { "alias": "filterSelectOptions"; "required": false; }; "groupable": { "alias": "groupable"; "required": false; }; "freezable": { "alias": "freezable"; "required": false; }; "endFreezable": { "alias": "endFreezable"; "required": false; }; "nonInteractive": { "alias": "nonInteractive"; "required": false; }; "width": { "alias": "width"; "required": false; }; "noWrap": { "alias": "noWrap"; "required": false; }; "applyText": { "alias": "applyText"; "required": false; }; "role": { "alias": "role"; "required": false; }; }, {}, ["fdpCellDef", "fdpEditableCellDef", "fdpHeaderCellDef", "fdpHeaderCellPopover"], never, true, never>;
}
