import { AfterViewChecked, AfterViewInit, ChangeDetectorRef, ElementRef, EventEmitter, Injector, NgZone, OnChanges, OnDestroy, OnInit, QueryList, SimpleChanges, TrackByFunction } from '@angular/core';
import { FocusableCellPosition, FocusableGridDirective, Nullable, RtlService, TabbableElementService } from '@fundamental-ngx/cdk/utils';
import { ContentDensityObserver } from '@fundamental-ngx/core/content-density';
import { SearchInput } from '@fundamental-ngx/platform/search-field';
import { CollectionFilter, CollectionGroup, CollectionSort, EditableTableCell, PlatformTableManagedPreset, RowComparator, SaveRowsEvent, SelectionModeValue, Table, TableCellActivateEvent, TableColumn, TableColumnFreezeEvent, TableColumnResizeService, TableColumnsChangeEvent, TableDataSource, TableDataSourceDirective, TableDraggable, TableFilterChangeEvent, TableGroupChangeEvent, TableInitialState, TablePageChangeEvent, TableRow, TableRowActivateEvent, TableRowClass, TableRowSelectionChangeEvent, TableRowService, TableRowToggleOpenStateEvent, TableScrollable, TableScrollDispatcherService, TableService, TableSortChangeEvent, TableState, TableVirtualScroll } from '@fundamental-ngx/platform/table-helpers';
import { Observable } from 'rxjs';
import { NoDataWrapperComponent, TableToolbarInterface, ToolbarContext } from './components';
import * as i0 from "@angular/core";
import * as i1 from "@fundamental-ngx/platform/table-helpers";
/**
 * The component that represents a table.
 * A table is a set of tabular data. Line items can support data, images and actions.
 * ```html
 * <fdp-table
 *  [dataSource]="source"
 *  fdCompact
 *  selectionMode="multiple"
 *  emptyTableMessage="No data found">
 *
 *  <fdp-table-toolbar
 *   title="Order Line Items"
 *   [hideItemCount]="false">
 *  </fdp-table-toolbar>
 *
 *  <fdp-column
 *   name="name"
 *   key="name"
 *   label="Name"
 *   align="start">
 *  </fdp-column>
 *
 *  <fdp-column
 *   name="description"
 *   key="description"
 *   label="Description">
 *  </fdp-column>
 * </fdp-table>
 * ```
 */
export declare class TableComponent<T = any> extends Table<T> implements AfterViewInit, OnDestroy, OnChanges, OnInit, AfterViewChecked {
    private readonly _ngZone;
    private readonly _cdr;
    readonly _tableService: TableService;
    private readonly _tableScrollDispatcher;
    readonly _tableColumnResizeService: TableColumnResizeService;
    private readonly _rtlService;
    readonly contentDensityObserver: ContentDensityObserver;
    readonly injector: Injector;
    private readonly _tabbableService;
    /** Component name used in a Preset-managed component. */
    name: string;
    /** ID for the Table. */
    id: string;
    /** Whether to allow resizing columns by dragging the column edge with mouse. */
    enableDragResize: boolean;
    /**
     * Whether to fix the table header and footer. Default is true.
     * Note that if the table contains freezable columns, the header and
     * footer will be fixed automatically, regardless of this input value.
     * */
    fixed: boolean;
    /** The column `name` to freeze columns up to and including. */
    freezeColumnsTo: string;
    /** The column `name` to freeze columns after and including. */
    freezeEndColumnsTo: string;
    /** Toggle for page scrolling feature. */
    pageScrolling: boolean;
    /** Whether the table should render growing button at the bottom. */
    showGrowingButton: boolean;
    /**
     * Whether to show total items amount in growing button text content.
     */
    showItemsCount: boolean;
    /** Number of items per page. */
    pageSize: Nullable<number>;
    /** Page scrolling threshold in px. */
    pageScrollingThreshold: number;
    /** Table body height. */
    bodyHeight: string;
    /** Text displayed when table has no items. */
    emptyTableMessage: string;
    /** Text displayed when table has no visible columns. */
    noVisibleColumnsMessage: string;
    /** Table without horizontal borders. */
    noHorizontalBorders: boolean;
    /** If the table scrolling should happen outside and stick the toolbar and thead */
    outerScroll: boolean;
    /** Table without vertical borders. */
    noVerticalBorders: boolean;
    /** Table without borders. */
    noBorders: boolean;
    /** Table body without any borders, but header with borders. */
    noBodyBorders: boolean;
    /** Table without outer borders */
    noOuterBorders: boolean;
    /** Table body without horizontal borders. */
    noBorderX: boolean;
    /** Table body without vertical borders. */
    noBorderY: boolean;
    /** Accessor to a children nodes of tree. */
    relationKey: string;
    /** Table row expanded state key. Used to set the initial state of tree row. */
    expandedStateKey: string;
    /** Property key indicating that the row has children accessible via [childDataSource]. */
    hasChildrenKey: string;
    /**
     * Whether row is navigatable.
     * Pass boolean value to set state for the all rows.
     * Pass string value with the key of the row item's field to compute state for every single row.
     */
    rowNavigatable: string | boolean;
    /**
     * Whether to highlight navigated row.
     */
    highlightNavigatedRow: boolean;
    /** Whether table row can be clicked */
    rowsActivable: boolean;
    /** Value with the key of the row item's field to enable selecting.  */
    selectedKey: string;
    /** Value with the key of the row item's field to enable selecting.  */
    selectableKey: string;
    /**
     * Tracking function that will be used to check the differences in data changes.
     * Used similarly to `ngFor` `trackBy` function.
     * Accepts a function that takes two parameters, index and item.
     */
    trackBy: TrackByFunction<T>;
    /**
     * An optional function that identifies uniqueness of a particular row.
     * Table component uses it to be able to preserve selection when a data list is changed.
     */
    rowComparator: RowComparator<T>;
    /** String or function to calculate additional rows' CSS classes. */
    rowsClass: TableRowClass<T>;
    /** Used to construct empty row object for editing. */
    editableRowSkeleton: T;
    /** Whether all rows should be collapsed by default after the table is loaded. */
    expandOnInit: boolean;
    /**
     * Minimum height of the row, required for the virtualScroll,
     * default is 44px in cozy, 32px in compact and 24px in condensed (set automatically)
     */
    rowHeight: number;
    /**
     * Minimum height of the popping column when displayed in pop-in mode. Required when using popping columns and virtual scroll.
     */
    secondaryRowHeight: number | undefined;
    /** Event emitted when the current preset configuration has been changed. */
    /**
     * Whether to load previous pages.
     * This option works only when `pageScrolling` is true, and the initial page > 1
     */
    loadPagesBefore: boolean;
    /**
     * Whether or not the header cells should be interactive
     * (have hover and active states)
     * default: false
     */
    nonInteractiveHeader: boolean;
    /** @hidden */
    _selectionMode: SelectionModeValue;
    /** Sets selection mode for the table. 'single' | 'multiple' | 'none' */
    set selectionMode(value: SelectionModeValue);
    get selectionMode(): SelectionModeValue;
    /** @hidden */
    private _enableTristateMode;
    /**
     *  When True, the checked state of each tree item depends on the checked
     *  state of its parent or direct child.
     */
    set enableTristateMode(value: boolean);
    get enableTristateMode(): boolean;
    /** Value with the key of the row item's field to compute semantic state of the row.  */
    set semanticHighlighting(value: string);
    get semanticHighlighting(): string;
    /**
     * Whether to force rows to follow 'checked all' state of the table.
     * If true, all new rows that are coming from the dataSource will follow 'checked all' state.
     * If false, row will respect `checkedKey` property to define checked state of itself.
     * Default is false.
     */
    forceCheckedAllState: boolean;
    /**
     * aria-labelledby attribute value for the table.
     */
    ariaLabelledBy: string;
    /** The minimum width at which the user can resize a column, in pixels. Default is 50. */
    minimumColumnWidth: number;
    /**
     * Specifies minimal width of the non-frozen columns.
     * Useful when table has large amount of freezable columns and table still should have some free space available.
     */
    set nonFrozenColumnsMinWidth(value: number);
    get nonFrozenColumnsMinWidth(): number;
    /** @hidden */
    private _nonFrozenColumnsMinWidth;
    /** @hidden */
    private _shouldCheckNewRows;
    /** @hidden */
    private _calculateFrozenColumnRefresh$;
    /** @hidden */
    private _defaultBodyHeight;
    /** Event emitted when current preset configuration has been changed. */
    presetChanged: EventEmitter<Partial<Omit<TableState, "columnKeys">>>;
    /** Event fired when table selection has changed. */
    readonly rowSelectionChange: EventEmitter<TableRowSelectionChangeEvent<T>>;
    /** Event fired when table sort order has changed. */
    readonly sortChange: EventEmitter<TableSortChangeEvent>;
    /** Event fired when the table filter has changed. */
    readonly filterChange: EventEmitter<TableFilterChangeEvent>;
    /** Event fired when table grouping has changed. */
    readonly groupChange: EventEmitter<TableGroupChangeEvent>;
    /** Event fired when visible columns list has been changed. */
    readonly columnsChange: EventEmitter<TableColumnsChangeEvent>;
    /** Event emitted when pagination state has been changed. */
    readonly pageChange: EventEmitter<TablePageChangeEvent>;
    /** Event fired when there is a change in the frozen column. */
    readonly columnFreeze: EventEmitter<TableColumnFreezeEvent>;
    /** Event fired when group/tree row collapsed/expanded. */
    readonly rowToggleOpenState: EventEmitter<TableRowToggleOpenStateEvent<T>>;
    /** Event fired when a cell is clicked or focused. */
    readonly cellActivate: EventEmitter<TableCellActivateEvent<T>>;
    /** Event fired when row navigated. */
    readonly rowActivate: EventEmitter<TableRowActivateEvent<T>>;
    /** Event fired when row navigated. */
    readonly rowNavigate: EventEmitter<TableRowActivateEvent<T>>;
    /** Event fired when empty row added. */
    readonly emptyRowAdded: EventEmitter<void>;
    /** Event fired when save button pressed. */
    readonly save: EventEmitter<SaveRowsEvent<T>>;
    /** Event fired when cancel button pressed. */
    readonly cancel: EventEmitter<void>;
    /** Event emitted when table body being scrolled. */
    readonly tableScrolled: EventEmitter<number>;
    /** Event emitted when new rows has been set and rendered. */
    readonly tableRowsSet: EventEmitter<void>;
    /** Event emitted when all rows being expanded. */
    allRowsExpanded: EventEmitter<void>;
    /** Event emitted when all rows being collapsed. */
    allRowsCollapsed: EventEmitter<void>;
    /** Event emitted when a cell is focused. */
    readonly cellFocused: EventEmitter<any>;
    /** @hidden */
    readonly tableScrollable: TableScrollable;
    /** @hidden */
    readonly tableContainer: ElementRef<HTMLDivElement>;
    /** @hidden */
    readonly table: ElementRef<HTMLDivElement>;
    /** @hidden */
    readonly _focusableGrid: FocusableGridDirective;
    /** @hidden */
    private readonly _tableBody;
    /** @hidden */
    private readonly _dndDirective;
    /** @hidden */
    readonly columns: QueryList<TableColumn>;
    /** @hidden */
    readonly customEditableCells: QueryList<EditableTableCell>;
    /** @hidden */
    readonly tableToolbar: TableToolbarInterface;
    /** @hidden */
    readonly _noDataWrapper: Nullable<NoDataWrapperComponent>;
    /** Total loaded items. */
    loadedRows$: import("@angular/core").WritableSignal<number>;
    /** @hidden */
    hoveredRowIndex$: import("@angular/core").WritableSignal<Nullable<number>>;
    /** @hidden */
    get initialSortBy(): CollectionSort[];
    /**
     * @hidden
     * verify if the bodyheight provided is valid in percentage or pixels
     */
    get isBodyHeightValid(): boolean;
    /**
     * @hidden
     * if the bodyHeight is valid, then set that as value as body height for table
     * else set default body height for table as 100%
     */
    get tableBodyHeight(): string;
    /**
     * @hidden
     * Columns to be rendered in the template
     */
    get _visibleColumns(): TableColumn[];
    /** @hidden */
    get isTreeTable(): boolean;
    /** @hidden */
    get enableRowReordering(): boolean;
    /** Whether the table rows are draggable. */
    get isDraggable(): boolean;
    /** @hidden */
    get _semanticHighlightingColumnWidth(): number;
    /** @hidden Sum of widths of fixed columns (semantic highlighting, selection) */
    get _fixedColumnsPadding(): number;
    /** @hidden */
    get _tableWidthPx(): number;
    /** @hidden */
    get loadingState(): boolean;
    /**
     *  Whether the selection column should be shown,
     *  If there are no visible rows or visible columns
     *  or selection mode is none, then selection column should not be shown.
     **/
    get isSelectionColumnShown(): boolean;
    /** @hidden */
    get _ariaLabelledBy(): string | null;
    /**
     * @hidden
     * Representation of combined table rows.
     * Contains all rows including group rows.
     */
    _tableRows: TableRow<T>[];
    /**
     * @hidden
     * Representation of table rows that came from dataSource.
     * Contains all rows including group rows.
     */
    _dataSourceTableRows: TableRow<T>[];
    /**
     * @hidden
     * Representation of added table rows.
     */
    _newTableRows: TableRow<T>[];
    /**
     * @hidden
     * Visible table rows.
     * Rows list that is used to be rendered in the ui.
     * Based on _tableRows and excludes hidden rows.
     */
    _tableRowsVisible: TableRow<T>[];
    /**
     * @hidden
     * Table Column Map. Where key is column key and value is column
     */
    _keyToColumnMap: Map<string, TableColumn>;
    /**
     * @hidden
     * Freezable column names and their respective indexes
     */
    _freezableColumns: Map<string, number>;
    /**
     * @hidden
     * Freezable column names and their respective indexes for columns that will be frozen to the end of the table
     */
    _freezableEndColumns: Map<string, number>;
    /** @hidden */
    _tableColumnsLength: number;
    /** @hidden */
    _checkedState: boolean | null;
    /** @hidden */
    _isGroupTable$: import("@angular/core").Signal<boolean>;
    /**
     * @hidden
     * Used to create a row component placeholder and set data in it rather than re-create the row component when data changes.
     * Optimizes performance due to skipping initial setup of the component.
     */
    _tableCurrentlyRenderedRowsPlaceholder: number[];
    /** @hidden */
    _dndTableRowsPlaceholder: TableRow[];
    /** @hidden */
    _isSelectionColumnShown: boolean;
    /** @hidden */
    readonly _toolbarContext: ToolbarContext;
    /** @hidden */
    _navigatedRowIndex: number;
    /** @hidden */
    _selectionColumnWidth: number;
    /** @hidden */
    readonly tableColumnsStream: Observable<TableColumn[]>;
    /** @hidden */
    _loadPreviousPages: boolean;
    /** @hidden */
    _rtl: boolean;
    /** @hidden */
    readonly _dataSourceDirective: TableDataSourceDirective<T>;
    /** @hidden */
    readonly _tableRowService: TableRowService<any>;
    /** @hidden */
    readonly initialState: TableInitialState | null;
    /** @hidden */
    readonly _virtualScrollDirective: TableVirtualScroll | null;
    /** @hidden */
    readonly _dndTableDirective: TableDraggable<any> | null;
    /** Loading state */
    loading: boolean | undefined;
    /** @hidden */
    private _semanticHighlightingKey;
    /** @hidden */
    private _forceSemanticHighlighting;
    /** @hidden */
    private readonly _isShownSortSettingsInToolbar$;
    /** @hidden */
    private readonly _isShownFilterSettingsInToolbar$;
    /** @hidden */
    private readonly _isShownGroupSettingsInToolbar$;
    /** @hidden */
    private _appliedFilterNames;
    /** @hidden */
    private readonly _isShownColumnSettingsInToolbar$;
    /** @hidden */
    private readonly _isShownSettingsInToolbar$;
    /**
     * @hidden
     * Indicates when all items are checked
     */
    private _checkedAll;
    /**
     * @hidden
     * Indicates whether at least 1 item is checked
     */
    private _checkedAny;
    /** @hidden */
    private _subscriptions;
    /** @hidden */
    private _viewInitiated;
    /** @hidden */
    private _addedItems;
    /** @hidden */
    private _columnsWidthSet;
    /** @hidden */
    private _dndLoadingState;
    /** @hidden */
    private readonly _rangeSelector;
    /** @hidden */
    private _currentPreset;
    /** @hidden */
    private _initialStateSet;
    /** @hidden */
    private _rowHeightManuallySet;
    /** @hidden */
    private _shouldEmitRowsChange;
    /** @hidden */
    private _lastFreezableColumnCalculation;
    /** @hidden */
    private readonly _tableHeaderResizer;
    /**
     * @hidden
     * Mapping function for the trackBy, provided by the user.
     * Is needed, because we are wrapping user supplied data into a `TableRow` class.
     */
    _rowTrackBy: TrackByFunction<number>;
    /** @hidden */
    private readonly _defaultTrackBy;
    /** @hidden */
    constructor(_ngZone: NgZone, _cdr: ChangeDetectorRef, _tableService: TableService, _tableScrollDispatcher: TableScrollDispatcherService, _tableColumnResizeService: TableColumnResizeService, _rtlService: RtlService, contentDensityObserver: ContentDensityObserver, injector: Injector, _tabbableService: TabbableElementService);
    /** Returns array of rows that are currently rendered in the table body. */
    getCurrentlyRenderedRows(): number[];
    /**
     * Sets an array of rows that are currently rendered in the table body.
     * @param startIndex Start index of all rows.
     * @param length Length of rows.
     */
    setCurrentlyRenderedRows(startIndex: number | undefined, length: number): void;
    /** @hidden */
    ngOnChanges(changes: SimpleChanges): void;
    /** @hidden */
    ngOnInit(): void;
    /** @hidden */
    ngAfterViewInit(): void;
    /** @hidden */
    ngAfterViewChecked(): void;
    /** @hidden */
    ngOnDestroy(): void;
    /** Get current state/settings of the Table. */
    getTableState(): TableState;
    /** Set current state/settings of the Table. */
    setTableState(state: TableState): void;
    /** Get table columns definition list. */
    getTableColumns(): TableColumn[];
    /** Get a list of visible table columns. */
    getVisibleTableColumns(): TableColumn[];
    /** Set Sorting rules */
    sort(sortRules: CollectionSort[]): void;
    /** Add Sorting rules to the existing ones */
    addSort(sortRules: CollectionSort[]): void;
    /** Set Filter rules */
    filter(filterRules: CollectionFilter[]): void;
    /** Add Filter rule */
    addFilter(filterRules: CollectionFilter[]): void;
    /** Removes filters for the provided fields */
    removeFilter(fields: string[]): void;
    /** Set Groups */
    group(groups: CollectionGroup[]): void;
    /** Add Groups */
    addGroup(groups: CollectionGroup[]): void;
    /**
     * Set visible table columns
     * The order is matter
     * @param columns table columns names
     */
    setColumns(columns: string[]): void;
    /** Freeze table to column */
    freezeToColumn(columnName: string, end?: boolean): void;
    /** Unfreeze column */
    unfreeze(columnName: string, end?: boolean): void;
    /** expand all rows */
    expandAll(): void;
    /** collapse all rows */
    collapseAll(): void;
    /** Search in table */
    search(searchInput: SearchInput): void;
    /** Set current page number in table */
    setCurrentPage(currentPage: number): void;
    /** Set items per page number in table  */
    setPageSize(pageSize: number, resetPageNumber?: boolean): void;
    /** Toolbar Sort Settings button visibility */
    showSortSettingsInToolbar(showSortSettings: boolean): void;
    /** Toolbar Filter Settings button visibility */
    showFilterSettingsInToolbar(showFilterSettings: boolean): void;
    /** Toolbar Group Settings button visibility */
    showGroupSettingsInToolbar(showGroupSettings: boolean): void;
    /** Toolbar Columns Settings button visibility */
    showColumnSettingsInToolbar(showColumnSettings: boolean): void;
    /** Toolbar Settings button visibility */
    showSettingsInToolbar(showSettings: boolean): void;
    /** Disable filter from column heder menu */
    setHeaderColumnFilteringDisabled(disabled: boolean): void;
    /** Set the row navigation */
    setRowNavigation(rowIndex: number, rowNavigatable: string | boolean): void;
    /**
     * Toggle row checked state.
     * @param rowIndex Index of the row.
     */
    toggleSelectableRow(rowIndex: number): void;
    /**
     * Sets the checked state of the row.
     * @param rowIndex Index of the row.
     * @param value Value of the checked state.
     */
    setRowCheckedState(rowIndex: number, value: boolean): void;
    /** Remove the row navigation */
    removeRowNavigation(rowIndex: number): void;
    /** Manually triggers column's width recalculation */
    recalculateTableColumnWidth(): void;
    /** Gets the max allowed width for all freezable columns */
    getMaxAllowedFreezableColumnsWidth(): number;
    /** Get table data source */
    getDataSource(): TableDataSource<T>;
    /**
     * Adds empty row for editing at the beginning of the rows array.
     */
    addRow(): void;
    /** Cancels editing and discards newly added rows */
    cancelEditing(): void;
    /**
     * Emits save event and resets editable rows array.
     */
    saveRows(): void;
    /** Sets current preset for the Table. */
    setPreset(data: PlatformTableManagedPreset): void;
    /** Returns current preset configuration. */
    getCurrentPreset(): PlatformTableManagedPreset;
    /**
     * Manually triggers list item event listeners for drag&drop list directive.
     */
    refreshDndList(): void;
    /**
     * Programmatically toggles `expand` state of the row with defined index.
     * @param rowIndexes row indexes including nested rows.
     *
     * Usage example:
     * * To toggle first level row, pass one argument:
     * ```
     * table.toggleGroupRows(0) // Will result in toggling first row if it's type is `TableRowType.TREE` or `TableRowType.GROUP`.
     * ```
     * * To toggle nested rows, pass additional arguments of indexes:
     * ```
     * table.toggleGroupRows(0, 1) // Will result in toggling first root row, then it's second child.
     * ```
     */
    toggleGroupRows(...rowIndexes: number[]): void;
    /**
     * Function used to focus a cell in the table utilizing a FocusableCellPosition interface.
     */
    focusCell(position: FocusableCellPosition): void;
    /**
     * Clears the array of table rows. Triggered when new data source is applied.
     */
    clearTableRows(): void;
    /** @hidden */
    _scrollToOverlappedCell(): void;
    /** @hidden */
    _calculatingLoading(isLoading: boolean): void;
    /**
     * @hidden
     * Toggle selectable row in SelectionMode.MULTIPLE
     */
    _toggleMultiSelectRow(rowToToggle: TableRow, event?: Event): void;
    /**
     * @hidden
     * Toggle selectable row in SelectionMode.SINGLE
     */
    _toggleSingleSelectableRow(rowToToggle: TableRow): void;
    /**
     * @hidden
     * Select / Unselect all selectable rows
     */
    _toggleAllSelectableRows(selectAll: boolean): void;
    /**
     * @hidden
     * Create table row selection event
     */
    _emitRowSelectionChangeEvent(added: TableRow[], removed: TableRow[], all?: boolean): void;
    /** @hidden */
    _onCellClick(colIdx: number, row: TableRow<T>): void;
    /** @hidden */
    _onRowClick(row: TableRow<T> | null, event: Event): void;
    /** @hidden */
    _emitCellActivate(columnIndex: number, row: TableRow<T>): void;
    /** @hidden */
    _emitRowActivate(row: TableRow<T>, ctrlKey: boolean): void;
    /**
     * @hidden
     * Expand/Collapse group row
     */
    _toggleGroupRow(groupRow: TableRow<T>): void;
    /** @hidden */
    _emitRowNavigate(row: TableRow<T>, ctrlKey: boolean): void;
    /** Fetch data source data. */
    fetch(): void;
    /** @hidden */
    onTableRowsChanged(): void;
    /** @hidden */
    toggleExpandableTableRow(rowToToggle: TableRow, forceFetch?: boolean): void;
    /** @hidden */
    _onSpyIntersect(intersected: boolean): void;
    /** @hidden */
    handleMouseEnter(rowIndex: number): void;
    /** @hidden */
    handleMouseLeave(): void;
    /** Manually update index after we add new items to the main array */
    private _reIndexTableRows;
    /** @hidden */
    private _isValidInteger;
    /** @hidden */
    private _markAsExpanded;
    /** @hidden */
    private _listenToTableRowsPipe;
    /** @hidden */
    private _listenToTableStateChanges;
    /** @hidden */
    private _setAppliedFilterNames;
    private _formatParams;
    /** @hidden */
    private _formatColumnName;
    /** @hidden */
    private _listenToTableRowStateChange;
    /** @hidden */
    private _createTableRowsByDataSourceItems;
    /** @hidden */
    private _createTreeTableRowsByDataSourceItems;
    /** @hidden */
    private _setTableRows;
    /** @hidden */
    private _calculateVisibleTableRows;
    /** @hidden */
    private _listenToRowHeightChange;
    /** @hidden */
    private _listenToColumns;
    /**
     * Construct visible columns for rendering purpose.
     */
    private _calculateVisibleColumns;
    /** @hidden */
    private _calculateTableColumnsLength;
    /** @hidden */
    private _calculateIsShownNavigationColumn;
    /** @hidden */
    private _setFreezableInfo;
    /** @hidden */
    private _buildColumnsMap;
    /** @hidden */
    private _calculateCheckedAll;
    /**
     * Propagates tristate selection mode to the tree. It starts with updating a state for all the parents and then
     * to children
     *
     * @hidden
     */
    private _applyTristateSelection;
    /** @hidden */
    private _applySelectionToParents;
    /** @hidden */
    private _applySelectionToChildren;
    /** @hidden */
    private _listenToPageScrolling;
    /** @hidden */
    private _listenToColumnPropertiesChange;
    /** @hidden */
    private _listenToTableWidthChanges;
    /** @hidden */
    private _listenToTableContainerMouseLeave;
    /** @hidden */
    private _initScrollPosition;
    /**
     * @hidden
     * Resets editable rows discarding the editable rows array.
     */
    private _resetEditState;
    /** @hidden */
    private _listenToLoadingAndRefocusCell;
    /** @hidden */
    private _onZoneFree;
    /** @hidden */
    private _setSemanticHighlighting;
    /** @hidden */
    private _setSelectionColumnWidth;
    /** @hidden */
    private _checkCellMock;
    /**
     * @hidden
     * Listens to fixed column width subject and calculates last applicable frozen column based on `nonFrozenColumnsMinWidth` vaue.
     */
    private _listenToFixedColumnsWidth;
    private _calculateLastFreezableColumn;
    static ɵfac: i0.ɵɵFactoryDeclaration<TableComponent<any>, [null, null, null, null, null, { optional: true; }, null, null, null]>;
    static ɵcmp: i0.ɵɵComponentDeclaration<TableComponent<any>, "fdp-table", never, { "name": { "alias": "name"; "required": false; }; "id": { "alias": "id"; "required": false; }; "enableDragResize": { "alias": "enableDragResize"; "required": false; }; "fixed": { "alias": "fixed"; "required": false; }; "freezeColumnsTo": { "alias": "freezeColumnsTo"; "required": false; }; "freezeEndColumnsTo": { "alias": "freezeEndColumnsTo"; "required": false; }; "pageScrolling": { "alias": "pageScrolling"; "required": false; }; "showGrowingButton": { "alias": "showGrowingButton"; "required": false; }; "showItemsCount": { "alias": "showItemsCount"; "required": false; }; "pageSize": { "alias": "pageSize"; "required": false; }; "pageScrollingThreshold": { "alias": "pageScrollingThreshold"; "required": false; }; "bodyHeight": { "alias": "bodyHeight"; "required": false; }; "emptyTableMessage": { "alias": "emptyTableMessage"; "required": false; }; "noVisibleColumnsMessage": { "alias": "noVisibleColumnsMessage"; "required": false; }; "noHorizontalBorders": { "alias": "noHorizontalBorders"; "required": false; }; "outerScroll": { "alias": "outerScroll"; "required": false; }; "noVerticalBorders": { "alias": "noVerticalBorders"; "required": false; }; "noBorders": { "alias": "noBorders"; "required": false; }; "noBodyBorders": { "alias": "noBodyBorders"; "required": false; }; "noOuterBorders": { "alias": "noOuterBorders"; "required": false; }; "noBorderX": { "alias": "noBorderX"; "required": false; }; "noBorderY": { "alias": "noBorderY"; "required": false; }; "relationKey": { "alias": "relationKey"; "required": false; }; "expandedStateKey": { "alias": "expandedStateKey"; "required": false; }; "hasChildrenKey": { "alias": "hasChildrenKey"; "required": false; }; "rowNavigatable": { "alias": "rowNavigatable"; "required": false; }; "highlightNavigatedRow": { "alias": "highlightNavigatedRow"; "required": false; }; "rowsActivable": { "alias": "rowsActivable"; "required": false; }; "selectedKey": { "alias": "selectedKey"; "required": false; }; "selectableKey": { "alias": "selectableKey"; "required": false; }; "trackBy": { "alias": "trackBy"; "required": false; }; "rowComparator": { "alias": "rowComparator"; "required": false; }; "rowsClass": { "alias": "rowsClass"; "required": false; }; "editableRowSkeleton": { "alias": "editableRowSkeleton"; "required": false; }; "expandOnInit": { "alias": "expandOnInit"; "required": false; }; "rowHeight": { "alias": "rowHeight"; "required": false; }; "secondaryRowHeight": { "alias": "secondaryRowHeight"; "required": false; }; "loadPagesBefore": { "alias": "loadPagesBefore"; "required": false; }; "nonInteractiveHeader": { "alias": "nonInteractiveHeader"; "required": false; }; "selectionMode": { "alias": "selectionMode"; "required": false; }; "enableTristateMode": { "alias": "enableTristateMode"; "required": false; }; "semanticHighlighting": { "alias": "semanticHighlighting"; "required": false; }; "forceCheckedAllState": { "alias": "forceCheckedAllState"; "required": false; }; "ariaLabelledBy": { "alias": "ariaLabelledBy"; "required": false; }; "minimumColumnWidth": { "alias": "minimumColumnWidth"; "required": false; }; "nonFrozenColumnsMinWidth": { "alias": "nonFrozenColumnsMinWidth"; "required": false; }; "loading": { "alias": "loading"; "required": false; }; }, { "presetChanged": "presetChanged"; "rowSelectionChange": "rowSelectionChange"; "sortChange": "sortChange"; "filterChange": "filterChange"; "groupChange": "groupChange"; "columnsChange": "columnsChange"; "pageChange": "pageChange"; "columnFreeze": "columnFreeze"; "rowToggleOpenState": "rowToggleOpenState"; "cellActivate": "cellActivate"; "rowActivate": "rowActivate"; "rowNavigate": "rowNavigate"; "emptyRowAdded": "emptyRowAdded"; "save": "save"; "cancel": "cancel"; "tableScrolled": "tableScrolled"; "tableRowsSet": "tableRowsSet"; "allRowsExpanded": "allRowsExpanded"; "allRowsCollapsed": "allRowsCollapsed"; "cellFocused": "cellFocused"; }, ["tableToolbar", "_noDataWrapper", "columns", "customEditableCells"], ["fdp-table-no-data-wrapper"], true, [{ directive: typeof i1.TableDataSourceDirective; inputs: { "dataSource": "dataSource"; "childDataSource": "childDataSource"; }; outputs: { "childDataSourceChanged": "childDataSourceChanged"; "onDataRequested": "onDataRequested"; "onDataReceived": "onDataReceived"; "dataSourceChanged": "dataSourceChanged"; "dataChanged": "dataChanged"; "isLoading": "isLoading"; }; }, { directive: typeof i1.TableHeaderResizerDirective; inputs: {}; outputs: {}; }]>;
    static ngAcceptInputType_outerScroll: unknown;
}
