import { Observable } from 'rxjs';
import { FormGeneratorService } from '@fundamental-ngx/platform/form';
import { WizardGeneratorFormGroup } from './interfaces/wizard-generator-form-group.interface';
import { WizardGeneratorFormsValue, WizardStepSubmittedForms } from './interfaces/wizard-generator-forms.interface';
import { PreparedWizardGeneratorItem, WizardGeneratorItem } from './interfaces/wizard-generator-item.interface';
import { WizardGeneratorStep } from './interfaces/wizard-step.interface';
import * as i0 from "@angular/core";
export type StepsComponents = Map<string, WizardGeneratorStep>;
export declare enum WizardGeneratorRefreshStrategy {
    REFRESH_STEP_VISIBILITY = "refreshStepVisibility",
    REVALIDATE_STEP_FORMS = "revalidateStepForms",
    REFRESH_FORM_VISIBILITY = "refreshFormVisibility"
}
export interface DependencySteps {
    /** Step ID */
    [key: string]: {
        /** Form ID with array of form control id's */
        [key: string]: {
            [key in WizardGeneratorRefreshStrategy]?: string[];
        };
    };
}
export interface StepDependencyFields {
    /** Dependent step */
    [key: string]: DependencySteps;
}
/**
 * @description Helper service to keep all transformations and data in one place.
 */
export declare class WizardGeneratorService {
    private _formGeneratorService;
    /** @hidden */
    items: PreparedWizardGeneratorItem[];
    /**
     * @description Visible steps components
     */
    stepsComponents: StepsComponents;
    /**
     * @description Object with steps that are dependencies for another steps.
     */
    dependencySteps: StepDependencyFields;
    /**
     * @description Whether to append the step to the wizard. If false, each step will be displayed on a different page.
     * Default is true.
     */
    appendToWizard: boolean;
    /** @hidden */
    _shouldRedirectToSummary: boolean;
    /**
     * @description Steps that are visible to the user.
     */
    visibleWizardSteps: PreparedWizardGeneratorItem[];
    /** @hidden */
    private _visibleWizardSteps$;
    /** @hidden */
    private _appendToWizard$;
    /** @hidden */
    private _stepsComponents$;
    /** @hidden */
    private _stepsOrderChanged$;
    /** @hidden */
    private _nextStepIndex$;
    /** @hidden */
    private _submittedFormRawValues;
    /** @hidden */
    private _wizardStepIds;
    /** @hidden */
    constructor(_formGeneratorService: FormGeneratorService);
    /**
     * @description Returns current step ID in Wizard.
     * @returns {string | undefined} Current step ID.
     */
    getCurrentStepId(): string | undefined;
    /**
     * @description Returns current step index.
     * @returns {number} Index of the current step in array of Wizard steps.
     */
    getCurrentStepIndex(): number;
    /**
     * Searches for the index of the step based on it's ID
     * @param stepId Step ID
     * @returns Index in the array of steps for defined step ID.
     */
    getStepIndex(stepId?: string): number;
    /**
     * @description Sends command to submit all forms inside the step.
     * @param stepId Step ID for which forms need's to be submitted.
     * @param skipIfUntouched Skip validation if form haven't been touched.
     * @returns {Observable<WizardStepSubmittedForms>} Observable, which will emit
     * when all visible forms in step are submitted.
     */
    submitStepForms(stepId?: string, skipIfUntouched?: boolean): Observable<WizardStepSubmittedForms | null>;
    /**
     * @description Triggers validation for current step forms.
     * @returns {Observable<boolean>} Observable with validation result.
     */
    validateStepForms(skipIfUntouched?: boolean): Observable<boolean>;
    /**
     * @description Sets new array of visible steps for Wizard.
     * @param steps steps that needs to be shown in Wizard.
     */
    setVisibleSteps(steps: PreparedWizardGeneratorItem[]): void;
    /**
     * @returns {Observable<PreparedWizardGeneratorItem[]>} Observable, which will emit every time
     * when array of visible step items has been changed.
     */
    getVisibleSteps(): Observable<PreparedWizardGeneratorItem[]>;
    /**
     * @description Runs initial transformation for passed Wizard steps.
     * @param items All Wizard steps.
     * @returns {Promise<PreparedWizardGeneratorItem[]>} Array of transformed Wizard steps.
     */
    prepareWizardItems(items: WizardGeneratorItem[]): Promise<PreparedWizardGeneratorItem[]>;
    /**
     * @description Runs conditional function for each step to define if step should be shown.
     */
    refreshStepVisibility(): Promise<void>;
    /**
     * Checks whether current form group should be visible.
     * @param formGroup Form Group.
     */
    refreshFormVisibility(formGroup: WizardGeneratorFormGroup): Promise<boolean>;
    /**
     * @description Clears array of Wizard steps components.
     */
    clearWizardStepComponents(): void;
    /**
     * @returns {Observable<StepsComponents>} Observable, which will emit every time
     * when Set visible steps components has been changed.
     */
    trackStepsComponents(): Observable<StepsComponents>;
    /**
     * @description Adds new component to the set of Wizard steps components.
     * @param component Wizard Step component to add.
     * @param key Wizard Step ID.
     */
    addWizardStepComponent(component: WizardGeneratorStep, key: string): void;
    /**
     * @description Removes Component from the set of Wizard steps components.
     * @param key
     */
    removeWizardStepComponent(key: string): void;
    /**
     * @param step Step ID which includes dependency fields.
     * @returns Object with forms and their fields that are dependencies for other steps.
     */
    getStepDependencyFields(step: string): DependencySteps;
    /**
     * Notifies step components to revalidate inner forms.
     * @param stepIds Step ID's which needs to be revalidated.
     */
    notifyStepsToRevalidateForms(stepIds: string[]): void;
    /**
     * Notifies step components to refresh form groups visibility.
     * @param stepIds Step ID's which needs to be checked.
     */
    refreshFormsVisibility(stepIds: string[]): Promise<void>;
    /**
     * @param formatted Flag defining whether form value should be formatted by form control
     * transformers, or return raw value
     * @returns {WizardGeneratorFormsValue} Wizard form value
     */
    getWizardFormValue(formatted?: boolean): Promise<WizardGeneratorFormsValue>;
    /**
     *
     * @param stepId Step ID to check.
     * @returns {Boolean} indicator if step was previously validated and submitted.
     */
    stepSubmitted(stepId: string): boolean;
    /**
     *
     * @returns {Boolean} if steps are untouched, will return true, if yes - false
     */
    isStepsUntouched(): boolean;
    /**
     * Reverts wizard to defined step.
     * @param stepId Step ID to edit
     */
    editStep(stepId: string): void;
    /**
     * Set's current set of step ID's
     * @param ids step ID's
     */
    setWizardStepIds(ids: string[]): void;
    /**
     * Set next step index.
     * @param index Next step index.
     */
    setNextStepIndex(index: number): void;
    /**
     * Returns observable of next step index which emits when index is changed.
     */
    trackNextStepIndex(): Observable<number>;
    /**
     * Returns observable which emits when steps order has been changed.
     */
    trackStepsOrder(): Observable<number>;
    /**
     * Stores original 'appendToWizard' input property.
     * @param value
     */
    setOriginalAppendToWizardState(value: boolean): void;
    /**
     * Resets modified 'appendToWizard' property to original one.
     */
    restoreAppendToWizardState(): void;
    /**
     * Returns observable which emits when 'appendToWizard' property has been changed.
     */
    trackAppendToWizardState(): Observable<boolean>;
    /**
     * @hidden
     * @param items
     * @param index
     * @param key
     * @returns
     */
    private _getFormItemPropertyValue;
    /**
     * @hidden
     * @description Returns value from Promise-like, Observable-like, simple function or just some object.
     * @param obj
     * @returns
     */
    private _getFunctionValue;
    /**
     * @hidden
     * @description Sets `branching` property for the steps that will create new branches.
     * @param items
     * @returns
     */
    private _setBranchingSteps;
    /** @hidden */
    private _normalizeDependencyStep;
    /** @hidden */
    private _buildDependencyMap;
    /** @hidden */
    private _getCompletedStepIds;
    static ɵfac: i0.ɵɵFactoryDeclaration<WizardGeneratorService, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<WizardGeneratorService>;
}
