import * as i0 from '@angular/core';
import { EventEmitter, signal, input, inject, ChangeDetectorRef, NgZone, DestroyRef, isDevMode, ElementRef, ViewChild, Output, Input, Directive, ViewChildren, Component, ChangeDetectionStrategy, ViewEncapsulation, booleanAttribute, viewChild, contentChild, TemplateRef, contentChildren, model, output, viewChildren, computed, HostBinding, Optional, NgModule } from '@angular/core';
import { take, debounceTime, distinctUntilChanged, first } from 'rxjs/operators';
import { SPACE, ENTER, RIGHT_ARROW, DOWN_ARROW, LEFT_ARROW, UP_ARROW, BACKSPACE, DELETE } from '@angular/cdk/keycodes';
import { takeUntilDestroyed } from '@angular/core/rxjs-interop';
import * as i1$1 from '@fundamental-ngx/cdk/utils';
import { KeyUtil, OverflowListDirective, AsyncOrSyncPipe, OverflowListItemDirective, scrollTop } from '@fundamental-ngx/cdk/utils';
import { FD_DYNAMIC_PAGE } from '@fundamental-ngx/core/dynamic-page';
import { NgClass, SlicePipe, NgTemplateOutlet } from '@angular/common';
import { ButtonComponent } from '@fundamental-ngx/core/button';
import { IconComponent, FD_DEFAULT_ICON_FONT_FAMILY } from '@fundamental-ngx/core/icon';
import { PopoverComponent, PopoverControlComponent, PopoverBodyComponent } from '@fundamental-ngx/core/popover';
import { cloneDeep } from 'lodash-es';
import * as i1 from '@angular/cdk/drag-drop';
import { ReplaySubject, Subject, fromEvent } from 'rxjs';
import { ScrollSpyDirective } from '@fundamental-ngx/core/scroll-spy';
import { ScrollbarDirective } from '@fundamental-ngx/core/scrollbar';
import { FD_TABLIST } from '@fundamental-ngx/core/shared';

const UNIQUE_KEY_SEPARATOR = '.';
const FLIPPER_SIZE = {
    width: 20,
    height: 44,
    verticalHeight: 20
};
const ICON_TAB_HIDDEN_CLASS_NAME = 'fd-icon-tab-bar__item--hidden';

class IconTabBarBase {
    constructor() {
        /**
         * @description Emits when some tab is selected.
         */
        this.selected = new EventEmitter();
        /** Emits when selected tab Unique ID changes. */
        this.selectedUidChange = new EventEmitter();
        /** @hidden */
        this._extraTabs$ = signal([]);
        /** @hidden */
        this._extraItems$ = signal(false);
        /** @hidden Heading level of the tab. */
        this.tabHeadingLevel = input(null);
        /**
         * @description Associations for colors of the tabs.
         * If any of the color associations provided, they'll be read by screenreader instead of the actual color
         */
        this.colorAssociations = input();
        /** @hidden */
        this._inDynamicPage = !!inject(FD_DYNAMIC_PAGE, { optional: true });
        /** @hidden */
        this._cd = inject(ChangeDetectorRef);
        /** @Hidden */
        this._ngZone = inject(NgZone);
        /** @hidden */
        this._tabs = [];
        /** @hidden */
        this._destroyRef = inject(DestroyRef);
        /** @hidden */
        this._destroyed = false;
    }
    /**
     * @description A tab bar configuration that stores the state of each tab. Based on this configuration, a tab bar is representing.
     */
    set tabs(value) {
        this._tabs = value;
    }
    /** @hidden */
    get tabs() {
        return this._tabs;
    }
    /**
     * @description densityMode setter triggers tabs to re-calculation overflowed tabs
     */
    set densityMode(value) {
        // Skip first value && value doesn't equal to previous one
        if (this._densityMode && value !== this._densityMode) {
            this._triggerRecalculationVisibleItems();
        }
        this._densityMode = value;
    }
    /** @hidden */
    ngOnChanges(changes) {
        if (changes.tabs && !changes.tabs.firstChange) {
            this._initTabs();
            this._triggerRecalculationVisibleItems();
            return;
        }
        if (changes.isRtl && !changes.isRtl.firstChange) {
            this._triggerRecalculationVisibleItems();
        }
        if (changes.selectedUid && !changes.selectedUid.firstChange) {
            this._handleSelectedUidChange();
        }
    }
    /** @hidden */
    ngOnInit() {
        this._initTabs();
    }
    /** @hidden */
    ngAfterViewInit() {
        this._triggerRecalculationVisibleItems();
    }
    /** @hidden */
    ngOnDestroy() {
        this._destroyed = true;
    }
    /**
     * @hidden
     * @param selectedItem
     * @param event
     */
    _selectItem(selectedItem, event) {
        event?.stopPropagation();
        this.selectedUid = selectedItem.uId;
        this.selectedUidChange.emit(this.selectedUid);
        selectedItem.badge = false;
        this.selected.emit(selectedItem);
    }
    /** @hidden */
    _keyDownHandler(event, tab, currentIndex, popover) {
        if (tab && KeyUtil.isKeyCode(event, [SPACE, ENTER])) {
            event.preventDefault();
            this._selectItem(tab);
        }
        else if (KeyUtil.isKeyCode(event, [RIGHT_ARROW, DOWN_ARROW])) {
            event.preventDefault();
            if (KeyUtil.isKeyCode(event, DOWN_ARROW) && tab?.subItems?.length) {
                popover?._openPopover();
                return;
            }
            this.isRtl ? this._focusPreviousItem(currentIndex) : this._focusNextItem(currentIndex);
        }
        else if (KeyUtil.isKeyCode(event, [LEFT_ARROW, UP_ARROW])) {
            event.preventDefault();
            this.isRtl ? this._focusNextItem(currentIndex) : this._focusPreviousItem(currentIndex);
        }
    }
    /** @hidden focuses next tab */
    _focusNextItem(currentIndex) {
        const nextIndex = currentIndex === this.tabs.length - 1 ? 0 : currentIndex + 1;
        this._focusItem(nextIndex);
    }
    /** @hidden focuses previous tab */
    _focusPreviousItem(currentIndex) {
        if (currentIndex === 0) {
            // if current focused element is first, focus the last one. If tabs are overflowed, focus the last tab in the popover
            this._focusItem(this.tabs.length - 1, true);
        }
        else {
            this._focusItem(currentIndex - 1);
        }
    }
    /**
     * @hidden
     * @param tabIndex index of the tab to apply focus
     * @param focusLast whether to focus first or last item in the popover
     */
    _focusItem(tabIndex, focusLast = false) {
        if (tabIndex > this._lastVisibleTabIndex) {
            this._tabBarPopover._openPopover(focusLast);
        }
        else {
            const itemToFocus = this._getTabUIElementFocusable(this._tabUIElements.get(tabIndex));
            itemToFocus?.focus();
        }
    }
    /**
     * @hidden
     * @param selectedItem
     * @description select extra item inside popover
     */
    _selectExtraItem(selectedItem) {
        if (selectedItem.color) {
            selectedItem.cssClasses = [`fd-icon-tab-bar__item--${selectedItem.color}`];
        }
        this._selectItem(selectedItem);
        this._triggerRecalculationVisibleItems();
    }
    /**
     * @hidden
     * @param extraItems
     * @description recalculate _nextSteps and _prevSteps array if we have extra items
     */
    _recalculateVisibleItems(extraItems) {
        this._extraItems$.set(extraItems > 0);
        const tabs = [...this._tabs];
        const extraTabs = [];
        const selected = tabs.find((tab) => tab.uId === this.selectedUid);
        this._lastVisibleTabIndex = this._getLastVisibleTabIndex(tabs.length, extraItems, selected);
        this._resetAndHideExtraTabs(tabs, extraTabs);
        this._extraTabs$.set(extraTabs);
        this._cd.detectChanges();
    }
    /**
     * @hidden
     * Mapping function to resolve UI tab element to it's focusable part/descendant
     */
    _getTabUIElementFocusable(tabUIElement) {
        if (tabUIElement && typeof tabUIElement === 'object' && 'nativeElement' in tabUIElement) {
            if (tabUIElement.nativeElement instanceof HTMLElement) {
                return tabUIElement.nativeElement;
            }
        }
        if (isDevMode()) {
            console.warn('Failed to get focusable tab element');
        }
        return null;
    }
    /**
     * @hidden
     * @description trigger recalculation items, need to do it asynchronously after dom was rerendered
     */
    _triggerRecalculationVisibleItems() {
        this._ngZone.onMicrotaskEmpty.pipe(take(1), takeUntilDestroyed(this._destroyRef)).subscribe(() => {
            if (this.overflowDirective && !this._destroyed) {
                const extra = this.overflowDirective.getAmountOfExtraItems();
                this._recalculateVisibleItems(extra);
                this._cd.detectChanges();
            }
        });
    }
    /**
     * @hidden
     * @description initialize state of tabs
     */
    _initTabs() {
        this._lastVisibleTabIndex = this.tabs.length - 1;
        if (this.selectedUid) {
            return;
        }
        const selectedItem = this._findActiveTabId(this.tabs) || this.tabs[0];
        this.selectedUid = selectedItem?.uId;
        if (!selectedItem) {
            return;
        }
        this.selectedUidChange.emit(this.selectedUid);
        this.selected.emit(selectedItem);
    }
    /**
     * @hidden
     * Resets and hides extra tabs.
     * @param tabs - The list of tab items.
     * @param extraTabs - The list of extra tab items.
     */
    _resetAndHideExtraTabs(tabs, extraTabs) {
        tabs.forEach((item) => this._resetTab(item));
        this._hideExtraTabs(tabs, extraTabs);
    }
    /**
     * @hidden
     * Handles changes to the selected UID.
     */
    _handleSelectedUidChange() {
        const isExtraTab = this._extraTabs$().find((tab) => tab.uId === this.selectedUid);
        const selected = this.tabs.find((tab) => tab.uId === this.selectedUid);
        if (isExtraTab) {
            this._selectExtraItem(isExtraTab);
        }
        else if (selected) {
            this._selectExtraItem(selected);
        }
    }
    /**
     * @hidden
     * Calculates the index of the last visible tab item.
     * @param totalTabs - The total number of tabs.
     * @param extraItems - The number of extra items.
     * @param selected - The currently selected tab item (optional).
     * @returns The index of the last visible tab item.
     */
    _getLastVisibleTabIndex(totalTabs, extraItems, selected) {
        if (selected && selected.index >= totalTabs - extraItems) {
            return totalTabs - extraItems - 2;
        }
        else {
            return totalTabs - extraItems - 1;
        }
    }
    /**
     * @hidden
     * Resets the visibility and CSS classes of a tab item.
     * @param tab - The tab item to reset.
     */
    _resetTab(tab) {
        tab.hidden = false;
        tab.cssClasses = tab.cssClasses.filter((cssClass) => cssClass !== ICON_TAB_HIDDEN_CLASS_NAME);
    }
    /**
     * @hidden
     * Hides extra tabs and updates the extraTabs array with the hidden tabs.
     * @param tabs - The array of all tab items.
     * @param extraTabs - The array to store the hidden tab items.
     */
    _hideExtraTabs(tabs, extraTabs) {
        for (let i = this._lastVisibleTabIndex + 1; i < tabs.length; i++) {
            const tab = tabs[i];
            if (tab && tab?.uId !== this.selectedUid) {
                extraTabs.push({ ...tab });
                tab.hidden = true;
                tab.cssClasses.push(ICON_TAB_HIDDEN_CLASS_NAME);
            }
        }
    }
    /**
     * @hidden
     * Recursively finds the active tab item in a list of tabs.
     * @param tabs - The array of tab items.
     * @returns The active tab item, or undefined if no active tab is found.
     */
    _findActiveTabId(tabs) {
        let activeTab;
        for (const tab of tabs) {
            if (tab.active) {
                activeTab = tab;
                break;
            }
            else if (tab.subItems) {
                activeTab = this._findActiveTabId(tab.subItems);
                if (activeTab) {
                    break;
                }
            }
        }
        return activeTab;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarBase, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.0.7", type: IconTabBarBase, isStandalone: true, inputs: { tabs: { classPropertyName: "tabs", publicName: "tabs", isSignal: false, isRequired: false, transformFunction: null }, isRtl: { classPropertyName: "isRtl", publicName: "isRtl", isSignal: false, isRequired: false, transformFunction: null }, densityMode: { classPropertyName: "densityMode", publicName: "densityMode", isSignal: false, isRequired: false, transformFunction: null }, selectedUid: { classPropertyName: "selectedUid", publicName: "selectedUid", isSignal: false, isRequired: false, transformFunction: null }, tabHeadingLevel: { classPropertyName: "tabHeadingLevel", publicName: "tabHeadingLevel", isSignal: true, isRequired: false, transformFunction: null }, colorAssociations: { classPropertyName: "colorAssociations", publicName: "colorAssociations", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { selected: "selected", selectedUidChange: "selectedUidChange" }, viewQueries: [{ propertyName: "overflowDirective", first: true, predicate: OverflowListDirective, descendants: true }, { propertyName: "headerElement", first: true, predicate: ["tablist"], descendants: true, read: ElementRef }], usesOnChanges: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarBase, decorators: [{
            type: Directive
        }], propDecorators: { tabs: [{
                type: Input
            }], isRtl: [{
                type: Input
            }], densityMode: [{
                type: Input
            }], selectedUid: [{
                type: Input
            }], selected: [{
                type: Output
            }], selectedUidChange: [{
                type: Output
            }], overflowDirective: [{
                type: ViewChild,
                args: [OverflowListDirective]
            }], headerElement: [{
                type: ViewChild,
                args: ['tablist', { read: ElementRef }]
            }] } });

class ClosableIconTabBar extends IconTabBarBase {
    constructor() {
        super(...arguments);
        /**
         * Event emits when user clicks on x button in tab.
         */
        this.closeTab = new EventEmitter();
    }
    /** @hidden */
    _closeTab(itemIndex, checkPopover = false) {
        const popoverPresent = !!this._tabBarPopover;
        const tab = this._getTabInfoFromMainList(itemIndex);
        if (!tab?.parent) {
            return;
        }
        tab.parent.splice(tab.parent.findIndex((item) => item.uId === itemIndex), 1);
        this.tabs = this._updateTabs(this.tabs);
        this._triggerRecalculationVisibleItems();
        this.closeTab.emit(tab.tab);
        if (!checkPopover) {
            return;
        }
        // Delay to ensure popover is recalculated.
        setTimeout(() => {
            // Popover has been removed, shift focus to the last item of the tab bar.
            if (popoverPresent && !this._tabBarPopover) {
                this._focusItem(this._lastVisibleTabIndex);
            }
        });
    }
    /** @hidden */
    _keyDownHandler(event, tab, currentIndex, popover) {
        if (tab?.closable && KeyUtil.isKeyCode(event, [BACKSPACE, DELETE])) {
            event.preventDefault();
            this._closeTab(tab.uId);
            // Need to detect changes to trigger query list refresh.
            this._cd.detectChanges();
            // If last tab was closed, focus new last item, otherwhise set focus to the item that took place of removed one.
            const newFocusIndex = this.tabs.length - 1 < currentIndex ? this.tabs.length - 1 : currentIndex;
            this._focusItem(newFocusIndex);
            return;
        }
        super._keyDownHandler(event, tab, currentIndex, popover);
    }
    /**
     * @hidden
     * @param arr
     * @param parentUid
     * @description Update indexes, uIds, styles.
     */
    _updateTabs(arr, parentUid, flatIndexRef = { value: 0 }) {
        return arr.map((item, index) => {
            item.index = index;
            item.uId = parentUid ? `${parentUid}${UNIQUE_KEY_SEPARATOR}${index}` : `${index}`;
            item.flatIndex = flatIndexRef.value++;
            item.parentUId = parentUid;
            if (!parentUid && item.color) {
                item.cssClasses = [`fd-icon-tab-bar__item--${item.color}`];
            }
            if (Array.isArray(item.subItems)) {
                item.subItems = this._updateTabs(item.subItems, item.uId, flatIndexRef);
            }
            return { ...item };
        });
    }
    /**
     * @hidden
     * @param uid
     * @param arr
     * @returns {parent, tab}
     * @description Get tab reference inside main tab array and  reference to parent array.
     */
    _getTabInfoFromMainList(uid, arr = this.tabs) {
        let result;
        for (let i = 0; i < arr.length; i++) {
            const item = arr[i];
            if (item.uId === uid) {
                result = { parent: arr, tab: item };
                break;
            }
            else if (Array.isArray(item.subItems)) {
                result = this._getTabInfoFromMainList(uid, item.subItems);
                if (result) {
                    break;
                }
            }
        }
        return result;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: ClosableIconTabBar, deps: null, target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: ClosableIconTabBar, isStandalone: true, outputs: { closeTab: "closeTab" }, usesInheritance: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: ClosableIconTabBar, decorators: [{
            type: Directive
        }], propDecorators: { closeTab: [{
                type: Output
            }] } });

class IconTabBarPopoverBase {
    /** @hidden */
    constructor(_cd) {
        this._cd = _cd;
        /**
         * @description Should we show separator between subItems
         */
        this.isSeparators = false;
        /**
         * @description Emits when some tab is selected.
         */
        this.selectedExtraItem = new EventEmitter();
        /** @description Emits when popover is closed and focus should be applied to first visible tab */
        this.focusFirstVisibleItem = new EventEmitter();
        /** @description Emits when popover is closed and focus should be applied to last visible tab */
        this.focusLastVisibleItem = new EventEmitter();
        /** Event emits when user clicks on x button in tab. */
        this.closeTab = new EventEmitter();
        /**
         * @description Associations for colors of the tabs.
         * If any of the color associations provided, they'll be read by screenreader instead of the actual color
         */
        this.colorAssociations = input();
        /**
         * @hidden
         * @description state of popover
         */
        this._isOpen = false;
    }
    /** @hidden */
    ngOnChanges(changes) {
        if (changes.extraTabs) {
            this._setStyles(this.extraTabs);
        }
    }
    /**
     * @hidden
     * @param focusLast whether to focus first or last item in the popover
     */
    _openPopover(focusLast = false) {
        if (!this._isOpen) {
            this.popover.open();
        }
        if (focusLast) {
            this._tabExtraUIElements.get(this._tabExtraUIElements.length - 1)?.nativeElement.focus();
        }
    }
    /** @hidden */
    _focusPreviousPopoverItem(currentIndex) {
        if (currentIndex === 0) {
            this.popover.close();
            this.focusLastVisibleItem.emit();
        }
        else {
            this._focusPopoverItem(currentIndex - 1);
        }
    }
    /** @hidden */
    _focusNextPopoverItem(currentIndex) {
        if (currentIndex === this._tabExtraUIElements.length - 1) {
            this.popover.close();
            this.focusFirstVisibleItem.emit();
        }
        else {
            this._focusPopoverItem(currentIndex + 1);
        }
    }
    /** @hidden */
    _focusPopoverItem(index) {
        const ordered = this._getOrderedTabExtraUIElements();
        ordered[index]?.focus();
    }
    /**
     * @hidden
     * @param selectedItem
     */
    _selectItem(selectedItem) {
        this.selectedExtraItem.emit(selectedItem);
        this.popover.close();
    }
    /** @hidden */
    _keyDownHandler(event, tab, currentIndex) {
        if (tab?.closable && KeyUtil.isKeyCode(event, [BACKSPACE, DELETE])) {
            event.preventDefault();
            this.closeTab.emit(tab.uId);
            this._focusPopoverItem(currentIndex);
        }
        else if (KeyUtil.isKeyCode(event, [SPACE, ENTER])) {
            event.preventDefault();
            this._selectItem(tab);
        }
        else if (KeyUtil.isKeyCode(event, [RIGHT_ARROW, DOWN_ARROW])) {
            event.preventDefault();
            this.isRtl ? this._focusPreviousPopoverItem(currentIndex) : this._focusNextPopoverItem(currentIndex);
        }
        else if (KeyUtil.isKeyCode(event, [LEFT_ARROW, UP_ARROW])) {
            event.preventDefault();
            this.isRtl ? this._focusNextPopoverItem(currentIndex) : this._focusPreviousPopoverItem(currentIndex);
        }
    }
    /**
     * @hidden
     * @description Generate styles for subItems
     */
    _setStyles(items) {
        items.forEach((item) => {
            if (item.color) {
                item.cssClasses = [`fd-icon-tab-bar__list-item--${item.color}`];
            }
            if (Array.isArray(item.subItems)) {
                this._setStyles(item.subItems);
            }
        });
    }
    /** @hidden */
    _getOrderedTabExtraUIElements() {
        return this._tabExtraUIElements
            .toArray()
            .map((e) => ({
            flatIndex: parseInt(e.nativeElement.getAttribute('data-flatIndex'), 10),
            nativeElement: e.nativeElement
        }))
            .sort(({ flatIndex: aFlatIndex }, { flatIndex: bFlatIndex }) => aFlatIndex - bFlatIndex)
            .map(({ nativeElement }) => nativeElement);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarPopoverBase, deps: [{ token: i0.ChangeDetectorRef }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.0.7", type: IconTabBarPopoverBase, isStandalone: true, inputs: { isRtl: { classPropertyName: "isRtl", publicName: "isRtl", isSignal: false, isRequired: false, transformFunction: null }, extraTabs: { classPropertyName: "extraTabs", publicName: "extraTabs", isSignal: false, isRequired: false, transformFunction: null }, isSeparators: { classPropertyName: "isSeparators", publicName: "isSeparators", isSignal: false, isRequired: false, transformFunction: null }, colorAssociations: { classPropertyName: "colorAssociations", publicName: "colorAssociations", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { selectedExtraItem: "selectedExtraItem", focusFirstVisibleItem: "focusFirstVisibleItem", focusLastVisibleItem: "focusLastVisibleItem", closeTab: "closeTab" }, viewQueries: [{ propertyName: "popover", first: true, predicate: ["popover"], descendants: true }], usesOnChanges: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarPopoverBase, decorators: [{
            type: Directive
        }], ctorParameters: () => [{ type: i0.ChangeDetectorRef }], propDecorators: { isRtl: [{
                type: Input
            }], popover: [{
                type: ViewChild,
                args: ['popover']
            }], extraTabs: [{
                type: Input
            }], isSeparators: [{
                type: Input
            }], selectedExtraItem: [{
                type: Output
            }], focusFirstVisibleItem: [{
                type: Output
            }], focusLastVisibleItem: [{
                type: Output
            }], closeTab: [{
                type: Output
            }] } });

class IconTabBarPopoverComponent extends IconTabBarPopoverBase {
    constructor() {
        super(...arguments);
        /**
         * @description Flag representing position inside container left/right
         */
        this.leftSide = false;
        /**
         * @description Label for button
         */
        this.label = 'more';
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarPopoverComponent, deps: null, target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.0.7", type: IconTabBarPopoverComponent, isStandalone: true, selector: "fdp-icon-tab-bar-popover", inputs: { leftSide: "leftSide", label: "label", showItemLabel: "showItemLabel" }, viewQueries: [{ propertyName: "_tabExtraUIElements", predicate: ["tabItem"], descendants: true }], usesInheritance: true, ngImport: i0, template: "<fd-popover placement=\"bottom-end\" [focusAutoCapture]=\"true\" [focusTrapped]=\"true\" #popover>\n    <fd-popover-control>\n        <button class=\"fd-icon-tab-bar__overflow\" tabindex=\"-1\">\n            <span class=\"fd-icon-tab-bar__overflow-text\">{{ label }}</span>\n            <fd-icon glyph=\"slim-arrow-down\" role=\"presentation\"></fd-icon>\n        </button>\n    </fd-popover-control>\n    <fd-popover-body class=\"fd-icon-tab-bar\">\n        <ul class=\"fd-list fd-list--navigation fd-list--no-border fd-icon-tab-bar__list\" role=\"list\">\n            @for (item of extraTabs; track item.uId; let idx = $index) {\n                @if (isSeparators) {\n                    <li tabindex=\"-1\" class=\"fd-icon-tab-bar__icon-separator\">\n                        <fd-icon glyph=\"process\"></fd-icon>\n                    </li>\n                }\n                <li\n                    class=\"fd-list__item fd-list__item--link fd-icon-tab-bar__list-item\"\n                    role=\"listitem\"\n                    [class.fd-icon-tab-bar__list-item--closable]=\"item.closable\"\n                    [ngClass]=\"item.cssClasses\"\n                    tabindex=\"-1\"\n                >\n                    <a\n                        class=\"fd-list__link fd-icon-tab-bar__list-link\"\n                        #tabItem\n                        [attr.data-flatIndex]=\"item.flatIndex\"\n                        tabindex=\"0\"\n                        (click)=\"_selectItem(item)\"\n                        (keydown)=\"_keyDownHandler($event, item, idx)\"\n                    >\n                        <span class=\"fd-icon-tab-bar__list-item-icon-container\">\n                            <span class=\"fd-list__icon fd-icon-tab-bar__list-item-icon\">\n                                <fd-icon [glyph]=\"item.icon | fdkAsyncOrSync\"></fd-icon>\n                            </span>\n                        </span>\n                        @if (showItemLabel && (item.label | fdkAsyncOrSync)) {\n                            <span class=\"fd-list__title fd-icon-tab-bar__list-item-title\">{{\n                                item.label | fdkAsyncOrSync\n                            }}</span>\n                        }\n                        @if (item.badge) {\n                            <span class=\"fd-icon-tab-bar__badge\"></span>\n                        }\n                        @if (item.counter | fdkAsyncOrSync; as itemCounter) {\n                            <span class=\"fd-list__counter fd-icon-tab-bar__list-item-counter\"\n                                >({{ itemCounter }})\n                            </span>\n                        }\n                        @if (item.color | fdkAsyncOrSync; as itemColor) {\n                            <div class=\"screenreader-only\">\n                                {{ colorAssociations()?.[itemColor] || itemColor }}\n                            </div>\n                        }\n                    </a>\n                    @if (item.closable) {\n                        <div class=\"fd-icon-tab-bar__button-container\">\n                            <button\n                                fd-button\n                                tabindex=\"-1\"\n                                class=\"fd-icon-tab-bar__button\"\n                                fdType=\"transparent\"\n                                (click)=\"closeTab.emit(item.uId)\"\n                            >\n                                <fd-icon glyph=\"decline\"></fd-icon>\n                            </button>\n                        </div>\n                    }\n                </li>\n            }\n        </ul>\n    </fd-popover-body>\n</fd-popover>\n", dependencies: [{ kind: "component", type: PopoverComponent, selector: "fd-popover", inputs: ["title", "trigger", "fixedPosition", "id", "mobile", "mobileConfig", "preventSpaceKeyScroll"] }, { kind: "component", type: PopoverControlComponent, selector: "fd-popover-control, [fdPopoverControl]" }, { kind: "component", type: PopoverBodyComponent, selector: "fd-popover-body", inputs: ["minWidth", "maxWidth", "minHeight", "maxHeight"] }, { kind: "component", type: IconComponent, selector: "fd-icon", inputs: ["glyph", "font", "color", "background", "class", "ariaLabel", "ariaHidden"] }, { kind: "directive", type: NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "component", type: ButtonComponent, selector: "button[fd-button], a[fd-button], span[fd-button]", inputs: ["class"], exportAs: ["fd-button"] }, { kind: "pipe", type: AsyncOrSyncPipe, name: "fdkAsyncOrSync" }] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarPopoverComponent, decorators: [{
            type: Component,
            args: [{ selector: 'fdp-icon-tab-bar-popover', imports: [
                        PopoverComponent,
                        PopoverControlComponent,
                        PopoverBodyComponent,
                        IconComponent,
                        NgClass,
                        ButtonComponent,
                        AsyncOrSyncPipe
                    ], template: "<fd-popover placement=\"bottom-end\" [focusAutoCapture]=\"true\" [focusTrapped]=\"true\" #popover>\n    <fd-popover-control>\n        <button class=\"fd-icon-tab-bar__overflow\" tabindex=\"-1\">\n            <span class=\"fd-icon-tab-bar__overflow-text\">{{ label }}</span>\n            <fd-icon glyph=\"slim-arrow-down\" role=\"presentation\"></fd-icon>\n        </button>\n    </fd-popover-control>\n    <fd-popover-body class=\"fd-icon-tab-bar\">\n        <ul class=\"fd-list fd-list--navigation fd-list--no-border fd-icon-tab-bar__list\" role=\"list\">\n            @for (item of extraTabs; track item.uId; let idx = $index) {\n                @if (isSeparators) {\n                    <li tabindex=\"-1\" class=\"fd-icon-tab-bar__icon-separator\">\n                        <fd-icon glyph=\"process\"></fd-icon>\n                    </li>\n                }\n                <li\n                    class=\"fd-list__item fd-list__item--link fd-icon-tab-bar__list-item\"\n                    role=\"listitem\"\n                    [class.fd-icon-tab-bar__list-item--closable]=\"item.closable\"\n                    [ngClass]=\"item.cssClasses\"\n                    tabindex=\"-1\"\n                >\n                    <a\n                        class=\"fd-list__link fd-icon-tab-bar__list-link\"\n                        #tabItem\n                        [attr.data-flatIndex]=\"item.flatIndex\"\n                        tabindex=\"0\"\n                        (click)=\"_selectItem(item)\"\n                        (keydown)=\"_keyDownHandler($event, item, idx)\"\n                    >\n                        <span class=\"fd-icon-tab-bar__list-item-icon-container\">\n                            <span class=\"fd-list__icon fd-icon-tab-bar__list-item-icon\">\n                                <fd-icon [glyph]=\"item.icon | fdkAsyncOrSync\"></fd-icon>\n                            </span>\n                        </span>\n                        @if (showItemLabel && (item.label | fdkAsyncOrSync)) {\n                            <span class=\"fd-list__title fd-icon-tab-bar__list-item-title\">{{\n                                item.label | fdkAsyncOrSync\n                            }}</span>\n                        }\n                        @if (item.badge) {\n                            <span class=\"fd-icon-tab-bar__badge\"></span>\n                        }\n                        @if (item.counter | fdkAsyncOrSync; as itemCounter) {\n                            <span class=\"fd-list__counter fd-icon-tab-bar__list-item-counter\"\n                                >({{ itemCounter }})\n                            </span>\n                        }\n                        @if (item.color | fdkAsyncOrSync; as itemColor) {\n                            <div class=\"screenreader-only\">\n                                {{ colorAssociations()?.[itemColor] || itemColor }}\n                            </div>\n                        }\n                    </a>\n                    @if (item.closable) {\n                        <div class=\"fd-icon-tab-bar__button-container\">\n                            <button\n                                fd-button\n                                tabindex=\"-1\"\n                                class=\"fd-icon-tab-bar__button\"\n                                fdType=\"transparent\"\n                                (click)=\"closeTab.emit(item.uId)\"\n                            >\n                                <fd-icon glyph=\"decline\"></fd-icon>\n                            </button>\n                        </div>\n                    }\n                </li>\n            }\n        </ul>\n    </fd-popover-body>\n</fd-popover>\n" }]
        }], propDecorators: { _tabExtraUIElements: [{
                type: ViewChildren,
                args: ['tabItem']
            }], leftSide: [{
                type: Input
            }], label: [{
                type: Input
            }], showItemLabel: [{
                type: Input
            }] } });

class IconTabBarFilterTypeComponent extends ClosableIconTabBar {
    constructor() {
        super(...arguments);
        /**
         * @description Boolean flag indicating to show total tab
         */
        this.showTotalTab = true;
    }
    /**
     * @hidden
     * @description initialize state of tabs
     */
    _initTabs() {
        super._initTabs();
        this._totalTab = this.showTotalTab ? this.tabs[0] : undefined;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarFilterTypeComponent, deps: null, target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.0.7", type: IconTabBarFilterTypeComponent, isStandalone: true, selector: "fdp-icon-tab-bar-filter-type", inputs: { showTotalTab: "showTotalTab" }, providers: [
            {
                provide: IconTabBarBase,
                useExisting: IconTabBarFilterTypeComponent
            }
        ], viewQueries: [{ propertyName: "_tabBarPopover", first: true, predicate: IconTabBarPopoverComponent, descendants: true }, { propertyName: "_tabUIElements", predicate: ["tabItem"], descendants: true }], usesInheritance: true, ngImport: i0, template: "<ul\n    class=\"fd-icon-tab-bar__header\"\n    [class.fd-dynamic-page__tabs]=\"_inDynamicPage\"\n    role=\"tablist\"\n    fdkOverflowList\n    [isRtl]=\"isRtl\"\n    #tablist\n    [overflowOffset]=\"100\"\n    (overflowChanged)=\"_recalculateVisibleItems($event)\"\n>\n    @if (showTotalTab) {\n        <li class=\"fd-icon-tab-bar__item\" role=\"presentation\" fdkOverflowListItem>\n            <a\n                (click)=\"_totalTab && _selectItem(_totalTab)\"\n                (keydown)=\"_keyDownHandler($event, _totalTab, 0)\"\n                [attr.aria-selected]=\"_totalTab?.uId === selectedUid\"\n                class=\"fd-icon-tab-bar__tab\"\n                tabindex=\"0\"\n                #tabItem\n                role=\"tab\"\n                [attr.aria-level]=\"tabHeadingLevel()\"\n            >\n                <div class=\"fd-icon-tab-bar__container fd-icon-tab-bar__container--filter\">\n                    @if (_totalTab?.counter) {\n                        <span class=\"fd-icon-tab-bar__filter-counter\">{{ _totalTab?.counter }}</span>\n                    }\n                    <span class=\"fd-icon-tab-bar__filter-label\">{{ _totalTab?.label | fdkAsyncOrSync }}</span>\n                </div>\n            </a>\n        </li>\n    }\n    @for (item of tabs | slice: 1; track item.uId; let idx = $index) {\n        <li\n            [class]=\"item.cssClasses\"\n            class=\"fd-icon-tab-bar__item\"\n            role=\"presentation\"\n            fdkOverflowListItem\n            [attr.aria-hidden]=\"item.hidden\"\n        >\n            <a\n                (click)=\"_selectItem(item)\"\n                [attr.aria-selected]=\"item.uId === selectedUid\"\n                class=\"fd-icon-tab-bar__tab\"\n                #tabItem\n                [attr.tabindex]=\"showTotalTab ? -1 : idx > 0 ? -1 : 0\"\n                (keydown)=\"_keyDownHandler($event, item, idx + (showTotalTab ? 1 : 0))\"\n                role=\"tab\"\n                [attr.aria-level]=\"tabHeadingLevel()\"\n            >\n                @if (item?.titleTemplate) {\n                    <ng-container [ngTemplateOutlet]=\"item.titleTemplate || null\"></ng-container>\n                } @else {\n                    <div class=\"fd-icon-tab-bar__container\">\n                        <span class=\"fd-icon-tab-bar__icon\">\n                            <fd-icon [glyph]=\"item.icon\"></fd-icon>\n                        </span>\n                        @if (item.counter) {\n                            <span class=\"fd-icon-tab-bar__counter\">{{ item.counter }}</span>\n                        }\n                    </div>\n                    <div class=\"fd-icon-tab-bar__label\">{{ item.label | fdkAsyncOrSync }}</div>\n                }\n                @if (item.color | fdkAsyncOrSync; as itemColor) {\n                    <div class=\"screenreader-only\">\n                        {{ colorAssociations()?.[itemColor] || itemColor }}\n                    </div>\n                }\n            </a>\n            @if (item.closable) {\n                <div class=\"fd-icon-tab-bar__button-container\">\n                    <button\n                        fd-button\n                        tabindex=\"-1\"\n                        class=\"fd-icon-tab-bar__button\"\n                        fdType=\"transparent\"\n                        (click)=\"_closeTab(item.uId)\"\n                    >\n                        <fd-icon glyph=\"decline\"></fd-icon>\n                    </button>\n                </div>\n            }\n        </li>\n    }\n    @if (_extraItems$()) {\n        <li role=\"presentation\" class=\"fd-icon-tab-bar__item fd-icon-tab-bar__item--overflow\">\n            <fdp-icon-tab-bar-popover\n                [isRtl]=\"isRtl\"\n                [extraTabs]=\"_extraTabs$()\"\n                [colorAssociations]=\"colorAssociations()\"\n                (selectedExtraItem)=\"_selectExtraItem($event)\"\n                (focusFirstVisibleItem)=\"_focusItem(0)\"\n                (focusLastVisibleItem)=\"_focusItem(_lastVisibleTabIndex)\"\n                (closeTab)=\"_closeTab($event, true)\"\n            ></fdp-icon-tab-bar-popover>\n        </li>\n    }\n</ul>\n", dependencies: [{ kind: "directive", type: OverflowListDirective, selector: "[fdkOverflowList]", inputs: ["overflowOffset", "isRtl", "itemCssBlockValue"], outputs: ["overflowChanged"] }, { kind: "directive", type: OverflowListItemDirective, selector: "[fdkOverflowListItem]" }, { kind: "component", type: IconComponent, selector: "fd-icon", inputs: ["glyph", "font", "color", "background", "class", "ariaLabel", "ariaHidden"] }, { kind: "component", type: IconTabBarPopoverComponent, selector: "fdp-icon-tab-bar-popover", inputs: ["leftSide", "label", "showItemLabel"] }, { kind: "pipe", type: SlicePipe, name: "slice" }, { kind: "component", type: ButtonComponent, selector: "button[fd-button], a[fd-button], span[fd-button]", inputs: ["class"], exportAs: ["fd-button"] }, { kind: "pipe", type: AsyncOrSyncPipe, name: "fdkAsyncOrSync" }, { kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarFilterTypeComponent, decorators: [{
            type: Component,
            args: [{ selector: 'fdp-icon-tab-bar-filter-type', providers: [
                        {
                            provide: IconTabBarBase,
                            useExisting: IconTabBarFilterTypeComponent
                        }
                    ], imports: [
                        OverflowListDirective,
                        OverflowListItemDirective,
                        IconComponent,
                        IconTabBarPopoverComponent,
                        SlicePipe,
                        ButtonComponent,
                        AsyncOrSyncPipe,
                        NgTemplateOutlet
                    ], template: "<ul\n    class=\"fd-icon-tab-bar__header\"\n    [class.fd-dynamic-page__tabs]=\"_inDynamicPage\"\n    role=\"tablist\"\n    fdkOverflowList\n    [isRtl]=\"isRtl\"\n    #tablist\n    [overflowOffset]=\"100\"\n    (overflowChanged)=\"_recalculateVisibleItems($event)\"\n>\n    @if (showTotalTab) {\n        <li class=\"fd-icon-tab-bar__item\" role=\"presentation\" fdkOverflowListItem>\n            <a\n                (click)=\"_totalTab && _selectItem(_totalTab)\"\n                (keydown)=\"_keyDownHandler($event, _totalTab, 0)\"\n                [attr.aria-selected]=\"_totalTab?.uId === selectedUid\"\n                class=\"fd-icon-tab-bar__tab\"\n                tabindex=\"0\"\n                #tabItem\n                role=\"tab\"\n                [attr.aria-level]=\"tabHeadingLevel()\"\n            >\n                <div class=\"fd-icon-tab-bar__container fd-icon-tab-bar__container--filter\">\n                    @if (_totalTab?.counter) {\n                        <span class=\"fd-icon-tab-bar__filter-counter\">{{ _totalTab?.counter }}</span>\n                    }\n                    <span class=\"fd-icon-tab-bar__filter-label\">{{ _totalTab?.label | fdkAsyncOrSync }}</span>\n                </div>\n            </a>\n        </li>\n    }\n    @for (item of tabs | slice: 1; track item.uId; let idx = $index) {\n        <li\n            [class]=\"item.cssClasses\"\n            class=\"fd-icon-tab-bar__item\"\n            role=\"presentation\"\n            fdkOverflowListItem\n            [attr.aria-hidden]=\"item.hidden\"\n        >\n            <a\n                (click)=\"_selectItem(item)\"\n                [attr.aria-selected]=\"item.uId === selectedUid\"\n                class=\"fd-icon-tab-bar__tab\"\n                #tabItem\n                [attr.tabindex]=\"showTotalTab ? -1 : idx > 0 ? -1 : 0\"\n                (keydown)=\"_keyDownHandler($event, item, idx + (showTotalTab ? 1 : 0))\"\n                role=\"tab\"\n                [attr.aria-level]=\"tabHeadingLevel()\"\n            >\n                @if (item?.titleTemplate) {\n                    <ng-container [ngTemplateOutlet]=\"item.titleTemplate || null\"></ng-container>\n                } @else {\n                    <div class=\"fd-icon-tab-bar__container\">\n                        <span class=\"fd-icon-tab-bar__icon\">\n                            <fd-icon [glyph]=\"item.icon\"></fd-icon>\n                        </span>\n                        @if (item.counter) {\n                            <span class=\"fd-icon-tab-bar__counter\">{{ item.counter }}</span>\n                        }\n                    </div>\n                    <div class=\"fd-icon-tab-bar__label\">{{ item.label | fdkAsyncOrSync }}</div>\n                }\n                @if (item.color | fdkAsyncOrSync; as itemColor) {\n                    <div class=\"screenreader-only\">\n                        {{ colorAssociations()?.[itemColor] || itemColor }}\n                    </div>\n                }\n            </a>\n            @if (item.closable) {\n                <div class=\"fd-icon-tab-bar__button-container\">\n                    <button\n                        fd-button\n                        tabindex=\"-1\"\n                        class=\"fd-icon-tab-bar__button\"\n                        fdType=\"transparent\"\n                        (click)=\"_closeTab(item.uId)\"\n                    >\n                        <fd-icon glyph=\"decline\"></fd-icon>\n                    </button>\n                </div>\n            }\n        </li>\n    }\n    @if (_extraItems$()) {\n        <li role=\"presentation\" class=\"fd-icon-tab-bar__item fd-icon-tab-bar__item--overflow\">\n            <fdp-icon-tab-bar-popover\n                [isRtl]=\"isRtl\"\n                [extraTabs]=\"_extraTabs$()\"\n                [colorAssociations]=\"colorAssociations()\"\n                (selectedExtraItem)=\"_selectExtraItem($event)\"\n                (focusFirstVisibleItem)=\"_focusItem(0)\"\n                (focusLastVisibleItem)=\"_focusItem(_lastVisibleTabIndex)\"\n                (closeTab)=\"_closeTab($event, true)\"\n            ></fdp-icon-tab-bar-popover>\n        </li>\n    }\n</ul>\n" }]
        }], propDecorators: { _tabUIElements: [{
                type: ViewChildren,
                args: ['tabItem']
            }], _tabBarPopover: [{
                type: ViewChild,
                args: [IconTabBarPopoverComponent]
            }], showTotalTab: [{
                type: Input
            }] } });

class IconTabBarIconTypeComponent extends ClosableIconTabBar {
    constructor() {
        super(...arguments);
        /** @hidden */
        this._defaultFontFamily = FD_DEFAULT_ICON_FONT_FAMILY;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarIconTypeComponent, deps: null, target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.0.7", type: IconTabBarIconTypeComponent, isStandalone: true, selector: "fdp-icon-tab-bar-icon-type", inputs: { showLabel: "showLabel" }, providers: [
            {
                provide: IconTabBarBase,
                useExisting: IconTabBarIconTypeComponent
            }
        ], viewQueries: [{ propertyName: "_tabBarPopover", first: true, predicate: IconTabBarPopoverComponent, descendants: true }, { propertyName: "_tabUIElements", predicate: ["tabItem"], descendants: true }], usesInheritance: true, ngImport: i0, template: "<ul\n    class=\"fd-icon-tab-bar__header\"\n    [class.fd-dynamic-page__tabs]=\"_inDynamicPage\"\n    role=\"tablist\"\n    fdkOverflowList\n    [isRtl]=\"isRtl\"\n    #tablist\n    [overflowOffset]=\"100\"\n    (overflowChanged)=\"_recalculateVisibleItems($event)\"\n>\n    @for (item of tabs; track item.uId; let idx = $index) {\n        <li\n            [ngClass]=\"item.cssClasses\"\n            [class.fd-icon-tab-bar__item--closable]=\"item.closable\"\n            fdkOverflowListItem\n            [attr.aria-hidden]=\"item.hidden\"\n            class=\"fd-icon-tab-bar__item\"\n            role=\"presentation\"\n        >\n            <a\n                class=\"fd-icon-tab-bar__tab\"\n                #tabItem\n                role=\"tab\"\n                [attr.tabindex]=\"idx === 0 ? 0 : -1\"\n                [attr.aria-selected]=\"item.uId === selectedUid\"\n                (click)=\"_selectItem(item)\"\n                (keydown)=\"_keyDownHandler($event, item, idx)\"\n                [attr.aria-level]=\"tabHeadingLevel()\"\n            >\n                @if (item?.titleTemplate) {\n                    <ng-template [ngTemplateOutlet]=\"item.titleTemplate || null\"></ng-template>\n                } @else {\n                    <div class=\"fd-icon-tab-bar__container\">\n                        <span class=\"fd-icon-tab-bar__icon\">\n                            <fd-icon [glyph]=\"item.icon\" [font]=\"item.iconFont || _defaultFontFamily\"></fd-icon>\n                        </span>\n                        @if (item.counter && (!item.label || !showLabel)) {\n                            <span class=\"fd-icon-tab-bar__counter\">{{ item.counter }}</span>\n                        }\n                    </div>\n                    @if (showLabel && item.label) {\n                        <div class=\"fd-icon-tab-bar__details\">\n                            <span class=\"fd-icon-tab-bar__counter\">{{ item.counter }}</span>\n                            <span class=\"fd-icon-tab-bar__label\">{{ item.label }}</span>\n                        </div>\n                    }\n                    @if (item.color | fdkAsyncOrSync; as itemColor) {\n                        <div class=\"screenreader-only\">\n                            {{ colorAssociations()?.[itemColor] || itemColor }}\n                        </div>\n                    }\n                }\n            </a>\n            @if (item.closable) {\n                <div class=\"fd-icon-tab-bar__button-container\">\n                    <button\n                        fd-button\n                        tabindex=\"-1\"\n                        class=\"fd-icon-tab-bar__button\"\n                        fdType=\"transparent\"\n                        (click)=\"_closeTab(item.uId)\"\n                    >\n                        <fd-icon glyph=\"decline\"></fd-icon>\n                    </button>\n                </div>\n            }\n        </li>\n    }\n    @if (_extraItems$()) {\n        <li role=\"presentation\" class=\"fd-icon-tab-bar__item fd-icon-tab-bar__item--overflow\">\n            <fdp-icon-tab-bar-popover\n                [showItemLabel]=\"showLabel\"\n                [isRtl]=\"isRtl\"\n                [extraTabs]=\"_extraTabs$()\"\n                [colorAssociations]=\"colorAssociations()\"\n                (selectedExtraItem)=\"_selectExtraItem($event)\"\n                (focusFirstVisibleItem)=\"_focusItem(0)\"\n                (focusLastVisibleItem)=\"_focusItem(_lastVisibleTabIndex)\"\n                (closeTab)=\"_closeTab($event, true)\"\n            ></fdp-icon-tab-bar-popover>\n        </li>\n    }\n</ul>\n", dependencies: [{ kind: "directive", type: OverflowListDirective, selector: "[fdkOverflowList]", inputs: ["overflowOffset", "isRtl", "itemCssBlockValue"], outputs: ["overflowChanged"] }, { kind: "directive", type: OverflowListItemDirective, selector: "[fdkOverflowListItem]" }, { kind: "directive", type: NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "component", type: IconComponent, selector: "fd-icon", inputs: ["glyph", "font", "color", "background", "class", "ariaLabel", "ariaHidden"] }, { kind: "component", type: ButtonComponent, selector: "button[fd-button], a[fd-button], span[fd-button]", inputs: ["class"], exportAs: ["fd-button"] }, { kind: "component", type: IconTabBarPopoverComponent, selector: "fdp-icon-tab-bar-popover", inputs: ["leftSide", "label", "showItemLabel"] }, { kind: "pipe", type: AsyncOrSyncPipe, name: "fdkAsyncOrSync" }, { kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarIconTypeComponent, decorators: [{
            type: Component,
            args: [{ selector: 'fdp-icon-tab-bar-icon-type', providers: [
                        {
                            provide: IconTabBarBase,
                            useExisting: IconTabBarIconTypeComponent
                        }
                    ], imports: [
                        OverflowListDirective,
                        OverflowListItemDirective,
                        NgClass,
                        IconComponent,
                        ButtonComponent,
                        IconTabBarPopoverComponent,
                        AsyncOrSyncPipe,
                        NgTemplateOutlet
                    ], template: "<ul\n    class=\"fd-icon-tab-bar__header\"\n    [class.fd-dynamic-page__tabs]=\"_inDynamicPage\"\n    role=\"tablist\"\n    fdkOverflowList\n    [isRtl]=\"isRtl\"\n    #tablist\n    [overflowOffset]=\"100\"\n    (overflowChanged)=\"_recalculateVisibleItems($event)\"\n>\n    @for (item of tabs; track item.uId; let idx = $index) {\n        <li\n            [ngClass]=\"item.cssClasses\"\n            [class.fd-icon-tab-bar__item--closable]=\"item.closable\"\n            fdkOverflowListItem\n            [attr.aria-hidden]=\"item.hidden\"\n            class=\"fd-icon-tab-bar__item\"\n            role=\"presentation\"\n        >\n            <a\n                class=\"fd-icon-tab-bar__tab\"\n                #tabItem\n                role=\"tab\"\n                [attr.tabindex]=\"idx === 0 ? 0 : -1\"\n                [attr.aria-selected]=\"item.uId === selectedUid\"\n                (click)=\"_selectItem(item)\"\n                (keydown)=\"_keyDownHandler($event, item, idx)\"\n                [attr.aria-level]=\"tabHeadingLevel()\"\n            >\n                @if (item?.titleTemplate) {\n                    <ng-template [ngTemplateOutlet]=\"item.titleTemplate || null\"></ng-template>\n                } @else {\n                    <div class=\"fd-icon-tab-bar__container\">\n                        <span class=\"fd-icon-tab-bar__icon\">\n                            <fd-icon [glyph]=\"item.icon\" [font]=\"item.iconFont || _defaultFontFamily\"></fd-icon>\n                        </span>\n                        @if (item.counter && (!item.label || !showLabel)) {\n                            <span class=\"fd-icon-tab-bar__counter\">{{ item.counter }}</span>\n                        }\n                    </div>\n                    @if (showLabel && item.label) {\n                        <div class=\"fd-icon-tab-bar__details\">\n                            <span class=\"fd-icon-tab-bar__counter\">{{ item.counter }}</span>\n                            <span class=\"fd-icon-tab-bar__label\">{{ item.label }}</span>\n                        </div>\n                    }\n                    @if (item.color | fdkAsyncOrSync; as itemColor) {\n                        <div class=\"screenreader-only\">\n                            {{ colorAssociations()?.[itemColor] || itemColor }}\n                        </div>\n                    }\n                }\n            </a>\n            @if (item.closable) {\n                <div class=\"fd-icon-tab-bar__button-container\">\n                    <button\n                        fd-button\n                        tabindex=\"-1\"\n                        class=\"fd-icon-tab-bar__button\"\n                        fdType=\"transparent\"\n                        (click)=\"_closeTab(item.uId)\"\n                    >\n                        <fd-icon glyph=\"decline\"></fd-icon>\n                    </button>\n                </div>\n            }\n        </li>\n    }\n    @if (_extraItems$()) {\n        <li role=\"presentation\" class=\"fd-icon-tab-bar__item fd-icon-tab-bar__item--overflow\">\n            <fdp-icon-tab-bar-popover\n                [showItemLabel]=\"showLabel\"\n                [isRtl]=\"isRtl\"\n                [extraTabs]=\"_extraTabs$()\"\n                [colorAssociations]=\"colorAssociations()\"\n                (selectedExtraItem)=\"_selectExtraItem($event)\"\n                (focusFirstVisibleItem)=\"_focusItem(0)\"\n                (focusLastVisibleItem)=\"_focusItem(_lastVisibleTabIndex)\"\n                (closeTab)=\"_closeTab($event, true)\"\n            ></fdp-icon-tab-bar-popover>\n        </li>\n    }\n</ul>\n" }]
        }], propDecorators: { _tabUIElements: [{
                type: ViewChildren,
                args: ['tabItem']
            }], _tabBarPopover: [{
                type: ViewChild,
                args: [IconTabBarPopoverComponent]
            }], showLabel: [{
                type: Input
            }] } });

class IconTabBarProcessTypeComponent extends ClosableIconTabBar {
    constructor() {
        super(...arguments);
        /** @hidden */
        this._offsetOverflowDirective = 30;
        /**
         * @hidden
         * An array containing extra tabs for the next steps
         */
        this._nextSteps = [];
        /**
         * @hidden
         * An array containing extra tabs for the next steps
         */
        this._prevSteps = [];
        /** @hidden */
        this._showLeftBtn = false;
        /** @hidden */
        this._showRightBtn = false;
        /** @hidden */
        this._firstVisibleTabIndex = 0;
        /** @hidden */
        this._currentStepIndex = 0;
    }
    /**
     * @hidden
     * @param selectedItem
     */
    _selectItem(selectedItem) {
        this._currentStepIndex = selectedItem.index;
        super._selectItem(selectedItem);
    }
    /**
     * @hidden
     * @param selectedItem
     * @description select extra item inside popover
     */
    async _selectExtraItem(selectedItem) {
        this._currentStepIndex = selectedItem.index;
        let amountOfPreviousSteps;
        let amountOfNextSteps;
        // We have to cases (strategy) for overflow extra tabs.
        // 1) When user click on step inside previous steps popover
        // 2) When user click on step inside next steps popover
        let isPreviousStepsStrategy;
        if (this._currentStepIndex > this._lastVisibleTabIndex) {
            amountOfNextSteps = this._nextSteps.length - (this._currentStepIndex - this._lastVisibleTabIndex);
            this._showRightBtn = amountOfNextSteps > 0;
            this._showLeftBtn = true;
            isPreviousStepsStrategy = false;
        }
        else {
            amountOfPreviousSteps = this._prevSteps.length - (this._firstVisibleTabIndex - this._currentStepIndex);
            this._showRightBtn = true;
            isPreviousStepsStrategy = true;
            this._showLeftBtn = amountOfPreviousSteps > 0;
        }
        this._selectItem(selectedItem);
        this._ngZone.onMicrotaskEmpty.pipe(take(1)).subscribe(() => {
            if (this.overflowDirective) {
                const extra = this.overflowDirective.getAmountOfExtraItems();
                isPreviousStepsStrategy
                    ? this.recalculateItemsByPrevArr(extra, amountOfPreviousSteps)
                    : this._recalculateItemsByNextArr(extra, amountOfNextSteps);
                this._cd.detectChanges();
            }
        });
    }
    /**
     * @hidden
     * @param extraItems
     * @description recalculate _nextSteps and _prevSteps array if we have extra items
     */
    _recalculateVisibleItems(extraItems) {
        const amountOfPrevSteps = this._currentStepIndex > extraItems ? extraItems : this._currentStepIndex;
        this.recalculateItemsByPrevArr(extraItems, amountOfPrevSteps);
    }
    /** @hidden */
    _clearExtraList() {
        this._nextSteps = [];
        this._prevSteps = [];
        this.tabs.forEach((item) => {
            item.hidden = false;
            item.cssClasses = item.cssClasses.filter((cssClass) => cssClass !== ICON_TAB_HIDDEN_CLASS_NAME);
        });
    }
    /**
     * @hidden
     * @param extraItems
     * @param amountOfNextSteps
     * @description We fill the array of the next steps to the selected tab,
     * based on this we generate the array of the previous steps if it is necessary
     */
    _recalculateItemsByNextArr(extraItems, amountOfNextSteps) {
        this._clearExtraList();
        if (!extraItems) {
            return;
        }
        const tabs = [...this.tabs];
        const visibleAmountOfItems = tabs.length - extraItems;
        for (let i = tabs.length - amountOfNextSteps; i < tabs.length; i++) {
            this._nextSteps.push(cloneDeep(tabs[i]));
            tabs[i].hidden = true;
            tabs[i].cssClasses.push(ICON_TAB_HIDDEN_CLASS_NAME);
        }
        this._showRightBtn = !!this._nextSteps.length;
        this._firstVisibleTabIndex = tabs.length - visibleAmountOfItems - this._nextSteps.length;
        this._lastVisibleTabIndex = tabs.length - amountOfNextSteps - 1;
        if (this._nextSteps.length === extraItems) {
            return;
        }
        let amountOfPrevSteps = extraItems - this._nextSteps.length;
        let nextIndex = this._firstVisibleTabIndex - 1;
        while (amountOfPrevSteps > 0) {
            this._prevSteps.push(cloneDeep(tabs[nextIndex]));
            tabs[nextIndex].hidden = true;
            tabs[nextIndex].cssClasses.push(ICON_TAB_HIDDEN_CLASS_NAME);
            --nextIndex;
            --amountOfPrevSteps;
        }
        // Added +1 for left button
        this._anchorIndexForExtraBtnDirective = this._prevSteps.length
            ? this._prevSteps.length + visibleAmountOfItems
            : this._prevSteps.length + visibleAmountOfItems - 1;
        this._showLeftBtn = !!this._prevSteps.length;
        this._cd.detectChanges();
    }
    /**
     * @hidden
     * @param extraItems
     * @param amountOfPreviousSteps
     * @description We fill the array of the previous steps to the selected tab,
     * based on this we generate the array of the next steps if it is necessary
     */
    recalculateItemsByPrevArr(extraItems, amountOfPreviousSteps) {
        this._clearExtraList();
        if (!extraItems) {
            return;
        }
        const tabs = [...this.tabs];
        const visibleAmountOfItems = tabs.length - extraItems;
        for (let i = amountOfPreviousSteps - 1; i >= 0; i--) {
            this._prevSteps.push(cloneDeep(tabs[i]));
            tabs[i].hidden = true;
            tabs[i].cssClasses.push(ICON_TAB_HIDDEN_CLASS_NAME);
        }
        this._firstVisibleTabIndex = this._prevSteps.length;
        this._lastVisibleTabIndex = this._prevSteps.length + visibleAmountOfItems - 1;
        this._showLeftBtn = !!this._prevSteps.length;
        if (this._prevSteps.length === extraItems) {
            this._showRightBtn = false;
            return;
        }
        let amountOfNextSteps = extraItems - this._prevSteps.length;
        let nextIndex = this._prevSteps.length ? this._prevSteps.length + visibleAmountOfItems : visibleAmountOfItems;
        while (amountOfNextSteps > 0) {
            this._nextSteps.push(cloneDeep(tabs[nextIndex]));
            tabs[nextIndex].hidden = true;
            tabs[nextIndex].cssClasses.push(ICON_TAB_HIDDEN_CLASS_NAME);
            ++nextIndex;
            --amountOfNextSteps;
        }
        // Add +1 for left btn
        this._anchorIndexForExtraBtnDirective = this._prevSteps.length
            ? this._prevSteps.length + visibleAmountOfItems
            : this._prevSteps.length + visibleAmountOfItems - 1;
        this._showRightBtn = !!this._nextSteps.length;
        this._offsetOverflowDirective = this._nextSteps.length ? 70 : 0;
        this._cd.detectChanges();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarProcessTypeComponent, deps: null, target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.0.7", type: IconTabBarProcessTypeComponent, isStandalone: true, selector: "fdp-icon-tab-bar-process-type", providers: [
            {
                provide: IconTabBarBase,
                useExisting: IconTabBarProcessTypeComponent
            }
        ], viewQueries: [{ propertyName: "_tabBarPopover", first: true, predicate: IconTabBarPopoverComponent, descendants: true }, { propertyName: "_tabUIElements", predicate: ["tabItem"], descendants: true }], usesInheritance: true, ngImport: i0, template: "<ul\n    class=\"fd-icon-tab-bar__header\"\n    [class.fd-dynamic-page__tabs]=\"_inDynamicPage\"\n    role=\"tablist\"\n    #tablist\n    [class.fd-icon-tab-bar__header--left-offset]=\"_showLeftBtn\"\n    fdkOverflowList\n    [isRtl]=\"isRtl\"\n    [overflowOffset]=\"_offsetOverflowDirective\"\n    (overflowChanged)=\"_recalculateVisibleItems($event)\"\n>\n    @if (_showLeftBtn) {\n        <li\n            role=\"presentation\"\n            class=\"fd-icon-tab-bar__item fd-icon-tab-bar__item--overflow fd-icon-tab-bar__item--overflow-left\"\n        >\n            <fdp-icon-tab-bar-popover\n                [extraTabs]=\"_prevSteps\"\n                [label]=\"_prevSteps.length.toString()\"\n                [isSeparators]=\"true\"\n                [colorAssociations]=\"colorAssociations()\"\n                (selectedExtraItem)=\"_selectExtraItem($event)\"\n            ></fdp-icon-tab-bar-popover>\n        </li>\n    }\n    @for (item of tabs; track item.uId; let idx = $index) {\n        <li\n            [class]=\"item.cssClasses\"\n            class=\"fd-icon-tab-bar__item\"\n            role=\"presentation\"\n            fdkOverflowListItem\n            [attr.aria-hidden]=\"item.hidden\"\n        >\n            <a\n                (click)=\"_selectItem(item)\"\n                (keydown)=\"_keyDownHandler($event, item, idx)\"\n                [attr.aria-selected]=\"item.uId === selectedUid\"\n                class=\"fd-icon-tab-bar__tab\"\n                #tabItem\n                [attr.tabindex]=\"idx === 0 ? 0 : -1\"\n                role=\"tab\"\n                [attr.aria-level]=\"tabHeadingLevel()\"\n            >\n                @if (item?.titleTemplate) {\n                    <ng-container [ngTemplateOutlet]=\"item.titleTemplate || null\"></ng-container>\n                } @else {\n                    @if (item.icon) {\n                        <div class=\"fd-icon-tab-bar__container\">\n                            <span class=\"fd-icon-tab-bar__icon\">\n                                <fd-icon [glyph]=\"item.icon\"></fd-icon>\n                            </span>\n                            @if (item.badge) {\n                                <span class=\"fd-icon-tab-bar__badge\"></span>\n                            }\n                        </div>\n                        <div class=\"fd-icon-tab-bar__details\">\n                            <span class=\"fd-icon-tab-bar__counter\">{{ item.counter }}</span>\n                            <span class=\"fd-icon-tab-bar__label\">{{ item.label | fdkAsyncOrSync }}</span>\n                        </div>\n                    } @else {\n                        <span class=\"fd-icon-tab-bar__tag\">{{ item.label | fdkAsyncOrSync }}</span>\n                        @if (item.badge) {\n                            <span class=\"fd-icon-tab-bar__badge\"></span>\n                        }\n                    }\n                    @if (item.color | fdkAsyncOrSync; as itemColor) {\n                        <div class=\"screenreader-only\">\n                            {{ colorAssociations()?.[itemColor] || itemColor }}\n                        </div>\n                    }\n                }\n            </a>\n            @if (item.closable) {\n                <div class=\"fd-icon-tab-bar__button-container\">\n                    <button\n                        fd-button\n                        tabindex=\"-1\"\n                        class=\"fd-icon-tab-bar__button\"\n                        fdType=\"transparent\"\n                        (click)=\"_closeTab(item.uId)\"\n                    >\n                        <fd-icon glyph=\"decline\"></fd-icon>\n                    </button>\n                </div>\n            }\n            @if (_lastVisibleTabIndex !== idx) {\n                <span class=\"fd-icon-tab-bar__separator\">\n                    <fd-icon glyph=\"process\"></fd-icon>\n                </span>\n            }\n        </li>\n    }\n    @if (_showRightBtn) {\n        <li role=\"presentation\" class=\"fd-icon-tab-bar__item fd-icon-tab-bar__item--overflow\">\n            <fdp-icon-tab-bar-popover\n                [isRtl]=\"isRtl\"\n                [extraTabs]=\"_nextSteps\"\n                [isSeparators]=\"true\"\n                [label]=\"_nextSteps.length.toString()\"\n                [colorAssociations]=\"colorAssociations()\"\n                (selectedExtraItem)=\"_selectExtraItem($event)\"\n                (focusFirstVisibleItem)=\"_focusItem(0)\"\n                (focusLastVisibleItem)=\"_focusItem(_lastVisibleTabIndex)\"\n                (closeTab)=\"_closeTab($event, true)\"\n            >\n            </fdp-icon-tab-bar-popover>\n        </li>\n    }\n</ul>\n", dependencies: [{ kind: "directive", type: OverflowListDirective, selector: "[fdkOverflowList]", inputs: ["overflowOffset", "isRtl", "itemCssBlockValue"], outputs: ["overflowChanged"] }, { kind: "component", type: IconTabBarPopoverComponent, selector: "fdp-icon-tab-bar-popover", inputs: ["leftSide", "label", "showItemLabel"] }, { kind: "directive", type: OverflowListItemDirective, selector: "[fdkOverflowListItem]" }, { kind: "component", type: IconComponent, selector: "fd-icon", inputs: ["glyph", "font", "color", "background", "class", "ariaLabel", "ariaHidden"] }, { kind: "component", type: ButtonComponent, selector: "button[fd-button], a[fd-button], span[fd-button]", inputs: ["class"], exportAs: ["fd-button"] }, { kind: "pipe", type: AsyncOrSyncPipe, name: "fdkAsyncOrSync" }, { kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarProcessTypeComponent, decorators: [{
            type: Component,
            args: [{ selector: 'fdp-icon-tab-bar-process-type', providers: [
                        {
                            provide: IconTabBarBase,
                            useExisting: IconTabBarProcessTypeComponent
                        }
                    ], imports: [
                        OverflowListDirective,
                        IconTabBarPopoverComponent,
                        OverflowListItemDirective,
                        IconComponent,
                        ButtonComponent,
                        AsyncOrSyncPipe,
                        NgTemplateOutlet
                    ], template: "<ul\n    class=\"fd-icon-tab-bar__header\"\n    [class.fd-dynamic-page__tabs]=\"_inDynamicPage\"\n    role=\"tablist\"\n    #tablist\n    [class.fd-icon-tab-bar__header--left-offset]=\"_showLeftBtn\"\n    fdkOverflowList\n    [isRtl]=\"isRtl\"\n    [overflowOffset]=\"_offsetOverflowDirective\"\n    (overflowChanged)=\"_recalculateVisibleItems($event)\"\n>\n    @if (_showLeftBtn) {\n        <li\n            role=\"presentation\"\n            class=\"fd-icon-tab-bar__item fd-icon-tab-bar__item--overflow fd-icon-tab-bar__item--overflow-left\"\n        >\n            <fdp-icon-tab-bar-popover\n                [extraTabs]=\"_prevSteps\"\n                [label]=\"_prevSteps.length.toString()\"\n                [isSeparators]=\"true\"\n                [colorAssociations]=\"colorAssociations()\"\n                (selectedExtraItem)=\"_selectExtraItem($event)\"\n            ></fdp-icon-tab-bar-popover>\n        </li>\n    }\n    @for (item of tabs; track item.uId; let idx = $index) {\n        <li\n            [class]=\"item.cssClasses\"\n            class=\"fd-icon-tab-bar__item\"\n            role=\"presentation\"\n            fdkOverflowListItem\n            [attr.aria-hidden]=\"item.hidden\"\n        >\n            <a\n                (click)=\"_selectItem(item)\"\n                (keydown)=\"_keyDownHandler($event, item, idx)\"\n                [attr.aria-selected]=\"item.uId === selectedUid\"\n                class=\"fd-icon-tab-bar__tab\"\n                #tabItem\n                [attr.tabindex]=\"idx === 0 ? 0 : -1\"\n                role=\"tab\"\n                [attr.aria-level]=\"tabHeadingLevel()\"\n            >\n                @if (item?.titleTemplate) {\n                    <ng-container [ngTemplateOutlet]=\"item.titleTemplate || null\"></ng-container>\n                } @else {\n                    @if (item.icon) {\n                        <div class=\"fd-icon-tab-bar__container\">\n                            <span class=\"fd-icon-tab-bar__icon\">\n                                <fd-icon [glyph]=\"item.icon\"></fd-icon>\n                            </span>\n                            @if (item.badge) {\n                                <span class=\"fd-icon-tab-bar__badge\"></span>\n                            }\n                        </div>\n                        <div class=\"fd-icon-tab-bar__details\">\n                            <span class=\"fd-icon-tab-bar__counter\">{{ item.counter }}</span>\n                            <span class=\"fd-icon-tab-bar__label\">{{ item.label | fdkAsyncOrSync }}</span>\n                        </div>\n                    } @else {\n                        <span class=\"fd-icon-tab-bar__tag\">{{ item.label | fdkAsyncOrSync }}</span>\n                        @if (item.badge) {\n                            <span class=\"fd-icon-tab-bar__badge\"></span>\n                        }\n                    }\n                    @if (item.color | fdkAsyncOrSync; as itemColor) {\n                        <div class=\"screenreader-only\">\n                            {{ colorAssociations()?.[itemColor] || itemColor }}\n                        </div>\n                    }\n                }\n            </a>\n            @if (item.closable) {\n                <div class=\"fd-icon-tab-bar__button-container\">\n                    <button\n                        fd-button\n                        tabindex=\"-1\"\n                        class=\"fd-icon-tab-bar__button\"\n                        fdType=\"transparent\"\n                        (click)=\"_closeTab(item.uId)\"\n                    >\n                        <fd-icon glyph=\"decline\"></fd-icon>\n                    </button>\n                </div>\n            }\n            @if (_lastVisibleTabIndex !== idx) {\n                <span class=\"fd-icon-tab-bar__separator\">\n                    <fd-icon glyph=\"process\"></fd-icon>\n                </span>\n            }\n        </li>\n    }\n    @if (_showRightBtn) {\n        <li role=\"presentation\" class=\"fd-icon-tab-bar__item fd-icon-tab-bar__item--overflow\">\n            <fdp-icon-tab-bar-popover\n                [isRtl]=\"isRtl\"\n                [extraTabs]=\"_nextSteps\"\n                [isSeparators]=\"true\"\n                [label]=\"_nextSteps.length.toString()\"\n                [colorAssociations]=\"colorAssociations()\"\n                (selectedExtraItem)=\"_selectExtraItem($event)\"\n                (focusFirstVisibleItem)=\"_focusItem(0)\"\n                (focusLastVisibleItem)=\"_focusItem(_lastVisibleTabIndex)\"\n                (closeTab)=\"_closeTab($event, true)\"\n            >\n            </fdp-icon-tab-bar-popover>\n        </li>\n    }\n</ul>\n" }]
        }], propDecorators: { _tabUIElements: [{
                type: ViewChildren,
                args: ['tabItem']
            }], _tabBarPopover: [{
                type: ViewChild,
                args: [IconTabBarPopoverComponent]
            }] } });

class IconBarDndContainerDirective {
    /**
     * @description Defines if drag and drop feature should be enabled for list items
     */
    set draggable(draggable) {
        this._draggable = draggable;
        this._changeDraggableState(draggable);
    }
    /** @hidden */
    constructor(elementRef) {
        this.elementRef = elementRef;
        /**
         * @description Event that is thrown, when the item is dropped
         */
        this.dropped = new EventEmitter();
        /** @hidden */
        this._dragRefItems = [];
        /** @hidden  */
        this.dndItemDirectives = [];
        /** @hidden  */
        this._dndListDirectives = new Set();
        /** @hidden */
        this._virtualSeparatorsCoordinates = [];
        /** @hidden */
        this._closestItemIndex = null;
        /** @hidden */
        this._closestSeparatorIndex = null;
        /** @hidden */
        this._draggable = true;
        /** @hidden */
        this._destroyRef = inject(DestroyRef);
    }
    /**
     * @description Method called, when element is started to be dragged
     */
    dragStart() {
        /** Counting all of the elements's chords */
        this._elementsCoordinates = this.dndItemDirectives.map((item) => item.getElementCoordinates());
        this._generateVirtualSeparators();
    }
    /**
     * @param mousePosition
     * @description Method called, when the item is being moved by 1 px
     */
    onMove(mousePosition) {
        let newClosestIndex = null;
        let newClosestSeparatorIndex = null;
        this._elementsCoordinates.find((element, index) => {
            /** Check if element can be replaced */
            if (newClosestIndex !== index) {
                const isMouseOnElement = this._isMouseOnElement(element, mousePosition);
                if (isMouseOnElement) {
                    newClosestIndex = index;
                    return true;
                }
            }
        });
        if (newClosestIndex !== null && newClosestIndex === this._closestItemIndex) {
            return;
        }
        if (this._closestItemIndex !== newClosestIndex) {
            this.dndItemDirectives[this._closestItemIndex]?.toggleHoveredStyles();
            this.dndItemDirectives[this._closestSeparatorIndex]?.toggleSeparatorStyles();
        }
        this._closestItemIndex = newClosestIndex;
        if (newClosestIndex !== null) {
            this._closestSeparatorIndex = null;
            this.dndItemDirectives[newClosestIndex].toggleHoveredStyles(true);
            return;
        }
        // If we aren't on tab then check separator element.
        this._virtualSeparatorsCoordinates.find((element, index) => {
            const isMouseOnFlipper = this._isMouseOnFlipper(element, mousePosition);
            if (isMouseOnFlipper) {
                newClosestSeparatorIndex = index;
                return true;
            }
        });
        if (newClosestSeparatorIndex !== null && newClosestSeparatorIndex === this._closestSeparatorIndex) {
            return;
        }
        if (this._closestSeparatorIndex !== newClosestSeparatorIndex) {
            this.dndItemDirectives[this._closestItemIndex]?.toggleHoveredStyles();
            this.dndItemDirectives[this._closestSeparatorIndex]?.toggleSeparatorStyles();
        }
        this._closestSeparatorIndex = newClosestSeparatorIndex;
        if (newClosestSeparatorIndex !== null) {
            this._closestItemIndex = null;
            this.dndItemDirectives[newClosestSeparatorIndex].toggleSeparatorStyles(true);
            return;
        }
    }
    /**
     * @param dragDir
     * @description Method called, when element is released
     */
    dragEnd(dragDir) {
        if (this._closestSeparatorIndex || this._closestSeparatorIndex === 0) {
            this.dndItemDirectives[this._closestSeparatorIndex].toggleSeparatorStyles();
            this.dropped.emit({
                draggableItem: dragDir.dndItemData,
                targetItem: this.dndItemDirectives[this._closestSeparatorIndex].dndItemData,
                action: 'replace'
            });
        }
        if (this._closestItemIndex || this._closestItemIndex === 0) {
            this.dndItemDirectives[this._closestItemIndex].toggleHoveredStyles();
            this.dropped.emit({
                draggableItem: dragDir.dndItemData,
                targetItem: this.dndItemDirectives[this._closestItemIndex].dndItemData,
                action: 'insert'
            });
        }
        /** Reset */
        this._elementsCoordinates = [];
        this._virtualSeparatorsCoordinates = [];
        this._closestItemIndex = null;
        this._closestSeparatorIndex = null;
    }
    /**
     * @param dragItem
     * @description Register IconTabBarDndItem to current container
     */
    registerDragItem(dragItem) {
        this._dragRefItems.push(dragItem.dragRef);
        this.dndItemDirectives.push(dragItem);
        dragItem.moved.pipe(takeUntilDestroyed(this._destroyRef)).subscribe((position) => this.onMove(position));
        dragItem.started.pipe(takeUntilDestroyed(this._destroyRef)).subscribe(() => this.dragStart());
        dragItem.released.pipe(takeUntilDestroyed(this._destroyRef)).subscribe(() => this.dragEnd(dragItem));
    }
    /**
     * @param dragItem
     * @description Remove registered IconTabBarDndItem to current container
     */
    removeDragItem(dragItem) {
        this._dragRefItems = this._dragRefItems.filter((item) => item !== dragItem.dragRef);
        this.dndItemDirectives = this.dndItemDirectives.filter((item) => item !== dragItem);
    }
    /**
     * @param listDir
     * @description Register IconTabBarDndList to current container
     */
    registerDndList(listDir) {
        this._dndListDirectives.add(listDir);
        listDir.changeDraggableState(this._draggable);
    }
    /**
     * @param listDir
     * @description Remove registered IconTabBarDndList to current container
     */
    removeDndList(listDir) {
        this._dndListDirectives.delete(listDir);
    }
    /**
     * @description Generate virtual elements as separator between tabs
     */
    _generateVirtualSeparators() {
        this._elementsCoordinates.forEach((item, index) => {
            const isVertical = this.dndItemDirectives[index].isVertical;
            this._virtualSeparatorsCoordinates.push({
                x: isVertical ? item.x : item.x - FLIPPER_SIZE.width,
                y: isVertical ? item.y + FLIPPER_SIZE.verticalHeight : item.y,
                width: isVertical ? item.width : FLIPPER_SIZE.width,
                height: isVertical ? FLIPPER_SIZE.verticalHeight : FLIPPER_SIZE.height
            });
        });
    }
    /** @hidden */
    _changeDraggableState(draggable) {
        for (const list of this._dndListDirectives) {
            list.changeDraggableState(draggable);
        }
    }
    /** @hidden */
    _isMouseOnFlipper(element, mousePosition) {
        const startX = element.x;
        const endX = element.x + element.width;
        const startY = element.y;
        const endY = element.y + element.height;
        return _between(mousePosition.x, startX, endX) && _between(mousePosition.y, startY, endY);
    }
    /** @hidden */
    _isMouseOnElement(element, mousePosition, isVertical = false) {
        const startX = element.x;
        const endX = element.x + element.width;
        const startY = isVertical ? element.y - FLIPPER_SIZE.verticalHeight / 2 : element.y;
        const endY = element.y + element.height - FLIPPER_SIZE.verticalHeight / 2;
        return _between(mousePosition.x, startX, endX) && _between(mousePosition.y, startY, endY);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconBarDndContainerDirective, deps: [{ token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: IconBarDndContainerDirective, isStandalone: true, selector: "[fdpIconBarDndContainer], [fdp-icon-bar-dnd-container]", inputs: { draggable: "draggable" }, outputs: { dropped: "dropped" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconBarDndContainerDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[fdpIconBarDndContainer], [fdp-icon-bar-dnd-container]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: i0.ElementRef }], propDecorators: { draggable: [{
                type: Input
            }], dropped: [{
                type: Output
            }] } });
/** @hidden */
function _between(x, min, max) {
    return x >= min && x <= max;
}

class IconBarDndListDirective {
    /** @hidden */
    constructor(elementRef, _dragDrop, _dndContainer) {
        this.elementRef = elementRef;
        this._dragDrop = _dragDrop;
        this._dndContainer = _dndContainer;
        /**
         * @description Direction in which the list is oriented.
         */
        this.dndOrientation = 'vertical';
        /**
         * @description Is disabled autoscroll inside draggable list
         */
        this.dndAutoScroll = true;
        /** @hidden */
        this._destroyRef = inject(DestroyRef);
        /** @hidden */
        this._dragRefItems = [];
        /** @hidden */
        this._dndItems$ = new ReplaySubject(1);
    }
    /** @hidden */
    ngAfterViewInit() {
        this._dropListRef = this._dragDrop.createDropList(this.elementRef.nativeElement);
        this._dropListRef.sortingDisabled = true;
        this._dropListRef.autoScrollDisabled = this.dndAutoScroll;
        this._dropListRef.disabled = true;
        this._dropListRef.withOrientation(this.dndOrientation);
        this._dndContainer.registerDndList(this);
        this._dndItems$
            .pipe(debounceTime(100), takeUntilDestroyed(this._destroyRef))
            .subscribe(() => this._dropListRef.withItems(this._dragRefItems));
    }
    /** @hidden */
    ngOnDestroy() {
        this._dropListRef.dispose();
        this._dndContainer.removeDndList(this);
    }
    /**
     * @param dragItem
     * @description Register IconTabBarDndItem to current list
     */
    registerDragItem(dragItem) {
        this._dragRefItems.push(dragItem.dragRef);
        this._dndItems$.next();
    }
    /**
     * @param dragItem
     * @description Remove registered IconTabBarDndItem to current list
     */
    removeDragItem(dragItem) {
        this._dragRefItems = this._dragRefItems.filter((item) => item !== dragItem.dragRef);
        this._dndItems$.next();
    }
    /**
     * @param draggable
     */
    changeDraggableState(draggable) {
        this._dropListRef.disabled = !draggable;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconBarDndListDirective, deps: [{ token: i0.ElementRef }, { token: i1.DragDrop }, { token: IconBarDndContainerDirective }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: IconBarDndListDirective, isStandalone: true, selector: "[fdpIconBarDndList], [fdp-icon-bar-dnd-list]", inputs: { dndOrientation: "dndOrientation", dndAutoScroll: "dndAutoScroll" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconBarDndListDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[fdpIconBarDndList], [fdp-icon-bar-dnd-list]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: i0.ElementRef }, { type: i1.DragDrop }, { type: IconBarDndContainerDirective }], propDecorators: { dndOrientation: [{
                type: Input
            }], dndAutoScroll: [{
                type: Input
            }] } });

class IconBarDndItemDirective {
    /** @hidden */
    constructor(elementRef, _dragDrop, _dndListDir, _dndContainerDir) {
        this.elementRef = elementRef;
        this._dragDrop = _dragDrop;
        this._dndListDir = _dndListDir;
        this._dndContainerDir = _dndContainerDir;
        /**
         * @description Classes for draggable preview
         */
        this.previewClass = ['fd-icon-tab-bar__item--dnd-preview'];
        /**
         * @description Classes for hovered state
         */
        this.dndHoveredClass = ['fd-icon-tab-bar__item--dnd-hovered'];
        /**
         * @description Classes for separator, when draggable element between two tabs
         */
        this.dndSeparatorClass = [
            'fd-icon-tab-bar__item--dnd-separator',
            'fd-icon-tab-bar__item--dnd-separator-vertical'
        ];
        /**
         * @description Event thrown when the element is moved by 1px
         */
        this.moved = new Subject();
        /**
         * @description Event thrown when the element is released
         */
        this.released = new Subject();
        /**
         * @description Event thrown when the element is started to be dragged
         */
        this.started = new Subject();
        /** @hidden */
        this.isVertical = false;
        /** @hidden */
        this._destroyRef = inject(DestroyRef);
    }
    /** @hidden */
    ngAfterViewInit() {
        this.isVertical = this._dndListDir?.dndOrientation === 'vertical';
        this._setCDKDrag();
    }
    /** @hidden */
    ngOnDestroy() {
        this._dndListDir.removeDragItem(this);
        this._dndContainerDir.removeDragItem(this);
        this.dragRef.dispose();
    }
    /** @hidden */
    onCdkDragStart() {
        /** Adds class */
        this.started.next();
    }
    /** @hidden */
    onCdkMove(position) {
        this.moved.next(position);
    }
    /** @hidden */
    onCdkDragReleased() {
        this.released.next();
    }
    /** @hidden */
    getElementCoordinates() {
        /** Takes distance from the beginning of window page */
        const rect = this.elementRef.nativeElement.getBoundingClientRect();
        /** Vertically distance is counted by distance from top of the side + half of the element height */
        return {
            x: rect.left,
            y: rect.top,
            width: rect.width,
            height: rect.height
        };
    }
    /**
     * @description Add/delete separator classes
     */
    toggleSeparatorStyles(force = false) {
        force
            ? this.elementRef.nativeElement.classList.add(...this.dndSeparatorClass)
            : this.elementRef.nativeElement.classList.remove(...this.dndSeparatorClass);
    }
    /**
     * @description Add/delete hovered state classes
     */
    toggleHoveredStyles(force = false) {
        force
            ? this.elementRef.nativeElement.classList.add(...this.dndHoveredClass)
            : this.elementRef.nativeElement.classList.remove(...this.dndHoveredClass);
    }
    /** @hidden */
    _setCDKDrag() {
        this.dragRef = this._dragDrop.createDrag(this.elementRef);
        this.dragRef.previewClass = this.previewClass;
        this._dndListDir?.registerDragItem(this);
        this._dndContainerDir?.registerDragItem(this);
        this.dragRef.moved
            .pipe(takeUntilDestroyed(this._destroyRef))
            .subscribe((event) => this.onCdkMove(event.pointerPosition));
        this.dragRef.released.pipe(takeUntilDestroyed(this._destroyRef)).subscribe(() => this.onCdkDragReleased());
        this.dragRef.started.pipe(takeUntilDestroyed(this._destroyRef)).subscribe(() => this.onCdkDragStart());
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconBarDndItemDirective, deps: [{ token: i0.ElementRef }, { token: i1.DragDrop }, { token: IconBarDndListDirective }, { token: IconBarDndContainerDirective }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: IconBarDndItemDirective, isStandalone: true, selector: "[fdpIconBarDndItem], [fdp-icon-bar-dnd-item]", inputs: { dndItemData: "dndItemData", previewClass: "previewClass", dndHoveredClass: "dndHoveredClass", dndSeparatorClass: "dndSeparatorClass" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconBarDndItemDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[fdpIconBarDndItem], [fdp-icon-bar-dnd-item]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: i0.ElementRef }, { type: i1.DragDrop }, { type: IconBarDndListDirective }, { type: IconBarDndContainerDirective }], propDecorators: { dndItemData: [{
                type: Input
            }], previewClass: [{
                type: Input
            }], dndHoveredClass: [{
                type: Input
            }], dndSeparatorClass: [{
                type: Input
            }] } });

class IconTabBarTextTypeTabItemComponent {
    constructor() {
        /**
         * @description Associations for colors of the tabs.
         * If any of the color associations provided, they'll be read by screenreader instead of the actual color
         */
        this.colorAssociations = input();
        /** @hiddem */
        this._elementRef = inject(ElementRef);
    }
    /** @hidden */
    get nativeElement() {
        return this._elementRef.nativeElement;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarTextTypeTabItemComponent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.0.7", type: IconTabBarTextTypeTabItemComponent, isStandalone: true, selector: "[fdp-icon-tab-bar-text-type-tab-item]", inputs: { item: { classPropertyName: "item", publicName: "item", isSignal: false, isRequired: false, transformFunction: null }, layoutMode: { classPropertyName: "layoutMode", publicName: "layoutMode", isSignal: false, isRequired: false, transformFunction: null }, colorAssociations: { classPropertyName: "colorAssociations", publicName: "colorAssociations", isSignal: true, isRequired: false, transformFunction: null } }, host: { attributes: { "role": "tab" }, classAttribute: "fd-icon-tab-bar__tab" }, ngImport: i0, template: "@if (item?.titleTemplate) {\n    <ng-template [ngTemplateOutlet]=\"item?.titleTemplate || null\"></ng-template>\n} @else {\n    @if (layoutMode === 'row') {\n        <span class=\"fd-icon-tab-bar__tag\">\n            {{ item?.label | fdkAsyncOrSync }}\n            @if (item?.counter | fdkAsyncOrSync; as itemCounter) {\n                ({{ itemCounter }})\n            }\n        </span>\n    } @else {\n        <span class=\"fd-icon-tab-bar__counter\">{{ item?.counter }}</span>\n        <span class=\"fd-icon-tab-bar__tag\">{{ item?.label | fdkAsyncOrSync }}</span>\n    }\n    @if (item?.badge) {\n        <span class=\"fd-icon-tab-bar__badge\"></span>\n    }\n    @if (item?.color | fdkAsyncOrSync; as itemColor) {\n        <div class=\"screenreader-only\">\n            {{ colorAssociations()?.[itemColor] || itemColor }}\n        </div>\n    }\n}\n", dependencies: [{ kind: "pipe", type: AsyncOrSyncPipe, name: "fdkAsyncOrSync" }, { kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarTextTypeTabItemComponent, decorators: [{
            type: Component,
            args: [{ selector: '[fdp-icon-tab-bar-text-type-tab-item]', imports: [AsyncOrSyncPipe, NgTemplateOutlet], encapsulation: ViewEncapsulation.None, changeDetection: ChangeDetectionStrategy.OnPush, host: {
                        class: 'fd-icon-tab-bar__tab',
                        role: 'tab'
                    }, template: "@if (item?.titleTemplate) {\n    <ng-template [ngTemplateOutlet]=\"item?.titleTemplate || null\"></ng-template>\n} @else {\n    @if (layoutMode === 'row') {\n        <span class=\"fd-icon-tab-bar__tag\">\n            {{ item?.label | fdkAsyncOrSync }}\n            @if (item?.counter | fdkAsyncOrSync; as itemCounter) {\n                ({{ itemCounter }})\n            }\n        </span>\n    } @else {\n        <span class=\"fd-icon-tab-bar__counter\">{{ item?.counter }}</span>\n        <span class=\"fd-icon-tab-bar__tag\">{{ item?.label | fdkAsyncOrSync }}</span>\n    }\n    @if (item?.badge) {\n        <span class=\"fd-icon-tab-bar__badge\"></span>\n    }\n    @if (item?.color | fdkAsyncOrSync; as itemColor) {\n        <div class=\"screenreader-only\">\n            {{ colorAssociations()?.[itemColor] || itemColor }}\n        </div>\n    }\n}\n" }]
        }], propDecorators: { item: [{
                type: Input
            }], layoutMode: [{
                type: Input
            }] } });

class TextTypePopoverComponent extends IconTabBarPopoverBase {
    constructor() {
        super(...arguments);
        /** Whether to render item as a multi-click variant. */
        this.multiClick = false;
        /**
         * @description Is extra items mode or subitems mode
         */
        this.isExtraItemsMode = false;
        /**
         * @description tabIndex of the particular item. Aplicable only for subitems mode
         */
        this.tabindex = -1;
        /**
         * @description Emits when some subTab is selected and isExtraItemsMode is disabled
         */
        this.selectedSubItem = new EventEmitter();
        /** Emits when root item is being selected. Applicable for multi-click variant only. */
        this.selectedItem = new EventEmitter();
        /** @hidden */
        this._containsSelected = false;
        /** @hidden */
        this._popoverTriggers = [
            { trigger: 'click', openAction: true, closeAction: true, stopPropagation: true }
        ];
    }
    /** @hidden */
    ngOnChanges(changes) {
        super.ngOnChanges(changes);
        if (!this.isExtraItemsMode && changes.parentTab && this.parentTab.subItems) {
            this._setStyles(this.parentTab.subItems);
        }
        if (changes.selectedSubItemUid) {
            this._calculateIfContainsSelected();
        }
    }
    /**
     * @hidden
     * @param selectedItem
     */
    _selectItem(selectedItem) {
        this.isExtraItemsMode ? this.selectedExtraItem.emit(selectedItem) : this.selectedSubItem.emit(selectedItem);
        this.popover.close();
    }
    /** @hidden */
    _textPopoverKeyDownHandler(event, tab) {
        // using relative index within popover (first popover tab should start from 0)
        const baseIndex = this.parentTab ? this.parentTab.flatIndex + 1 : this.extraTabs[0].flatIndex;
        const currentIndex = tab.flatIndex - baseIndex;
        super._keyDownHandler(event, tab, currentIndex);
    }
    /** @hidden */
    _calculateIfContainsSelected() {
        this._containsSelected =
            (this.multiClick && this.parentTab.uId === this.selectedSubItemUid) ||
                (!!this.parentTab.subItems &&
                    this._getChildren(this.parentTab.subItems).some(({ uId }) => uId === this.selectedSubItemUid));
    }
    /** @hidden */
    _getChildren(items) {
        return items.reduce((acc, item) => {
            acc = acc.concat(item);
            if (item.subItems) {
                acc = acc.concat(this._getChildren(item.subItems));
            }
            return acc;
        }, []);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TextTypePopoverComponent, deps: null, target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.0.7", type: TextTypePopoverComponent, isStandalone: true, selector: "fdp-text-type-popover", inputs: { multiClick: "multiClick", layoutMode: "layoutMode", isExtraItemsMode: "isExtraItemsMode", parentTab: "parentTab", selectedSubItemUid: "selectedSubItemUid", tabindex: "tabindex" }, outputs: { selectedSubItem: "selectedSubItem", selectedItem: "selectedItem" }, viewQueries: [{ propertyName: "_dropdownTrigger", first: true, predicate: ["dropdownTrigger"], descendants: true, read: ElementRef }, { propertyName: "_tabExtraUIElements", predicate: ["tabItem"], descendants: true, read: ElementRef }], usesInheritance: true, usesOnChanges: true, ngImport: i0, template: "@if (isExtraItemsMode) {\n    <fd-popover placement=\"bottom-end\" [focusAutoCapture]=\"true\" [focusTrapped]=\"true\" #popover>\n        <fd-popover-control>\n            <button class=\"fd-icon-tab-bar__overflow\" tabindex=\"-1\">\n                <span class=\"fd-icon-tab-bar__overflow-text\">More</span>\n                <fd-icon glyph=\"slim-arrow-down\" role=\"presentation\"></fd-icon>\n            </button>\n        </fd-popover-control>\n        <fd-popover-body class=\"fd-icon-tab-bar\">\n            <ul\n                fdpIconBarDndList\n                class=\"fd-list fd-list--navigation fd-list--no-border fd-icon-tab-bar__list\"\n                role=\"list\"\n            >\n                <ng-template [ngTemplateOutletContext]=\"{ level: 1, items: extraTabs }\" [ngTemplateOutlet]=\"subItem\">\n                </ng-template>\n            </ul>\n        </fd-popover-body>\n    </fd-popover>\n}\n@if (multiClick) {\n    <a\n        #dropdownTrigger\n        fdp-icon-tab-bar-text-type-tab-item\n        [item]=\"parentTab\"\n        [attr.aria-selected]=\"_containsSelected\"\n        [colorAssociations]=\"colorAssociations()\"\n        [layoutMode]=\"layoutMode\"\n        [attr.tabindex]=\"tabindex\"\n        (click)=\"selectedItem.emit()\"\n    >\n    </a>\n}\n@if (!isExtraItemsMode && parentTab && parentTab.subItems?.length) {\n    <fd-popover\n        [class.fd-icon-tab-bar__popover]=\"multiClick\"\n        placement=\"bottom-end\"\n        [focusAutoCapture]=\"true\"\n        [triggers]=\"_popoverTriggers\"\n        [focusTrapped]=\"true\"\n        #popover\n    >\n        <fd-popover-control>\n            @if (multiClick) {\n                <div class=\"fd-icon-tab-bar__button-container\">\n                    <button fd-button fdType=\"transparent\" tabindex=\"-1\" class=\"fd-icon-tab-bar__button\">\n                        <fd-icon glyph=\"slim-arrow-down\"></fd-icon>\n                    </button>\n                </div>\n            } @else {\n                <a\n                    [attr.aria-selected]=\"_containsSelected\"\n                    class=\"fd-icon-tab-bar__tab\"\n                    role=\"tab\"\n                    [attr.tabindex]=\"tabindex\"\n                    #dropdownTrigger\n                >\n                    <div class=\"fd-icon-tab-bar__tab-container\">\n                        <span class=\"fd-icon-tab-bar__tag\">{{ parentTab.label }}</span>\n                        <span class=\"fd-icon-tab-bar__arrow\">\n                            <fd-icon glyph=\"slim-arrow-down\" role=\"presentation\"></fd-icon>\n                        </span>\n                        @if (parentTab.badge) {\n                            <span class=\"fd-icon-tab-bar__badge\"></span>\n                        }\n                    </div>\n                </a>\n            }\n        </fd-popover-control>\n        <fd-popover-body class=\"fd-icon-tab-bar\">\n            <ul\n                fdpIconBarDndList\n                class=\"fd-list fd-list--navigation fd-list--no-border fd-icon-tab-bar__list\"\n                role=\"list\"\n            >\n                <ng-container\n                    [ngTemplateOutletContext]=\"{ level: 1, items: parentTab.subItems }\"\n                    [ngTemplateOutlet]=\"subItem\"\n                ></ng-container>\n            </ul>\n        </fd-popover-body>\n    </fd-popover>\n}\n<ng-template #subItem let-items=\"items\" let-level=\"level\">\n    @for (item of items; track item.uId) {\n        <li\n            [attr.aria-level]=\"level\"\n            class=\"fd-list__item fd-list__item--link fd-icon-tab-bar__list-item\"\n            [class.fd-icon-tab-bar__list-item--closable]=\"item.closable\"\n            [ngClass]=\"item.cssClasses\"\n            fdpIconBarDndItem\n            [dndItemData]=\"item\"\n            role=\"listitem\"\n            tabindex=\"-1\"\n        >\n            <a\n                class=\"fd-list__link fd-icon-tab-bar__list-link\"\n                tabindex=\"0\"\n                #tabItem\n                [attr.data-flatIndex]=\"item.flatIndex\"\n                [attr.aria-selected]=\"item.uId === selectedSubItemUid\"\n                (click)=\"_selectItem(item)\"\n                (keydown)=\"_textPopoverKeyDownHandler($event, item)\"\n            >\n                <span class=\"fd-list__title fd-icon-tab-bar__list-item-title\">{{ item.label }}</span>\n                @if (item.badge) {\n                    <span class=\"fd-icon-tab-bar__badge\"></span>\n                }\n                @if (item.counter) {\n                    <span class=\"fd-list__counter fd-icon-tab-bar__list-item-counter\">({{ item.counter }}) </span>\n                }\n                @if (item.color) {\n                    <div class=\"screenreader-only\">\n                        {{ (colorAssociations() && colorAssociations()?.[item.color]) || item.color }}\n                    </div>\n                }\n            </a>\n            @if (item.closable) {\n                <div class=\"fd-icon-tab-bar__button-container\">\n                    <button\n                        fd-button\n                        tabindex=\"-1\"\n                        class=\"fd-icon-tab-bar__button\"\n                        fdType=\"transparent\"\n                        (click)=\"closeTab.emit(item.uId)\"\n                    >\n                        <fd-icon glyph=\"decline\"></fd-icon>\n                    </button>\n                </div>\n            }\n        </li>\n        @if (item.subItems?.length) {\n            <ng-container\n                [ngTemplateOutletContext]=\"{ level: level + 1, items: item.subItems }\"\n                [ngTemplateOutlet]=\"subItem\"\n            ></ng-container>\n        }\n    }\n</ng-template>\n", dependencies: [{ kind: "component", type: PopoverComponent, selector: "fd-popover", inputs: ["title", "trigger", "fixedPosition", "id", "mobile", "mobileConfig", "preventSpaceKeyScroll"] }, { kind: "component", type: PopoverControlComponent, selector: "fd-popover-control, [fdPopoverControl]" }, { kind: "component", type: PopoverBodyComponent, selector: "fd-popover-body", inputs: ["minWidth", "maxWidth", "minHeight", "maxHeight"] }, { kind: "directive", type: IconBarDndListDirective, selector: "[fdpIconBarDndList], [fdp-icon-bar-dnd-list]", inputs: ["dndOrientation", "dndAutoScroll"] }, { kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }, { kind: "directive", type: IconBarDndItemDirective, selector: "[fdpIconBarDndItem], [fdp-icon-bar-dnd-item]", inputs: ["dndItemData", "previewClass", "dndHoveredClass", "dndSeparatorClass"] }, { kind: "directive", type: NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "component", type: ButtonComponent, selector: "button[fd-button], a[fd-button], span[fd-button]", inputs: ["class"], exportAs: ["fd-button"] }, { kind: "component", type: IconComponent, selector: "fd-icon", inputs: ["glyph", "font", "color", "background", "class", "ariaLabel", "ariaHidden"] }, { kind: "component", type: IconTabBarTextTypeTabItemComponent, selector: "[fdp-icon-tab-bar-text-type-tab-item]", inputs: ["item", "layoutMode", "colorAssociations"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: TextTypePopoverComponent, decorators: [{
            type: Component,
            args: [{ selector: 'fdp-text-type-popover', changeDetection: ChangeDetectionStrategy.OnPush, imports: [
                        PopoverComponent,
                        PopoverControlComponent,
                        PopoverBodyComponent,
                        IconBarDndListDirective,
                        NgTemplateOutlet,
                        IconBarDndItemDirective,
                        NgClass,
                        ButtonComponent,
                        IconComponent,
                        IconTabBarTextTypeTabItemComponent
                    ], template: "@if (isExtraItemsMode) {\n    <fd-popover placement=\"bottom-end\" [focusAutoCapture]=\"true\" [focusTrapped]=\"true\" #popover>\n        <fd-popover-control>\n            <button class=\"fd-icon-tab-bar__overflow\" tabindex=\"-1\">\n                <span class=\"fd-icon-tab-bar__overflow-text\">More</span>\n                <fd-icon glyph=\"slim-arrow-down\" role=\"presentation\"></fd-icon>\n            </button>\n        </fd-popover-control>\n        <fd-popover-body class=\"fd-icon-tab-bar\">\n            <ul\n                fdpIconBarDndList\n                class=\"fd-list fd-list--navigation fd-list--no-border fd-icon-tab-bar__list\"\n                role=\"list\"\n            >\n                <ng-template [ngTemplateOutletContext]=\"{ level: 1, items: extraTabs }\" [ngTemplateOutlet]=\"subItem\">\n                </ng-template>\n            </ul>\n        </fd-popover-body>\n    </fd-popover>\n}\n@if (multiClick) {\n    <a\n        #dropdownTrigger\n        fdp-icon-tab-bar-text-type-tab-item\n        [item]=\"parentTab\"\n        [attr.aria-selected]=\"_containsSelected\"\n        [colorAssociations]=\"colorAssociations()\"\n        [layoutMode]=\"layoutMode\"\n        [attr.tabindex]=\"tabindex\"\n        (click)=\"selectedItem.emit()\"\n    >\n    </a>\n}\n@if (!isExtraItemsMode && parentTab && parentTab.subItems?.length) {\n    <fd-popover\n        [class.fd-icon-tab-bar__popover]=\"multiClick\"\n        placement=\"bottom-end\"\n        [focusAutoCapture]=\"true\"\n        [triggers]=\"_popoverTriggers\"\n        [focusTrapped]=\"true\"\n        #popover\n    >\n        <fd-popover-control>\n            @if (multiClick) {\n                <div class=\"fd-icon-tab-bar__button-container\">\n                    <button fd-button fdType=\"transparent\" tabindex=\"-1\" class=\"fd-icon-tab-bar__button\">\n                        <fd-icon glyph=\"slim-arrow-down\"></fd-icon>\n                    </button>\n                </div>\n            } @else {\n                <a\n                    [attr.aria-selected]=\"_containsSelected\"\n                    class=\"fd-icon-tab-bar__tab\"\n                    role=\"tab\"\n                    [attr.tabindex]=\"tabindex\"\n                    #dropdownTrigger\n                >\n                    <div class=\"fd-icon-tab-bar__tab-container\">\n                        <span class=\"fd-icon-tab-bar__tag\">{{ parentTab.label }}</span>\n                        <span class=\"fd-icon-tab-bar__arrow\">\n                            <fd-icon glyph=\"slim-arrow-down\" role=\"presentation\"></fd-icon>\n                        </span>\n                        @if (parentTab.badge) {\n                            <span class=\"fd-icon-tab-bar__badge\"></span>\n                        }\n                    </div>\n                </a>\n            }\n        </fd-popover-control>\n        <fd-popover-body class=\"fd-icon-tab-bar\">\n            <ul\n                fdpIconBarDndList\n                class=\"fd-list fd-list--navigation fd-list--no-border fd-icon-tab-bar__list\"\n                role=\"list\"\n            >\n                <ng-container\n                    [ngTemplateOutletContext]=\"{ level: 1, items: parentTab.subItems }\"\n                    [ngTemplateOutlet]=\"subItem\"\n                ></ng-container>\n            </ul>\n        </fd-popover-body>\n    </fd-popover>\n}\n<ng-template #subItem let-items=\"items\" let-level=\"level\">\n    @for (item of items; track item.uId) {\n        <li\n            [attr.aria-level]=\"level\"\n            class=\"fd-list__item fd-list__item--link fd-icon-tab-bar__list-item\"\n            [class.fd-icon-tab-bar__list-item--closable]=\"item.closable\"\n            [ngClass]=\"item.cssClasses\"\n            fdpIconBarDndItem\n            [dndItemData]=\"item\"\n            role=\"listitem\"\n            tabindex=\"-1\"\n        >\n            <a\n                class=\"fd-list__link fd-icon-tab-bar__list-link\"\n                tabindex=\"0\"\n                #tabItem\n                [attr.data-flatIndex]=\"item.flatIndex\"\n                [attr.aria-selected]=\"item.uId === selectedSubItemUid\"\n                (click)=\"_selectItem(item)\"\n                (keydown)=\"_textPopoverKeyDownHandler($event, item)\"\n            >\n                <span class=\"fd-list__title fd-icon-tab-bar__list-item-title\">{{ item.label }}</span>\n                @if (item.badge) {\n                    <span class=\"fd-icon-tab-bar__badge\"></span>\n                }\n                @if (item.counter) {\n                    <span class=\"fd-list__counter fd-icon-tab-bar__list-item-counter\">({{ item.counter }}) </span>\n                }\n                @if (item.color) {\n                    <div class=\"screenreader-only\">\n                        {{ (colorAssociations() && colorAssociations()?.[item.color]) || item.color }}\n                    </div>\n                }\n            </a>\n            @if (item.closable) {\n                <div class=\"fd-icon-tab-bar__button-container\">\n                    <button\n                        fd-button\n                        tabindex=\"-1\"\n                        class=\"fd-icon-tab-bar__button\"\n                        fdType=\"transparent\"\n                        (click)=\"closeTab.emit(item.uId)\"\n                    >\n                        <fd-icon glyph=\"decline\"></fd-icon>\n                    </button>\n                </div>\n            }\n        </li>\n        @if (item.subItems?.length) {\n            <ng-container\n                [ngTemplateOutletContext]=\"{ level: level + 1, items: item.subItems }\"\n                [ngTemplateOutlet]=\"subItem\"\n            ></ng-container>\n        }\n    }\n</ng-template>\n" }]
        }], propDecorators: { _dropdownTrigger: [{
                type: ViewChild,
                args: ['dropdownTrigger', { read: ElementRef }]
            }], _tabExtraUIElements: [{
                type: ViewChildren,
                args: ['tabItem', { read: ElementRef }]
            }], multiClick: [{
                type: Input
            }], layoutMode: [{
                type: Input
            }], isExtraItemsMode: [{
                type: Input
            }], parentTab: [{
                type: Input
            }], selectedSubItemUid: [{
                type: Input
            }], tabindex: [{
                type: Input
            }], selectedSubItem: [{
                type: Output
            }], selectedItem: [{
                type: Output
            }] } });

class IconTabBarTextTypeComponent extends ClosableIconTabBar {
    constructor() {
        super(...arguments);
        /**
         * @description Disable or enable reordering(drag and drop) feature.
         */
        this.enableTabReordering = false;
        /** Whether to render icon tab item as multi-click variant. */
        this.multiClick = false;
        /**
         * @description Emits when user drops the tab.
         */
        this.reordered = new EventEmitter();
    }
    /**
     * @hidden
     * @param selectedItem
     */
    _selectExtraItem(selectedItem) {
        // Check if the selected item is subItem
        // Then to find root tab, and pass it to the parent method.
        if (selectedItem?.uId.includes(UNIQUE_KEY_SEPARATOR)) {
            const rootTabUid = selectedItem.uId.split(UNIQUE_KEY_SEPARATOR)[0];
            selectedItem = this.tabs.find((tab) => tab.uId === rootTabUid);
        }
        if (!selectedItem) {
            return;
        }
        super._selectExtraItem(selectedItem);
    }
    /**
     * @hidden
     * @param draggableItem
     * @param targetItem
     * @param action
     */
    _onDropped(event) {
        if (!this._canDrop(event)) {
            return;
        }
        const replacedParsedUidArr = event.targetItem.uId.split(UNIQUE_KEY_SEPARATOR);
        replacedParsedUidArr.length = replacedParsedUidArr.length - 1;
        const draggableParsedUidArr = event.draggableItem.uId.split(UNIQUE_KEY_SEPARATOR);
        draggableParsedUidArr.length = draggableParsedUidArr.length - 1;
        const tabInfoForNewList = this._getTabInfoFromMainList(event.targetItem.uId);
        const tabInfoForPrevList = this._getTabInfoFromMainList(event.draggableItem.uId);
        if (!tabInfoForNewList || !tabInfoForPrevList) {
            return;
        }
        const dataForAction = {
            replacedItemInfo: {
                arr: tabInfoForNewList.parent,
                item: tabInfoForNewList.tab,
                parentUid: replacedParsedUidArr.length ? replacedParsedUidArr.join(UNIQUE_KEY_SEPARATOR) : ''
            },
            draggableItemInfo: {
                arr: tabInfoForPrevList.parent,
                item: tabInfoForPrevList.tab,
                parentUid: draggableParsedUidArr.length ? draggableParsedUidArr.join(UNIQUE_KEY_SEPARATOR) : ''
            }
        };
        event.action === 'replace' ? this._replaceAsSibling(dataForAction) : this._insertItemAsChild(dataForAction);
        this.reordered.emit(this.tabs);
    }
    /** @hidden */
    _getTabUIElementFocusable(tabUIElement) {
        if (!tabUIElement) {
            return tabUIElement;
        }
        if (typeof tabUIElement === 'object' && 'nativeElement' in tabUIElement) {
            return tabUIElement.nativeElement;
        }
        return tabUIElement._dropdownTrigger.nativeElement;
    }
    /** @hidden */
    _canDrop(event) {
        let parentUId = event.action === 'replace' ? event.targetItem.parentUId : event.targetItem.uId;
        while (parentUId) {
            if (parentUId === event.draggableItem.uId) {
                return false;
            }
            const nextSeparatorIdx = parentUId.lastIndexOf(UNIQUE_KEY_SEPARATOR);
            parentUId = nextSeparatorIdx > 0 ? parentUId.slice(0, nextSeparatorIdx) : null;
        }
        return true;
    }
    /**
     * @hidden
     * @param data
     * @description Insert tab into another tab.
     */
    _insertItemAsChild(data) {
        const { replacedItemInfo, draggableItemInfo } = data;
        // Remove draggable tab from previous list
        draggableItemInfo.arr.splice(draggableItemInfo.item.index, 1);
        if (!replacedItemInfo.item.subItems?.length) {
            replacedItemInfo.item.subItems = [];
        }
        // Add tab to subitem of the target tab
        replacedItemInfo.item.subItems.push(draggableItemInfo.item);
        this.tabs = this._updateTabs(this.tabs);
        this._triggerRecalculationVisibleItems();
    }
    /**
     * @hidden
     * @param data
     * @description Insert tab between tabs
     */
    _replaceAsSibling(data) {
        const { replacedItemInfo, draggableItemInfo } = data;
        draggableItemInfo.arr.splice(draggableItemInfo.item.index, 1);
        const newIndex = replacedItemInfo?.item?.index || 0;
        replacedItemInfo.arr.splice(newIndex, 0, draggableItemInfo.item);
        this.tabs = this._updateTabs(this.tabs);
        this._triggerRecalculationVisibleItems();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarTextTypeComponent, deps: null, target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.0.7", type: IconTabBarTextTypeComponent, isStandalone: true, selector: "fdp-icon-tab-bar-text-type", inputs: { enableTabReordering: "enableTabReordering", layoutMode: "layoutMode", multiClick: "multiClick" }, outputs: { reordered: "reordered" }, providers: [
            {
                provide: IconTabBarBase,
                useExisting: IconTabBarTextTypeComponent
            }
        ], viewQueries: [{ propertyName: "_tabBarPopover", first: true, predicate: ["extraItemsPopover"], descendants: true }, { propertyName: "_tabUIElements", predicate: ["tabItem"], descendants: true }], usesInheritance: true, ngImport: i0, template: "<ng-container fdpIconBarDndContainer [draggable]=\"enableTabReordering\" (dropped)=\"_onDropped($event)\">\n    <ul\n        class=\"fd-icon-tab-bar__header\"\n        [class.fd-dynamic-page__tabs]=\"_inDynamicPage\"\n        role=\"tablist\"\n        #tablist\n        fdkOverflowList\n        [isRtl]=\"isRtl\"\n        [overflowOffset]=\"100\"\n        fdpIconBarDndList\n        dndOrientation=\"horizontal\"\n        (overflowChanged)=\"_recalculateVisibleItems($event)\"\n    >\n        @for (item of tabs; track item.uId; let idx = $index) {\n            <li\n                [ngClass]=\"item.cssClasses\"\n                [class.fd-icon-tab-bar__item--multi-click]=\"multiClick && item.subItems?.length\"\n                fdkOverflowListItem\n                fdpIconBarDndItem\n                [dndSeparatorClass]=\"['fd-icon-tab-bar__item--dnd-separator']\"\n                [dndItemData]=\"item\"\n                class=\"fd-icon-tab-bar__item\"\n                [class.fd-icon-tab-bar__item--closable]=\"item.closable\"\n                [attr.aria-hidden]=\"item.hidden\"\n                role=\"presentation\"\n            >\n                @if (!item.subItems?.length && !multiClick) {\n                    <a\n                        #tabItem\n                        fdp-icon-tab-bar-text-type-tab-item\n                        [item]=\"item\"\n                        [layoutMode]=\"layoutMode\"\n                        [colorAssociations]=\"colorAssociations()\"\n                        [attr.tabindex]=\"idx === 0 ? 0 : -1\"\n                        [attr.aria-selected]=\"item.uId === selectedUid\"\n                        (click)=\"_selectItem(item, $event)\"\n                        (keydown)=\"_keyDownHandler($event, item, idx)\"\n                        [attr.aria-level]=\"tabHeadingLevel()\"\n                    >\n                    </a>\n                }\n                @if (item.subItems?.length || multiClick) {\n                    <fdp-text-type-popover\n                        #tabItem\n                        #itemPopover\n                        [tabindex]=\"idx === 0 ? 0 : -1\"\n                        [parentTab]=\"item\"\n                        [layoutMode]=\"layoutMode\"\n                        [multiClick]=\"multiClick\"\n                        [selectedSubItemUid]=\"selectedUid\"\n                        [colorAssociations]=\"colorAssociations()\"\n                        (selectedSubItem)=\"_selectItem($event)\"\n                        (keydown)=\"_keyDownHandler($event, item, idx, itemPopover)\"\n                        (closeTab)=\"_closeTab($event)\"\n                        (selectedItem)=\"_selectItem(item)\"\n                        [attr.aria-level]=\"tabHeadingLevel()\"\n                    >\n                    </fdp-text-type-popover>\n                }\n                @if (item.closable) {\n                    <div class=\"fd-icon-tab-bar__button-container\">\n                        <button\n                            fd-button\n                            tabindex=\"-1\"\n                            class=\"fd-icon-tab-bar__button\"\n                            fdType=\"transparent\"\n                            (click)=\"_closeTab(item.uId)\"\n                        >\n                            <fd-icon glyph=\"decline\"></fd-icon>\n                        </button>\n                    </div>\n                }\n            </li>\n        }\n        @if (_extraItems$()) {\n            <li role=\"presentation\" class=\"fd-icon-tab-bar__item fd-icon-tab-bar__item--overflow\">\n                <fdp-text-type-popover\n                    #extraItemsPopover\n                    [isRtl]=\"isRtl\"\n                    [extraTabs]=\"_extraTabs$()\"\n                    [isExtraItemsMode]=\"true\"\n                    [colorAssociations]=\"colorAssociations()\"\n                    (selectedExtraItem)=\"_selectExtraItem($event)\"\n                    (focusFirstVisibleItem)=\"_focusItem(0)\"\n                    (focusLastVisibleItem)=\"_focusItem(_lastVisibleTabIndex)\"\n                    (closeTab)=\"_closeTab($event, true)\"\n                >\n                </fdp-text-type-popover>\n            </li>\n        }\n    </ul>\n</ng-container>\n", dependencies: [{ kind: "directive", type: IconBarDndContainerDirective, selector: "[fdpIconBarDndContainer], [fdp-icon-bar-dnd-container]", inputs: ["draggable"], outputs: ["dropped"] }, { kind: "directive", type: OverflowListDirective, selector: "[fdkOverflowList]", inputs: ["overflowOffset", "isRtl", "itemCssBlockValue"], outputs: ["overflowChanged"] }, { kind: "directive", type: IconBarDndListDirective, selector: "[fdpIconBarDndList], [fdp-icon-bar-dnd-list]", inputs: ["dndOrientation", "dndAutoScroll"] }, { kind: "directive", type: OverflowListItemDirective, selector: "[fdkOverflowListItem]" }, { kind: "directive", type: IconBarDndItemDirective, selector: "[fdpIconBarDndItem], [fdp-icon-bar-dnd-item]", inputs: ["dndItemData", "previewClass", "dndHoveredClass", "dndSeparatorClass"] }, { kind: "directive", type: NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "component", type: TextTypePopoverComponent, selector: "fdp-text-type-popover", inputs: ["multiClick", "layoutMode", "isExtraItemsMode", "parentTab", "selectedSubItemUid", "tabindex"], outputs: ["selectedSubItem", "selectedItem"] }, { kind: "component", type: ButtonComponent, selector: "button[fd-button], a[fd-button], span[fd-button]", inputs: ["class"], exportAs: ["fd-button"] }, { kind: "component", type: IconComponent, selector: "fd-icon", inputs: ["glyph", "font", "color", "background", "class", "ariaLabel", "ariaHidden"] }, { kind: "component", type: IconTabBarTextTypeTabItemComponent, selector: "[fdp-icon-tab-bar-text-type-tab-item]", inputs: ["item", "layoutMode", "colorAssociations"] }] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarTextTypeComponent, decorators: [{
            type: Component,
            args: [{ selector: 'fdp-icon-tab-bar-text-type', providers: [
                        {
                            provide: IconTabBarBase,
                            useExisting: IconTabBarTextTypeComponent
                        }
                    ], imports: [
                        IconBarDndContainerDirective,
                        OverflowListDirective,
                        IconBarDndListDirective,
                        OverflowListItemDirective,
                        IconBarDndItemDirective,
                        NgClass,
                        TextTypePopoverComponent,
                        ButtonComponent,
                        IconComponent,
                        AsyncOrSyncPipe,
                        IconTabBarTextTypeTabItemComponent,
                        NgTemplateOutlet
                    ], template: "<ng-container fdpIconBarDndContainer [draggable]=\"enableTabReordering\" (dropped)=\"_onDropped($event)\">\n    <ul\n        class=\"fd-icon-tab-bar__header\"\n        [class.fd-dynamic-page__tabs]=\"_inDynamicPage\"\n        role=\"tablist\"\n        #tablist\n        fdkOverflowList\n        [isRtl]=\"isRtl\"\n        [overflowOffset]=\"100\"\n        fdpIconBarDndList\n        dndOrientation=\"horizontal\"\n        (overflowChanged)=\"_recalculateVisibleItems($event)\"\n    >\n        @for (item of tabs; track item.uId; let idx = $index) {\n            <li\n                [ngClass]=\"item.cssClasses\"\n                [class.fd-icon-tab-bar__item--multi-click]=\"multiClick && item.subItems?.length\"\n                fdkOverflowListItem\n                fdpIconBarDndItem\n                [dndSeparatorClass]=\"['fd-icon-tab-bar__item--dnd-separator']\"\n                [dndItemData]=\"item\"\n                class=\"fd-icon-tab-bar__item\"\n                [class.fd-icon-tab-bar__item--closable]=\"item.closable\"\n                [attr.aria-hidden]=\"item.hidden\"\n                role=\"presentation\"\n            >\n                @if (!item.subItems?.length && !multiClick) {\n                    <a\n                        #tabItem\n                        fdp-icon-tab-bar-text-type-tab-item\n                        [item]=\"item\"\n                        [layoutMode]=\"layoutMode\"\n                        [colorAssociations]=\"colorAssociations()\"\n                        [attr.tabindex]=\"idx === 0 ? 0 : -1\"\n                        [attr.aria-selected]=\"item.uId === selectedUid\"\n                        (click)=\"_selectItem(item, $event)\"\n                        (keydown)=\"_keyDownHandler($event, item, idx)\"\n                        [attr.aria-level]=\"tabHeadingLevel()\"\n                    >\n                    </a>\n                }\n                @if (item.subItems?.length || multiClick) {\n                    <fdp-text-type-popover\n                        #tabItem\n                        #itemPopover\n                        [tabindex]=\"idx === 0 ? 0 : -1\"\n                        [parentTab]=\"item\"\n                        [layoutMode]=\"layoutMode\"\n                        [multiClick]=\"multiClick\"\n                        [selectedSubItemUid]=\"selectedUid\"\n                        [colorAssociations]=\"colorAssociations()\"\n                        (selectedSubItem)=\"_selectItem($event)\"\n                        (keydown)=\"_keyDownHandler($event, item, idx, itemPopover)\"\n                        (closeTab)=\"_closeTab($event)\"\n                        (selectedItem)=\"_selectItem(item)\"\n                        [attr.aria-level]=\"tabHeadingLevel()\"\n                    >\n                    </fdp-text-type-popover>\n                }\n                @if (item.closable) {\n                    <div class=\"fd-icon-tab-bar__button-container\">\n                        <button\n                            fd-button\n                            tabindex=\"-1\"\n                            class=\"fd-icon-tab-bar__button\"\n                            fdType=\"transparent\"\n                            (click)=\"_closeTab(item.uId)\"\n                        >\n                            <fd-icon glyph=\"decline\"></fd-icon>\n                        </button>\n                    </div>\n                }\n            </li>\n        }\n        @if (_extraItems$()) {\n            <li role=\"presentation\" class=\"fd-icon-tab-bar__item fd-icon-tab-bar__item--overflow\">\n                <fdp-text-type-popover\n                    #extraItemsPopover\n                    [isRtl]=\"isRtl\"\n                    [extraTabs]=\"_extraTabs$()\"\n                    [isExtraItemsMode]=\"true\"\n                    [colorAssociations]=\"colorAssociations()\"\n                    (selectedExtraItem)=\"_selectExtraItem($event)\"\n                    (focusFirstVisibleItem)=\"_focusItem(0)\"\n                    (focusLastVisibleItem)=\"_focusItem(_lastVisibleTabIndex)\"\n                    (closeTab)=\"_closeTab($event, true)\"\n                >\n                </fdp-text-type-popover>\n            </li>\n        }\n    </ul>\n</ng-container>\n" }]
        }], propDecorators: { _tabUIElements: [{
                type: ViewChildren,
                args: ['tabItem']
            }], _tabBarPopover: [{
                type: ViewChild,
                args: ['extraItemsPopover']
            }], enableTabReordering: [{
                type: Input
            }], layoutMode: [{
                type: Input
            }], multiClick: [{
                type: Input
            }], reordered: [{
                type: Output
            }] } });

class IconTabTitleDirective {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabTitleDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: IconTabTitleDirective, isStandalone: true, selector: "[fdpIconTabBarTitleTemplate], [fdp-icon-tab-bar-title-template]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabTitleDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[fdpIconTabBarTitleTemplate], [fdp-icon-tab-bar-title-template]',
                    standalone: true
                }]
        }] });

let defaultIdIndex = 0;
class IconTabBarTabContentDirective {
    constructor() {
        /** Tab ID */
        this.id = input();
        /** Tab Unique ID. */
        this.uId = input();
        /** Element reference. */
        this.elementRef = inject(ElementRef);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarTabContentDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.0.7", type: IconTabBarTabContentDirective, isStandalone: true, selector: "fdp-icon-tab-bar-tab-content", inputs: { id: { classPropertyName: "id", publicName: "id", isSignal: true, isRequired: false, transformFunction: null }, uId: { classPropertyName: "uId", publicName: "uId", isSignal: true, isRequired: false, transformFunction: null } }, host: { properties: { "attr.id": "id()" } }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarTabContentDirective, decorators: [{
            type: Directive,
            args: [{
                    // eslint-disable-next-line @angular-eslint/directive-selector
                    selector: 'fdp-icon-tab-bar-tab-content',
                    standalone: true,
                    host: { '[attr.id]': 'id()' }
                }]
        }] });
class IconTabBarTabComponent {
    constructor() {
        /** Tab Label. */
        this.label = input();
        /** Tab text color. */
        this.color = input('default');
        /** Tab icon. */
        this.icon = input();
        /** Tab icon font family. */
        this.iconFont = input(FD_DEFAULT_ICON_FONT_FAMILY);
        /** Tab counter. */
        this.counter = input();
        /** whether the tab is selected */
        this.active = input(false, { transform: booleanAttribute });
        /** if set to true, will show red circle in top-right corner of tab */
        this.badge = input(false, { transform: booleanAttribute });
        /** Whether the tab can be closed. */
        this.closable = input(false, { transform: booleanAttribute });
        /** Tab ID. */
        this.id = input(`fdp-icon-tab-bar-tab-${defaultIdIndex++}`);
        /** Content renderer. */
        this.renderer = viewChild('renderer');
        /** Title template. */
        this.titleTemplate = contentChild(IconTabTitleDirective, { read: TemplateRef });
        /** @hidden */
        this.children = contentChildren(IconTabBarTabComponent);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarTabComponent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.0.7", type: IconTabBarTabComponent, isStandalone: true, selector: "fdp-icon-tab-bar-tab", inputs: { label: { classPropertyName: "label", publicName: "label", isSignal: true, isRequired: false, transformFunction: null }, color: { classPropertyName: "color", publicName: "color", isSignal: true, isRequired: false, transformFunction: null }, icon: { classPropertyName: "icon", publicName: "icon", isSignal: true, isRequired: false, transformFunction: null }, iconFont: { classPropertyName: "iconFont", publicName: "iconFont", isSignal: true, isRequired: false, transformFunction: null }, counter: { classPropertyName: "counter", publicName: "counter", isSignal: true, isRequired: false, transformFunction: null }, active: { classPropertyName: "active", publicName: "active", isSignal: true, isRequired: false, transformFunction: null }, badge: { classPropertyName: "badge", publicName: "badge", isSignal: true, isRequired: false, transformFunction: null }, closable: { classPropertyName: "closable", publicName: "closable", isSignal: true, isRequired: false, transformFunction: null }, id: { classPropertyName: "id", publicName: "id", isSignal: true, isRequired: false, transformFunction: null } }, queries: [{ propertyName: "titleTemplate", first: true, predicate: IconTabTitleDirective, descendants: true, read: TemplateRef, isSignal: true }, { propertyName: "children", predicate: IconTabBarTabComponent, isSignal: true }], viewQueries: [{ propertyName: "renderer", first: true, predicate: ["renderer"], descendants: true, isSignal: true }], ngImport: i0, template: "<ng-template #renderer>\n    <ng-content></ng-content>\n</ng-template>\n", changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarTabComponent, decorators: [{
            type: Component,
            args: [{ selector: 'fdp-icon-tab-bar-tab', imports: [], changeDetection: ChangeDetectionStrategy.OnPush, template: "<ng-template #renderer>\n    <ng-content></ng-content>\n</ng-template>\n" }]
        }] });

class IconTabBarFreeContentDirective {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarFreeContentDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.7", type: IconTabBarFreeContentDirective, isStandalone: true, selector: "[fdpIconTabBarFreeContent]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarFreeContentDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[fdpIconTabBarFreeContent]',
                    standalone: true
                }]
        }] });

class IconTabBarComponent {
    get _customTabs() {
        return this._inDynamicPage;
    }
    /** @hidden */
    get headerContainer() {
        return this._iconTabBarCmp()?.headerElement;
    }
    /** Scrollable element reference. */
    get scrollableElement() {
        return this._scrollbar()?.elementRef;
    }
    /** @hidden */
    constructor(_cd, _contentDensityService, _rtlService) {
        this._cd = _cd;
        this._contentDensityService = _contentDensityService;
        this._rtlService = _rtlService;
        /**
         * Whether to open tab content one under another without collapsing.
         * Works only for content-projected tab content.
         */
        this.stackContent = false;
        /** Heading level of the tab. */
        this.tabHeadingLevel = input(null);
        /** @description Type of tab bar view. */
        this.iconTabType = input('text');
        /**
         * @description A tab bar configuration that stores the state of each tab. Based on this configuration, a tab bar is representing.
         */
        this.tabsConfig = model();
        /** @description Destiny mode. */
        this.densityMode = model('inherit');
        /** @description Icon font */
        this.iconTabFont = input('SAP-icons');
        /** @description Disable or enable reordering(drag and drop) feature. (supported by text type only) */
        this.enableTabReordering = input(false);
        /** @description Boolean flag indicating to show total tab.(supported by filter type only) */
        this.showTotalTab = input(true);
        /** Whether to render icon tab item as multi-click variant. */
        this.multiClick = input(false);
        /** @description Layout type for tab (supported by text type only) */
        this.layoutMode = input('row');
        /** @description Icon tab bar background type. */
        this.iconTabBackground = input('solid');
        /** @description Icon tab bar size. */
        this.iconTabSize = input();
        /**
         * @description Associations for colors of the tabs.
         * If any of the color associations provided, they'll be read by screenreader instead of the actual color
         */
        this.colorAssociations = input();
        /**
         * Maximum height of the content.
         * Works only for content-projected tab content.
         */
        this.maxContentHeight = input('100%');
        /** @description Emits when some tab is selected. */
        this.iconTabSelected = output();
        /** @description Emits when user drop tab. */
        this.iconTabReordered = output();
        /** Event emitted when user clicks on x icon in tab. */
        this.closeTab = output();
        /** @hidden */
        this.children = contentChildren(IconTabBarTabComponent, { descendants: false });
        /** @hidden */
        this._scrollbar = viewChild(ScrollbarDirective);
        /** @hidden */
        this.tabDirectives = viewChildren(IconTabBarTabContentDirective);
        /** @hidden */
        this._disableScrollSpy = false;
        /** @hidden */
        this._init = true;
        /** @hidden */
        this._selectedUid = signal(undefined);
        /** @hidden */
        this._tabs$ = computed(() => this._generateTabBarItems(this._tabsConfig$()));
        /** @hidden */
        this._flatTabs$ = computed(() => this._generateFlatTabs(this._tabs$()));
        /** @hidden */
        this._tabRenderer$ = signal(null);
        /** @hidden */
        this._rtl$ = computed(() => !!this._rtlService?.rtlSignal());
        /** @hidden */
        this._tabsConfig$ = computed(() => {
            const tabConfig = this.tabsConfig();
            if (tabConfig) {
                return this.tabsConfig();
            }
            const children = this.children();
            if (!children) {
                return [];
            }
            return children.map((t) => this._generateTabConfig(t));
        });
        /** @hidden */
        this._destroyRef = inject(DestroyRef);
        this._inDynamicPage = !!inject(FD_DYNAMIC_PAGE, { optional: true });
        this._iconTabBarCmp = viewChild(IconTabBarBase);
    }
    /** @hidden */
    ngOnInit() {
        this._cssClassForContainer = this._generateContainerStyles();
        if (this.densityMode() === 'inherit') {
            this._contentDensityService?._contentDensityListener
                .pipe(distinctUntilChanged(), takeUntilDestroyed(this._destroyRef))
                .subscribe((density) => {
                this.densityMode.set(density);
                if (density !== 'compact') {
                    this._cssClassForContainer = this._cssClassForContainer.filter((cssClass) => cssClass !== 'fd-icon-tab-bar--compact');
                }
                else {
                    this._cssClassForContainer.push('fd-icon-tab-bar--compact');
                }
                this._cd.detectChanges();
            });
        }
    }
    /**
     * @hidden
     * @param event reordered array of IconTabBarItem
     */
    _onReorder(event) {
        this.iconTabReordered.emit(event);
    }
    /**
     * @hidden
     * @param selectedItem
     */
    _selectItem(selectedItem) {
        this._tabRenderer$.set(selectedItem);
        this.iconTabSelected.emit(selectedItem);
        if (this.stackContent) {
            this._scrollToPanel(this.tabDirectives().find((tab) => tab.uId() === selectedItem.uId));
        }
    }
    /** @hidden */
    _closeTab(item) {
        this.closeTab.emit(item);
    }
    /** Programmatically select tab. */
    selectTab(id) {
        const selectedItem = this._flatTabs$().find((t) => t.id === id);
        if (!selectedItem) {
            return;
        }
        this._selectedUid.set(selectedItem.uId);
        this._selectItem(selectedItem);
    }
    /** @hidden */
    highlightActiveTab(id, scroll = false) {
        if (this._disableScrollSpy) {
            return;
        }
        const activeTab = this.tabDirectives().find((tab) => tab.id() === id);
        this._selectedUid.set(activeTab?.uId() || undefined);
        if (scroll && activeTab) {
            this._scrollToPanel(activeTab);
        }
    }
    /**
     * @hidden
     * @returns array of css classes for icon-tab-bar container
     */
    _generateContainerStyles() {
        const styles = [`fd-icon-tab-bar--${this.iconTabType()}`];
        const tabsConfig = this.tabsConfig();
        if (tabsConfig && this.iconTabType() === 'process' && tabsConfig[0].icon) {
            styles.push('fd-icon-tab-bar--icon');
        }
        if (this.iconTabBackground() !== 'solid') {
            styles.push(`fd-icon-tab-bar--${this.iconTabBackground()}`);
        }
        if (this.iconTabSize()) {
            styles.push(`fd-icon-tab-bar--${this.iconTabSize()}`);
        }
        if (this.densityMode() === 'compact') {
            styles.push('fd-icon-tab-bar--compact');
        }
        if (this.layoutMode() === 'column') {
            styles.push('fd-icon-tab-bar--counters');
        }
        return styles;
    }
    _generateTabConfig(tab) {
        const tabConfig = {
            label: tab.label(),
            color: tab.color(),
            active: tab.active(),
            badge: tab.badge(),
            counter: tab.counter(),
            renderer: tab.renderer(),
            id: tab.id(),
            titleTemplate: tab.titleTemplate(),
            subItems: tab.children().map((c) => this._generateTabConfig(c))
        };
        return tabConfig;
    }
    /** @hidden */
    _scrollToPanel(tabPanel) {
        if (!tabPanel) {
            return;
        }
        const scrollbar = this._scrollbar();
        if (!scrollbar) {
            return;
        }
        const panelElement = tabPanel.elementRef.nativeElement;
        const containerElement = scrollbar.elementRef.nativeElement;
        const distanceToScroll = panelElement.offsetTop - containerElement.offsetTop;
        const maximumScrollTop = containerElement.scrollHeight - containerElement.clientHeight;
        const currentScrollPosition = Math.ceil(containerElement.scrollTop);
        if (!(currentScrollPosition === maximumScrollTop && distanceToScroll > maximumScrollTop)) {
            this._disableScrollSpy = true;
            fromEvent(containerElement, 'scroll')
                .pipe(debounceTime(100), first(), takeUntilDestroyed(this._destroyRef))
                .subscribe(() => (this._disableScrollSpy = false));
            scrollTop(containerElement, distanceToScroll);
        }
    }
    /**
     * @hidden
     * @description generate IconTabItems from TabConfig array
     */
    _generateTabBarItems(config, indexPrefix = '', flatIndexRef, parentUId) {
        flatIndexRef = flatIndexRef || { value: 0 };
        return config.map((item, index) => {
            const uId = `${indexPrefix}${index}`;
            item.color = item.color || 'default';
            const result = {
                ...item,
                index,
                cssClasses: item.color ? [`fd-icon-tab-bar__item--${item.color}`] : [],
                uId,
                hidden: false,
                parentUId,
                flatIndex: flatIndexRef.value++,
                subItems: this._generateTabBarItems(item.subItems || [], `${uId}${UNIQUE_KEY_SEPARATOR}`, flatIndexRef, uId)
            };
            return result;
        });
    }
    _generateFlatTabs(tabs) {
        const flatTabs = tabs.reduce((acc, tab) => {
            acc.push(tab);
            if (tab.subItems) {
                acc.push(...this._generateFlatTabs(tab.subItems));
            }
            return acc;
        }, []);
        return flatTabs;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarComponent, deps: [{ token: i0.ChangeDetectorRef }, { token: i1$1.ContentDensityService, optional: true }, { token: i1$1.RtlService, optional: true }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.0.7", type: IconTabBarComponent, isStandalone: true, selector: "fdp-icon-tab-bar", inputs: { stackContent: { classPropertyName: "stackContent", publicName: "stackContent", isSignal: false, isRequired: false, transformFunction: booleanAttribute }, tabHeadingLevel: { classPropertyName: "tabHeadingLevel", publicName: "tabHeadingLevel", isSignal: true, isRequired: false, transformFunction: null }, iconTabType: { classPropertyName: "iconTabType", publicName: "iconTabType", isSignal: true, isRequired: false, transformFunction: null }, tabsConfig: { classPropertyName: "tabsConfig", publicName: "tabsConfig", isSignal: true, isRequired: false, transformFunction: null }, densityMode: { classPropertyName: "densityMode", publicName: "densityMode", isSignal: true, isRequired: false, transformFunction: null }, iconTabFont: { classPropertyName: "iconTabFont", publicName: "iconTabFont", isSignal: true, isRequired: false, transformFunction: null }, enableTabReordering: { classPropertyName: "enableTabReordering", publicName: "enableTabReordering", isSignal: true, isRequired: false, transformFunction: null }, showTotalTab: { classPropertyName: "showTotalTab", publicName: "showTotalTab", isSignal: true, isRequired: false, transformFunction: null }, multiClick: { classPropertyName: "multiClick", publicName: "multiClick", isSignal: true, isRequired: false, transformFunction: null }, layoutMode: { classPropertyName: "layoutMode", publicName: "layoutMode", isSignal: true, isRequired: false, transformFunction: null }, iconTabBackground: { classPropertyName: "iconTabBackground", publicName: "iconTabBackground", isSignal: true, isRequired: false, transformFunction: null }, iconTabSize: { classPropertyName: "iconTabSize", publicName: "iconTabSize", isSignal: true, isRequired: false, transformFunction: null }, colorAssociations: { classPropertyName: "colorAssociations", publicName: "colorAssociations", isSignal: true, isRequired: false, transformFunction: null }, maxContentHeight: { classPropertyName: "maxContentHeight", publicName: "maxContentHeight", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { tabsConfig: "tabsConfigChange", densityMode: "densityModeChange", iconTabSelected: "iconTabSelected", iconTabReordered: "iconTabReordered", closeTab: "closeTab" }, host: { properties: { "class.fd-tabs-custom": "this._customTabs" } }, providers: [
            {
                provide: FD_TABLIST,
                useExisting: IconTabBarComponent
            }
        ], queries: [{ propertyName: "children", predicate: IconTabBarTabComponent, isSignal: true }], viewQueries: [{ propertyName: "_scrollbar", first: true, predicate: ScrollbarDirective, descendants: true, isSignal: true }, { propertyName: "tabDirectives", predicate: IconTabBarTabContentDirective, descendants: true, isSignal: true }, { propertyName: "_iconTabBarCmp", first: true, predicate: IconTabBarBase, descendants: true, isSignal: true }], ngImport: i0, template: "<div class=\"fd-icon-tab-bar\" [class]=\"_cssClassForContainer\">\n    @if (iconTabType() === 'text') {\n        <ng-template [ngTemplateOutlet]=\"textType\"></ng-template>\n    }\n    @if (iconTabType() === 'icon' || iconTabType() === 'icon-only') {\n        <ng-template [ngTemplateOutlet]=\"iconType\"></ng-template>\n    }\n    @if (iconTabType() === 'filter') {\n        <ng-template [ngTemplateOutlet]=\"filterType\"></ng-template>\n    }\n    @if (iconTabType() === 'process') {\n        <ng-template [ngTemplateOutlet]=\"processType\"></ng-template>\n    }\n    <div\n        class=\"fd-icon-tab-bar__content\"\n        fd-scrollbar\n        (spyChange)=\"highlightActiveTab($event.id)\"\n        [scrollSpyDisabled]=\"!stackContent || _disableScrollSpy\"\n        fdScrollSpy\n        [trackedTags]=\"['fdp-icon-tab-bar-tab-content']\"\n        [style.max-height]=\"maxContentHeight()\"\n    >\n        @if (stackContent) {\n            @for (tab of _flatTabs$(); track tab) {\n                <fdp-icon-tab-bar-tab-content [id]=\"tab.id\" [uId]=\"tab.uId\" class=\"fd-icon-tab-bar__tab-content\">\n                    <ng-template [ngTemplateOutlet]=\"tab.renderer || null\"></ng-template>\n                </fdp-icon-tab-bar-tab-content>\n            }\n        } @else {\n            @if (_tabRenderer$(); as tabRenderer) {\n                <fdp-icon-tab-bar-tab-content\n                    [id]=\"tabRenderer.id\"\n                    [uId]=\"tabRenderer.uId\"\n                    class=\"fd-icon-tab-bar__tab-content\"\n                >\n                    <ng-template [ngTemplateOutlet]=\"tabRenderer.renderer || null\"></ng-template>\n                </fdp-icon-tab-bar-tab-content>\n            }\n        }\n        <ng-content select=\"[fdpIconTabBarFreeContent]\"></ng-content>\n    </div>\n</div>\n<ng-template #processType>\n    <fdp-icon-tab-bar-process-type\n        [tabs]=\"_tabs$()\"\n        [isRtl]=\"_rtl$()\"\n        [selectedUid]=\"_selectedUid()\"\n        (selectedUidChange)=\"_selectedUid.set($event)\"\n        [densityMode]=\"densityMode()\"\n        [colorAssociations]=\"colorAssociations()\"\n        (selected)=\"_selectItem($event)\"\n        [tabHeadingLevel]=\"tabHeadingLevel()\"\n    ></fdp-icon-tab-bar-process-type>\n</ng-template>\n<ng-template #filterType>\n    <fdp-icon-tab-bar-filter-type\n        [tabs]=\"_tabs$()\"\n        [showTotalTab]=\"showTotalTab()\"\n        [isRtl]=\"_rtl$()\"\n        [selectedUid]=\"_selectedUid()\"\n        (selectedUidChange)=\"_selectedUid.set($event)\"\n        [densityMode]=\"densityMode()\"\n        [colorAssociations]=\"colorAssociations()\"\n        (selected)=\"_selectItem($event)\"\n        [tabHeadingLevel]=\"tabHeadingLevel()\"\n    ></fdp-icon-tab-bar-filter-type>\n</ng-template>\n<ng-template #iconType>\n    <fdp-icon-tab-bar-icon-type\n        [showLabel]=\"iconTabType() === 'icon'\"\n        [tabs]=\"_tabs$()\"\n        [isRtl]=\"_rtl$()\"\n        [selectedUid]=\"_selectedUid()\"\n        (selectedUidChange)=\"_selectedUid.set($event)\"\n        [densityMode]=\"densityMode()\"\n        [colorAssociations]=\"colorAssociations()\"\n        (selected)=\"_selectItem($event)\"\n        (closeTab)=\"_closeTab($event)\"\n        [tabHeadingLevel]=\"tabHeadingLevel()\"\n    ></fdp-icon-tab-bar-icon-type>\n</ng-template>\n<ng-template #textType>\n    <fdp-icon-tab-bar-text-type\n        [tabs]=\"_tabs$()\"\n        [multiClick]=\"multiClick()\"\n        [layoutMode]=\"layoutMode()\"\n        [isRtl]=\"_rtl$()\"\n        [selectedUid]=\"_selectedUid()\"\n        (selectedUidChange)=\"_selectedUid.set($event)\"\n        [densityMode]=\"densityMode()\"\n        [enableTabReordering]=\"enableTabReordering()\"\n        [colorAssociations]=\"colorAssociations()\"\n        (reordered)=\"_onReorder($event)\"\n        (selected)=\"_selectItem($event)\"\n        (closeTab)=\"_closeTab($event)\"\n        [tabHeadingLevel]=\"tabHeadingLevel()\"\n    ></fdp-icon-tab-bar-text-type>\n</ng-template>\n", styles: ["@charset \"UTF-8\";.fd-icon-tab-bar__counter,.fd-icon-tab-bar__tag{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:var(--sapContent_LabelColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;line-height:1rem;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;white-space:nowrap}.fd-icon-tab-bar__counter:after,.fd-icon-tab-bar__counter:before,.fd-icon-tab-bar__tag:after,.fd-icon-tab-bar__tag:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__label:before,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__label[dir=rtl]:after,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__label:before,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__label[dir=rtl]:after,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__label:before,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__label[dir=rtl]:after,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__label:before,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__label[dir=rtl]:after,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon[dir=rtl]:after,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tag:before,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tag[dir=rtl]:after,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon[dir=rtl]:after,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tag:before,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tag[dir=rtl]:after,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon[dir=rtl]:after,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tag:before,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tag[dir=rtl]:after,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon[dir=rtl]:after,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tag:before,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tag[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__label:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__label:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__label:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__label:after,[dir=rtl] .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tag:after,[dir=rtl] .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tag:after,[dir=rtl] .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tag:after,[dir=rtl] .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tag:after{speak:none;font-family:SAP-icons;font-size:.75rem;-webkit-font-smoothing:antialiased;color:var(--fdIconTabBar_Tag_Icon_Color);display:var(--fdIcon_Tab_Bar_Semantic_Icon_Display)}.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon[dir=rtl]:after,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon[dir=rtl]:after,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon[dir=rtl]:after,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon:after{bottom:0;margin-block:0;margin-inline:0;position:absolute;width:.75rem}.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon[dir=rtl]:before,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tag[dir=rtl]:before,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon[dir=rtl]:before,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tag[dir=rtl]:before,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon[dir=rtl]:before,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tag[dir=rtl]:before,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon[dir=rtl]:before,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tag[dir=rtl]:before,[dir=rtl] .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon:before,[dir=rtl] .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tag:before,[dir=rtl] .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon:before,[dir=rtl] .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tag:before,[dir=rtl] .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon:before,[dir=rtl] .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tag:before,[dir=rtl] .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon:before,[dir=rtl] .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tag:before{content:none;margin-block:0;margin-inline:0}.fd-icon-tab-bar{--fdIconTabBar_Icon_Size:2.75rem;--fdIconTabBar_Icon_Font_Size:1.25rem;--fdIcon_Tab_Bar_Container_Badge_Size:.5rem;border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0}.fd-icon-tab-bar:after,.fd-icon-tab-bar:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__header{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-align:center;-ms-flex-align:center;align-items:center;background:var(--sapObjectHeader_Background);-webkit-box-shadow:var(--sapContent_HeaderShadow);box-shadow:var(--sapContent_HeaderShadow);list-style:none;min-height:2.75rem;position:relative}.fd-icon-tab-bar__header:after,.fd-icon-tab-bar__header:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__header:has(.fd-icon-tab-bar__item--overflow){overflow:visible}.fd-icon-tab-bar__panel{background:var(--sapBackgroundColor)}.fd-icon-tab-bar__item{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-margin-end:2rem;margin-inline-end:2rem}.fd-icon-tab-bar__item:after,.fd-icon-tab-bar__item:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__item--negative{--fdIconTabBar_Badge_Background:var(--fdIcon_Tab_Bar_Semantic_Badge_Color_Negative);--fdIconTabBar_Badge_Background_After:var(--fdIcon_Tab_Bar_Semantic_Badge_Color_Negative);--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Negative);--fdIconTabBar_Tag_Icon_Color:var(--sapNegativeElementColor);--fdIconTabBar_Tag_Icon_Content:\\e1ec;--fdIconTabBar_Icon_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Negative);--fdIconTabBar_Icon_Border_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Negative)}.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tag:before,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tag[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tag:after{-webkit-margin-end:.5rem;content:\"\\e1ec\";margin-inline-end:.5rem}.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tab.is-hover,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tab:hover{--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Negative)}.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tab.is-selected,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tab[aria-selected=true]{--fdIconTabBar_Selection_Bar_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Negative);--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Negative);--fdIconTabBar_Icon_Background:var(--fdIcon_Tab_Bar_Semantic_Background_Color_Negative)}.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tab.is-selected.is-hover,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tab.is-selected:hover,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tab[aria-selected=true].is-hover,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tab[aria-selected=true]:hover{--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Negative)}.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon{position:relative}.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon:before{content:\"\\e1ec\";right:-.9rem}.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon:after{content:\"\\e1ec\";left:-.9rem;right:auto}.fd-icon-tab-bar__item--positive{--fdIconTabBar_Badge_Background:var(--fdIcon_Tab_Bar_Semantic_Badge_Color_Positive);--fdIconTabBar_Badge_Background_After:var(--fdIcon_Tab_Bar_Semantic_Badge_Color_Positive);--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Positive);--fdIconTabBar_Tag_Icon_Color:var(--sapPositiveElementColor);--fdIconTabBar_Tag_Icon_Content:\\e1c1;--fdIconTabBar_Icon_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Positive);--fdIconTabBar_Icon_Border_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Positive)}.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tag:before,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tag[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tag:after{-webkit-margin-end:.5rem;content:\"\\e1c1\";margin-inline-end:.5rem}.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tab.is-hover,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tab:hover{--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Positive)}.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tab.is-selected,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tab[aria-selected=true]{--fdIconTabBar_Selection_Bar_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Positive);--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Positive);--fdIconTabBar_Icon_Background:var(--fdIcon_Tab_Bar_Semantic_Background_Color_Positive)}.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tab.is-selected.is-hover,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tab.is-selected:hover,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tab[aria-selected=true].is-hover,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tab[aria-selected=true]:hover{--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Positive)}.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon{position:relative}.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon:before{content:\"\\e1c1\";right:-.9rem}.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon:after{content:\"\\e1c1\";left:-.9rem;right:auto}.fd-icon-tab-bar__item--critical{--fdIconTabBar_Badge_Background:var(--fdIcon_Tab_Bar_Semantic_Badge_Color_Critical);--fdIconTabBar_Badge_Background_After:var(--fdIcon_Tab_Bar_Semantic_Badge_Color_Critical);--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Critical);--fdIconTabBar_Tag_Icon_Color:var(--sapCriticalElementColor);--fdIconTabBar_Tag_Icon_Content:\\e053;--fdIconTabBar_Icon_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Critical);--fdIconTabBar_Icon_Border_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Critical)}.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tag:before,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tag[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tag:after{-webkit-margin-end:.5rem;content:\"\\e053\";margin-inline-end:.5rem}.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tab.is-hover,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tab:hover{--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Critical)}.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tab.is-selected,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tab[aria-selected=true]{--fdIconTabBar_Selection_Bar_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Critical);--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Critical);--fdIconTabBar_Icon_Background:var(--fdIcon_Tab_Bar_Semantic_Background_Color_Critical)}.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tab.is-selected.is-hover,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tab.is-selected:hover,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tab[aria-selected=true].is-hover,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tab[aria-selected=true]:hover{--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Critical)}.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon{position:relative}.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon:before{content:\"\\e053\";right:-.9rem}.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon:after{content:\"\\e053\";left:-.9rem;right:auto}.fd-icon-tab-bar__item--informative{--fdIconTabBar_Badge_Background:var(--fdIcon_Tab_Bar_Semantic_Badge_Color_Informative);--fdIconTabBar_Badge_Background_After:var(--fdIcon_Tab_Bar_Semantic_Badge_Color_Informative);--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Informative);--fdIconTabBar_Tag_Icon_Color:var(--sapInformativeElementColor);--fdIconTabBar_Tag_Icon_Content:\\e289;--fdIconTabBar_Icon_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Informative);--fdIconTabBar_Icon_Border_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Informative)}.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tag:before,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tag[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tag:after{-webkit-margin-end:.5rem;content:\"\\e289\";margin-inline-end:.5rem}.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tab.is-hover,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tab:hover{--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Informative)}.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tab.is-selected,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tab[aria-selected=true]{--fdIconTabBar_Selection_Bar_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Informative);--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Informative);--fdIconTabBar_Icon_Background:var(--fdIcon_Tab_Bar_Semantic_Background_Color_Informative)}.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tab.is-selected.is-hover,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tab.is-selected:hover,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tab[aria-selected=true].is-hover,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tab[aria-selected=true]:hover{--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Informative)}.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon{position:relative}.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon:before{content:\"\\e289\";right:-.9rem}.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon:after{content:\"\\e289\";left:-.9rem;right:auto}.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab{-webkit-box-orient:horizontal;-webkit-box-direction:normal;-ms-flex-direction:row;flex-direction:row;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab.is-focus,.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab:focus{z-index:5}.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tab-container,.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tab-container{position:relative}.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tab-container .fd-icon-tab-bar__tag,.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tab-container .fd-icon-tab-bar__tag{outline:none}.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tab-container .fd-icon-tab-bar__tag:after,.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tab-container .fd-icon-tab-bar__tag:after{border:none}.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tab-container:after,.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tab-container:after{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);border-radius:var(--fdIcon_Tab_Bar_Text_Focus_Radius);content:\"\";display:block;inset:0;inset:var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset);position:absolute}.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__badge,.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__badge{right:-.4375rem;top:-.125rem}.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__badge[dir=rtl],.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__badge[dir=rtl],[dir=rtl] .fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__badge,[dir=rtl] .fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__badge{left:-.4375rem;right:auto}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;position:relative}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click .fd-icon-tab-bar__tab{padding-inline:.188rem 1.938rem}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click .fd-icon-tab-bar__tab .fd-icon-tab-bar__badge{right:1.625rem}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click .fd-icon-tab-bar__tab .fd-icon-tab-bar__badge[dir=rtl],[dir=rtl] .fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click .fd-icon-tab-bar__tab .fd-icon-tab-bar__badge{left:1.625rem;right:auto}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click .fd-icon-tab-bar__tab.is-selected+.fd-icon-tab-bar__popover .fd-icon-tab-bar__button-container:before,.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click .fd-icon-tab-bar__tab[aria-selected=true]+.fd-icon-tab-bar__popover .fd-icon-tab-bar__button-container:before{--fdIconTabBar_Multi_Click_Separator_Background:var(--sapContent_Selected_ForegroundColor)}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__tab{padding-inline:.188rem 3.938rem}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__popover{height:100%;max-width:1.5rem;min-width:1.5rem;right:2.188rem;width:1.5rem}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__popover[dir=rtl],[dir=rtl] .fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__popover{left:2.188rem;right:auto}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__popover .fd-icon-tab-bar__button-container{max-width:1.5rem;min-width:1.5rem;width:1.5rem}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__popover .fd-icon-tab-bar__button-container:before{display:var(--fdIcon_Tab_Bar_Multi_Click_Button_Separator_Display)}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--closable{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;position:relative}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__tab{padding-inline:.188rem 2.188rem}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container{max-width:2rem;min-width:2rem;position:absolute;right:0;width:2rem;z-index:10}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container[dir=rtl],[dir=rtl] .fd-icon-tab-bar__item.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container{left:0;right:auto}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container:before{display:none}.fd-icon-tab-bar__item--dnd-preview{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;font-size:var(--sapFontSize);line-height:var(--sapContent_LineHeight);opacity:.8;z-index:9999!important}.fd-icon-tab-bar__item--dnd-preview,.fd-icon-tab-bar__item--dnd-preview *{background:var(--sapObjectHeader_Background)}.fd-icon-tab-bar__item--dnd-hovered{background:var(--sapButton_Hover_Background);-webkit-box-shadow:inset 0 0 0 .125rem var(--sapButton_Hover_BorderColor);box-shadow:inset 0 0 0 .125rem var(--sapButton_Hover_BorderColor)}.fd-icon-tab-bar__item--dnd-hovered.fd-icon-tab-bar__list-item{background:var(--sapButton_Hover_Background)!important}.fd-icon-tab-bar__item--dnd-hovered.fd-icon-tab-bar__list-item .fd-icon-tab-bar__list-link{background:none}.fd-icon-tab-bar__item--dnd-separator{position:relative}.fd-icon-tab-bar__item--dnd-separator:before{background:var(--fdIcon_Tab_Bar_Separator_Color);content:\"\";height:100%;left:-.5rem;position:absolute;top:0;width:.125rem}[dir=rtl] .fd-icon-tab-bar__item--dnd-separator:before{left:auto;right:-.5rem}.fd-icon-tab-bar__item--dnd-separator:after{background:var(--sapObjectHeader_Background);border:.125rem solid var(--fdIcon_Tab_Bar_Separator_Color);border-radius:50%;content:\"\";height:.5rem;left:-.7rem;position:absolute;top:0;width:.5rem}[dir=rtl] .fd-icon-tab-bar__item--dnd-separator:after{left:auto;right:-.7rem}.fd-icon-tab-bar__item--dnd-separator.fd-icon-tab-bar__list-item:before{height:.125rem;left:.0625rem;right:unset;top:.125rem;width:100%}[dir=rtl] .fd-icon-tab-bar__item--dnd-separator.fd-icon-tab-bar__list-item:before{left:auto;right:.0625rem}.fd-icon-tab-bar__item--dnd-separator.fd-icon-tab-bar__list-item:after{left:0;right:unset;top:-.0625rem;z-index:1}[dir=rtl] .fd-icon-tab-bar__item--dnd-separator.fd-icon-tab-bar__list-item:after{left:auto;right:0}.fd-icon-tab-bar__item--dnd-separator-end{position:relative}.fd-icon-tab-bar__item--dnd-separator-end:before{background:var(--fdIcon_Tab_Bar_Separator_Color);content:\"\";height:100%;position:absolute;right:-.5rem;top:0;width:.125rem}[dir=rtl] .fd-icon-tab-bar__item--dnd-separator-end:before{left:-.5rem;right:auto}.fd-icon-tab-bar__item--dnd-separator-end:after{background:var(--sapObjectHeader_Background);border:.125rem solid var(--fdIcon_Tab_Bar_Separator_Color);border-radius:50%;content:\"\";height:.5rem;position:absolute;right:-.7rem;top:0;width:.5rem}[dir=rtl] .fd-icon-tab-bar__item--dnd-separator-end:after{left:-.7rem;right:auto}.fd-icon-tab-bar__item--dnd-separator-end.fd-icon-tab-bar__list-item:before{bottom:.125rem;height:.125rem;left:.0625rem;right:unset;top:unset;width:100%}[dir=rtl] .fd-icon-tab-bar__item--dnd-separator-end.fd-icon-tab-bar__list-item:before{left:auto;right:.0625rem}.fd-icon-tab-bar__item--dnd-separator-end.fd-icon-tab-bar__list-item:after{bottom:-.0625rem;left:0;right:unset;top:unset;z-index:1}[dir=rtl] .fd-icon-tab-bar__item--dnd-separator-end.fd-icon-tab-bar__list-item:after{left:auto;right:0}.fd-icon-tab-bar__button-container{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;height:2.75rem;max-height:2.75rem;max-width:1.5rem;min-height:2.75rem;min-width:1.5rem;position:relative;width:1.5rem}.fd-icon-tab-bar__button-container:after,.fd-icon-tab-bar__button-container:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__button-container:before{background:var(--fdIconTabBar_Multi_Click_Separator_Background);content:\"\";display:var(--fdIcon_Tab_Bar_Multi_Click_Button_Separator_Display);height:1rem;left:0;position:absolute;top:.75rem;width:.0625rem}.fd-icon-tab-bar__button-container[dir=rtl]:before,[dir=rtl] .fd-icon-tab-bar__button-container:before{left:auto;right:0}.fd-icon-tab-bar .fd-icon-tab-bar__button{border-radius:.25rem;font-family:var(--sapContent_IconFontFamily);height:1.5rem;min-width:1.5rem;width:1.5rem;--fdButton_Hover_Shadow:none}.fd-icon-tab-bar .fd-icon-tab-bar__button.is-focus,.fd-icon-tab-bar .fd-icon-tab-bar__button:focus{z-index:5}.fd-icon-tab-bar .fd-icon-tab-bar__button.is-focus:after,.fd-icon-tab-bar .fd-icon-tab-bar__button:focus:after{border-radius:var(--fdIcon_Tab_Bar_Multi_Click_Button_Focus_Radius)}.fd-icon-tab-bar__tab{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:start;-ms-flex-align:start;align-items:flex-start;cursor:pointer;min-width:2rem;padding-block:.875rem;padding-inline:.188rem;position:relative;text-decoration:none}.fd-icon-tab-bar__tab:after,.fd-icon-tab-bar__tab:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__tab.is-selected,.fd-icon-tab-bar__tab[aria-selected=true]{--fdIconTabBar_Icon_Background:var(--sapSelectedColor);--fdIconTabBar_Icon_Color:var(--sapContent_ContrastIconColor);--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Selection_Color);--fdIconTabBar_Arrow_Color:var(--sapContent_Selected_ForegroundColor);position:relative}.fd-icon-tab-bar__tab.is-selected:after,.fd-icon-tab-bar__tab[aria-selected=true]:after{background:var(--fdIconTabBar_Selection_Bar_Color);border-radius:.125rem .125rem 0 0;bottom:0;content:\"\";display:inline-block;height:var(--fdIcon_Tab_Bar_Selection_Bar_Height);left:0;position:absolute;-webkit-transition:all .125s ease-in;transition:all .125s ease-in;width:100%}.fd-icon-tab-bar__tab.is-selected[dir=rtl]:after,.fd-icon-tab-bar__tab[aria-selected=true][dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__tab.is-selected:after,[dir=rtl] .fd-icon-tab-bar__tab[aria-selected=true]:after{left:auto;right:0}.fd-icon-tab-bar__tab.is-selected+.fd-icon-tab-bar__popover .fd-icon-tab-bar__button,.fd-icon-tab-bar__tab[aria-selected=true]+.fd-icon-tab-bar__popover .fd-icon-tab-bar__button{--fdButtonColor:var(--sapContent_Selected_ForegroundColor)}.fd-icon-tab-bar__tab.is-hover,.fd-icon-tab-bar__tab:hover{--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Label_Hover_Color);--fdIconTabBar_Arrow_Color:var(--sapContent_Selected_ForegroundColor)}.fd-icon-tab-bar__tab.is-focus,.fd-icon-tab-bar__tab:focus{-webkit-box-shadow:none;box-shadow:none;outline:none;z-index:5}.fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__icon:after,.fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__icon:after{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);border-radius:var(--fdIcon_Tab_Bar_Icon_Focus_Radius);content:\"\";display:block;inset:0;inset:var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset);inset:var(--fdIcon_Tab_Bar_Icon_Focus_Offset) var(--fdIcon_Tab_Bar_Icon_Focus_Offset) var(--fdIcon_Tab_Bar_Icon_Focus_Offset) var(--fdIcon_Tab_Bar_Icon_Focus_Offset);position:absolute}.fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__container--filter:before,.fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tag:after,.fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__container--filter:before,.fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tag:after{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);border-radius:var(--fdIcon_Tab_Bar_Text_Focus_Radius);content:\"\";display:block;inset:0;inset:var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset);position:absolute}.fd-icon-tab-bar__tab-container{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.fd-icon-tab-bar__tab-container:after,.fd-icon-tab-bar__tab-container:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__arrow{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;height:1rem;margin-inline:var(--fdIcon_Tab_Bar_Single_Click_Icon_Container_Padding_Left) var(--fdIcon_Tab_Bar_Single_Click_Icon_Container_Padding_Right);width:var(--fdIcon_Tab_Bar_Single_Click_Icon_Container_Width)}.fd-icon-tab-bar__arrow:after,.fd-icon-tab-bar__arrow:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__arrow [class*=sap-icon]{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;color:var(--fdIconTabBar_Arrow_Color);font-size:1rem;height:1rem;line-height:1rem;width:1rem}.fd-icon-tab-bar__tag{color:var(--fdIconTabBar_Tag_Color);font-family:var(--fdIcon_Tab_Bar_Label_Font_Family);font-size:var(--sapFontSize);position:relative}.fd-icon-tab-bar__counter{-webkit-margin-after:.125rem;margin-block-end:.125rem}.fd-icon-tab-bar__icon{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;background:var(--fdIconTabBar_Icon_Background);border:var(--fdIcon_Tab_Bar_Icon_Border_Weight) solid var(--fdIconTabBar_Icon_Border_Color);border-radius:50%;height:var(--fdIconTabBar_Icon_Size);margin-inline:.188rem;position:relative;width:var(--fdIconTabBar_Icon_Size)}.fd-icon-tab-bar__icon:after,.fd-icon-tab-bar__icon:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__icon [class*=sap-icon],.fd-icon-tab-bar__icon[class*=sap-icon]{color:var(--fdIconTabBar_Icon_Color);display:-webkit-inline-box;display:-ms-inline-flexbox;display:inline-flex;font-size:var(--fdIconTabBar_Icon_Font_Size);-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.fd-icon-tab-bar__details{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:start;-ms-flex-align:start;align-items:flex-start;-webkit-box-pack:end;-ms-flex-pack:end;height:2.75rem;justify-content:flex-end;max-width:7.5rem;padding-block:.375rem .25rem;padding-inline:.125rem .25rem}.fd-icon-tab-bar__details:after,.fd-icon-tab-bar__details:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__label{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:var(--fdIcon_Tab_Bar_Label_Color);font-family:var(--sapFontFamily);font-family:var(--fdIcon_Tab_Bar_Label_Font_Family);font-size:var(--sapFontSize);font-size:var(--sapFontSmallSize);font-weight:400;forced-color-adjust:none;line-height:normal;line-height:.875rem;margin-block:0;margin-inline:0;overflow:hidden;padding-block:0;padding-inline:0;text-overflow:ellipsis;white-space:nowrap}.fd-icon-tab-bar__label:after,.fd-icon-tab-bar__label:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__separator{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;padding-inline:.5rem}.fd-icon-tab-bar__separator:after,.fd-icon-tab-bar__separator:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__separator [class*=sap-icon],.fd-icon-tab-bar__separator[class*=sap-icon]{color:var(--sapContent_NonInteractiveIconColor);font-size:1rem}.fd-icon-tab-bar__separator [class*=sap-icon][dir=rtl],.fd-icon-tab-bar__separator[class*=sap-icon][dir=rtl],[dir=rtl] .fd-icon-tab-bar__separator [class*=sap-icon],[dir=rtl] .fd-icon-tab-bar__separator[class*=sap-icon]{-webkit-transform:rotate(180deg);transform:rotate(180deg)}.fd-icon-tab-bar__container{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;position:relative}.fd-icon-tab-bar__container:after,.fd-icon-tab-bar__container:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__container .fd-icon-tab-bar__badge{border:.125rem solid var(--sapObjectHeader_Background);border-radius:50%;height:var(--fdIcon_Tab_Bar_Container_Badge_Size);right:.188rem;top:0;width:var(--fdIcon_Tab_Bar_Container_Badge_Size)}.fd-icon-tab-bar__container .fd-icon-tab-bar__badge[dir=rtl],[dir=rtl] .fd-icon-tab-bar__container .fd-icon-tab-bar__badge{left:.188rem;right:auto}.fd-icon-tab-bar__container .fd-icon-tab-bar__counter{left:2.75rem;position:absolute;top:0}.fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],[dir=rtl] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter{left:auto;right:2.75rem}.fd-icon-tab-bar__container--filter{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center;height:2rem}.fd-icon-tab-bar__container--filter:after{background:var(--sapGroup_TitleBorderColor);content:\"\";height:2rem;padding-block:0;padding-inline:0;position:absolute;right:-.5625rem;width:.0625rem}.fd-icon-tab-bar__container--filter[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__container--filter:after{left:-.5625rem;right:auto}.fd-icon-tab-bar__filter-counter,.fd-icon-tab-bar__filter-label{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:var(--sapContent_LabelColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-size:var(--sapFontSmallSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0}.fd-icon-tab-bar__filter-counter:after,.fd-icon-tab-bar__filter-counter:before,.fd-icon-tab-bar__filter-label:after,.fd-icon-tab-bar__filter-label:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__filter-counter{font-size:1.5rem;-webkit-margin-end:.5rem;margin-inline-end:.5rem}.fd-icon-tab-bar__overflow{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;padding-inline:.5rem;-webkit-box-align:center;-ms-flex-align:center;align-items:center;background:var(--fdIconTabBar_Overflow_Button_Background);border:.0625rem solid var(--fdIconTabBar_Overflow_Button_Border_Color);border-radius:var(--fdIcon_Tab_Bar_Overflow_Button_Border_Radius);cursor:pointer;gap:.375rem;height:1.5rem;position:relative}.fd-icon-tab-bar__overflow:after,.fd-icon-tab-bar__overflow:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__overflow [class*=sap-icon]{color:var(--fdIconTabBar_Overflow_Button_Icon_Color);font-family:var(--sapContent_IconFontFamily);font-size:1rem}.fd-icon-tab-bar__overflow-text{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:var(--fdIconTabBar_Overflow_Button_Text_Color);font-family:var(--sapFontFamily);font-family:var(--fdIcon_Tab_Bar_Overflow_Button_Font_Family);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;line-height:1.5rem;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0}.fd-icon-tab-bar__overflow-text:after,.fd-icon-tab-bar__overflow-text:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__overflow.is-hover,.fd-icon-tab-bar__overflow:hover{--fdIconTabBar_Overflow_Button_Background:var(--sapButton_Hover_Background);--fdIconTabBar_Overflow_Button_Border_Color:var(--sapButton_Hover_BorderColor);--fdIconTabBar_Overflow_Button_Text_Color:var(--fdIcon_Tab_Bar_Overflow_Button_Text_Color_Hover);--fdIconTabBar_Overflow_Button_Icon_Color:var(--fdIcon_Tab_Bar_Overflow_Button_Icon_Color_Hover)}.fd-icon-tab-bar__overflow.is-active,.fd-icon-tab-bar__overflow:active{--fdIconTabBar_Overflow_Button_Background:var(--sapButton_Active_Background);--fdIconTabBar_Overflow_Button_Border_Color:var(--sapButton_Active_BorderColor);--fdIconTabBar_Overflow_Button_Text_Color:var(--sapButton_Active_TextColor);--fdIconTabBar_Overflow_Button_Icon_Color:var(--sapButton_Active_TextColor)}.fd-icon-tab-bar__overflow.is-focus,.fd-icon-tab-bar__overflow:focus{outline:none;z-index:5}.fd-icon-tab-bar__overflow.is-focus:after,.fd-icon-tab-bar__overflow:focus:after{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);border-radius:var(--fdIcon_Tab_Bar_Overflow_Button_Border_Radius);content:\"\";display:block;inset:var(--fdIcon_Tab_Bar_Overflow_Button_Focus_Offset) var(--fdIcon_Tab_Bar_Overflow_Button_Focus_Offset) var(--fdIcon_Tab_Bar_Overflow_Button_Focus_Offset) var(--fdIcon_Tab_Bar_Overflow_Button_Focus_Offset);position:absolute}.fd-icon-tab-bar__overflow.is-focus.is-active:after,.fd-icon-tab-bar__overflow.is-focus:active:after,.fd-icon-tab-bar__overflow:focus.is-active:after,.fd-icon-tab-bar__overflow:focus:active:after{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_ContrastFocusColor)}.fd-icon-tab-bar .fd-icon-tab-bar__popover{position:absolute;right:.188rem;top:0}.fd-icon-tab-bar .fd-icon-tab-bar__popover[dir=rtl],[dir=rtl] .fd-icon-tab-bar .fd-icon-tab-bar__popover{left:.188rem;right:auto}.fd-icon-tab-bar .fd-icon-tab-bar__popover-body{-webkit-margin-end:-.25rem;margin-inline-end:-.25rem;-webkit-margin-before:-.25rem;margin-block-start:-.25rem;z-index:10}.fd-icon-tab-bar .fd-icon-tab-bar__popover .fd-icon-tab-bar__button{--fdButtonColor:var(--sapTextColor)}.fd-icon-tab-bar__badge{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;background:var(--fdIconTabBar_Badge_Background);border:.0625rem solid var(--sapObjectHeader_Background);border-radius:50%;-webkit-box-sizing:content-box;box-sizing:content-box;height:.375rem;position:absolute;right:-.2505rem;top:.75rem;width:.375rem;z-index:5}.fd-icon-tab-bar__badge:after,.fd-icon-tab-bar__badge:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__badge[dir=rtl],[dir=rtl] .fd-icon-tab-bar__badge{left:-.2505rem;right:auto}.fd-icon-tab-bar__badge--animated{-webkit-animation-delay:0s,1.5s;animation-delay:0s,1.5s;-webkit-animation-duration:1.5s,.75s;animation-duration:1.5s,.75s;-webkit-animation-iteration-count:1,1;animation-iteration-count:1,1;-webkit-animation-name:pulse-appear,pulse-disappear;animation-name:pulse-appear,pulse-disappear;-webkit-animation-timing-function:ease-in,ease-out;animation-timing-function:ease-in,ease-out;-webkit-transform:scale(0);transform:scale(0)}.fd-icon-tab-bar__badge--animated:after{-webkit-animation:pulse-after 1s ease-in;animation:pulse-after 1s ease-in;-webkit-animation-delay:1s;animation-delay:1s;-webkit-animation-iteration-count:1;animation-iteration-count:1;background:var(--fdIconTabBar_Badge_Background_After);border-radius:50%;content:\"\";height:.75rem;opacity:0;position:absolute;-webkit-transform:scale(0);transform:scale(0);width:.75rem}@-webkit-keyframes pulse-appear{0%{-webkit-transform:scale(.25);transform:scale(.25)}64%{-webkit-transform:scale(1.25);transform:scale(1.25)}71%{-webkit-transform:scale(1);transform:scale(1)}to{-webkit-transform:scale(1);transform:scale(1)}}@keyframes pulse-appear{0%{-webkit-transform:scale(.25);transform:scale(.25)}64%{-webkit-transform:scale(1.25);transform:scale(1.25)}71%{-webkit-transform:scale(1);transform:scale(1)}to{-webkit-transform:scale(1);transform:scale(1)}}@-webkit-keyframes pulse-disappear{0%{-webkit-transform:scale(1);transform:scale(1)}to{-webkit-transform:scale(0);transform:scale(0)}}@keyframes pulse-disappear{0%{-webkit-transform:scale(1);transform:scale(1)}to{-webkit-transform:scale(0);transform:scale(0)}}@-webkit-keyframes pulse-after{0%{opacity:0;-webkit-transform:scale(0);transform:scale(0)}71%{opacity:.2;-webkit-transform:scale(1);transform:scale(1)}80%{opacity:.2;-webkit-transform:scale(1.17);transform:scale(1.17)}86%{opacity:.1;-webkit-transform:scale(1.33);transform:scale(1.33)}93%{opacity:0;-webkit-transform:scale(1.5);transform:scale(1.5)}to{opacity:0;-webkit-transform:scale(0);transform:scale(0)}}@keyframes pulse-after{0%{opacity:0;-webkit-transform:scale(0);transform:scale(0)}71%{opacity:.2;-webkit-transform:scale(1);transform:scale(1)}80%{opacity:.2;-webkit-transform:scale(1.17);transform:scale(1.17)}86%{opacity:.1;-webkit-transform:scale(1.33);transform:scale(1.33)}93%{opacity:0;-webkit-transform:scale(1.5);transform:scale(1.5)}to{opacity:0;-webkit-transform:scale(0);transform:scale(0)}}.fd-icon-tab-bar__icon-separator{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-inline:0;-webkit-box-align:end;-ms-flex-align:end;align-items:flex-end;-webkit-box-pack:center;-ms-flex-pack:center;height:1rem;justify-content:center;padding-block:0;padding-inline:.5rem}.fd-icon-tab-bar__icon-separator:after,.fd-icon-tab-bar__icon-separator:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__icon-separator [class*=sap-icon]{color:var(--sapContent_NonInteractiveIconColor);font-size:1rem;height:1rem;line-height:1rem;width:1rem}.fd-icon-tab-bar__icon-separator[dir=rtl] [class*=sap-icon],[dir=rtl] .fd-icon-tab-bar__icon-separator [class*=sap-icon]{-webkit-transform:rotate(180deg);transform:rotate(180deg)}.fd-icon-tab-bar__line-separator{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-margin-before:.25rem;border-bottom:.0625rem solid var(--sapGroup_TitleBorderColor);margin-block-start:.25rem}.fd-icon-tab-bar__line-separator:after,.fd-icon-tab-bar__line-separator:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__line-separator:first-child{-webkit-margin-before:.5rem;margin-block-start:.5rem;margin-inline:1rem 1rem}.fd-icon-tab-bar__list{width:100%}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"1\"]{--fdList_Navigation_Link_Padding_Left:1rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"1\"]+.fd-icon-tab-bar__line-separator{margin-inline:1.5rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"2\"]{--fdList_Navigation_Link_Padding_Left:1.5rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"2\"]+.fd-icon-tab-bar__line-separator{margin-inline:2rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"3\"]{--fdList_Navigation_Link_Padding_Left:2rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"3\"]+.fd-icon-tab-bar__line-separator{margin-inline:2.5rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"4\"]{--fdList_Navigation_Link_Padding_Left:2.5rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"4\"]+.fd-icon-tab-bar__line-separator{margin-inline:3rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"5\"]{--fdList_Navigation_Link_Padding_Left:3rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"5\"]+.fd-icon-tab-bar__line-separator{margin-inline:3.5rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"6\"]{--fdList_Navigation_Link_Padding_Left:3.5rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"6\"]+.fd-icon-tab-bar__line-separator{margin-inline:4rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"7\"]{--fdList_Navigation_Link_Padding_Left:4rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"7\"]+.fd-icon-tab-bar__line-separator{margin-inline:4.5rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"8\"]{--fdList_Navigation_Link_Padding_Left:4.5rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"8\"]+.fd-icon-tab-bar__line-separator{margin-inline:5rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"9\"]{--fdList_Navigation_Link_Padding_Left:5rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"9\"]+.fd-icon-tab-bar__line-separator{margin-inline:5.5rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"10\"]{--fdList_Navigation_Link_Padding_Left:5.5rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"10\"]+.fd-icon-tab-bar__line-separator{margin-inline:6rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item span{-webkit-box-flex:0;-ms-flex:0 1 auto;flex:0 1 auto}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item .fd-icon-tab-bar__badge{position:static;-webkit-margin-after:.625rem;margin-block-end:.625rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item .fd-icon-tab-bar__badge [class*=sap-icon]{-webkit-margin-after:0;margin-block-end:0}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--negative .fd-icon-tab-bar__list-item-counter,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--negative .fd-icon-tab-bar__list-item-icon [class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--negative .fd-icon-tab-bar__list-item-icon[class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--negative .fd-icon-tab-bar__list-item-title{color:var(--fdIcon_Tab_Bar_Semantic_Color_Negative)}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--positive .fd-icon-tab-bar__list-item-counter,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--positive .fd-icon-tab-bar__list-item-icon [class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--positive .fd-icon-tab-bar__list-item-icon[class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--positive .fd-icon-tab-bar__list-item-title{color:var(--fdIcon_Tab_Bar_Semantic_Color_Positive)}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--critical .fd-icon-tab-bar__list-item-counter,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--critical .fd-icon-tab-bar__list-item-icon [class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--critical .fd-icon-tab-bar__list-item-icon[class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--critical .fd-icon-tab-bar__list-item-title{color:var(--fdIcon_Tab_Bar_Semantic_Color_Critical)}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--informative .fd-icon-tab-bar__list-item-counter,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--informative .fd-icon-tab-bar__list-item-icon [class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--informative .fd-icon-tab-bar__list-item-icon[class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--informative .fd-icon-tab-bar__list-item-title{color:var(--fdIcon_Tab_Bar_Semantic_Color_Informative)}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--closable .fd-icon-tab-bar__list-link{--fdList_Navigation_Link_Padding_Right:2rem;position:relative}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--closable .fd-icon-tab-bar__button-container{max-width:2rem;min-width:2rem;position:absolute;right:0;width:2rem;z-index:10}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--closable .fd-icon-tab-bar__button-container[dir=rtl],[dir=rtl] .fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--closable .fd-icon-tab-bar__button-container{left:0;right:auto}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--closable .fd-icon-tab-bar__button-container:before{display:none}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item-icon-container{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;width:1.25rem;-webkit-margin-end:.25rem;margin-inline-end:.25rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link.is-selected,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link[aria-selected=true]{border-bottom:none}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link.is-active .fd-icon-tab-bar__list-item-counter,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link.is-active .fd-icon-tab-bar__list-item-icon [class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link.is-active .fd-icon-tab-bar__list-item-icon[class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link.is-active .fd-icon-tab-bar__list-item-title,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link:active .fd-icon-tab-bar__list-item-counter,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link:active .fd-icon-tab-bar__list-item-icon [class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link:active .fd-icon-tab-bar__list-item-icon[class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link:active .fd-icon-tab-bar__list-item-title{color:var(--sapList_Active_TextColor)}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link.is-focus,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link:focus{z-index:5}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link.is-focus:before,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link:focus:before{border-radius:.375rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link span{font-size:var(--sapFontSize)}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item-icon [class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item-icon [class*=sap-icon]:last-child{font-size:1rem;margin-block:0;margin-inline:0;width:auto}.fd-icon-tab-bar__list .fd-icon-tab-bar__line-separator:first-child{padding-inline:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__line-separator:first-child:after{-webkit-margin-before:.75rem;margin-block-start:.75rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__icon-separator:first-child{height:1.5rem;padding-inline:1rem}.fd-icon-tab-bar__list[dir=rtl] .fd-icon-tab-bar__list-item-icon,[dir=rtl] .fd-icon-tab-bar__list .fd-icon-tab-bar__list-item-icon{-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center}.fd-icon-tab-bar--transparent .fd-icon-tab-bar__header,.fd-icon-tab-bar--transparent .fd-icon-tab-bar__panel{background:transparent}.fd-icon-tab-bar--translucent .fd-icon-tab-bar__header{background:var(--sapObjectHeader_Background)}.fd-icon-tab-bar--translucent .fd-icon-tab-bar__panel{background:var(--sapGroup_ContentBackground)}.fd-icon-tab-bar[class*=-compact],.fd-icon-tab-bar[class*=-condensed],[class*=-compact] .fd-icon-tab-bar:not([class*=-cozy]),[class*=-condensed] .fd-icon-tab-bar:not([class*=-cozy]){--fdIconTabBar_Icon_Size:2rem;--fdIconTabBar_Icon_Font_Size:1rem;--fdIcon_Tab_Bar_Container_Badge_Size:.375rem}.fd-icon-tab-bar[class*=-compact] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,.fd-icon-tab-bar[class*=-condensed] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[class*=-compact] .fd-icon-tab-bar:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[class*=-condensed] .fd-icon-tab-bar:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter{left:2rem}.fd-icon-tab-bar[class*=-compact] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],.fd-icon-tab-bar[class*=-condensed] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],[class*=-compact] .fd-icon-tab-bar:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],[class*=-condensed] .fd-icon-tab-bar:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],[dir=rtl] .fd-icon-tab-bar[class*=-compact] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[dir=rtl] .fd-icon-tab-bar[class*=-condensed] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[dir=rtl] [class*=-compact] .fd-icon-tab-bar:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[dir=rtl] [class*=-condensed] .fd-icon-tab-bar:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter{left:auto;right:2rem}.fd-icon-tab-bar--counters .fd-icon-tab-bar__tab{height:3.875rem;-webkit-box-pack:end;-ms-flex-pack:end;justify-content:flex-end}.fd-icon-tab-bar--counters .fd-icon-tab-bar__badge{top:1.9375rem}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__tab{padding-block:1rem;padding-inline:0;position:relative;-webkit-box-orient:horizontal;-webkit-box-direction:normal;-ms-flex-direction:row;flex-direction:row;margin-inline:.75rem}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item{margin-inline:.25rem}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item:first-child{margin-block:0;margin-inline:0 .25rem}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item:first-child .fd-icon-tab-bar__tab{margin-inline:0 .75rem}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item--closable .fd-icon-tab-bar__tab{-webkit-padding-end:2rem;padding-inline-end:2rem}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container{right:.75rem}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container[dir=rtl],[dir=rtl] .fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container{left:.75rem;right:auto}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__container .fd-icon-tab-bar__badge{right:.2505rem}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__container .fd-icon-tab-bar__badge[dir=rtl],[dir=rtl] .fd-icon-tab-bar--icon-only .fd-icon-tab-bar__container .fd-icon-tab-bar__badge{left:.2505rem;right:auto}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__container .fd-icon-tab-bar__counter{left:2.938rem}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],[dir=rtl] .fd-icon-tab-bar--icon-only .fd-icon-tab-bar__container .fd-icon-tab-bar__counter{left:auto;right:2.938rem}.fd-icon-tab-bar--icon-only[class*=-compact] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,.fd-icon-tab-bar--icon-only[class*=-condensed] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[class*=-compact] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[class*=-condensed] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter{left:2.188rem}.fd-icon-tab-bar--icon-only[class*=-compact] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],.fd-icon-tab-bar--icon-only[class*=-condensed] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],[class*=-compact] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],[class*=-condensed] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],[dir=rtl] .fd-icon-tab-bar--icon-only[class*=-compact] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[dir=rtl] .fd-icon-tab-bar--icon-only[class*=-condensed] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[dir=rtl] [class*=-compact] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[dir=rtl] [class*=-condensed] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter{left:auto;right:2.188rem}.fd-icon-tab-bar--icon-only[class*=-compact] .fd-icon-tab-bar__container .fd-icon-tab-bar__badge,.fd-icon-tab-bar--icon-only[class*=-condensed] .fd-icon-tab-bar__container .fd-icon-tab-bar__badge,[class*=-compact] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__badge,[class*=-condensed] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__badge{right:.188rem}.fd-icon-tab-bar--icon-only[class*=-compact] .fd-icon-tab-bar__container .fd-icon-tab-bar__badge[dir=rtl],.fd-icon-tab-bar--icon-only[class*=-condensed] .fd-icon-tab-bar__container .fd-icon-tab-bar__badge[dir=rtl],[class*=-compact] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__badge[dir=rtl],[class*=-condensed] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__badge[dir=rtl],[dir=rtl] .fd-icon-tab-bar--icon-only[class*=-compact] .fd-icon-tab-bar__container .fd-icon-tab-bar__badge,[dir=rtl] .fd-icon-tab-bar--icon-only[class*=-condensed] .fd-icon-tab-bar__container .fd-icon-tab-bar__badge,[dir=rtl] [class*=-compact] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__badge,[dir=rtl] [class*=-condensed] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__badge{left:.188rem;right:auto}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item{margin-block:0;margin-inline:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative[dir=rtl] .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon:after{content:none;margin-block:0;margin-inline:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__label:before{-webkit-margin-end:.5rem;color:var(--sapNegativeElementColor);content:\"\\e1ec\";margin-inline-end:.5rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__label[dir=rtl]:before,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__label:before{content:none;margin-block:0;margin-inline:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__label[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__label:after{-webkit-margin-end:.5rem;color:var(--sapNegativeElementColor);content:\"\\e1ec\";margin-inline-end:.5rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive[dir=rtl] .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon:after{content:none;margin-block:0;margin-inline:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__label:before{-webkit-margin-end:.5rem;color:var(--sapPositiveElementColor);content:\"\\e1c1\";margin-inline-end:.5rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__label[dir=rtl]:before,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__label:before{content:none;margin-block:0;margin-inline:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__label[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__label:after{-webkit-margin-end:.5rem;color:var(--sapPositiveElementColor);content:\"\\e1c1\";margin-inline-end:.5rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical[dir=rtl] .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon:after{content:none;margin-block:0;margin-inline:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__label:before{-webkit-margin-end:.5rem;color:var(--sapCriticalElementColor);content:\"\\e053\";margin-inline-end:.5rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__label[dir=rtl]:before,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__label:before{content:none;margin-block:0;margin-inline:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__label[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__label:after{-webkit-margin-end:.5rem;color:var(--sapCriticalElementColor);content:\"\\e053\";margin-inline-end:.5rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative[dir=rtl] .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon:after{content:none;margin-block:0;margin-inline:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__label:before{-webkit-margin-end:.5rem;color:var(--sapInformativeElementColor);content:\"\\e289\";margin-inline-end:.5rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__label[dir=rtl]:before,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__label:before{content:none;margin-block:0;margin-inline:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__label[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__label:after{-webkit-margin-end:.5rem;color:var(--sapInformativeElementColor);content:\"\\e289\";margin-inline-end:.5rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--closable .fd-icon-tab-bar__tab{-webkit-padding-end:2rem;padding-inline-end:2rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container{right:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container[dir=rtl],[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container{left:0;right:auto}.fd-icon-tab-bar--icon .fd-icon-tab-bar__tab{padding-block:1rem;padding-inline:0;-webkit-box-orient:horizontal;-webkit-box-direction:normal;-ms-flex-direction:row;flex-direction:row}.fd-icon-tab-bar--icon .fd-icon-tab-bar__tab:after{width:3.126rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__counter{-webkit-margin-after:0;font-size:var(--sapFontSmallSize);line-height:.875rem;margin-block-end:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__counter+.fd-icon-tab-bar__label{-webkit-margin-before:.375rem;margin-block-start:.375rem}.fd-icon-tab-bar--icon[class*=-compact] .fd-icon-tab-bar__details,.fd-icon-tab-bar--icon[class*=-condensed] .fd-icon-tab-bar__details,[class*=-compact] .fd-icon-tab-bar--icon:not([class*=-cozy]) .fd-icon-tab-bar__details,[class*=-condensed] .fd-icon-tab-bar--icon:not([class*=-cozy]) .fd-icon-tab-bar__details{height:2rem;padding-block:0}.fd-icon-tab-bar--icon[class*=-compact] .fd-icon-tab-bar__tab:after,.fd-icon-tab-bar--icon[class*=-condensed] .fd-icon-tab-bar__tab:after,[class*=-compact] .fd-icon-tab-bar--icon:not([class*=-cozy]) .fd-icon-tab-bar__tab:after,[class*=-condensed] .fd-icon-tab-bar--icon:not([class*=-cozy]) .fd-icon-tab-bar__tab:after{width:2.376rem}.fd-icon-tab-bar--icon[class*=-compact] .fd-icon-tab-bar__counter+.fd-icon-tab-bar__label,.fd-icon-tab-bar--icon[class*=-condensed] .fd-icon-tab-bar__counter+.fd-icon-tab-bar__label,[class*=-compact] .fd-icon-tab-bar--icon:not([class*=-cozy]) .fd-icon-tab-bar__counter+.fd-icon-tab-bar__label,[class*=-condensed] .fd-icon-tab-bar--icon:not([class*=-cozy]) .fd-icon-tab-bar__counter+.fd-icon-tab-bar__label{-webkit-margin-before:.25rem;margin-block-start:.25rem}.fd-icon-tab-bar--process .fd-icon-tab-bar__item{margin-inline:0}.fd-icon-tab-bar--filter .fd-icon-tab-bar__item{margin-inline:.25rem}.fd-icon-tab-bar--filter .fd-icon-tab-bar__item:first-child{margin-inline:0 1.063rem}.fd-icon-tab-bar--filter .fd-icon-tab-bar__item:first-child .fd-icon-tab-bar__tab{padding-block:1.625rem 1.875rem;position:relative}.fd-icon-tab-bar--filter .fd-icon-tab-bar__item:nth-child(2){margin-block:0;margin-inline:0 .25rem}.fd-icon-tab-bar--filter .fd-icon-tab-bar__tab{margin-inline:0;padding-block:.875rem .625rem;padding-inline:.188rem;position:relative;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.fd-icon-tab-bar--filter .fd-icon-tab-bar__icon{margin-inline:0}.fd-icon-tab-bar--filter .fd-icon-tab-bar__label{text-align:center;width:5rem;-webkit-margin-before:.375rem;margin-block-start:.375rem}.fd-icon-tab-bar--filter[class*=-compact] .fd-icon-tab-bar__item:first-child .fd-icon-tab-bar__tab,.fd-icon-tab-bar--filter[class*=-condensed] .fd-icon-tab-bar__item:first-child .fd-icon-tab-bar__tab,[class*=-compact] .fd-icon-tab-bar--filter:not([class*=-cozy]) .fd-icon-tab-bar__item:first-child .fd-icon-tab-bar__tab,[class*=-condensed] .fd-icon-tab-bar--filter:not([class*=-cozy]) .fd-icon-tab-bar__item:first-child .fd-icon-tab-bar__tab{padding-block:1.375rem}.fd-icon-tab-bar--filter[class*=-compact] .fd-icon-tab-bar__badge,.fd-icon-tab-bar--filter[class*=-condensed] .fd-icon-tab-bar__badge,[class*=-compact] .fd-icon-tab-bar--filter:not([class*=-cozy]) .fd-icon-tab-bar__badge,[class*=-condensed] .fd-icon-tab-bar--filter:not([class*=-cozy]) .fd-icon-tab-bar__badge{right:.063rem}.fd-icon-tab-bar--filter[class*=-compact] .fd-icon-tab-bar__badge[dir=rtl],.fd-icon-tab-bar--filter[class*=-condensed] .fd-icon-tab-bar__badge[dir=rtl],[class*=-compact] .fd-icon-tab-bar--filter:not([class*=-cozy]) .fd-icon-tab-bar__badge[dir=rtl],[class*=-condensed] .fd-icon-tab-bar--filter:not([class*=-cozy]) .fd-icon-tab-bar__badge[dir=rtl],[dir=rtl] .fd-icon-tab-bar--filter[class*=-compact] .fd-icon-tab-bar__badge,[dir=rtl] .fd-icon-tab-bar--filter[class*=-condensed] .fd-icon-tab-bar__badge,[dir=rtl] [class*=-compact] .fd-icon-tab-bar--filter:not([class*=-cozy]) .fd-icon-tab-bar__badge,[dir=rtl] [class*=-condensed] .fd-icon-tab-bar--filter:not([class*=-cozy]) .fd-icon-tab-bar__badge{left:.063rem;right:auto}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__header{background:var(--sapShell_Navigation_Background);-webkit-box-shadow:var(--sapShell_Shadow);box-shadow:var(--sapShell_Shadow)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__item{color:var(--sapShell_Navigation_TextColor)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab [class*=sap-icon]{color:var(--sapShell_Navigation_TextColor);font-size:1rem;height:1rem;line-height:1;-webkit-margin-end:.5rem;margin-inline-end:.5rem}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab .fd-icon-tab-bar__tag{color:var(--sapShell_Navigation_TextColor)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-selected .fd-icon-tab-bar__tag,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-selected [class*=sap-icon],.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab[aria-selected=true] .fd-icon-tab-bar__tag,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab[aria-selected=true] [class*=sap-icon]{color:var(--sapShell_Navigation_Selected_TextColor)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-selected:after,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab[aria-selected=true]:after{background:var(--sapShell_Navigation_SelectedColor)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-selected+.fd-icon-tab-bar__popover .fd-icon-tab-bar__button,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab[aria-selected=true]+.fd-icon-tab-bar__popover .fd-icon-tab-bar__button{--fdButtonColor:var(--fdIconTabBar_Button_Shell_Color_Active)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-hover .fd-icon-tab-bar__tag,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-hover [class*=sap-icon],.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab:hover .fd-icon-tab-bar__tag,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab:hover [class*=sap-icon]{color:var(--sapShell_Navigation_Selected_TextColor)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-focus,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab:focus{z-index:5}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tag:after,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tag:after{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapShell_Navigation_TextColor);content:\"\";display:block;inset:0;inset:var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset);position:absolute}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tab-container,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tab-container{position:relative}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tab-container .fd-icon-tab-bar__tag,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tab-container .fd-icon-tab-bar__tag{outline:none}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tab-container .fd-icon-tab-bar__tag:after,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tab-container .fd-icon-tab-bar__tag:after{border:none}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tab-container:after,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tab-container:after{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapShell_Navigation_TextColor);content:\"\";display:block;inset:0;inset:var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset);position:absolute}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab .fd-icon-tab-bar__arrow [class*=sap-icon]{margin-block:0;margin-inline:0}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__overflow{--fdIconTabBar_Overflow_Button_Background:var(--fdIconTabBar_Overflow_Button_Shell_Background);--fdIconTabBar_Overflow_Button_Border_Color:var(--fdIconTabBar_Overflow_Button_Shell_Border_Color);--fdIconTabBar_Overflow_Button_Text_Color:var(--fdIconTabBar_Overflow_Button_Shell_Text_Color);--fdIconTabBar_Overflow_Button_Icon_Color:var(--fdIconTabBar_Overflow_Button_Shell_Icon_Color)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__overflow.is-hover,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__overflow:hover{--fdIconTabBar_Overflow_Button_Background:var(--fdIconTabBar_Overflow_Button_Shell_Background_Hover);--fdIconTabBar_Overflow_Button_Border_Color:var( --fdIconTabBar_Overflow_Button_Shell_Border_Color_Hover );--fdIconTabBar_Overflow_Button_Text_Color:var(--fdIconTabBar_Overflow_Button_Shell_Text_Color_Hover);--fdIconTabBar_Overflow_Button_Icon_Color:var(--fdIconTabBar_Overflow_Button_Shell_Icon_Color_Hover)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__overflow.is-active,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__overflow:active{--fdIconTabBar_Overflow_Button_Background:var(--fdIconTabBar_Overflow_Button_Shell_Background_Active);--fdIconTabBar_Overflow_Button_Border_Color:var( --fdIconTabBar_Overflow_Button_Shell_Border_Color_Active );--fdIconTabBar_Overflow_Button_Text_Color:var(--fdIconTabBar_Overflow_Button_Shell_Text_Color_Active);--fdIconTabBar_Overflow_Button_Icon_Color:var(--fdIconTabBar_Overflow_Button_Shell_Icon_Color_Active)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__overflow.is-focus,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__overflow:focus{z-index:5}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__overflow.is-focus:after,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__overflow:focus:after{border-color:var(--fdIconTabBar_Overflow_Button_Shell_Focus_Color)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button{--fdButtonColor:var(--fdIconTabBar_Button_Shell_Color)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button.is-hover,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button:hover{--fdButtonColor:var(--fdIconTabBar_Button_Shell_Color_Hover);--fdButtonBorderColor:var(--fdIconTabBar_Button_Shell_Background_Color_Hover);--fdButtonBackgroundColor:var(--fdIconTabBar_Button_Shell_Background_Color_Hover)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button.is-active,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button:active{--fdButtonColor:var(--fdIconTabBar_Button_Shell_Color_Active);--fdButtonBackgroundColor:var(--fdIconTabBar_Button_Shell_Background_Color_Active);--fdButtonBorderColor:var(--fdIconTabBar_Button_Shell_Background_Color_Active)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button.is-focus,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button:focus{z-index:5;--fdButtonColor:var(--fdIconTabBar_Button_Shell_Color_Active);--fdButtonBackgroundColor:var(--sapShell_Navigation_Background);--fdButtonBorderColor:var(--sapShell_Navigation_Background)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button.is-focus:after,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button:focus:after{border-color:var(--fdIconTabBar_Button_Shell_Focus_Color)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button.is-selected,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button[aria-selected=true]{--fdButtonColor:var(--fdIconTabBar_Button_Shell_Color_Active);--fdButtonBackgroundColor:var(--fdIconTabBar_Button_Shell_Background_Color_Selected);--fdButtonBorderColor:var(--fdIconTabBar_Button_Shell_Background_Color_Selected)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button.is-selected.is-hover,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button.is-selected:hover,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button[aria-selected=true].is-hover,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button[aria-selected=true]:hover{--fdButtonColor:var(--fdIconTabBar_Button_Shell_Color_Hover);--fdButtonBorderColor:transparent;--fdButtonBackgroundColor:var(--fdIconTabBar_Button_Shell_Background_Color_Hover)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation-flat .fd-icon-tab-bar__header{background:var(--sapShellColor)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation-flat .fd-icon-tab-bar__item--with-separator{position:relative;-webkit-margin-end:2rem;margin-inline-end:2rem}.fd-icon-tab-bar.fd-icon-tab-bar--navigation-flat .fd-icon-tab-bar__item--with-separator:after{background:var(--sapShell_InteractiveBorderColor);content:\"\";height:1.5rem;position:absolute;right:-1rem;width:.063rem}.fd-icon-tab-bar.fd-icon-tab-bar--navigation-flat .fd-icon-tab-bar__item--with-separator[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar.fd-icon-tab-bar--navigation-flat .fd-icon-tab-bar__item--with-separator:after{left:-1rem;right:auto}.fd-icon-tab-bar--navigation-horizontal .fd-icon-tab-bar__header{-webkit-box-shadow:none;box-shadow:none}.fd-icon-tab-bar--navigation-horizontal .fd-icon-tab-bar__tab{padding-block:.75rem 1rem}.fd-icon-tab-bar--navigation-horizontal .fd-icon-tab-bar__tab.is-selected:after,.fd-icon-tab-bar--navigation-horizontal .fd-icon-tab-bar__tab[aria-selected=true]:after{border-radius:.125rem;-webkit-margin-after:.188rem;margin-block-end:.188rem}.fd-icon-tab-bar--navigation-horizontal .fd-icon-tab-bar__button-container{-webkit-box-align:normal;-ms-flex-align:normal;align-items:normal}.fd-icon-tab-bar--navigation-horizontal .fd-icon-tab-bar__button-container .fd-icon-tab-bar__button{-webkit-margin-before:.5rem;margin-block-start:.5rem}.fd-icon-tab-bar--sm .fd-icon-tab-bar__header{padding-block:0;padding-inline:1rem}.fd-icon-tab-bar--lg .fd-icon-tab-bar__header,.fd-icon-tab-bar--md .fd-icon-tab-bar__header{padding-block:0;padding-inline:2rem}.fd-icon-tab-bar--xl .fd-icon-tab-bar__header,.fd-icon-tab-bar--xxl .fd-icon-tab-bar__header{padding-block:0;padding-inline:3rem}.fd-icon-tab-bar--responsive-paddings .fd-icon-tab-bar__header{padding-block:0}@media (width <= 599px){.fd-icon-tab-bar--responsive-paddings .fd-icon-tab-bar__header{padding-inline:1rem}}@media (width >= 600px) and (width <= 1023px){.fd-icon-tab-bar--responsive-paddings .fd-icon-tab-bar__header{padding-inline:2rem}}@media (width >= 1024px) and (width <= 1439px){.fd-icon-tab-bar--responsive-paddings .fd-icon-tab-bar__header{padding-inline:2rem}}@media (width >= 1440px){.fd-icon-tab-bar--responsive-paddings .fd-icon-tab-bar__header{padding-inline:3rem}}.fd-icon-tab-bar .fd-icon-tab-bar__item--overflow,.fd-icon-tab-bar--filter .fd-icon-tab-bar__item--overflow,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--overflow,.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item--overflow,.fd-icon-tab-bar--navigation .fd-icon-tab-bar__item--overflow,.fd-icon-tab-bar--navigation-flat .fd-icon-tab-bar__item--overflow,.fd-icon-tab-bar--process .fd-icon-tab-bar__item--overflow{margin-block:0;margin-inline:0}.fd-icon-tab-bar .fd-icon-tab-bar__item--overflow:last-of-type,.fd-icon-tab-bar--filter .fd-icon-tab-bar__item--overflow:last-of-type,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--overflow:last-of-type,.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item--overflow:last-of-type,.fd-icon-tab-bar--navigation .fd-icon-tab-bar__item--overflow:last-of-type,.fd-icon-tab-bar--navigation-flat .fd-icon-tab-bar__item--overflow:last-of-type,.fd-icon-tab-bar--process .fd-icon-tab-bar__item--overflow:last-of-type{-webkit-margin-start:auto;margin-inline-start:auto}.fd-icon-tab-bar .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover,.fd-icon-tab-bar--filter .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover,.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover,.fd-icon-tab-bar--navigation .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover,.fd-icon-tab-bar--navigation-flat .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover,.fd-icon-tab-bar--process .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover{position:relative}.fd-icon-tab-bar .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover-body,.fd-icon-tab-bar--filter .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover-body,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover-body,.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover-body,.fd-icon-tab-bar--navigation .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover-body,.fd-icon-tab-bar--navigation-flat .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover-body,.fd-icon-tab-bar--process .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover-body{-webkit-margin-end:-.25rem;margin-inline-end:-.25rem;-webkit-margin-before:.375rem;margin-block-start:.375rem}.fd-list{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;list-style:none;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;position:relative;width:100%}.fd-list:after,.fd-list:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list__footer,.fd-list__group-header,.fd-list__item{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-inline:0;-webkit-box-align:center;-ms-flex-align:center;align-items:center;padding-block:0;padding-inline:.9375rem}.fd-list__footer:after,.fd-list__footer:before,.fd-list__group-header:after,.fd-list__group-header:before,.fd-list__item:after,.fd-list__item:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list__item{background:var(--fdList_Item_Background_Color);border-bottom:var(--sapList_BorderWidth) solid var(--fdList_Item_Border_Color);height:var(--fdList_Item_Height);position:relative}.fd-list__item.is-focus,.fd-list__item:focus{outline:none;pointer-events:all}.fd-list__item.is-focus:before,.fd-list__item:focus:before{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);content:\"\";display:block;inset:.125rem .125rem .0625rem;pointer-events:none;position:absolute}.fd-list__item.is-focus.is-active:before,.fd-list__item.is-focus:active:before,.fd-list__item:focus.is-active:before,.fd-list__item:focus:active:before{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--fdList_Active_Outline_Color)}.fd-list__item--unread .fd-list__title{font-weight:700}.fd-list__item--no-data .fd-list__title{text-align:center}.fd-list__item--interactive.is-hover,.fd-list__item--interactive:hover,.fd-list__item--interractive.is-hover,.fd-list__item--interractive:hover{--fdList_Item_Background_Color:var(--sapList_Hover_Background);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_BorderColor)}.fd-list__item--interactive.is-selected,.fd-list__item--interactive[aria-selected=true],.fd-list__item--interractive.is-selected,.fd-list__item--interractive[aria-selected=true]{--fdList_Item_Background_Color:var(--sapList_SelectionBackgroundColor);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_SelectionBorderColor)}.fd-list__item--interactive.is-selected .fd-list__link,.fd-list__item--interactive[aria-selected=true] .fd-list__link,.fd-list__item--interractive.is-selected .fd-list__link,.fd-list__item--interractive[aria-selected=true] .fd-list__link{background:inherit}.fd-list__item--interactive.is-selected .fd-list__link.is-active,.fd-list__item--interactive.is-selected .fd-list__link:active,.fd-list__item--interactive[aria-selected=true] .fd-list__link.is-active,.fd-list__item--interactive[aria-selected=true] .fd-list__link:active,.fd-list__item--interractive.is-selected .fd-list__link.is-active,.fd-list__item--interractive.is-selected .fd-list__link:active,.fd-list__item--interractive[aria-selected=true] .fd-list__link.is-active,.fd-list__item--interractive[aria-selected=true] .fd-list__link:active{--fdList_Item_Background_Color:var(--sapList_Active_Background)}.fd-list__item--interactive.is-selected.is-hover,.fd-list__item--interactive.is-selected:hover,.fd-list__item--interactive[aria-selected=true].is-hover,.fd-list__item--interactive[aria-selected=true]:hover,.fd-list__item--interractive.is-selected.is-hover,.fd-list__item--interractive.is-selected:hover,.fd-list__item--interractive[aria-selected=true].is-hover,.fd-list__item--interractive[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Hover_SelectionBackground)}.fd-list__item--interactive.is-active,.fd-list__item--interactive.is-active.is-hover,.fd-list__item--interactive.is-active.is-selected,.fd-list__item--interactive.is-active.is-selected.is-hover,.fd-list__item--interactive.is-active.is-selected:hover,.fd-list__item--interactive.is-active:hover,.fd-list__item--interactive.is-active[aria-selected=true],.fd-list__item--interactive.is-active[aria-selected=true].is-hover,.fd-list__item--interactive.is-active[aria-selected=true]:hover,.fd-list__item--interactive:active,.fd-list__item--interactive:active.is-hover,.fd-list__item--interactive:active.is-selected,.fd-list__item--interactive:active.is-selected.is-hover,.fd-list__item--interactive:active.is-selected:hover,.fd-list__item--interactive:active:hover,.fd-list__item--interactive:active[aria-selected=true],.fd-list__item--interactive:active[aria-selected=true].is-hover,.fd-list__item--interactive:active[aria-selected=true]:hover,.fd-list__item--interractive.is-active,.fd-list__item--interractive.is-active.is-hover,.fd-list__item--interractive.is-active.is-selected,.fd-list__item--interractive.is-active.is-selected.is-hover,.fd-list__item--interractive.is-active.is-selected:hover,.fd-list__item--interractive.is-active:hover,.fd-list__item--interractive.is-active[aria-selected=true],.fd-list__item--interractive.is-active[aria-selected=true].is-hover,.fd-list__item--interractive.is-active[aria-selected=true]:hover,.fd-list__item--interractive:active,.fd-list__item--interractive:active.is-hover,.fd-list__item--interractive:active.is-selected,.fd-list__item--interractive:active.is-selected.is-hover,.fd-list__item--interractive:active.is-selected:hover,.fd-list__item--interractive:active:hover,.fd-list__item--interractive:active[aria-selected=true],.fd-list__item--interractive:active[aria-selected=true].is-hover,.fd-list__item--interractive:active[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Active_Background);--fdList_Item_Text_Color:var(--sapList_Active_TextColor);--fdList_Item_Border_Color:var(--sapList_Active_Background)}.fd-list__item--interactive.is-active,.fd-list__item--interactive.is-active .fd-list__byline,.fd-list__item--interactive.is-active .fd-list__byline-left,.fd-list__item--interactive.is-active .fd-list__byline-right,.fd-list__item--interactive.is-active .fd-list__byline-right--critical,.fd-list__item--interactive.is-active .fd-list__byline-right--informative,.fd-list__item--interactive.is-active .fd-list__byline-right--negative,.fd-list__item--interactive.is-active .fd-list__byline-right--neutral,.fd-list__item--interactive.is-active .fd-list__byline-right--positive,.fd-list__item--interactive.is-active .fd-list__content,.fd-list__item--interactive.is-active .fd-list__icon,.fd-list__item--interactive.is-active .fd-list__secondary,.fd-list__item--interactive.is-active .fd-list__thumbnail,.fd-list__item--interactive.is-active .fd-list__title,.fd-list__item--interactive.is-active.fd-list__link--navigation-indicator:after,.fd-list__item--interactive:active,.fd-list__item--interactive:active .fd-list__byline,.fd-list__item--interactive:active .fd-list__byline-left,.fd-list__item--interactive:active .fd-list__byline-right,.fd-list__item--interactive:active .fd-list__byline-right--critical,.fd-list__item--interactive:active .fd-list__byline-right--informative,.fd-list__item--interactive:active .fd-list__byline-right--negative,.fd-list__item--interactive:active .fd-list__byline-right--neutral,.fd-list__item--interactive:active .fd-list__byline-right--positive,.fd-list__item--interactive:active .fd-list__content,.fd-list__item--interactive:active .fd-list__icon,.fd-list__item--interactive:active .fd-list__secondary,.fd-list__item--interactive:active .fd-list__thumbnail,.fd-list__item--interactive:active .fd-list__title,.fd-list__item--interactive:active.fd-list__link--navigation-indicator:after,.fd-list__item--interractive.is-active,.fd-list__item--interractive.is-active .fd-list__byline,.fd-list__item--interractive.is-active .fd-list__byline-left,.fd-list__item--interractive.is-active .fd-list__byline-right,.fd-list__item--interractive.is-active .fd-list__byline-right--critical,.fd-list__item--interractive.is-active .fd-list__byline-right--informative,.fd-list__item--interractive.is-active .fd-list__byline-right--negative,.fd-list__item--interractive.is-active .fd-list__byline-right--neutral,.fd-list__item--interractive.is-active .fd-list__byline-right--positive,.fd-list__item--interractive.is-active .fd-list__content,.fd-list__item--interractive.is-active .fd-list__icon,.fd-list__item--interractive.is-active .fd-list__secondary,.fd-list__item--interractive.is-active .fd-list__thumbnail,.fd-list__item--interractive.is-active .fd-list__title,.fd-list__item--interractive.is-active.fd-list__link--navigation-indicator:after,.fd-list__item--interractive:active,.fd-list__item--interractive:active .fd-list__byline,.fd-list__item--interractive:active .fd-list__byline-left,.fd-list__item--interractive:active .fd-list__byline-right,.fd-list__item--interractive:active .fd-list__byline-right--critical,.fd-list__item--interractive:active .fd-list__byline-right--informative,.fd-list__item--interractive:active .fd-list__byline-right--negative,.fd-list__item--interractive:active .fd-list__byline-right--neutral,.fd-list__item--interractive:active .fd-list__byline-right--positive,.fd-list__item--interractive:active .fd-list__content,.fd-list__item--interractive:active .fd-list__icon,.fd-list__item--interractive:active .fd-list__secondary,.fd-list__item--interractive:active .fd-list__thumbnail,.fd-list__item--interractive:active .fd-list__title,.fd-list__item--interractive:active.fd-list__link--navigation-indicator:after{text-shadow:none}.fd-list__item--action{border-bottom:var(--fdList_Item_Action_Border);font-size:var(--sapFontSize);padding-block:0;padding-inline:0;text-align:center;--fdList_Item_Height:var(--sapElement_LineHeight)}.fd-list__item--action:first-child{border-top:var(--fdList_Item_Action_Border)}.fd-list__item--action .fd-list__title{background:transparent;border:var(--fdList_Item_Action_Button_Border);border-radius:var(--fdList_Item_Action_Button_Border_Radius);color:var(--sapButton_TextColor);cursor:pointer;height:var(--fdList_Item_Action_Button_Size);margin-block:var(--fdList_Item_Action_Button_Spacing);margin-inline:var(--fdList_Item_Action_Button_Spacing);padding-block:0;padding-inline:.9375rem;pointer-events:all;width:100%}.fd-list__item--action .fd-list__title.is-hover,.fd-list__item--action .fd-list__title:hover{--fdList_Item_Background_Color:var(--sapList_Hover_Background);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_BorderColor)}.fd-list__item--action .fd-list__title.is-selected,.fd-list__item--action .fd-list__title[aria-selected=true]{--fdList_Item_Background_Color:var(--sapList_SelectionBackgroundColor);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_SelectionBorderColor)}.fd-list__item--action .fd-list__title.is-selected .fd-list__link,.fd-list__item--action .fd-list__title[aria-selected=true] .fd-list__link{background:inherit}.fd-list__item--action .fd-list__title.is-selected .fd-list__link.is-active,.fd-list__item--action .fd-list__title.is-selected .fd-list__link:active,.fd-list__item--action .fd-list__title[aria-selected=true] .fd-list__link.is-active,.fd-list__item--action .fd-list__title[aria-selected=true] .fd-list__link:active{--fdList_Item_Background_Color:var(--sapList_Active_Background)}.fd-list__item--action .fd-list__title.is-selected.is-hover,.fd-list__item--action .fd-list__title.is-selected:hover,.fd-list__item--action .fd-list__title[aria-selected=true].is-hover,.fd-list__item--action .fd-list__title[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Hover_SelectionBackground)}.fd-list__item--action .fd-list__title.is-active,.fd-list__item--action .fd-list__title.is-active.is-hover,.fd-list__item--action .fd-list__title.is-active.is-selected,.fd-list__item--action .fd-list__title.is-active.is-selected.is-hover,.fd-list__item--action .fd-list__title.is-active.is-selected:hover,.fd-list__item--action .fd-list__title.is-active:hover,.fd-list__item--action .fd-list__title.is-active[aria-selected=true],.fd-list__item--action .fd-list__title.is-active[aria-selected=true].is-hover,.fd-list__item--action .fd-list__title.is-active[aria-selected=true]:hover,.fd-list__item--action .fd-list__title:active,.fd-list__item--action .fd-list__title:active.is-hover,.fd-list__item--action .fd-list__title:active.is-selected,.fd-list__item--action .fd-list__title:active.is-selected.is-hover,.fd-list__item--action .fd-list__title:active.is-selected:hover,.fd-list__item--action .fd-list__title:active:hover,.fd-list__item--action .fd-list__title:active[aria-selected=true],.fd-list__item--action .fd-list__title:active[aria-selected=true].is-hover,.fd-list__item--action .fd-list__title:active[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Active_Background);--fdList_Item_Text_Color:var(--sapList_Active_TextColor);--fdList_Item_Border_Color:var(--sapList_Active_Background)}.fd-list__item--action .fd-list__title.is-active,.fd-list__item--action .fd-list__title.is-active .fd-list__byline,.fd-list__item--action .fd-list__title.is-active .fd-list__byline-left,.fd-list__item--action .fd-list__title.is-active .fd-list__byline-right,.fd-list__item--action .fd-list__title.is-active .fd-list__byline-right--critical,.fd-list__item--action .fd-list__title.is-active .fd-list__byline-right--informative,.fd-list__item--action .fd-list__title.is-active .fd-list__byline-right--negative,.fd-list__item--action .fd-list__title.is-active .fd-list__byline-right--neutral,.fd-list__item--action .fd-list__title.is-active .fd-list__byline-right--positive,.fd-list__item--action .fd-list__title.is-active .fd-list__content,.fd-list__item--action .fd-list__title.is-active .fd-list__icon,.fd-list__item--action .fd-list__title.is-active .fd-list__secondary,.fd-list__item--action .fd-list__title.is-active .fd-list__thumbnail,.fd-list__item--action .fd-list__title.is-active .fd-list__title,.fd-list__item--action .fd-list__title.is-active.fd-list__link--navigation-indicator:after,.fd-list__item--action .fd-list__title:active,.fd-list__item--action .fd-list__title:active .fd-list__byline,.fd-list__item--action .fd-list__title:active .fd-list__byline-left,.fd-list__item--action .fd-list__title:active .fd-list__byline-right,.fd-list__item--action .fd-list__title:active .fd-list__byline-right--critical,.fd-list__item--action .fd-list__title:active .fd-list__byline-right--informative,.fd-list__item--action .fd-list__title:active .fd-list__byline-right--negative,.fd-list__item--action .fd-list__title:active .fd-list__byline-right--neutral,.fd-list__item--action .fd-list__title:active .fd-list__byline-right--positive,.fd-list__item--action .fd-list__title:active .fd-list__content,.fd-list__item--action .fd-list__title:active .fd-list__icon,.fd-list__item--action .fd-list__title:active .fd-list__secondary,.fd-list__item--action .fd-list__title:active .fd-list__thumbnail,.fd-list__item--action .fd-list__title:active .fd-list__title,.fd-list__item--action .fd-list__title:active.fd-list__link--navigation-indicator:after{text-shadow:none}.fd-list__item--action .fd-list__title.is-focus,.fd-list__item--action .fd-list__title:focus{outline:var(--sapContent_FocusColor) var(--sapContent_FocusStyle) var(--sapContent_FocusWidth);outline-offset:-.125rem}.fd-list__item--action .fd-list__title.is-focus.is-active,.fd-list__item--action .fd-list__title.is-focus:active,.fd-list__item--action .fd-list__title:focus.is-active,.fd-list__item--action .fd-list__title:focus:active{outline-color:var(--fdList_Active_Outline_Color)}.fd-list__item--action .fd-list__title::-moz-focus-inner{border:0}.fd-list__item--growing{--fdList_Item_Height:var(--sapElement_LineHeight)}.fd-list__item--growing .fd-list__title{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;gap:.5rem}.fd-list__item--growing .fd-list__title-text{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:inherit;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;font-weight:var(--fdList_Growing_Font_Weight);forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;overflow:hidden;padding-block:0;padding-inline:0;text-overflow:ellipsis;text-shadow:inherit;white-space:nowrap}.fd-list__item--growing .fd-list__title-text:after,.fd-list__item--growing .fd-list__title-text:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list__item--inactive,.fd-list__item--inactive .fd-list__secondary,.fd-list__item--inactive .fd-list__title{pointer-events:none}.fd-list__item.is-selected,.fd-list__item[aria-selected=true]{--fdList_Item_Background_Color:var(--sapList_SelectionBackgroundColor);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_SelectionBorderColor)}.fd-list__item.is-selected .fd-list__link,.fd-list__item[aria-selected=true] .fd-list__link{background:inherit}.fd-list__item.is-selected .fd-list__link.is-active,.fd-list__item.is-selected .fd-list__link:active,.fd-list__item[aria-selected=true] .fd-list__link.is-active,.fd-list__item[aria-selected=true] .fd-list__link:active{--fdList_Item_Background_Color:var(--sapList_Active_Background)}.fd-list__item.is-selected.is-hover,.fd-list__item.is-selected:hover,.fd-list__item[aria-selected=true].is-hover,.fd-list__item[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Hover_SelectionBackground)}.fd-list__secondary,.fd-list__title{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:var(--fdList_Item_Text_Color);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;overflow:hidden;padding-block:0;padding-inline:0;pointer-events:none;pointer-events:all;text-overflow:ellipsis;white-space:nowrap}.fd-list__secondary:after,.fd-list__secondary:before,.fd-list__title:after,.fd-list__title:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list__title{-webkit-box-flex:3;-ms-flex:3 3 10%;flex:3 3 10%;font-size:var(--fdList_Item_Text_Font_Size);min-height:var(--fdList_Item_Text_Font_Size)}.fd-list__title:last-child:first-child{-ms-flex-preferred-size:auto;flex-basis:auto}.fd-list__title .fd-list__title-text{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.fd-list__title~.fd-list__icon{-webkit-margin-end:-1rem;margin-inline-end:-1rem}.fd-list__secondary{color:var(--fdList_Status_Text_Color);font-size:var(--sapFontSize);max-width:40%;text-align:right;-webkit-padding-start:1rem;padding-inline-start:1rem}.fd-list__secondary--positive{--fdList_Status_Text_Color:var(--sapPositiveTextColor)}.fd-list__secondary--critical{--fdList_Status_Text_Color:var(--sapCriticalTextColor)}.fd-list__secondary--negative{--fdList_Status_Text_Color:var(--sapNegativeTextColor)}.fd-list__secondary--informative{--fdList_Status_Text_Color:var(--sapInformativeTextColor)}.fd-list__secondary[dir=rtl],[dir=rtl] .fd-list__secondary{text-align:left}.fd-list__icon [class*=sap-icon],.fd-list__icon[class*=sap-icon]{background:inherit;border-radius:inherit;color:inherit;display:-webkit-box;display:-ms-flexbox;display:flex;font-size:inherit;line-height:1;pointer-events:none;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;color:var(--sapContent_NonInteractiveIconColor);font-size:var(--fdList_Item_Icon_Font_Size);width:2.75rem;-webkit-box-flex:0;-ms-flex:0 0 auto;flex:0 0 auto}.fd-list__icon [class*=sap-icon]:first-child,.fd-list__icon[class*=sap-icon]:first-child{-webkit-margin-start:-1rem;margin-inline-start:-1rem}.fd-list__link{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-inline:0;-webkit-box-align:center;-ms-flex-align:center;align-items:center;--fdList_Item_Border_Color:transparent;background:var(--fdList_Item_Background_Color);border-bottom:var(--sapList_BorderWidth) solid var(--fdList_Item_Border_Color);cursor:pointer;height:100%;padding-block:0;padding-inline:.9375rem;text-decoration:none;width:100%}.fd-list__link:after,.fd-list__link:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list__link.is-focus,.fd-list__link:focus{outline:none;pointer-events:all}.fd-list__link.is-focus:before,.fd-list__link:focus:before{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);content:\"\";display:block;inset:.125rem;pointer-events:none;position:absolute}.fd-list__link.is-focus.is-active:before,.fd-list__link.is-focus:active:before,.fd-list__link:focus.is-active:before,.fd-list__link:focus:active:before{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--fdList_Active_Outline_Color)}.fd-list__link.is-hover,.fd-list__link:hover{--fdList_Item_Background_Color:var(--sapList_Hover_Background);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_BorderColor)}.fd-list__link.is-selected,.fd-list__link[aria-selected=true]{--fdList_Item_Background_Color:var(--sapList_SelectionBackgroundColor);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_SelectionBorderColor)}.fd-list__link.is-selected .fd-list__link,.fd-list__link[aria-selected=true] .fd-list__link{background:inherit}.fd-list__link.is-selected .fd-list__link.is-active,.fd-list__link.is-selected .fd-list__link:active,.fd-list__link[aria-selected=true] .fd-list__link.is-active,.fd-list__link[aria-selected=true] .fd-list__link:active{--fdList_Item_Background_Color:var(--sapList_Active_Background)}.fd-list__link.is-selected.is-hover,.fd-list__link.is-selected:hover,.fd-list__link[aria-selected=true].is-hover,.fd-list__link[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Hover_SelectionBackground)}.fd-list__link.is-active,.fd-list__link.is-active.is-hover,.fd-list__link.is-active.is-selected,.fd-list__link.is-active.is-selected.is-hover,.fd-list__link.is-active.is-selected:hover,.fd-list__link.is-active:hover,.fd-list__link.is-active[aria-selected=true],.fd-list__link.is-active[aria-selected=true].is-hover,.fd-list__link.is-active[aria-selected=true]:hover,.fd-list__link:active,.fd-list__link:active.is-hover,.fd-list__link:active.is-selected,.fd-list__link:active.is-selected.is-hover,.fd-list__link:active.is-selected:hover,.fd-list__link:active:hover,.fd-list__link:active[aria-selected=true],.fd-list__link:active[aria-selected=true].is-hover,.fd-list__link:active[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Active_Background);--fdList_Item_Text_Color:var(--sapList_Active_TextColor);--fdList_Item_Border_Color:var(--sapList_Active_Background)}.fd-list__link.is-active,.fd-list__link.is-active .fd-list__byline,.fd-list__link.is-active .fd-list__byline-left,.fd-list__link.is-active .fd-list__byline-right,.fd-list__link.is-active .fd-list__byline-right--critical,.fd-list__link.is-active .fd-list__byline-right--informative,.fd-list__link.is-active .fd-list__byline-right--negative,.fd-list__link.is-active .fd-list__byline-right--neutral,.fd-list__link.is-active .fd-list__byline-right--positive,.fd-list__link.is-active .fd-list__content,.fd-list__link.is-active .fd-list__icon,.fd-list__link.is-active .fd-list__secondary,.fd-list__link.is-active .fd-list__thumbnail,.fd-list__link.is-active .fd-list__title,.fd-list__link.is-active.fd-list__link--navigation-indicator:after,.fd-list__link:active,.fd-list__link:active .fd-list__byline,.fd-list__link:active .fd-list__byline-left,.fd-list__link:active .fd-list__byline-right,.fd-list__link:active .fd-list__byline-right--critical,.fd-list__link:active .fd-list__byline-right--informative,.fd-list__link:active .fd-list__byline-right--negative,.fd-list__link:active .fd-list__byline-right--neutral,.fd-list__link:active .fd-list__byline-right--positive,.fd-list__link:active .fd-list__content,.fd-list__link:active .fd-list__icon,.fd-list__link:active .fd-list__secondary,.fd-list__link:active .fd-list__thumbnail,.fd-list__link:active .fd-list__title,.fd-list__link:active.fd-list__link--navigation-indicator:after{text-shadow:none}.fd-list__group-header{background:var(--sapList_GroupHeaderBackground);border-bottom:var(--sapList_BorderWidth) solid var(--sapList_GroupHeaderBorderColor);height:var(--sapElement_LineHeight);-webkit-box-align:end;-ms-flex-align:end;align-items:flex-end}.fd-list__group-header .fd-list__title{color:var(--sapList_TableGroupHeaderTextColor);font-family:var(--sapFontHeaderFamily);font-size:var(--sapFontHeader6Size);font-weight:700;line-height:2rem;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.fd-list__footer{background:var(--sapList_FooterBackground);color:var(--sapList_FooterTextColor);font-size:var(--sapFontSize);height:2rem}.fd-list__footer--right{-webkit-box-pack:end;-ms-flex-pack:end;justify-content:flex-end}.fd-list__footer--left{-webkit-box-pack:start;-ms-flex-pack:start;justify-content:flex-start}.fd-list__bold{font-weight:700}.fd-list__message{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;height:2rem;left:0;line-height:normal;line-height:2rem;margin-block:0;margin-inline:0;overflow:hidden;padding-block:0;padding-inline:0;position:absolute;text-overflow:ellipsis;top:0;white-space:nowrap;width:100%;-webkit-padding-start:.5rem;background-color:var(--fdList_Message_Background_Color);font-size:var(--sapFontSmallSize);padding-inline-start:.5rem}.fd-list__message:after,.fd-list__message:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list__message--error{--fdList_Message_Background_Color:var(--sapErrorBackground)}.fd-list__message--warning{--fdList_Message_Background_Color:var(--sapWarningBackground)}.fd-list__message--success{--fdList_Message_Background_Color:var(--sapSuccessBackground)}.fd-list__message--information{--fdList_Message_Background_Color:var(--sapInformationBackground)}.fd-list .fd-list__button{-webkit-margin-start:var(--fdList_Button_Spacing);margin-inline-start:var(--fdList_Button_Spacing);-ms-flex-item-align:center;align-self:center}.fd-list[class*=-compact]:not(.fd-object-list):not(.fd-list--message-view),.fd-list[class*=-condensed]:not(.fd-object-list):not(.fd-list--message-view),[class*=-compact] .fd-list:not([class*=-cozy]):not(.fd-object-list):not(.fd-list--message-view),[class*=-condensed] .fd-list:not([class*=-cozy]):not(.fd-object-list):not(.fd-list--message-view){--fdList_Item_Icon_Font_Size:1rem;--fdList_Item_Text_Font_Size:var(--sapFontSize);--fdList_Item_Height:var(--sapElement_Compact_LineHeight)}.fd-list[class*=-compact]:not(.fd-object-list):not(.fd-list--message-view) .fd-list__item--growing,.fd-list[class*=-condensed]:not(.fd-object-list):not(.fd-list--message-view) .fd-list__item--growing,[class*=-compact] .fd-list:not([class*=-cozy]):not(.fd-object-list):not(.fd-list--message-view) .fd-list__item--growing,[class*=-condensed] .fd-list:not([class*=-cozy]):not(.fd-object-list):not(.fd-list--message-view) .fd-list__item--growing{--fdList_Item_Height:var(--sapElement_LineHeight)}.fd-list[class*=-compact]:not(.fd-object-list):not(.fd-list--message-view) .fd-list__item.fd-list__item--action,.fd-list[class*=-condensed]:not(.fd-object-list):not(.fd-list--message-view) .fd-list__item.fd-list__item--action,[class*=-compact] .fd-list:not([class*=-cozy]):not(.fd-object-list):not(.fd-list--message-view) .fd-list__item.fd-list__item--action,[class*=-condensed] .fd-list:not([class*=-cozy]):not(.fd-object-list):not(.fd-list--message-view) .fd-list__item.fd-list__item--action{--fdList_Item_Height:var(--sapElement_Compact_LineHeight)}.fd-list[class*=-compact]:not(.fd-object-list):not(.fd-list--message-view) .fd-list__item.fd-list__item--action .fd-list__title,.fd-list[class*=-condensed]:not(.fd-object-list):not(.fd-list--message-view) .fd-list__item.fd-list__item--action .fd-list__title,[class*=-compact] .fd-list:not([class*=-cozy]):not(.fd-object-list):not(.fd-list--message-view) .fd-list__item.fd-list__item--action .fd-list__title,[class*=-condensed] .fd-list:not([class*=-cozy]):not(.fd-object-list):not(.fd-list--message-view) .fd-list__item.fd-list__item--action .fd-list__title{height:var(--fdList_Item_Action_Button_Size_Compact)}.fd-list--no-border .fd-list__group-header,.fd-list--no-border .fd-list__item{border-bottom:0;border-top:0}.fd-list--no-border .fd-list__item.is-selected,.fd-list--no-border .fd-list__item[aria-selected=true]{border-bottom:var(--sapList_BorderWidth) solid var(--sapList_SelectionBorderColor)}.fd-list--has-message{-webkit-padding-before:2rem;padding-block-start:2rem}.fd-list__infinite-scroll{outline:none;overflow:scroll}.fd-list--dropdown.is-focus,.fd-list--dropdown:focus{outline:none;z-index:5}.fd-list--dropdown .fd-list__item:not(.fd-list__group-header){display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center;height:auto;min-height:2.5rem;padding-block:0;padding-inline:.9375rem}.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):hover{--fdList_Item_Background_Color:var(--sapList_Hover_Background);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_BorderColor)}.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-selected,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header)[aria-selected=true]{--fdList_Item_Background_Color:var(--sapList_SelectionBackgroundColor);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_SelectionBorderColor)}.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-selected .fd-list__link,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header)[aria-selected=true] .fd-list__link{background:inherit}.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-selected .fd-list__link.is-active,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-selected .fd-list__link:active,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header)[aria-selected=true] .fd-list__link.is-active,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header)[aria-selected=true] .fd-list__link:active{--fdList_Item_Background_Color:var(--sapList_Active_Background)}.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-selected.is-hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-selected:hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header)[aria-selected=true].is-hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header)[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Hover_SelectionBackground)}.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active.is-hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active.is-selected,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active.is-selected.is-hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active.is-selected:hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active:hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active[aria-selected=true],.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active[aria-selected=true].is-hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active[aria-selected=true]:hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active.is-hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active.is-selected,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active.is-selected.is-hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active.is-selected:hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active:hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active[aria-selected=true],.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active[aria-selected=true].is-hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Active_Background);--fdList_Item_Text_Color:var(--sapList_Active_TextColor);--fdList_Item_Border_Color:var(--sapList_Active_Background)}.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__byline,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__byline-left,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__byline-right,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__byline-right--critical,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__byline-right--informative,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__byline-right--negative,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__byline-right--neutral,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__byline-right--positive,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__content,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__icon,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__secondary,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__thumbnail,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__title,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active.fd-list__link--navigation-indicator:after,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__byline,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__byline-left,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__byline-right,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__byline-right--critical,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__byline-right--informative,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__byline-right--negative,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__byline-right--neutral,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__byline-right--positive,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__content,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__icon,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__secondary,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__thumbnail,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__title,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active.fd-list__link--navigation-indicator:after{text-shadow:none}.fd-list--dropdown[class*=-compact] .fd-list__item:not(.fd-list__group-header),.fd-list--dropdown[class*=-condensed] .fd-list__item:not(.fd-list__group-header),[class*=-compact] .fd-list--dropdown:not([class*=-cozy]) .fd-list__item:not(.fd-list__group-header),[class*=-condensed] .fd-list--dropdown:not([class*=-cozy]) .fd-list__item:not(.fd-list__group-header){height:auto;min-height:2rem;padding-block:0;padding-inline:.9375rem}.fd-list--multi-input .fd-list__item{height:auto;padding-block:0;padding-inline:0 1rem}.fd-list--multi-input .fd-list__item.is-hover,.fd-list--multi-input .fd-list__item:hover{--fdList_Item_Background_Color:var(--sapList_Hover_Background);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_BorderColor)}.fd-list--multi-input .fd-list__item.is-selected,.fd-list--multi-input .fd-list__item[aria-selected=true]{--fdList_Item_Background_Color:var(--sapList_SelectionBackgroundColor);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_SelectionBorderColor)}.fd-list--multi-input .fd-list__item.is-selected .fd-list__link,.fd-list--multi-input .fd-list__item[aria-selected=true] .fd-list__link{background:inherit}.fd-list--multi-input .fd-list__item.is-selected .fd-list__link.is-active,.fd-list--multi-input .fd-list__item.is-selected .fd-list__link:active,.fd-list--multi-input .fd-list__item[aria-selected=true] .fd-list__link.is-active,.fd-list--multi-input .fd-list__item[aria-selected=true] .fd-list__link:active{--fdList_Item_Background_Color:var(--sapList_Active_Background)}.fd-list--multi-input .fd-list__item.is-selected.is-hover,.fd-list--multi-input .fd-list__item.is-selected:hover,.fd-list--multi-input .fd-list__item[aria-selected=true].is-hover,.fd-list--multi-input .fd-list__item[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Hover_SelectionBackground)}.fd-list--multi-input .fd-list__item.is-active,.fd-list--multi-input .fd-list__item.is-active.is-hover,.fd-list--multi-input .fd-list__item.is-active.is-selected,.fd-list--multi-input .fd-list__item.is-active.is-selected.is-hover,.fd-list--multi-input .fd-list__item.is-active.is-selected:hover,.fd-list--multi-input .fd-list__item.is-active:hover,.fd-list--multi-input .fd-list__item.is-active[aria-selected=true],.fd-list--multi-input .fd-list__item.is-active[aria-selected=true].is-hover,.fd-list--multi-input .fd-list__item.is-active[aria-selected=true]:hover,.fd-list--multi-input .fd-list__item:active,.fd-list--multi-input .fd-list__item:active.is-hover,.fd-list--multi-input .fd-list__item:active.is-selected,.fd-list--multi-input .fd-list__item:active.is-selected.is-hover,.fd-list--multi-input .fd-list__item:active.is-selected:hover,.fd-list--multi-input .fd-list__item:active:hover,.fd-list--multi-input .fd-list__item:active[aria-selected=true],.fd-list--multi-input .fd-list__item:active[aria-selected=true].is-hover,.fd-list--multi-input .fd-list__item:active[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Active_Background);--fdList_Item_Text_Color:var(--sapList_Active_TextColor);--fdList_Item_Border_Color:var(--sapList_Active_Background)}.fd-list--multi-input .fd-list__item.is-active,.fd-list--multi-input .fd-list__item.is-active .fd-list__byline,.fd-list--multi-input .fd-list__item.is-active .fd-list__byline-left,.fd-list--multi-input .fd-list__item.is-active .fd-list__byline-right,.fd-list--multi-input .fd-list__item.is-active .fd-list__byline-right--critical,.fd-list--multi-input .fd-list__item.is-active .fd-list__byline-right--informative,.fd-list--multi-input .fd-list__item.is-active .fd-list__byline-right--negative,.fd-list--multi-input .fd-list__item.is-active .fd-list__byline-right--neutral,.fd-list--multi-input .fd-list__item.is-active .fd-list__byline-right--positive,.fd-list--multi-input .fd-list__item.is-active .fd-list__content,.fd-list--multi-input .fd-list__item.is-active .fd-list__icon,.fd-list--multi-input .fd-list__item.is-active .fd-list__secondary,.fd-list--multi-input .fd-list__item.is-active .fd-list__thumbnail,.fd-list--multi-input .fd-list__item.is-active .fd-list__title,.fd-list--multi-input .fd-list__item.is-active.fd-list__link--navigation-indicator:after,.fd-list--multi-input .fd-list__item:active,.fd-list--multi-input .fd-list__item:active .fd-list__byline,.fd-list--multi-input .fd-list__item:active .fd-list__byline-left,.fd-list--multi-input .fd-list__item:active .fd-list__byline-right,.fd-list--multi-input .fd-list__item:active .fd-list__byline-right--critical,.fd-list--multi-input .fd-list__item:active .fd-list__byline-right--informative,.fd-list--multi-input .fd-list__item:active .fd-list__byline-right--negative,.fd-list--multi-input .fd-list__item:active .fd-list__byline-right--neutral,.fd-list--multi-input .fd-list__item:active .fd-list__byline-right--positive,.fd-list--multi-input .fd-list__item:active .fd-list__content,.fd-list--multi-input .fd-list__item:active .fd-list__icon,.fd-list--multi-input .fd-list__item:active .fd-list__secondary,.fd-list--multi-input .fd-list__item:active .fd-list__thumbnail,.fd-list--multi-input .fd-list__item:active .fd-list__title,.fd-list--multi-input .fd-list__item:active.fd-list__link--navigation-indicator:after{text-shadow:none}.fd-list--multi-input .fd-list__label{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:inherit;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;overflow:hidden;padding-block:0;padding-inline:0;position:relative;text-overflow:ellipsis;width:100%;-webkit-margin-end:0;margin-inline-end:0}.fd-list--multi-input .fd-list__label:after,.fd-list--multi-input .fd-list__label:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list--multi-input .fd-list__label:before{margin-block:-.1875rem -.1875rem}.fd-list--multi-input .fd-list__input.is-focus,.fd-list--multi-input .fd-list__input:focus{-webkit-box-shadow:none;box-shadow:none;z-index:5}.fd-list--multi-input .fd-list__input.is-focus+.fd-list__label:after,.fd-list--multi-input .fd-list__input:focus+.fd-list__label:after{inset:.0625rem}.fd-list--multi-input .fd-list__footer{height:auto;text-align:right;-webkit-box-pack:end;-ms-flex-pack:end;background-color:var(--sapList_Background);justify-content:flex-end;line-height:1rem}.fd-list--multi-input .fd-list__footer,.fd-list--multi-input .fd-list__label{padding-block:.75rem;padding-inline:.9375rem}.fd-list--multi-input[class*=-compact] .fd-list__item,.fd-list--multi-input[class*=-condensed] .fd-list__item,[class*=-compact] .fd-list--multi-input:not([class*=-cozy]) .fd-list__item,[class*=-condensed] .fd-list--multi-input:not([class*=-cozy]) .fd-list__item{height:auto;padding-block:0;padding-inline:0}.fd-list--multi-input[class*=-compact] .fd-list__label:before,.fd-list--multi-input[class*=-condensed] .fd-list__label:before,[class*=-compact] .fd-list--multi-input:not([class*=-cozy]) .fd-list__label:before,[class*=-condensed] .fd-list--multi-input:not([class*=-cozy]) .fd-list__label:before{-webkit-margin-start:0;margin-inline-start:0;margin-top:0;-webkit-margin-after:0;margin-block-end:0}.fd-list--multi-input[class*=-compact] .fd-list__footer,.fd-list--multi-input[class*=-compact] .fd-list__label,.fd-list--multi-input[class*=-condensed] .fd-list__footer,.fd-list--multi-input[class*=-condensed] .fd-list__label,[class*=-compact] .fd-list--multi-input:not([class*=-cozy]) .fd-list__footer,[class*=-compact] .fd-list--multi-input:not([class*=-cozy]) .fd-list__label,[class*=-condensed] .fd-list--multi-input:not([class*=-cozy]) .fd-list__footer,[class*=-condensed] .fd-list--multi-input:not([class*=-cozy]) .fd-list__label{padding-block:.5rem;padding-inline:.9375rem}.fd-list--dropdown,.fd-list--multi-input{display:block;max-width:40rem;min-width:5rem}.fd-list--dropdown .fd-list__item,.fd-list--multi-input .fd-list__item{border:none;cursor:pointer}.fd-list--dropdown .fd-list__item.is-selected,.fd-list--dropdown .fd-list__item[aria-selected=true],.fd-list--multi-input .fd-list__item.is-selected,.fd-list--multi-input .fd-list__item[aria-selected=true]{border-bottom:var(--sapList_BorderWidth) solid var(--sapList_SelectionBorderColor)}.fd-list--dropdown .fd-list__secondary,.fd-list--dropdown .fd-list__title,.fd-list--multi-input .fd-list__secondary,.fd-list--multi-input .fd-list__title{font-size:var(--sapFontSize);white-space:normal;-webkit-box-flex:1;-ms-flex:auto;flex:auto;width:auto}.fd-list--dropdown .fd-list__secondary--no-wrap,.fd-list--dropdown .fd-list__title--no-wrap,.fd-list--multi-input .fd-list__secondary--no-wrap,.fd-list--multi-input .fd-list__title--no-wrap{white-space:nowrap}.fd-list--dropdown .fd-list__secondary,.fd-list--multi-input .fd-list__secondary{display:block}.fd-list--dropdown .fd-list__title,.fd-list--multi-input .fd-list__title{max-width:24rem}.fd-list--dropdown .fd-list__title:first-child:last-child,.fd-list--multi-input .fd-list__title:first-child:last-child{max-width:40rem}.fd-list--dropdown .fd-list__secondary,.fd-list--multi-input .fd-list__secondary{max-width:16rem}.fd-list--dropdown.fd-list--mobile,.fd-list--multi-input.fd-list--mobile{max-width:100%}.fd-list--dropdown .fd-list__icon,.fd-list--multi-input .fd-list__icon{line-height:1rem}.fd-list--dropdown.fd-list--large-dropdown,.fd-list--multi-input.fd-list--large-dropdown{max-width:62rem}.fd-list--dropdown.fd-list--large-dropdown .fd-list__title,.fd-list--multi-input.fd-list--large-dropdown .fd-list__title{max-width:37.2rem}.fd-list--dropdown.fd-list--large-dropdown .fd-list__title:first-child:last-child,.fd-list--multi-input.fd-list--large-dropdown .fd-list__title:first-child:last-child{max-width:62rem}.fd-list--dropdown.fd-list--large-dropdown .fd-list__secondary,.fd-list--multi-input.fd-list--large-dropdown .fd-list__secondary{max-width:24.8rem}.fd-list--dropdown.fd-list--large-dropdown.fd-list--mobile,.fd-list--multi-input.fd-list--large-dropdown.fd-list--mobile{max-width:100%}.fd-list--byline .fd-list__item{height:auto;min-height:5rem;padding-block:.9375rem;padding-inline:1rem;width:100%;-webkit-box-align:start;-ms-flex-align:start;align-items:flex-start}.fd-list--byline .fd-list__item.is-focus,.fd-list--byline .fd-list__item:focus{pointer-events:all;z-index:5}.fd-list--byline .fd-list__item .fd-list__item-counter{-ms-flex-align:center;-ms-flex-item-align:center;align-self:center;-webkit-box-pack:end;-ms-flex-pack:end;height:100%;justify-content:flex-end;min-width:2rem}.fd-list--byline .fd-list__item .fd-list__item-counter,.fd-list--byline .fd-list__thumbnail{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;align-items:center}.fd-list--byline .fd-list__thumbnail{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;pointer-events:none;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-ms-flex-align:center;-webkit-margin-end:.75rem;border-radius:.25rem;color:var(--sapContent_NonInteractiveIconColor);font-size:2rem;height:3rem;margin-inline-end:.75rem;max-height:3rem;max-width:3rem;min-height:3rem;min-width:3rem;width:3rem}.fd-list--byline .fd-list__thumbnail:after,.fd-list--byline .fd-list__thumbnail:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list--byline .fd-list__thumbnail [class*=sap-icon],.fd-list--byline .fd-list__thumbnail[class*=sap-icon]{background:inherit;border-radius:inherit;color:inherit;font-size:inherit;line-height:1}.fd-list--byline .fd-list__thumbnail>svg{height:100%}.fd-list--byline .fd-list__notification{color:var(--sapAccentColor6);font-size:.375rem;left:1rem;position:absolute;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%)}.fd-list--byline .fd-list__notification[dir=rtl],[dir=rtl] .fd-list--byline .fd-list__notification{left:auto;right:1rem}.fd-list--byline .fd-list__notification+.fd-list__link .fd-list__content .fd-list__title,.fd-list--byline .fd-list__notification~.fd-list__content .fd-list__title{font-weight:700}.fd-list--byline .fd-list__content{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;pointer-events:none;-ms-flex-line-pack:justify;align-content:space-between;height:100%;min-height:3rem;min-width:3rem;-webkit-box-flex:1;-ms-flex:1 1 auto;flex:1 1 auto;padding-block:.125rem;padding-inline:0}.fd-list--byline .fd-list__content:after,.fd-list--byline .fd-list__content:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list--byline .fd-list__title{font-size:var(--sapFontLargeSize);min-height:1rem;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.fd-list--byline .fd-list__byline{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:var(--sapContent_LabelColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;line-height:1rem;margin-block:0;margin-inline:0;overflow:hidden;padding-block:0;padding-inline:0;text-overflow:ellipsis;white-space:nowrap;-webkit-padding-before:.5rem;padding-block-start:.5rem}.fd-list--byline .fd-list__byline:after,.fd-list--byline .fd-list__byline:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list--byline .fd-list__byline--2-col{display:-webkit-box;display:-ms-flexbox;display:flex;pointer-events:all}.fd-list--byline .fd-list__byline--wrap .fd-list__byline-left,.fd-list--byline .fd-list__byline--wrap .fd-list__byline-right{display:inline;padding-block:0;padding-inline:0;white-space:normal;width:100%}.fd-list--byline .fd-list__byline--wrap .fd-list__byline-right{float:right;width:auto}.fd-list--byline .fd-list__link--more{pointer-events:all;text-transform:capitalize}.fd-list--byline .fd-list__byline--wrap,.fd-list--byline .fd-list__title--wrap{white-space:normal}.fd-list--byline .fd-list__byline-left{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:var(--sapContent_LabelColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;line-height:1rem;margin-block:0;margin-inline:0;overflow:hidden;padding-block:0;padding-inline:0;text-overflow:ellipsis;white-space:nowrap;width:60%;-ms-flex-item-align:end;align-self:flex-end;-webkit-padding-end:.5rem;padding-inline-end:.5rem}.fd-list--byline .fd-list__byline-left:after,.fd-list--byline .fd-list__byline-left:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list--byline .fd-list__byline-right{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:var(--sapContent_LabelColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;line-height:1rem;margin-block:0;margin-inline:0;overflow:hidden;padding-block:0;padding-inline:0;text-align:right;text-overflow:ellipsis;white-space:nowrap;width:40%;-ms-flex-item-align:end;align-self:flex-end;-webkit-padding-start:.5rem;padding-inline-start:.5rem}.fd-list--byline .fd-list__byline-right:after,.fd-list--byline .fd-list__byline-right:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list--byline .fd-list__byline-right[dir=rtl],[dir=rtl] .fd-list--byline .fd-list__byline-right{text-align:left}.fd-list--byline .fd-list__byline-right--neutral{color:var(--sapNeutralTextColor)}.fd-list--byline .fd-list__byline-right--positive{color:var(--sapPositiveTextColor)}.fd-list--byline .fd-list__byline-right--critical{color:var(--sapCriticalTextColor)}.fd-list--byline .fd-list__byline-right--negative{color:var(--sapNegativeTextColor)}.fd-list--byline .fd-list__byline-right--informative{color:var(--sapInformativeTextColor)}.fd-list--byline.fd-list--unread-indicator .fd-list__item{padding-inline:2.125rem 1rem}.fd-list--byline[class*=-compact] .fd-list__title,.fd-list--byline[class*=-condensed] .fd-list__title,[class*=-compact] .fd-list--byline:not([class*=-cozy]) .fd-list__title,[class*=-condensed] .fd-list--byline:not([class*=-cozy]) .fd-list__title{font-size:var(--sapFontSize)}.fd-list--byline.fd-list--no-border .fd-list__item{height:100%;max-height:5rem;padding-block:.5rem}.fd-list--byline.fd-list--no-border .fd-list__item:first-child{-webkit-padding-before:.9375rem;padding-block-start:.9375rem}.fd-list--byline.fd-list--no-border .fd-list__item:last-child{-webkit-padding-after:.9375rem;padding-block-end:.9375rem}.fd-list--message-view{--fdListNavigationIndicatorFontSize:.75rem;--fdList_Item_Height:calc(2.75rem + var(--sapList_BorderWidth));--fdList_Message_View_Item_Byline_Height:calc(3.325rem + var(--sapList_BorderWidth));--fdList_Message_View_Title_Size:var(--sapFontHeader5Size)}.fd-list--message-view .fd-list__item{height:auto}.fd-list--message-view .fd-list__item--byline{--fdList_Item_Height:var(--fdList_Message_View_Item_Byline_Height)}.fd-list--message-view .fd-list__content{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-flex:1;-ms-flex:1 1 auto;flex:1 1 auto;overflow:hidden;pointer-events:none;text-overflow:ellipsis;white-space:nowrap}.fd-list--message-view .fd-list__content:after,.fd-list--message-view .fd-list__content:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list--message-view .fd-list__title{color:var(--sapGroup_TitleTextColor);font-family:var(--sapFontHeaderFamily);font-size:var(--fdList_Message_View_Title_Size);font-weight:var(--sapFontHeaderWeight,normal);overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.fd-list--message-view .fd-list__subtitle{-webkit-padding-before:.25rem;color:var(--sapContent_LabelColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);overflow:hidden;padding-block-start:.25rem;text-overflow:ellipsis;white-space:nowrap}.fd-list--message-view .fd-list__link{padding-block:.5rem}.fd-list--message-view .fd-list__link.is-active .fd-list__subtitle,.fd-list--message-view .fd-list__link.is-active .fd-list__title,.fd-list--message-view .fd-list__link.is-active .fd-object-status .fd-object-status__icon,.fd-list--message-view .fd-list__link:active .fd-list__subtitle,.fd-list--message-view .fd-list__link:active .fd-list__title,.fd-list--message-view .fd-list__link:active .fd-object-status .fd-object-status__icon{color:var(--sapList_Active_TextColor)}.fd-list--message-view[class*=-compact],.fd-list--message-view[class*=-condensed],[class*=-compact] .fd-list--message-view:not([class*=-cozy]),[class*=-condensed] .fd-list--message-view:not([class*=-cozy]){--fdList_Message_View_Item_Height:calc(2rem + var(--sapList_BorderWidth));--fdList_Message_View_Title_Size:var(--sapFontHeader6Size);--fdList_Message_View_Item_Byline_Height:calc(3.25rem + var(--sapList_BorderWidth))}.fd-list--navigation .fd-list__item--link{padding-block:0;padding-inline:0}.fd-list--navigation.fd-icon-tab-bar__list .fd-list__link{gap:.25rem}.fd-list--navigation .fd-list__link{padding-inline:var(--fdList_Navigation_Link_Padding_Left,.9375rem) var(--fdList_Navigation_Link_Padding_Right,.9375rem)}.fd-list--navigation.fd-list--unread-indicator{--fdList_Navigation_Link_Padding_Left:2.125rem;--fdList_Navigation_Link_Padding_Right:.9375rem}.fd-list--navigation.fd-list--navigation-indication{--fdList_Navigation_Link_Padding_Left:.9375rem;--fdList_Navigation_Link_Padding_Right:0}.fd-list--navigation.fd-list--selection{--fdList_Navigation_Link_Padding_Left:2.75rem;--fdList_Navigation_Link_Padding_Right:.9375rem}.fd-list--navigation.fd-list--selection .fd-list__item--link{padding-block:0;padding-inline:0}.fd-list--navigation.fd-list--selection.fd-list--navigation-indication{--fdList_Navigation_Link_Padding_Left:2.75rem;--fdList_Navigation_Link_Padding_Right:0}.fd-list--navigation.fd-list--selection.fd-list--navigation-indication .fd-list__item--link{padding-block:0;padding-inline:0}.fd-list--navigation.fd-list--byline,.fd-list--navigation.fd-list--byline.fd-list--compact{--fdList_Navigation_Link_Padding_Left:1rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-list--navigation.fd-list--byline .fd-list__item--link,.fd-list--navigation.fd-list--byline.fd-list--compact .fd-list__item--link{padding-block:0;padding-inline:0}.fd-list--navigation.fd-list--byline .fd-list__link,.fd-list--navigation.fd-list--byline.fd-list--compact .fd-list__link{padding-block:.9375rem}.fd-list--navigation.fd-list--byline.fd-list--compact.fd-list--unread-indicator,.fd-list--navigation.fd-list--byline.fd-list--unread-indicator{--fdList_Navigation_Link_Padding_Left:2.125rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-list--navigation.fd-list--byline.fd-list--navigation-indication{--fdList_Navigation_Link_Padding_Left:.9375rem;--fdList_Navigation_Link_Padding_Right:0}.fd-list--navigation.fd-list--byline.fd-list--no-border .fd-list__item--link .fd-list__link{padding-block:.5rem}.fd-list--navigation.fd-list--byline.fd-list--no-border .fd-list__item--link:first-child{-webkit-padding-before:0;padding-block-start:0}.fd-list--navigation.fd-list--byline.fd-list--no-border .fd-list__item--link:first-child .fd-list__link{-webkit-padding-before:1rem;padding-block-start:1rem}.fd-list--navigation.fd-list--byline.fd-list--no-border .fd-list__item--link:first-child .fd-list__link--navigation-indicator:after{-webkit-margin-after:.5rem;margin-block-end:.5rem}.fd-list--navigation.fd-list--byline.fd-list--no-border .fd-list__item--link:last-child{-webkit-padding-after:0;padding-block-end:0}.fd-list--navigation.fd-list--byline.fd-list--no-border .fd-list__item--link:last-child .fd-list__link{-webkit-padding-after:1rem;padding-block-end:1rem}.fd-list--navigation.fd-list--byline.fd-list--no-border .fd-list__item--link:last-child .fd-list__link--navigation-indicator:after{margin-top:.25rem}.fd-list--navigation.fd-list--byline.fd-list--selection{--fdList_Navigation_Link_Padding_Left:2.75rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-list--navigation.fd-list--byline.fd-list--selection .fd-list__item--link{padding-block:0;padding-inline:0}.fd-list--navigation.fd-list--byline.fd-list--selection[class*=-compact],.fd-list--navigation.fd-list--byline.fd-list--selection[class*=-condensed],[class*=-compact] .fd-list--navigation.fd-list--byline.fd-list--selection:not([class*=-cozy]),[class*=-condensed] .fd-list--navigation.fd-list--byline.fd-list--selection:not([class*=-cozy]){--fdList_Navigation_Link_Padding_Left:2.75rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--no-border .fd-list__item--link,.fd-list--navigation.fd-list--byline.fd-list--selection[class*=-compact] .fd-list__item--link,.fd-list--navigation.fd-list--byline.fd-list--selection[class*=-condensed] .fd-list__item--link,[class*=-compact] .fd-list--navigation.fd-list--byline.fd-list--selection:not([class*=-cozy]) .fd-list__item--link,[class*=-condensed] .fd-list--navigation.fd-list--byline.fd-list--selection:not([class*=-cozy]) .fd-list__item--link{padding-block:0;padding-inline:0}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--no-border .fd-list__item--link .fd-list__link{padding-block:.5rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--no-border .fd-list__item--link:first-child{-webkit-padding-before:0;padding-block-start:0}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--no-border .fd-list__item--link:first-child .fd-list__link{-webkit-padding-before:1rem;padding-block-start:1rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--no-border .fd-list__item--link:first-child .fd-list__link--navigation-indicator:after{-webkit-margin-after:.5rem;margin-block-end:.5rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--no-border .fd-list__item--link:last-child{-webkit-padding-after:0;padding-block-end:0}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--no-border .fd-list__item--link:last-child .fd-list__link{-webkit-padding-after:1rem;padding-block-end:1rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--no-border .fd-list__item--link:last-child .fd-list__link--navigation-indicator:after{margin-top:.25rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication{--fdList_Navigation_Link_Padding_Left:2.75rem;--fdList_Navigation_Link_Padding_Right:0}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication .fd-list__item--link{padding-block:0;padding-inline:0}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication[class*=-compact],.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication[class*=-condensed],[class*=-compact] .fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication:not([class*=-cozy]),[class*=-condensed] .fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication:not([class*=-cozy]){--fdList_Navigation_Link_Padding_Left:2.75rem;--fdList_Navigation_Link_Padding_Right:0}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication.fd-list--no-border .fd-list__item--link .fd-list__link{padding-block:.5rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication.fd-list--no-border .fd-list__item--link:first-child{-webkit-padding-before:0;padding-block-start:0}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication.fd-list--no-border .fd-list__item--link:first-child .fd-list__link{-webkit-padding-before:1rem;padding-block-start:1rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication.fd-list--no-border .fd-list__item--link:first-child .fd-list__link--navigation-indicator:after{-webkit-margin-after:.5rem;margin-block-end:.5rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication.fd-list--no-border .fd-list__item--link:last-child{-webkit-padding-after:0;padding-block-end:0}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication.fd-list--no-border .fd-list__item--link:last-child .fd-list__link{-webkit-padding-after:1rem;padding-block-end:1rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication.fd-list--no-border .fd-list__item--link:last-child .fd-list__link--navigation-indicator:after{margin-top:.25rem}.fd-list--navigation.fd-list--navigation-object{--fdList_Navigation_Link_Padding_Left:1rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-list--navigation.fd-list--navigation-object .fd-list__item--link,.fd-list--navigation.fd-list--navigation-object[dir=rtl] .fd-list__item--link,[dir=rtl] .fd-list--navigation.fd-list--navigation-object .fd-list__item--link{padding-block:0;padding-inline:0}.fd-list--navigation.fd-list--navigation-object .fd-list__link{padding-block:1rem}.fd-list--navigation-indication .fd-list:after{content:\"\";min-width:1.5rem}.fd-list--navigation-indication fd-list__item.fd-list__item--link{-webkit-padding-end:0;padding-inline-end:0}.fd-list--navigation-indication fd-list__item.fd-list__item--link:after{content:none}.fd-list--navigation-indication .fd-list__link:after{content:\"\";min-width:2.5rem}.fd-list--navigation-indication .fd-list__link--navigation-indicator:after{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;color:var(--sapContent_NonInteractiveIconColor);content:\"\\e1ed\";font-family:SAP-icons;font-size:var(--fdListNavigationIndicatorFontSize,var(--sapFontLargeSize));height:100%;text-decoration:none;text-transform:none}.fd-list--navigation-indication .fd-list__link--navigation-indicator[dir=rtl]:after,[dir=rtl] .fd-list--navigation-indication .fd-list__link--navigation-indicator:after{content:\"\\e1ee\"}.fd-list--navigation-indication .fd-list__icon:last-child,.fd-list--navigation-indication .fd-list__icon[dir=rtl]:last-child,[dir=rtl] .fd-list--navigation-indication .fd-list__icon:last-child{margin-block:0;margin-inline:0}.fd-list--navigation-indication .is-navigated{-webkit-box-shadow:inset -.1875rem 0 0 0 var(--sapList_SelectionBorderColor);box-shadow:inset -.1875rem 0 0 0 var(--sapList_SelectionBorderColor)}.fd-list--navigation-indication .is-navigated[dir=rtl],[dir=rtl] .fd-list--navigation-indication .is-navigated{-webkit-box-shadow:inset .1875rem 0 0 0 var(--sapList_SelectionBorderColor);box-shadow:inset .1875rem 0 0 0 var(--sapList_SelectionBorderColor)}.fd-list--selection .fd-list__item{cursor:pointer;padding-inline:2.75rem .9375rem}.fd-list--selection .fd-list__item.is-hover,.fd-list--selection .fd-list__item:hover{--fdList_Item_Background_Color:var(--sapList_Hover_Background)}.fd-list--selection .fd-list__item.is-selected,.fd-list--selection .fd-list__item[aria-selected=true]{--fdList_Item_Background_Color:var(--sapList_SelectionBackgroundColor);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_SelectionBorderColor)}.fd-list--selection .fd-list__item.is-selected .fd-list__link,.fd-list--selection .fd-list__item[aria-selected=true] .fd-list__link{background:inherit}.fd-list--selection .fd-list__item.is-selected .fd-list__link.is-active,.fd-list--selection .fd-list__item.is-selected .fd-list__link:active,.fd-list--selection .fd-list__item[aria-selected=true] .fd-list__link.is-active,.fd-list--selection .fd-list__item[aria-selected=true] .fd-list__link:active{--fdList_Item_Background_Color:var(--sapList_Active_Background)}.fd-list--selection .fd-list__item.is-selected.is-hover,.fd-list--selection .fd-list__item.is-selected:hover,.fd-list--selection .fd-list__item[aria-selected=true].is-hover,.fd-list--selection .fd-list__item[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Hover_SelectionBackground)}.fd-list--selection .fd-list__form-item{left:0;position:absolute;top:0;z-index:5}.fd-list--selection .fd-list__form-item[dir=rtl],[dir=rtl] .fd-list--selection .fd-list__form-item{left:auto;right:0}.fd-list--selection[class*=-compact] .fd-list__form-item,.fd-list--selection[class*=-condensed] .fd-list__form-item,[class*=-compact] .fd-list--selection:not([class*=-cozy]) .fd-list__form-item,[class*=-condensed] .fd-list--selection:not([class*=-cozy]) .fd-list__form-item{left:.375rem}.fd-list--selection[class*=-compact] .fd-list__form-item[dir=rtl],.fd-list--selection[class*=-condensed] .fd-list__form-item[dir=rtl],[class*=-compact] .fd-list--selection:not([class*=-cozy]) .fd-list__form-item[dir=rtl],[class*=-condensed] .fd-list--selection:not([class*=-cozy]) .fd-list__form-item[dir=rtl],[dir=rtl] .fd-list--selection[class*=-compact] .fd-list__form-item,[dir=rtl] .fd-list--selection[class*=-condensed] .fd-list__form-item,[dir=rtl] [class*=-compact] .fd-list--selection:not([class*=-cozy]) .fd-list__form-item,[dir=rtl] [class*=-condensed] .fd-list--selection:not([class*=-cozy]) .fd-list__form-item{left:auto;right:.375rem}.fd-list--selection.fd-list--byline .fd-list__item{padding-inline:2.75rem 1rem}.fd-list--selection.fd-list--byline .fd-list__form-item{top:.3125rem}.fd-list--selection.fd-list--byline[class*=-compact] .fd-list__item,.fd-list--selection.fd-list--byline[class*=-condensed] .fd-list__item,[class*=-compact] .fd-list--selection.fd-list--byline:not([class*=-cozy]) .fd-list__item,[class*=-condensed] .fd-list--selection.fd-list--byline:not([class*=-cozy]) .fd-list__item{padding-inline:2.75rem 1rem}.fd-list--selection.fd-list--byline[class*=-compact] .fd-list__form-item,.fd-list--selection.fd-list--byline[class*=-condensed] .fd-list__form-item,[class*=-compact] .fd-list--selection.fd-list--byline:not([class*=-cozy]) .fd-list__form-item,[class*=-condensed] .fd-list--selection.fd-list--byline:not([class*=-cozy]) .fd-list__form-item{left:.375rem;top:.5rem}.fd-list--selection.fd-list--byline[class*=-compact] .fd-list__form-item[dir=rtl],.fd-list--selection.fd-list--byline[class*=-condensed] .fd-list__form-item[dir=rtl],[class*=-compact] .fd-list--selection.fd-list--byline:not([class*=-cozy]) .fd-list__form-item[dir=rtl],[class*=-condensed] .fd-list--selection.fd-list--byline:not([class*=-cozy]) .fd-list__form-item[dir=rtl],[dir=rtl] .fd-list--selection.fd-list--byline[class*=-compact] .fd-list__form-item,[dir=rtl] .fd-list--selection.fd-list--byline[class*=-condensed] .fd-list__form-item,[dir=rtl] [class*=-compact] .fd-list--selection.fd-list--byline:not([class*=-cozy]) .fd-list__form-item,[dir=rtl] [class*=-condensed] .fd-list--selection.fd-list--byline:not([class*=-cozy]) .fd-list__form-item{left:auto;right:.375rem}.fd-list--selection.fd-list--byline.fd-list--no-border .fd-list__item{padding-block:.5rem;padding-inline:2.75rem 1rem}.fd-list--selection.fd-list--byline.fd-list--no-border .fd-list__item:first-child{-webkit-padding-before:.9375rem;padding-block-start:.9375rem}.fd-list--selection.fd-list--byline.fd-list--no-border .fd-list__item:last-child{-webkit-padding-after:.9375rem;padding-block-end:.9375rem}.fd-list--selection.fd-list--byline.fd-list--no-border .fd-list__item:not(:first-child) .fd-list__form-item,.fd-list--selection.fd-list--byline.fd-list--no-border .fd-list__item:not(:first-child) .fd-list__form-item.fd-form-item{top:0}.fd-list--selection.fd-list--byline.fd-list--no-border .fd-list__item:not(:first-child) .fd-list__form-item label,.fd-list--selection.fd-list--byline.fd-list--no-border .fd-list__item:not(:first-child) .fd-list__form-item.fd-form-item label{-webkit-padding-before:.5rem;padding-block-start:.5rem}.fd-list--selection.fd-list--byline.fd-list--no-border .fd-list__item:not(:first-child) .fd-list__form-item label:after,.fd-list--selection.fd-list--byline.fd-list--no-border .fd-list__item:not(:first-child) .fd-list__form-item.fd-form-item label:after{top:.375rem}.fd-list--selection.fd-list--selection-row .fd-list__item{-webkit-padding-start:1rem;padding-inline-start:1rem}:root{--fdList_Item_Background_Color:var(--sapList_Background);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_BorderColor);--fdList_Item_Text_Font_Size:var(--sapFontLargeSize);--fdList_Item_Icon_Font_Size:1.125rem;--fdList_Item_Height:var(--sapElement_LineHeight);--fdList_Status_Text_Color:var(--sapNeutralTextColor);--fdList_Message_Background_Color:var(--sapNeutralBackground)}.fd-list__item-counter{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;overflow:hidden;padding-block:0;padding-inline:0;text-overflow:ellipsis;white-space:nowrap;-webkit-margin-start:1rem;color:var(--sapContent_MarkerTextColor);margin-inline-start:1rem}.fd-list__item-counter:after,.fd-list__item-counter:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar{display:flex;flex-direction:column;height:100%}.fd-icon-tab-bar .fd-icon-tab-bar__item--hidden{display:none}.fd-icon-tab-bar__list{max-height:20rem}.fd-icon-tab-bar__list-item-counter{padding-left:.313rem}[dir=rtl] .fd-icon-tab-bar .fd-icon-tab-bar__list-item-counter,.fd-icon-tab-bar[dir=rtl] .fd-icon-tab-bar__list-item-counter{padding-left:0;padding-right:.25rem}.fd-icon-tab-bar__popover,.fd-icon-tab-bar__popover .fd-popover__control{height:100%}.cdk-drag-placeholder{opacity:.3;pointer-events:none}.screenreader-only{position:absolute!important;height:1px;width:1px;overflow:hidden;opacity:0;clip:rect(1px 1px 1px 1px);clip:rect(1px,1px,1px,1px)}\n/*! Bundled license information:\n\nfundamental-styles/dist/list.css:\n  (*!\n   * Fundamental Library Styles v0.38.0\n   * Copyright (c) 2024 SAP SE or an SAP affiliate company.\n   * Licensed under Apache License 2.0 (https://github.com/SAP/fundamental-styles/blob/main/LICENSE)\n   *)\n*/\n"], dependencies: [{ kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }, { kind: "component", type: IconTabBarProcessTypeComponent, selector: "fdp-icon-tab-bar-process-type" }, { kind: "component", type: IconTabBarFilterTypeComponent, selector: "fdp-icon-tab-bar-filter-type", inputs: ["showTotalTab"] }, { kind: "component", type: IconTabBarIconTypeComponent, selector: "fdp-icon-tab-bar-icon-type", inputs: ["showLabel"] }, { kind: "component", type: IconTabBarTextTypeComponent, selector: "fdp-icon-tab-bar-text-type", inputs: ["enableTabReordering", "layoutMode", "multiClick"], outputs: ["reordered"] }, { kind: "directive", type: ScrollbarDirective, selector: "[fdScrollbar], [fd-scrollbar]", inputs: ["noHorizontalScroll", "noVerticalScroll", "alwaysVisible"] }, { kind: "directive", type: ScrollSpyDirective, selector: "[fdScrollSpy]", inputs: ["trackedTags", "fireEmpty", "targetPercent", "targetOffset", "scrollSpyDisabled"], outputs: ["spyChange"], exportAs: ["fdScrollSpy"] }, { kind: "directive", type: IconTabBarTabContentDirective, selector: "fdp-icon-tab-bar-tab-content", inputs: ["id", "uId"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: IconTabBarComponent, decorators: [{
            type: Component,
            args: [{ selector: 'fdp-icon-tab-bar', changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, providers: [
                        {
                            provide: FD_TABLIST,
                            useExisting: IconTabBarComponent
                        }
                    ], imports: [
                        NgTemplateOutlet,
                        IconTabBarProcessTypeComponent,
                        IconTabBarFilterTypeComponent,
                        IconTabBarIconTypeComponent,
                        IconTabBarTextTypeComponent,
                        ScrollbarDirective,
                        ScrollSpyDirective,
                        IconTabBarTabContentDirective
                    ], template: "<div class=\"fd-icon-tab-bar\" [class]=\"_cssClassForContainer\">\n    @if (iconTabType() === 'text') {\n        <ng-template [ngTemplateOutlet]=\"textType\"></ng-template>\n    }\n    @if (iconTabType() === 'icon' || iconTabType() === 'icon-only') {\n        <ng-template [ngTemplateOutlet]=\"iconType\"></ng-template>\n    }\n    @if (iconTabType() === 'filter') {\n        <ng-template [ngTemplateOutlet]=\"filterType\"></ng-template>\n    }\n    @if (iconTabType() === 'process') {\n        <ng-template [ngTemplateOutlet]=\"processType\"></ng-template>\n    }\n    <div\n        class=\"fd-icon-tab-bar__content\"\n        fd-scrollbar\n        (spyChange)=\"highlightActiveTab($event.id)\"\n        [scrollSpyDisabled]=\"!stackContent || _disableScrollSpy\"\n        fdScrollSpy\n        [trackedTags]=\"['fdp-icon-tab-bar-tab-content']\"\n        [style.max-height]=\"maxContentHeight()\"\n    >\n        @if (stackContent) {\n            @for (tab of _flatTabs$(); track tab) {\n                <fdp-icon-tab-bar-tab-content [id]=\"tab.id\" [uId]=\"tab.uId\" class=\"fd-icon-tab-bar__tab-content\">\n                    <ng-template [ngTemplateOutlet]=\"tab.renderer || null\"></ng-template>\n                </fdp-icon-tab-bar-tab-content>\n            }\n        } @else {\n            @if (_tabRenderer$(); as tabRenderer) {\n                <fdp-icon-tab-bar-tab-content\n                    [id]=\"tabRenderer.id\"\n                    [uId]=\"tabRenderer.uId\"\n                    class=\"fd-icon-tab-bar__tab-content\"\n                >\n                    <ng-template [ngTemplateOutlet]=\"tabRenderer.renderer || null\"></ng-template>\n                </fdp-icon-tab-bar-tab-content>\n            }\n        }\n        <ng-content select=\"[fdpIconTabBarFreeContent]\"></ng-content>\n    </div>\n</div>\n<ng-template #processType>\n    <fdp-icon-tab-bar-process-type\n        [tabs]=\"_tabs$()\"\n        [isRtl]=\"_rtl$()\"\n        [selectedUid]=\"_selectedUid()\"\n        (selectedUidChange)=\"_selectedUid.set($event)\"\n        [densityMode]=\"densityMode()\"\n        [colorAssociations]=\"colorAssociations()\"\n        (selected)=\"_selectItem($event)\"\n        [tabHeadingLevel]=\"tabHeadingLevel()\"\n    ></fdp-icon-tab-bar-process-type>\n</ng-template>\n<ng-template #filterType>\n    <fdp-icon-tab-bar-filter-type\n        [tabs]=\"_tabs$()\"\n        [showTotalTab]=\"showTotalTab()\"\n        [isRtl]=\"_rtl$()\"\n        [selectedUid]=\"_selectedUid()\"\n        (selectedUidChange)=\"_selectedUid.set($event)\"\n        [densityMode]=\"densityMode()\"\n        [colorAssociations]=\"colorAssociations()\"\n        (selected)=\"_selectItem($event)\"\n        [tabHeadingLevel]=\"tabHeadingLevel()\"\n    ></fdp-icon-tab-bar-filter-type>\n</ng-template>\n<ng-template #iconType>\n    <fdp-icon-tab-bar-icon-type\n        [showLabel]=\"iconTabType() === 'icon'\"\n        [tabs]=\"_tabs$()\"\n        [isRtl]=\"_rtl$()\"\n        [selectedUid]=\"_selectedUid()\"\n        (selectedUidChange)=\"_selectedUid.set($event)\"\n        [densityMode]=\"densityMode()\"\n        [colorAssociations]=\"colorAssociations()\"\n        (selected)=\"_selectItem($event)\"\n        (closeTab)=\"_closeTab($event)\"\n        [tabHeadingLevel]=\"tabHeadingLevel()\"\n    ></fdp-icon-tab-bar-icon-type>\n</ng-template>\n<ng-template #textType>\n    <fdp-icon-tab-bar-text-type\n        [tabs]=\"_tabs$()\"\n        [multiClick]=\"multiClick()\"\n        [layoutMode]=\"layoutMode()\"\n        [isRtl]=\"_rtl$()\"\n        [selectedUid]=\"_selectedUid()\"\n        (selectedUidChange)=\"_selectedUid.set($event)\"\n        [densityMode]=\"densityMode()\"\n        [enableTabReordering]=\"enableTabReordering()\"\n        [colorAssociations]=\"colorAssociations()\"\n        (reordered)=\"_onReorder($event)\"\n        (selected)=\"_selectItem($event)\"\n        (closeTab)=\"_closeTab($event)\"\n        [tabHeadingLevel]=\"tabHeadingLevel()\"\n    ></fdp-icon-tab-bar-text-type>\n</ng-template>\n", styles: ["@charset \"UTF-8\";.fd-icon-tab-bar__counter,.fd-icon-tab-bar__tag{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:var(--sapContent_LabelColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;line-height:1rem;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;white-space:nowrap}.fd-icon-tab-bar__counter:after,.fd-icon-tab-bar__counter:before,.fd-icon-tab-bar__tag:after,.fd-icon-tab-bar__tag:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__label:before,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__label[dir=rtl]:after,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__label:before,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__label[dir=rtl]:after,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__label:before,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__label[dir=rtl]:after,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__label:before,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__label[dir=rtl]:after,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon[dir=rtl]:after,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tag:before,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tag[dir=rtl]:after,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon[dir=rtl]:after,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tag:before,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tag[dir=rtl]:after,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon[dir=rtl]:after,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tag:before,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tag[dir=rtl]:after,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon[dir=rtl]:after,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tag:before,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tag[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__label:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__label:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__label:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__label:after,[dir=rtl] .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tag:after,[dir=rtl] .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tag:after,[dir=rtl] .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tag:after,[dir=rtl] .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tag:after{speak:none;font-family:SAP-icons;font-size:.75rem;-webkit-font-smoothing:antialiased;color:var(--fdIconTabBar_Tag_Icon_Color);display:var(--fdIcon_Tab_Bar_Semantic_Icon_Display)}.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon[dir=rtl]:after,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon[dir=rtl]:after,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon[dir=rtl]:after,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon:after{bottom:0;margin-block:0;margin-inline:0;position:absolute;width:.75rem}.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon[dir=rtl]:before,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tag[dir=rtl]:before,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon[dir=rtl]:before,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tag[dir=rtl]:before,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon[dir=rtl]:before,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tag[dir=rtl]:before,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon[dir=rtl]:before,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tag[dir=rtl]:before,[dir=rtl] .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon:before,[dir=rtl] .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tag:before,[dir=rtl] .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon:before,[dir=rtl] .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tag:before,[dir=rtl] .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon:before,[dir=rtl] .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tag:before,[dir=rtl] .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon:before,[dir=rtl] .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tag:before{content:none;margin-block:0;margin-inline:0}.fd-icon-tab-bar{--fdIconTabBar_Icon_Size:2.75rem;--fdIconTabBar_Icon_Font_Size:1.25rem;--fdIcon_Tab_Bar_Container_Badge_Size:.5rem;border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0}.fd-icon-tab-bar:after,.fd-icon-tab-bar:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__header{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-align:center;-ms-flex-align:center;align-items:center;background:var(--sapObjectHeader_Background);-webkit-box-shadow:var(--sapContent_HeaderShadow);box-shadow:var(--sapContent_HeaderShadow);list-style:none;min-height:2.75rem;position:relative}.fd-icon-tab-bar__header:after,.fd-icon-tab-bar__header:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__header:has(.fd-icon-tab-bar__item--overflow){overflow:visible}.fd-icon-tab-bar__panel{background:var(--sapBackgroundColor)}.fd-icon-tab-bar__item{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-margin-end:2rem;margin-inline-end:2rem}.fd-icon-tab-bar__item:after,.fd-icon-tab-bar__item:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__item--negative{--fdIconTabBar_Badge_Background:var(--fdIcon_Tab_Bar_Semantic_Badge_Color_Negative);--fdIconTabBar_Badge_Background_After:var(--fdIcon_Tab_Bar_Semantic_Badge_Color_Negative);--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Negative);--fdIconTabBar_Tag_Icon_Color:var(--sapNegativeElementColor);--fdIconTabBar_Tag_Icon_Content:\\e1ec;--fdIconTabBar_Icon_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Negative);--fdIconTabBar_Icon_Border_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Negative)}.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tag:before,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tag[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tag:after{-webkit-margin-end:.5rem;content:\"\\e1ec\";margin-inline-end:.5rem}.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tab.is-hover,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tab:hover{--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Negative)}.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tab.is-selected,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tab[aria-selected=true]{--fdIconTabBar_Selection_Bar_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Negative);--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Negative);--fdIconTabBar_Icon_Background:var(--fdIcon_Tab_Bar_Semantic_Background_Color_Negative)}.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tab.is-selected.is-hover,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tab.is-selected:hover,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tab[aria-selected=true].is-hover,.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__tab[aria-selected=true]:hover{--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Negative)}.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon{position:relative}.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon:before{content:\"\\e1ec\";right:-.9rem}.fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon:after{content:\"\\e1ec\";left:-.9rem;right:auto}.fd-icon-tab-bar__item--positive{--fdIconTabBar_Badge_Background:var(--fdIcon_Tab_Bar_Semantic_Badge_Color_Positive);--fdIconTabBar_Badge_Background_After:var(--fdIcon_Tab_Bar_Semantic_Badge_Color_Positive);--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Positive);--fdIconTabBar_Tag_Icon_Color:var(--sapPositiveElementColor);--fdIconTabBar_Tag_Icon_Content:\\e1c1;--fdIconTabBar_Icon_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Positive);--fdIconTabBar_Icon_Border_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Positive)}.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tag:before,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tag[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tag:after{-webkit-margin-end:.5rem;content:\"\\e1c1\";margin-inline-end:.5rem}.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tab.is-hover,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tab:hover{--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Positive)}.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tab.is-selected,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tab[aria-selected=true]{--fdIconTabBar_Selection_Bar_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Positive);--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Positive);--fdIconTabBar_Icon_Background:var(--fdIcon_Tab_Bar_Semantic_Background_Color_Positive)}.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tab.is-selected.is-hover,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tab.is-selected:hover,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tab[aria-selected=true].is-hover,.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__tab[aria-selected=true]:hover{--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Positive)}.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon{position:relative}.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon:before{content:\"\\e1c1\";right:-.9rem}.fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon:after{content:\"\\e1c1\";left:-.9rem;right:auto}.fd-icon-tab-bar__item--critical{--fdIconTabBar_Badge_Background:var(--fdIcon_Tab_Bar_Semantic_Badge_Color_Critical);--fdIconTabBar_Badge_Background_After:var(--fdIcon_Tab_Bar_Semantic_Badge_Color_Critical);--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Critical);--fdIconTabBar_Tag_Icon_Color:var(--sapCriticalElementColor);--fdIconTabBar_Tag_Icon_Content:\\e053;--fdIconTabBar_Icon_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Critical);--fdIconTabBar_Icon_Border_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Critical)}.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tag:before,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tag[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tag:after{-webkit-margin-end:.5rem;content:\"\\e053\";margin-inline-end:.5rem}.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tab.is-hover,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tab:hover{--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Critical)}.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tab.is-selected,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tab[aria-selected=true]{--fdIconTabBar_Selection_Bar_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Critical);--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Critical);--fdIconTabBar_Icon_Background:var(--fdIcon_Tab_Bar_Semantic_Background_Color_Critical)}.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tab.is-selected.is-hover,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tab.is-selected:hover,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tab[aria-selected=true].is-hover,.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__tab[aria-selected=true]:hover{--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Critical)}.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon{position:relative}.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon:before{content:\"\\e053\";right:-.9rem}.fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon:after{content:\"\\e053\";left:-.9rem;right:auto}.fd-icon-tab-bar__item--informative{--fdIconTabBar_Badge_Background:var(--fdIcon_Tab_Bar_Semantic_Badge_Color_Informative);--fdIconTabBar_Badge_Background_After:var(--fdIcon_Tab_Bar_Semantic_Badge_Color_Informative);--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Informative);--fdIconTabBar_Tag_Icon_Color:var(--sapInformativeElementColor);--fdIconTabBar_Tag_Icon_Content:\\e289;--fdIconTabBar_Icon_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Informative);--fdIconTabBar_Icon_Border_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Informative)}.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tag:before,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tag[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tag:after{-webkit-margin-end:.5rem;content:\"\\e289\";margin-inline-end:.5rem}.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tab.is-hover,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tab:hover{--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Informative)}.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tab.is-selected,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tab[aria-selected=true]{--fdIconTabBar_Selection_Bar_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Informative);--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Informative);--fdIconTabBar_Icon_Background:var(--fdIcon_Tab_Bar_Semantic_Background_Color_Informative)}.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tab.is-selected.is-hover,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tab.is-selected:hover,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tab[aria-selected=true].is-hover,.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__tab[aria-selected=true]:hover{--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Semantic_Color_Informative)}.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon{position:relative}.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon:before{content:\"\\e289\";right:-.9rem}.fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon:after{content:\"\\e289\";left:-.9rem;right:auto}.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab{-webkit-box-orient:horizontal;-webkit-box-direction:normal;-ms-flex-direction:row;flex-direction:row;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab.is-focus,.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab:focus{z-index:5}.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tab-container,.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tab-container{position:relative}.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tab-container .fd-icon-tab-bar__tag,.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tab-container .fd-icon-tab-bar__tag{outline:none}.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tab-container .fd-icon-tab-bar__tag:after,.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tab-container .fd-icon-tab-bar__tag:after{border:none}.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tab-container:after,.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tab-container:after{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);border-radius:var(--fdIcon_Tab_Bar_Text_Focus_Radius);content:\"\";display:block;inset:0;inset:var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset);position:absolute}.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__badge,.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__badge{right:-.4375rem;top:-.125rem}.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__badge[dir=rtl],.fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__badge[dir=rtl],[dir=rtl] .fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__badge,[dir=rtl] .fd-icon-tab-bar__item--single-click .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__badge{left:-.4375rem;right:auto}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;position:relative}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click .fd-icon-tab-bar__tab{padding-inline:.188rem 1.938rem}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click .fd-icon-tab-bar__tab .fd-icon-tab-bar__badge{right:1.625rem}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click .fd-icon-tab-bar__tab .fd-icon-tab-bar__badge[dir=rtl],[dir=rtl] .fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click .fd-icon-tab-bar__tab .fd-icon-tab-bar__badge{left:1.625rem;right:auto}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click .fd-icon-tab-bar__tab.is-selected+.fd-icon-tab-bar__popover .fd-icon-tab-bar__button-container:before,.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click .fd-icon-tab-bar__tab[aria-selected=true]+.fd-icon-tab-bar__popover .fd-icon-tab-bar__button-container:before{--fdIconTabBar_Multi_Click_Separator_Background:var(--sapContent_Selected_ForegroundColor)}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__tab{padding-inline:.188rem 3.938rem}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__popover{height:100%;max-width:1.5rem;min-width:1.5rem;right:2.188rem;width:1.5rem}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__popover[dir=rtl],[dir=rtl] .fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__popover{left:2.188rem;right:auto}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__popover .fd-icon-tab-bar__button-container{max-width:1.5rem;min-width:1.5rem;width:1.5rem}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--multi-click.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__popover .fd-icon-tab-bar__button-container:before{display:var(--fdIcon_Tab_Bar_Multi_Click_Button_Separator_Display)}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--closable{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;position:relative}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__tab{padding-inline:.188rem 2.188rem}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container{max-width:2rem;min-width:2rem;position:absolute;right:0;width:2rem;z-index:10}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container[dir=rtl],[dir=rtl] .fd-icon-tab-bar__item.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container{left:0;right:auto}.fd-icon-tab-bar__item.fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container:before{display:none}.fd-icon-tab-bar__item--dnd-preview{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;font-size:var(--sapFontSize);line-height:var(--sapContent_LineHeight);opacity:.8;z-index:9999!important}.fd-icon-tab-bar__item--dnd-preview,.fd-icon-tab-bar__item--dnd-preview *{background:var(--sapObjectHeader_Background)}.fd-icon-tab-bar__item--dnd-hovered{background:var(--sapButton_Hover_Background);-webkit-box-shadow:inset 0 0 0 .125rem var(--sapButton_Hover_BorderColor);box-shadow:inset 0 0 0 .125rem var(--sapButton_Hover_BorderColor)}.fd-icon-tab-bar__item--dnd-hovered.fd-icon-tab-bar__list-item{background:var(--sapButton_Hover_Background)!important}.fd-icon-tab-bar__item--dnd-hovered.fd-icon-tab-bar__list-item .fd-icon-tab-bar__list-link{background:none}.fd-icon-tab-bar__item--dnd-separator{position:relative}.fd-icon-tab-bar__item--dnd-separator:before{background:var(--fdIcon_Tab_Bar_Separator_Color);content:\"\";height:100%;left:-.5rem;position:absolute;top:0;width:.125rem}[dir=rtl] .fd-icon-tab-bar__item--dnd-separator:before{left:auto;right:-.5rem}.fd-icon-tab-bar__item--dnd-separator:after{background:var(--sapObjectHeader_Background);border:.125rem solid var(--fdIcon_Tab_Bar_Separator_Color);border-radius:50%;content:\"\";height:.5rem;left:-.7rem;position:absolute;top:0;width:.5rem}[dir=rtl] .fd-icon-tab-bar__item--dnd-separator:after{left:auto;right:-.7rem}.fd-icon-tab-bar__item--dnd-separator.fd-icon-tab-bar__list-item:before{height:.125rem;left:.0625rem;right:unset;top:.125rem;width:100%}[dir=rtl] .fd-icon-tab-bar__item--dnd-separator.fd-icon-tab-bar__list-item:before{left:auto;right:.0625rem}.fd-icon-tab-bar__item--dnd-separator.fd-icon-tab-bar__list-item:after{left:0;right:unset;top:-.0625rem;z-index:1}[dir=rtl] .fd-icon-tab-bar__item--dnd-separator.fd-icon-tab-bar__list-item:after{left:auto;right:0}.fd-icon-tab-bar__item--dnd-separator-end{position:relative}.fd-icon-tab-bar__item--dnd-separator-end:before{background:var(--fdIcon_Tab_Bar_Separator_Color);content:\"\";height:100%;position:absolute;right:-.5rem;top:0;width:.125rem}[dir=rtl] .fd-icon-tab-bar__item--dnd-separator-end:before{left:-.5rem;right:auto}.fd-icon-tab-bar__item--dnd-separator-end:after{background:var(--sapObjectHeader_Background);border:.125rem solid var(--fdIcon_Tab_Bar_Separator_Color);border-radius:50%;content:\"\";height:.5rem;position:absolute;right:-.7rem;top:0;width:.5rem}[dir=rtl] .fd-icon-tab-bar__item--dnd-separator-end:after{left:-.7rem;right:auto}.fd-icon-tab-bar__item--dnd-separator-end.fd-icon-tab-bar__list-item:before{bottom:.125rem;height:.125rem;left:.0625rem;right:unset;top:unset;width:100%}[dir=rtl] .fd-icon-tab-bar__item--dnd-separator-end.fd-icon-tab-bar__list-item:before{left:auto;right:.0625rem}.fd-icon-tab-bar__item--dnd-separator-end.fd-icon-tab-bar__list-item:after{bottom:-.0625rem;left:0;right:unset;top:unset;z-index:1}[dir=rtl] .fd-icon-tab-bar__item--dnd-separator-end.fd-icon-tab-bar__list-item:after{left:auto;right:0}.fd-icon-tab-bar__button-container{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;height:2.75rem;max-height:2.75rem;max-width:1.5rem;min-height:2.75rem;min-width:1.5rem;position:relative;width:1.5rem}.fd-icon-tab-bar__button-container:after,.fd-icon-tab-bar__button-container:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__button-container:before{background:var(--fdIconTabBar_Multi_Click_Separator_Background);content:\"\";display:var(--fdIcon_Tab_Bar_Multi_Click_Button_Separator_Display);height:1rem;left:0;position:absolute;top:.75rem;width:.0625rem}.fd-icon-tab-bar__button-container[dir=rtl]:before,[dir=rtl] .fd-icon-tab-bar__button-container:before{left:auto;right:0}.fd-icon-tab-bar .fd-icon-tab-bar__button{border-radius:.25rem;font-family:var(--sapContent_IconFontFamily);height:1.5rem;min-width:1.5rem;width:1.5rem;--fdButton_Hover_Shadow:none}.fd-icon-tab-bar .fd-icon-tab-bar__button.is-focus,.fd-icon-tab-bar .fd-icon-tab-bar__button:focus{z-index:5}.fd-icon-tab-bar .fd-icon-tab-bar__button.is-focus:after,.fd-icon-tab-bar .fd-icon-tab-bar__button:focus:after{border-radius:var(--fdIcon_Tab_Bar_Multi_Click_Button_Focus_Radius)}.fd-icon-tab-bar__tab{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:start;-ms-flex-align:start;align-items:flex-start;cursor:pointer;min-width:2rem;padding-block:.875rem;padding-inline:.188rem;position:relative;text-decoration:none}.fd-icon-tab-bar__tab:after,.fd-icon-tab-bar__tab:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__tab.is-selected,.fd-icon-tab-bar__tab[aria-selected=true]{--fdIconTabBar_Icon_Background:var(--sapSelectedColor);--fdIconTabBar_Icon_Color:var(--sapContent_ContrastIconColor);--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Selection_Color);--fdIconTabBar_Arrow_Color:var(--sapContent_Selected_ForegroundColor);position:relative}.fd-icon-tab-bar__tab.is-selected:after,.fd-icon-tab-bar__tab[aria-selected=true]:after{background:var(--fdIconTabBar_Selection_Bar_Color);border-radius:.125rem .125rem 0 0;bottom:0;content:\"\";display:inline-block;height:var(--fdIcon_Tab_Bar_Selection_Bar_Height);left:0;position:absolute;-webkit-transition:all .125s ease-in;transition:all .125s ease-in;width:100%}.fd-icon-tab-bar__tab.is-selected[dir=rtl]:after,.fd-icon-tab-bar__tab[aria-selected=true][dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__tab.is-selected:after,[dir=rtl] .fd-icon-tab-bar__tab[aria-selected=true]:after{left:auto;right:0}.fd-icon-tab-bar__tab.is-selected+.fd-icon-tab-bar__popover .fd-icon-tab-bar__button,.fd-icon-tab-bar__tab[aria-selected=true]+.fd-icon-tab-bar__popover .fd-icon-tab-bar__button{--fdButtonColor:var(--sapContent_Selected_ForegroundColor)}.fd-icon-tab-bar__tab.is-hover,.fd-icon-tab-bar__tab:hover{--fdIconTabBar_Tag_Color:var(--fdIcon_Tab_Bar_Label_Hover_Color);--fdIconTabBar_Arrow_Color:var(--sapContent_Selected_ForegroundColor)}.fd-icon-tab-bar__tab.is-focus,.fd-icon-tab-bar__tab:focus{-webkit-box-shadow:none;box-shadow:none;outline:none;z-index:5}.fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__icon:after,.fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__icon:after{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);border-radius:var(--fdIcon_Tab_Bar_Icon_Focus_Radius);content:\"\";display:block;inset:0;inset:var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset);inset:var(--fdIcon_Tab_Bar_Icon_Focus_Offset) var(--fdIcon_Tab_Bar_Icon_Focus_Offset) var(--fdIcon_Tab_Bar_Icon_Focus_Offset) var(--fdIcon_Tab_Bar_Icon_Focus_Offset);position:absolute}.fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__container--filter:before,.fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tag:after,.fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__container--filter:before,.fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tag:after{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);border-radius:var(--fdIcon_Tab_Bar_Text_Focus_Radius);content:\"\";display:block;inset:0;inset:var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset);position:absolute}.fd-icon-tab-bar__tab-container{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.fd-icon-tab-bar__tab-container:after,.fd-icon-tab-bar__tab-container:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__arrow{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;height:1rem;margin-inline:var(--fdIcon_Tab_Bar_Single_Click_Icon_Container_Padding_Left) var(--fdIcon_Tab_Bar_Single_Click_Icon_Container_Padding_Right);width:var(--fdIcon_Tab_Bar_Single_Click_Icon_Container_Width)}.fd-icon-tab-bar__arrow:after,.fd-icon-tab-bar__arrow:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__arrow [class*=sap-icon]{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;color:var(--fdIconTabBar_Arrow_Color);font-size:1rem;height:1rem;line-height:1rem;width:1rem}.fd-icon-tab-bar__tag{color:var(--fdIconTabBar_Tag_Color);font-family:var(--fdIcon_Tab_Bar_Label_Font_Family);font-size:var(--sapFontSize);position:relative}.fd-icon-tab-bar__counter{-webkit-margin-after:.125rem;margin-block-end:.125rem}.fd-icon-tab-bar__icon{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;background:var(--fdIconTabBar_Icon_Background);border:var(--fdIcon_Tab_Bar_Icon_Border_Weight) solid var(--fdIconTabBar_Icon_Border_Color);border-radius:50%;height:var(--fdIconTabBar_Icon_Size);margin-inline:.188rem;position:relative;width:var(--fdIconTabBar_Icon_Size)}.fd-icon-tab-bar__icon:after,.fd-icon-tab-bar__icon:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__icon [class*=sap-icon],.fd-icon-tab-bar__icon[class*=sap-icon]{color:var(--fdIconTabBar_Icon_Color);display:-webkit-inline-box;display:-ms-inline-flexbox;display:inline-flex;font-size:var(--fdIconTabBar_Icon_Font_Size);-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.fd-icon-tab-bar__details{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:start;-ms-flex-align:start;align-items:flex-start;-webkit-box-pack:end;-ms-flex-pack:end;height:2.75rem;justify-content:flex-end;max-width:7.5rem;padding-block:.375rem .25rem;padding-inline:.125rem .25rem}.fd-icon-tab-bar__details:after,.fd-icon-tab-bar__details:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__label{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:var(--fdIcon_Tab_Bar_Label_Color);font-family:var(--sapFontFamily);font-family:var(--fdIcon_Tab_Bar_Label_Font_Family);font-size:var(--sapFontSize);font-size:var(--sapFontSmallSize);font-weight:400;forced-color-adjust:none;line-height:normal;line-height:.875rem;margin-block:0;margin-inline:0;overflow:hidden;padding-block:0;padding-inline:0;text-overflow:ellipsis;white-space:nowrap}.fd-icon-tab-bar__label:after,.fd-icon-tab-bar__label:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__separator{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;padding-inline:.5rem}.fd-icon-tab-bar__separator:after,.fd-icon-tab-bar__separator:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__separator [class*=sap-icon],.fd-icon-tab-bar__separator[class*=sap-icon]{color:var(--sapContent_NonInteractiveIconColor);font-size:1rem}.fd-icon-tab-bar__separator [class*=sap-icon][dir=rtl],.fd-icon-tab-bar__separator[class*=sap-icon][dir=rtl],[dir=rtl] .fd-icon-tab-bar__separator [class*=sap-icon],[dir=rtl] .fd-icon-tab-bar__separator[class*=sap-icon]{-webkit-transform:rotate(180deg);transform:rotate(180deg)}.fd-icon-tab-bar__container{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;position:relative}.fd-icon-tab-bar__container:after,.fd-icon-tab-bar__container:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__container .fd-icon-tab-bar__badge{border:.125rem solid var(--sapObjectHeader_Background);border-radius:50%;height:var(--fdIcon_Tab_Bar_Container_Badge_Size);right:.188rem;top:0;width:var(--fdIcon_Tab_Bar_Container_Badge_Size)}.fd-icon-tab-bar__container .fd-icon-tab-bar__badge[dir=rtl],[dir=rtl] .fd-icon-tab-bar__container .fd-icon-tab-bar__badge{left:.188rem;right:auto}.fd-icon-tab-bar__container .fd-icon-tab-bar__counter{left:2.75rem;position:absolute;top:0}.fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],[dir=rtl] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter{left:auto;right:2.75rem}.fd-icon-tab-bar__container--filter{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center;height:2rem}.fd-icon-tab-bar__container--filter:after{background:var(--sapGroup_TitleBorderColor);content:\"\";height:2rem;padding-block:0;padding-inline:0;position:absolute;right:-.5625rem;width:.0625rem}.fd-icon-tab-bar__container--filter[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar__container--filter:after{left:-.5625rem;right:auto}.fd-icon-tab-bar__filter-counter,.fd-icon-tab-bar__filter-label{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:var(--sapContent_LabelColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-size:var(--sapFontSmallSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0}.fd-icon-tab-bar__filter-counter:after,.fd-icon-tab-bar__filter-counter:before,.fd-icon-tab-bar__filter-label:after,.fd-icon-tab-bar__filter-label:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__filter-counter{font-size:1.5rem;-webkit-margin-end:.5rem;margin-inline-end:.5rem}.fd-icon-tab-bar__overflow{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;padding-inline:.5rem;-webkit-box-align:center;-ms-flex-align:center;align-items:center;background:var(--fdIconTabBar_Overflow_Button_Background);border:.0625rem solid var(--fdIconTabBar_Overflow_Button_Border_Color);border-radius:var(--fdIcon_Tab_Bar_Overflow_Button_Border_Radius);cursor:pointer;gap:.375rem;height:1.5rem;position:relative}.fd-icon-tab-bar__overflow:after,.fd-icon-tab-bar__overflow:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__overflow [class*=sap-icon]{color:var(--fdIconTabBar_Overflow_Button_Icon_Color);font-family:var(--sapContent_IconFontFamily);font-size:1rem}.fd-icon-tab-bar__overflow-text{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:var(--fdIconTabBar_Overflow_Button_Text_Color);font-family:var(--sapFontFamily);font-family:var(--fdIcon_Tab_Bar_Overflow_Button_Font_Family);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;line-height:1.5rem;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0}.fd-icon-tab-bar__overflow-text:after,.fd-icon-tab-bar__overflow-text:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__overflow.is-hover,.fd-icon-tab-bar__overflow:hover{--fdIconTabBar_Overflow_Button_Background:var(--sapButton_Hover_Background);--fdIconTabBar_Overflow_Button_Border_Color:var(--sapButton_Hover_BorderColor);--fdIconTabBar_Overflow_Button_Text_Color:var(--fdIcon_Tab_Bar_Overflow_Button_Text_Color_Hover);--fdIconTabBar_Overflow_Button_Icon_Color:var(--fdIcon_Tab_Bar_Overflow_Button_Icon_Color_Hover)}.fd-icon-tab-bar__overflow.is-active,.fd-icon-tab-bar__overflow:active{--fdIconTabBar_Overflow_Button_Background:var(--sapButton_Active_Background);--fdIconTabBar_Overflow_Button_Border_Color:var(--sapButton_Active_BorderColor);--fdIconTabBar_Overflow_Button_Text_Color:var(--sapButton_Active_TextColor);--fdIconTabBar_Overflow_Button_Icon_Color:var(--sapButton_Active_TextColor)}.fd-icon-tab-bar__overflow.is-focus,.fd-icon-tab-bar__overflow:focus{outline:none;z-index:5}.fd-icon-tab-bar__overflow.is-focus:after,.fd-icon-tab-bar__overflow:focus:after{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);border-radius:var(--fdIcon_Tab_Bar_Overflow_Button_Border_Radius);content:\"\";display:block;inset:var(--fdIcon_Tab_Bar_Overflow_Button_Focus_Offset) var(--fdIcon_Tab_Bar_Overflow_Button_Focus_Offset) var(--fdIcon_Tab_Bar_Overflow_Button_Focus_Offset) var(--fdIcon_Tab_Bar_Overflow_Button_Focus_Offset);position:absolute}.fd-icon-tab-bar__overflow.is-focus.is-active:after,.fd-icon-tab-bar__overflow.is-focus:active:after,.fd-icon-tab-bar__overflow:focus.is-active:after,.fd-icon-tab-bar__overflow:focus:active:after{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_ContrastFocusColor)}.fd-icon-tab-bar .fd-icon-tab-bar__popover{position:absolute;right:.188rem;top:0}.fd-icon-tab-bar .fd-icon-tab-bar__popover[dir=rtl],[dir=rtl] .fd-icon-tab-bar .fd-icon-tab-bar__popover{left:.188rem;right:auto}.fd-icon-tab-bar .fd-icon-tab-bar__popover-body{-webkit-margin-end:-.25rem;margin-inline-end:-.25rem;-webkit-margin-before:-.25rem;margin-block-start:-.25rem;z-index:10}.fd-icon-tab-bar .fd-icon-tab-bar__popover .fd-icon-tab-bar__button{--fdButtonColor:var(--sapTextColor)}.fd-icon-tab-bar__badge{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;background:var(--fdIconTabBar_Badge_Background);border:.0625rem solid var(--sapObjectHeader_Background);border-radius:50%;-webkit-box-sizing:content-box;box-sizing:content-box;height:.375rem;position:absolute;right:-.2505rem;top:.75rem;width:.375rem;z-index:5}.fd-icon-tab-bar__badge:after,.fd-icon-tab-bar__badge:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__badge[dir=rtl],[dir=rtl] .fd-icon-tab-bar__badge{left:-.2505rem;right:auto}.fd-icon-tab-bar__badge--animated{-webkit-animation-delay:0s,1.5s;animation-delay:0s,1.5s;-webkit-animation-duration:1.5s,.75s;animation-duration:1.5s,.75s;-webkit-animation-iteration-count:1,1;animation-iteration-count:1,1;-webkit-animation-name:pulse-appear,pulse-disappear;animation-name:pulse-appear,pulse-disappear;-webkit-animation-timing-function:ease-in,ease-out;animation-timing-function:ease-in,ease-out;-webkit-transform:scale(0);transform:scale(0)}.fd-icon-tab-bar__badge--animated:after{-webkit-animation:pulse-after 1s ease-in;animation:pulse-after 1s ease-in;-webkit-animation-delay:1s;animation-delay:1s;-webkit-animation-iteration-count:1;animation-iteration-count:1;background:var(--fdIconTabBar_Badge_Background_After);border-radius:50%;content:\"\";height:.75rem;opacity:0;position:absolute;-webkit-transform:scale(0);transform:scale(0);width:.75rem}@-webkit-keyframes pulse-appear{0%{-webkit-transform:scale(.25);transform:scale(.25)}64%{-webkit-transform:scale(1.25);transform:scale(1.25)}71%{-webkit-transform:scale(1);transform:scale(1)}to{-webkit-transform:scale(1);transform:scale(1)}}@keyframes pulse-appear{0%{-webkit-transform:scale(.25);transform:scale(.25)}64%{-webkit-transform:scale(1.25);transform:scale(1.25)}71%{-webkit-transform:scale(1);transform:scale(1)}to{-webkit-transform:scale(1);transform:scale(1)}}@-webkit-keyframes pulse-disappear{0%{-webkit-transform:scale(1);transform:scale(1)}to{-webkit-transform:scale(0);transform:scale(0)}}@keyframes pulse-disappear{0%{-webkit-transform:scale(1);transform:scale(1)}to{-webkit-transform:scale(0);transform:scale(0)}}@-webkit-keyframes pulse-after{0%{opacity:0;-webkit-transform:scale(0);transform:scale(0)}71%{opacity:.2;-webkit-transform:scale(1);transform:scale(1)}80%{opacity:.2;-webkit-transform:scale(1.17);transform:scale(1.17)}86%{opacity:.1;-webkit-transform:scale(1.33);transform:scale(1.33)}93%{opacity:0;-webkit-transform:scale(1.5);transform:scale(1.5)}to{opacity:0;-webkit-transform:scale(0);transform:scale(0)}}@keyframes pulse-after{0%{opacity:0;-webkit-transform:scale(0);transform:scale(0)}71%{opacity:.2;-webkit-transform:scale(1);transform:scale(1)}80%{opacity:.2;-webkit-transform:scale(1.17);transform:scale(1.17)}86%{opacity:.1;-webkit-transform:scale(1.33);transform:scale(1.33)}93%{opacity:0;-webkit-transform:scale(1.5);transform:scale(1.5)}to{opacity:0;-webkit-transform:scale(0);transform:scale(0)}}.fd-icon-tab-bar__icon-separator{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-inline:0;-webkit-box-align:end;-ms-flex-align:end;align-items:flex-end;-webkit-box-pack:center;-ms-flex-pack:center;height:1rem;justify-content:center;padding-block:0;padding-inline:.5rem}.fd-icon-tab-bar__icon-separator:after,.fd-icon-tab-bar__icon-separator:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__icon-separator [class*=sap-icon]{color:var(--sapContent_NonInteractiveIconColor);font-size:1rem;height:1rem;line-height:1rem;width:1rem}.fd-icon-tab-bar__icon-separator[dir=rtl] [class*=sap-icon],[dir=rtl] .fd-icon-tab-bar__icon-separator [class*=sap-icon]{-webkit-transform:rotate(180deg);transform:rotate(180deg)}.fd-icon-tab-bar__line-separator{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-margin-before:.25rem;border-bottom:.0625rem solid var(--sapGroup_TitleBorderColor);margin-block-start:.25rem}.fd-icon-tab-bar__line-separator:after,.fd-icon-tab-bar__line-separator:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar__line-separator:first-child{-webkit-margin-before:.5rem;margin-block-start:.5rem;margin-inline:1rem 1rem}.fd-icon-tab-bar__list{width:100%}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"1\"]{--fdList_Navigation_Link_Padding_Left:1rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"1\"]+.fd-icon-tab-bar__line-separator{margin-inline:1.5rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"2\"]{--fdList_Navigation_Link_Padding_Left:1.5rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"2\"]+.fd-icon-tab-bar__line-separator{margin-inline:2rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"3\"]{--fdList_Navigation_Link_Padding_Left:2rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"3\"]+.fd-icon-tab-bar__line-separator{margin-inline:2.5rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"4\"]{--fdList_Navigation_Link_Padding_Left:2.5rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"4\"]+.fd-icon-tab-bar__line-separator{margin-inline:3rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"5\"]{--fdList_Navigation_Link_Padding_Left:3rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"5\"]+.fd-icon-tab-bar__line-separator{margin-inline:3.5rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"6\"]{--fdList_Navigation_Link_Padding_Left:3.5rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"6\"]+.fd-icon-tab-bar__line-separator{margin-inline:4rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"7\"]{--fdList_Navigation_Link_Padding_Left:4rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"7\"]+.fd-icon-tab-bar__line-separator{margin-inline:4.5rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"8\"]{--fdList_Navigation_Link_Padding_Left:4.5rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"8\"]+.fd-icon-tab-bar__line-separator{margin-inline:5rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"9\"]{--fdList_Navigation_Link_Padding_Left:5rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"9\"]+.fd-icon-tab-bar__line-separator{margin-inline:5.5rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"10\"]{--fdList_Navigation_Link_Padding_Left:5.5rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item[aria-level=\"10\"]+.fd-icon-tab-bar__line-separator{margin-inline:6rem 1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item span{-webkit-box-flex:0;-ms-flex:0 1 auto;flex:0 1 auto}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item .fd-icon-tab-bar__badge{position:static;-webkit-margin-after:.625rem;margin-block-end:.625rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item .fd-icon-tab-bar__badge [class*=sap-icon]{-webkit-margin-after:0;margin-block-end:0}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--negative .fd-icon-tab-bar__list-item-counter,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--negative .fd-icon-tab-bar__list-item-icon [class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--negative .fd-icon-tab-bar__list-item-icon[class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--negative .fd-icon-tab-bar__list-item-title{color:var(--fdIcon_Tab_Bar_Semantic_Color_Negative)}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--positive .fd-icon-tab-bar__list-item-counter,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--positive .fd-icon-tab-bar__list-item-icon [class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--positive .fd-icon-tab-bar__list-item-icon[class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--positive .fd-icon-tab-bar__list-item-title{color:var(--fdIcon_Tab_Bar_Semantic_Color_Positive)}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--critical .fd-icon-tab-bar__list-item-counter,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--critical .fd-icon-tab-bar__list-item-icon [class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--critical .fd-icon-tab-bar__list-item-icon[class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--critical .fd-icon-tab-bar__list-item-title{color:var(--fdIcon_Tab_Bar_Semantic_Color_Critical)}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--informative .fd-icon-tab-bar__list-item-counter,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--informative .fd-icon-tab-bar__list-item-icon [class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--informative .fd-icon-tab-bar__list-item-icon[class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--informative .fd-icon-tab-bar__list-item-title{color:var(--fdIcon_Tab_Bar_Semantic_Color_Informative)}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--closable .fd-icon-tab-bar__list-link{--fdList_Navigation_Link_Padding_Right:2rem;position:relative}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--closable .fd-icon-tab-bar__button-container{max-width:2rem;min-width:2rem;position:absolute;right:0;width:2rem;z-index:10}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--closable .fd-icon-tab-bar__button-container[dir=rtl],[dir=rtl] .fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--closable .fd-icon-tab-bar__button-container{left:0;right:auto}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item--closable .fd-icon-tab-bar__button-container:before{display:none}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item-icon-container{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;width:1.25rem;-webkit-margin-end:.25rem;margin-inline-end:.25rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link.is-selected,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link[aria-selected=true]{border-bottom:none}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link.is-active .fd-icon-tab-bar__list-item-counter,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link.is-active .fd-icon-tab-bar__list-item-icon [class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link.is-active .fd-icon-tab-bar__list-item-icon[class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link.is-active .fd-icon-tab-bar__list-item-title,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link:active .fd-icon-tab-bar__list-item-counter,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link:active .fd-icon-tab-bar__list-item-icon [class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link:active .fd-icon-tab-bar__list-item-icon[class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link:active .fd-icon-tab-bar__list-item-title{color:var(--sapList_Active_TextColor)}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link.is-focus,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link:focus{z-index:5}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link.is-focus:before,.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link:focus:before{border-radius:.375rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-link span{font-size:var(--sapFontSize)}.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item-icon [class*=sap-icon],.fd-icon-tab-bar__list .fd-icon-tab-bar__list-item-icon [class*=sap-icon]:last-child{font-size:1rem;margin-block:0;margin-inline:0;width:auto}.fd-icon-tab-bar__list .fd-icon-tab-bar__line-separator:first-child{padding-inline:1rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__line-separator:first-child:after{-webkit-margin-before:.75rem;margin-block-start:.75rem}.fd-icon-tab-bar__list .fd-icon-tab-bar__icon-separator:first-child{height:1.5rem;padding-inline:1rem}.fd-icon-tab-bar__list[dir=rtl] .fd-icon-tab-bar__list-item-icon,[dir=rtl] .fd-icon-tab-bar__list .fd-icon-tab-bar__list-item-icon{-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center}.fd-icon-tab-bar--transparent .fd-icon-tab-bar__header,.fd-icon-tab-bar--transparent .fd-icon-tab-bar__panel{background:transparent}.fd-icon-tab-bar--translucent .fd-icon-tab-bar__header{background:var(--sapObjectHeader_Background)}.fd-icon-tab-bar--translucent .fd-icon-tab-bar__panel{background:var(--sapGroup_ContentBackground)}.fd-icon-tab-bar[class*=-compact],.fd-icon-tab-bar[class*=-condensed],[class*=-compact] .fd-icon-tab-bar:not([class*=-cozy]),[class*=-condensed] .fd-icon-tab-bar:not([class*=-cozy]){--fdIconTabBar_Icon_Size:2rem;--fdIconTabBar_Icon_Font_Size:1rem;--fdIcon_Tab_Bar_Container_Badge_Size:.375rem}.fd-icon-tab-bar[class*=-compact] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,.fd-icon-tab-bar[class*=-condensed] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[class*=-compact] .fd-icon-tab-bar:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[class*=-condensed] .fd-icon-tab-bar:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter{left:2rem}.fd-icon-tab-bar[class*=-compact] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],.fd-icon-tab-bar[class*=-condensed] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],[class*=-compact] .fd-icon-tab-bar:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],[class*=-condensed] .fd-icon-tab-bar:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],[dir=rtl] .fd-icon-tab-bar[class*=-compact] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[dir=rtl] .fd-icon-tab-bar[class*=-condensed] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[dir=rtl] [class*=-compact] .fd-icon-tab-bar:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[dir=rtl] [class*=-condensed] .fd-icon-tab-bar:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter{left:auto;right:2rem}.fd-icon-tab-bar--counters .fd-icon-tab-bar__tab{height:3.875rem;-webkit-box-pack:end;-ms-flex-pack:end;justify-content:flex-end}.fd-icon-tab-bar--counters .fd-icon-tab-bar__badge{top:1.9375rem}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__tab{padding-block:1rem;padding-inline:0;position:relative;-webkit-box-orient:horizontal;-webkit-box-direction:normal;-ms-flex-direction:row;flex-direction:row;margin-inline:.75rem}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item{margin-inline:.25rem}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item:first-child{margin-block:0;margin-inline:0 .25rem}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item:first-child .fd-icon-tab-bar__tab{margin-inline:0 .75rem}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item--closable .fd-icon-tab-bar__tab{-webkit-padding-end:2rem;padding-inline-end:2rem}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container{right:.75rem}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container[dir=rtl],[dir=rtl] .fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container{left:.75rem;right:auto}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__container .fd-icon-tab-bar__badge{right:.2505rem}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__container .fd-icon-tab-bar__badge[dir=rtl],[dir=rtl] .fd-icon-tab-bar--icon-only .fd-icon-tab-bar__container .fd-icon-tab-bar__badge{left:.2505rem;right:auto}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__container .fd-icon-tab-bar__counter{left:2.938rem}.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],[dir=rtl] .fd-icon-tab-bar--icon-only .fd-icon-tab-bar__container .fd-icon-tab-bar__counter{left:auto;right:2.938rem}.fd-icon-tab-bar--icon-only[class*=-compact] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,.fd-icon-tab-bar--icon-only[class*=-condensed] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[class*=-compact] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[class*=-condensed] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter{left:2.188rem}.fd-icon-tab-bar--icon-only[class*=-compact] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],.fd-icon-tab-bar--icon-only[class*=-condensed] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],[class*=-compact] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],[class*=-condensed] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter[dir=rtl],[dir=rtl] .fd-icon-tab-bar--icon-only[class*=-compact] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[dir=rtl] .fd-icon-tab-bar--icon-only[class*=-condensed] .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[dir=rtl] [class*=-compact] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter,[dir=rtl] [class*=-condensed] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__counter{left:auto;right:2.188rem}.fd-icon-tab-bar--icon-only[class*=-compact] .fd-icon-tab-bar__container .fd-icon-tab-bar__badge,.fd-icon-tab-bar--icon-only[class*=-condensed] .fd-icon-tab-bar__container .fd-icon-tab-bar__badge,[class*=-compact] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__badge,[class*=-condensed] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__badge{right:.188rem}.fd-icon-tab-bar--icon-only[class*=-compact] .fd-icon-tab-bar__container .fd-icon-tab-bar__badge[dir=rtl],.fd-icon-tab-bar--icon-only[class*=-condensed] .fd-icon-tab-bar__container .fd-icon-tab-bar__badge[dir=rtl],[class*=-compact] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__badge[dir=rtl],[class*=-condensed] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__badge[dir=rtl],[dir=rtl] .fd-icon-tab-bar--icon-only[class*=-compact] .fd-icon-tab-bar__container .fd-icon-tab-bar__badge,[dir=rtl] .fd-icon-tab-bar--icon-only[class*=-condensed] .fd-icon-tab-bar__container .fd-icon-tab-bar__badge,[dir=rtl] [class*=-compact] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__badge,[dir=rtl] [class*=-condensed] .fd-icon-tab-bar--icon-only:not([class*=-cozy]) .fd-icon-tab-bar__container .fd-icon-tab-bar__badge{left:.188rem;right:auto}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item{margin-block:0;margin-inline:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative[dir=rtl] .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__icon:after{content:none;margin-block:0;margin-inline:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__label:before{-webkit-margin-end:.5rem;color:var(--sapNegativeElementColor);content:\"\\e1ec\";margin-inline-end:.5rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__label[dir=rtl]:before,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__label:before{content:none;margin-block:0;margin-inline:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__label[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--negative .fd-icon-tab-bar__label:after{-webkit-margin-end:.5rem;color:var(--sapNegativeElementColor);content:\"\\e1ec\";margin-inline-end:.5rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive[dir=rtl] .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__icon:after{content:none;margin-block:0;margin-inline:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__label:before{-webkit-margin-end:.5rem;color:var(--sapPositiveElementColor);content:\"\\e1c1\";margin-inline-end:.5rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__label[dir=rtl]:before,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__label:before{content:none;margin-block:0;margin-inline:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__label[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--positive .fd-icon-tab-bar__label:after{-webkit-margin-end:.5rem;color:var(--sapPositiveElementColor);content:\"\\e1c1\";margin-inline-end:.5rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical[dir=rtl] .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__icon:after{content:none;margin-block:0;margin-inline:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__label:before{-webkit-margin-end:.5rem;color:var(--sapCriticalElementColor);content:\"\\e053\";margin-inline-end:.5rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__label[dir=rtl]:before,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__label:before{content:none;margin-block:0;margin-inline:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__label[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--critical .fd-icon-tab-bar__label:after{-webkit-margin-end:.5rem;color:var(--sapCriticalElementColor);content:\"\\e053\";margin-inline-end:.5rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon:before,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative[dir=rtl] .fd-icon-tab-bar__icon:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__icon:after{content:none;margin-block:0;margin-inline:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__label:before{-webkit-margin-end:.5rem;color:var(--sapInformativeElementColor);content:\"\\e289\";margin-inline-end:.5rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__label[dir=rtl]:before,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__label:before{content:none;margin-block:0;margin-inline:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__label[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--informative .fd-icon-tab-bar__label:after{-webkit-margin-end:.5rem;color:var(--sapInformativeElementColor);content:\"\\e289\";margin-inline-end:.5rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--closable .fd-icon-tab-bar__tab{-webkit-padding-end:2rem;padding-inline-end:2rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container{right:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container[dir=rtl],[dir=rtl] .fd-icon-tab-bar--icon .fd-icon-tab-bar__item--closable .fd-icon-tab-bar__button-container{left:0;right:auto}.fd-icon-tab-bar--icon .fd-icon-tab-bar__tab{padding-block:1rem;padding-inline:0;-webkit-box-orient:horizontal;-webkit-box-direction:normal;-ms-flex-direction:row;flex-direction:row}.fd-icon-tab-bar--icon .fd-icon-tab-bar__tab:after{width:3.126rem}.fd-icon-tab-bar--icon .fd-icon-tab-bar__counter{-webkit-margin-after:0;font-size:var(--sapFontSmallSize);line-height:.875rem;margin-block-end:0}.fd-icon-tab-bar--icon .fd-icon-tab-bar__counter+.fd-icon-tab-bar__label{-webkit-margin-before:.375rem;margin-block-start:.375rem}.fd-icon-tab-bar--icon[class*=-compact] .fd-icon-tab-bar__details,.fd-icon-tab-bar--icon[class*=-condensed] .fd-icon-tab-bar__details,[class*=-compact] .fd-icon-tab-bar--icon:not([class*=-cozy]) .fd-icon-tab-bar__details,[class*=-condensed] .fd-icon-tab-bar--icon:not([class*=-cozy]) .fd-icon-tab-bar__details{height:2rem;padding-block:0}.fd-icon-tab-bar--icon[class*=-compact] .fd-icon-tab-bar__tab:after,.fd-icon-tab-bar--icon[class*=-condensed] .fd-icon-tab-bar__tab:after,[class*=-compact] .fd-icon-tab-bar--icon:not([class*=-cozy]) .fd-icon-tab-bar__tab:after,[class*=-condensed] .fd-icon-tab-bar--icon:not([class*=-cozy]) .fd-icon-tab-bar__tab:after{width:2.376rem}.fd-icon-tab-bar--icon[class*=-compact] .fd-icon-tab-bar__counter+.fd-icon-tab-bar__label,.fd-icon-tab-bar--icon[class*=-condensed] .fd-icon-tab-bar__counter+.fd-icon-tab-bar__label,[class*=-compact] .fd-icon-tab-bar--icon:not([class*=-cozy]) .fd-icon-tab-bar__counter+.fd-icon-tab-bar__label,[class*=-condensed] .fd-icon-tab-bar--icon:not([class*=-cozy]) .fd-icon-tab-bar__counter+.fd-icon-tab-bar__label{-webkit-margin-before:.25rem;margin-block-start:.25rem}.fd-icon-tab-bar--process .fd-icon-tab-bar__item{margin-inline:0}.fd-icon-tab-bar--filter .fd-icon-tab-bar__item{margin-inline:.25rem}.fd-icon-tab-bar--filter .fd-icon-tab-bar__item:first-child{margin-inline:0 1.063rem}.fd-icon-tab-bar--filter .fd-icon-tab-bar__item:first-child .fd-icon-tab-bar__tab{padding-block:1.625rem 1.875rem;position:relative}.fd-icon-tab-bar--filter .fd-icon-tab-bar__item:nth-child(2){margin-block:0;margin-inline:0 .25rem}.fd-icon-tab-bar--filter .fd-icon-tab-bar__tab{margin-inline:0;padding-block:.875rem .625rem;padding-inline:.188rem;position:relative;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.fd-icon-tab-bar--filter .fd-icon-tab-bar__icon{margin-inline:0}.fd-icon-tab-bar--filter .fd-icon-tab-bar__label{text-align:center;width:5rem;-webkit-margin-before:.375rem;margin-block-start:.375rem}.fd-icon-tab-bar--filter[class*=-compact] .fd-icon-tab-bar__item:first-child .fd-icon-tab-bar__tab,.fd-icon-tab-bar--filter[class*=-condensed] .fd-icon-tab-bar__item:first-child .fd-icon-tab-bar__tab,[class*=-compact] .fd-icon-tab-bar--filter:not([class*=-cozy]) .fd-icon-tab-bar__item:first-child .fd-icon-tab-bar__tab,[class*=-condensed] .fd-icon-tab-bar--filter:not([class*=-cozy]) .fd-icon-tab-bar__item:first-child .fd-icon-tab-bar__tab{padding-block:1.375rem}.fd-icon-tab-bar--filter[class*=-compact] .fd-icon-tab-bar__badge,.fd-icon-tab-bar--filter[class*=-condensed] .fd-icon-tab-bar__badge,[class*=-compact] .fd-icon-tab-bar--filter:not([class*=-cozy]) .fd-icon-tab-bar__badge,[class*=-condensed] .fd-icon-tab-bar--filter:not([class*=-cozy]) .fd-icon-tab-bar__badge{right:.063rem}.fd-icon-tab-bar--filter[class*=-compact] .fd-icon-tab-bar__badge[dir=rtl],.fd-icon-tab-bar--filter[class*=-condensed] .fd-icon-tab-bar__badge[dir=rtl],[class*=-compact] .fd-icon-tab-bar--filter:not([class*=-cozy]) .fd-icon-tab-bar__badge[dir=rtl],[class*=-condensed] .fd-icon-tab-bar--filter:not([class*=-cozy]) .fd-icon-tab-bar__badge[dir=rtl],[dir=rtl] .fd-icon-tab-bar--filter[class*=-compact] .fd-icon-tab-bar__badge,[dir=rtl] .fd-icon-tab-bar--filter[class*=-condensed] .fd-icon-tab-bar__badge,[dir=rtl] [class*=-compact] .fd-icon-tab-bar--filter:not([class*=-cozy]) .fd-icon-tab-bar__badge,[dir=rtl] [class*=-condensed] .fd-icon-tab-bar--filter:not([class*=-cozy]) .fd-icon-tab-bar__badge{left:.063rem;right:auto}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__header{background:var(--sapShell_Navigation_Background);-webkit-box-shadow:var(--sapShell_Shadow);box-shadow:var(--sapShell_Shadow)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__item{color:var(--sapShell_Navigation_TextColor)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab [class*=sap-icon]{color:var(--sapShell_Navigation_TextColor);font-size:1rem;height:1rem;line-height:1;-webkit-margin-end:.5rem;margin-inline-end:.5rem}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab .fd-icon-tab-bar__tag{color:var(--sapShell_Navigation_TextColor)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-selected .fd-icon-tab-bar__tag,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-selected [class*=sap-icon],.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab[aria-selected=true] .fd-icon-tab-bar__tag,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab[aria-selected=true] [class*=sap-icon]{color:var(--sapShell_Navigation_Selected_TextColor)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-selected:after,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab[aria-selected=true]:after{background:var(--sapShell_Navigation_SelectedColor)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-selected+.fd-icon-tab-bar__popover .fd-icon-tab-bar__button,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab[aria-selected=true]+.fd-icon-tab-bar__popover .fd-icon-tab-bar__button{--fdButtonColor:var(--fdIconTabBar_Button_Shell_Color_Active)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-hover .fd-icon-tab-bar__tag,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-hover [class*=sap-icon],.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab:hover .fd-icon-tab-bar__tag,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab:hover [class*=sap-icon]{color:var(--sapShell_Navigation_Selected_TextColor)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-focus,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab:focus{z-index:5}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tag:after,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tag:after{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapShell_Navigation_TextColor);content:\"\";display:block;inset:0;inset:var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset);position:absolute}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tab-container,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tab-container{position:relative}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tab-container .fd-icon-tab-bar__tag,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tab-container .fd-icon-tab-bar__tag{outline:none}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tab-container .fd-icon-tab-bar__tag:after,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tab-container .fd-icon-tab-bar__tag:after{border:none}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab.is-focus .fd-icon-tab-bar__tab-container:after,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab:focus .fd-icon-tab-bar__tab-container:after{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapShell_Navigation_TextColor);content:\"\";display:block;inset:0;inset:var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset) var(--fdIcon_Tab_Bar_Focus_Offset);position:absolute}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__tab .fd-icon-tab-bar__arrow [class*=sap-icon]{margin-block:0;margin-inline:0}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__overflow{--fdIconTabBar_Overflow_Button_Background:var(--fdIconTabBar_Overflow_Button_Shell_Background);--fdIconTabBar_Overflow_Button_Border_Color:var(--fdIconTabBar_Overflow_Button_Shell_Border_Color);--fdIconTabBar_Overflow_Button_Text_Color:var(--fdIconTabBar_Overflow_Button_Shell_Text_Color);--fdIconTabBar_Overflow_Button_Icon_Color:var(--fdIconTabBar_Overflow_Button_Shell_Icon_Color)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__overflow.is-hover,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__overflow:hover{--fdIconTabBar_Overflow_Button_Background:var(--fdIconTabBar_Overflow_Button_Shell_Background_Hover);--fdIconTabBar_Overflow_Button_Border_Color:var( --fdIconTabBar_Overflow_Button_Shell_Border_Color_Hover );--fdIconTabBar_Overflow_Button_Text_Color:var(--fdIconTabBar_Overflow_Button_Shell_Text_Color_Hover);--fdIconTabBar_Overflow_Button_Icon_Color:var(--fdIconTabBar_Overflow_Button_Shell_Icon_Color_Hover)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__overflow.is-active,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__overflow:active{--fdIconTabBar_Overflow_Button_Background:var(--fdIconTabBar_Overflow_Button_Shell_Background_Active);--fdIconTabBar_Overflow_Button_Border_Color:var( --fdIconTabBar_Overflow_Button_Shell_Border_Color_Active );--fdIconTabBar_Overflow_Button_Text_Color:var(--fdIconTabBar_Overflow_Button_Shell_Text_Color_Active);--fdIconTabBar_Overflow_Button_Icon_Color:var(--fdIconTabBar_Overflow_Button_Shell_Icon_Color_Active)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__overflow.is-focus,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__overflow:focus{z-index:5}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__overflow.is-focus:after,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__overflow:focus:after{border-color:var(--fdIconTabBar_Overflow_Button_Shell_Focus_Color)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button{--fdButtonColor:var(--fdIconTabBar_Button_Shell_Color)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button.is-hover,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button:hover{--fdButtonColor:var(--fdIconTabBar_Button_Shell_Color_Hover);--fdButtonBorderColor:var(--fdIconTabBar_Button_Shell_Background_Color_Hover);--fdButtonBackgroundColor:var(--fdIconTabBar_Button_Shell_Background_Color_Hover)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button.is-active,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button:active{--fdButtonColor:var(--fdIconTabBar_Button_Shell_Color_Active);--fdButtonBackgroundColor:var(--fdIconTabBar_Button_Shell_Background_Color_Active);--fdButtonBorderColor:var(--fdIconTabBar_Button_Shell_Background_Color_Active)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button.is-focus,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button:focus{z-index:5;--fdButtonColor:var(--fdIconTabBar_Button_Shell_Color_Active);--fdButtonBackgroundColor:var(--sapShell_Navigation_Background);--fdButtonBorderColor:var(--sapShell_Navigation_Background)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button.is-focus:after,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button:focus:after{border-color:var(--fdIconTabBar_Button_Shell_Focus_Color)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button.is-selected,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button[aria-selected=true]{--fdButtonColor:var(--fdIconTabBar_Button_Shell_Color_Active);--fdButtonBackgroundColor:var(--fdIconTabBar_Button_Shell_Background_Color_Selected);--fdButtonBorderColor:var(--fdIconTabBar_Button_Shell_Background_Color_Selected)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button.is-selected.is-hover,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button.is-selected:hover,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button[aria-selected=true].is-hover,.fd-icon-tab-bar.fd-icon-tab-bar--navigation .fd-icon-tab-bar__button[aria-selected=true]:hover{--fdButtonColor:var(--fdIconTabBar_Button_Shell_Color_Hover);--fdButtonBorderColor:transparent;--fdButtonBackgroundColor:var(--fdIconTabBar_Button_Shell_Background_Color_Hover)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation-flat .fd-icon-tab-bar__header{background:var(--sapShellColor)}.fd-icon-tab-bar.fd-icon-tab-bar--navigation-flat .fd-icon-tab-bar__item--with-separator{position:relative;-webkit-margin-end:2rem;margin-inline-end:2rem}.fd-icon-tab-bar.fd-icon-tab-bar--navigation-flat .fd-icon-tab-bar__item--with-separator:after{background:var(--sapShell_InteractiveBorderColor);content:\"\";height:1.5rem;position:absolute;right:-1rem;width:.063rem}.fd-icon-tab-bar.fd-icon-tab-bar--navigation-flat .fd-icon-tab-bar__item--with-separator[dir=rtl]:after,[dir=rtl] .fd-icon-tab-bar.fd-icon-tab-bar--navigation-flat .fd-icon-tab-bar__item--with-separator:after{left:-1rem;right:auto}.fd-icon-tab-bar--navigation-horizontal .fd-icon-tab-bar__header{-webkit-box-shadow:none;box-shadow:none}.fd-icon-tab-bar--navigation-horizontal .fd-icon-tab-bar__tab{padding-block:.75rem 1rem}.fd-icon-tab-bar--navigation-horizontal .fd-icon-tab-bar__tab.is-selected:after,.fd-icon-tab-bar--navigation-horizontal .fd-icon-tab-bar__tab[aria-selected=true]:after{border-radius:.125rem;-webkit-margin-after:.188rem;margin-block-end:.188rem}.fd-icon-tab-bar--navigation-horizontal .fd-icon-tab-bar__button-container{-webkit-box-align:normal;-ms-flex-align:normal;align-items:normal}.fd-icon-tab-bar--navigation-horizontal .fd-icon-tab-bar__button-container .fd-icon-tab-bar__button{-webkit-margin-before:.5rem;margin-block-start:.5rem}.fd-icon-tab-bar--sm .fd-icon-tab-bar__header{padding-block:0;padding-inline:1rem}.fd-icon-tab-bar--lg .fd-icon-tab-bar__header,.fd-icon-tab-bar--md .fd-icon-tab-bar__header{padding-block:0;padding-inline:2rem}.fd-icon-tab-bar--xl .fd-icon-tab-bar__header,.fd-icon-tab-bar--xxl .fd-icon-tab-bar__header{padding-block:0;padding-inline:3rem}.fd-icon-tab-bar--responsive-paddings .fd-icon-tab-bar__header{padding-block:0}@media (width <= 599px){.fd-icon-tab-bar--responsive-paddings .fd-icon-tab-bar__header{padding-inline:1rem}}@media (width >= 600px) and (width <= 1023px){.fd-icon-tab-bar--responsive-paddings .fd-icon-tab-bar__header{padding-inline:2rem}}@media (width >= 1024px) and (width <= 1439px){.fd-icon-tab-bar--responsive-paddings .fd-icon-tab-bar__header{padding-inline:2rem}}@media (width >= 1440px){.fd-icon-tab-bar--responsive-paddings .fd-icon-tab-bar__header{padding-inline:3rem}}.fd-icon-tab-bar .fd-icon-tab-bar__item--overflow,.fd-icon-tab-bar--filter .fd-icon-tab-bar__item--overflow,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--overflow,.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item--overflow,.fd-icon-tab-bar--navigation .fd-icon-tab-bar__item--overflow,.fd-icon-tab-bar--navigation-flat .fd-icon-tab-bar__item--overflow,.fd-icon-tab-bar--process .fd-icon-tab-bar__item--overflow{margin-block:0;margin-inline:0}.fd-icon-tab-bar .fd-icon-tab-bar__item--overflow:last-of-type,.fd-icon-tab-bar--filter .fd-icon-tab-bar__item--overflow:last-of-type,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--overflow:last-of-type,.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item--overflow:last-of-type,.fd-icon-tab-bar--navigation .fd-icon-tab-bar__item--overflow:last-of-type,.fd-icon-tab-bar--navigation-flat .fd-icon-tab-bar__item--overflow:last-of-type,.fd-icon-tab-bar--process .fd-icon-tab-bar__item--overflow:last-of-type{-webkit-margin-start:auto;margin-inline-start:auto}.fd-icon-tab-bar .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover,.fd-icon-tab-bar--filter .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover,.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover,.fd-icon-tab-bar--navigation .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover,.fd-icon-tab-bar--navigation-flat .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover,.fd-icon-tab-bar--process .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover{position:relative}.fd-icon-tab-bar .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover-body,.fd-icon-tab-bar--filter .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover-body,.fd-icon-tab-bar--icon .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover-body,.fd-icon-tab-bar--icon-only .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover-body,.fd-icon-tab-bar--navigation .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover-body,.fd-icon-tab-bar--navigation-flat .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover-body,.fd-icon-tab-bar--process .fd-icon-tab-bar__item--overflow .fd-icon-tab-bar__popover-body{-webkit-margin-end:-.25rem;margin-inline-end:-.25rem;-webkit-margin-before:.375rem;margin-block-start:.375rem}.fd-list{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;list-style:none;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;position:relative;width:100%}.fd-list:after,.fd-list:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list__footer,.fd-list__group-header,.fd-list__item{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-inline:0;-webkit-box-align:center;-ms-flex-align:center;align-items:center;padding-block:0;padding-inline:.9375rem}.fd-list__footer:after,.fd-list__footer:before,.fd-list__group-header:after,.fd-list__group-header:before,.fd-list__item:after,.fd-list__item:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list__item{background:var(--fdList_Item_Background_Color);border-bottom:var(--sapList_BorderWidth) solid var(--fdList_Item_Border_Color);height:var(--fdList_Item_Height);position:relative}.fd-list__item.is-focus,.fd-list__item:focus{outline:none;pointer-events:all}.fd-list__item.is-focus:before,.fd-list__item:focus:before{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);content:\"\";display:block;inset:.125rem .125rem .0625rem;pointer-events:none;position:absolute}.fd-list__item.is-focus.is-active:before,.fd-list__item.is-focus:active:before,.fd-list__item:focus.is-active:before,.fd-list__item:focus:active:before{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--fdList_Active_Outline_Color)}.fd-list__item--unread .fd-list__title{font-weight:700}.fd-list__item--no-data .fd-list__title{text-align:center}.fd-list__item--interactive.is-hover,.fd-list__item--interactive:hover,.fd-list__item--interractive.is-hover,.fd-list__item--interractive:hover{--fdList_Item_Background_Color:var(--sapList_Hover_Background);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_BorderColor)}.fd-list__item--interactive.is-selected,.fd-list__item--interactive[aria-selected=true],.fd-list__item--interractive.is-selected,.fd-list__item--interractive[aria-selected=true]{--fdList_Item_Background_Color:var(--sapList_SelectionBackgroundColor);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_SelectionBorderColor)}.fd-list__item--interactive.is-selected .fd-list__link,.fd-list__item--interactive[aria-selected=true] .fd-list__link,.fd-list__item--interractive.is-selected .fd-list__link,.fd-list__item--interractive[aria-selected=true] .fd-list__link{background:inherit}.fd-list__item--interactive.is-selected .fd-list__link.is-active,.fd-list__item--interactive.is-selected .fd-list__link:active,.fd-list__item--interactive[aria-selected=true] .fd-list__link.is-active,.fd-list__item--interactive[aria-selected=true] .fd-list__link:active,.fd-list__item--interractive.is-selected .fd-list__link.is-active,.fd-list__item--interractive.is-selected .fd-list__link:active,.fd-list__item--interractive[aria-selected=true] .fd-list__link.is-active,.fd-list__item--interractive[aria-selected=true] .fd-list__link:active{--fdList_Item_Background_Color:var(--sapList_Active_Background)}.fd-list__item--interactive.is-selected.is-hover,.fd-list__item--interactive.is-selected:hover,.fd-list__item--interactive[aria-selected=true].is-hover,.fd-list__item--interactive[aria-selected=true]:hover,.fd-list__item--interractive.is-selected.is-hover,.fd-list__item--interractive.is-selected:hover,.fd-list__item--interractive[aria-selected=true].is-hover,.fd-list__item--interractive[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Hover_SelectionBackground)}.fd-list__item--interactive.is-active,.fd-list__item--interactive.is-active.is-hover,.fd-list__item--interactive.is-active.is-selected,.fd-list__item--interactive.is-active.is-selected.is-hover,.fd-list__item--interactive.is-active.is-selected:hover,.fd-list__item--interactive.is-active:hover,.fd-list__item--interactive.is-active[aria-selected=true],.fd-list__item--interactive.is-active[aria-selected=true].is-hover,.fd-list__item--interactive.is-active[aria-selected=true]:hover,.fd-list__item--interactive:active,.fd-list__item--interactive:active.is-hover,.fd-list__item--interactive:active.is-selected,.fd-list__item--interactive:active.is-selected.is-hover,.fd-list__item--interactive:active.is-selected:hover,.fd-list__item--interactive:active:hover,.fd-list__item--interactive:active[aria-selected=true],.fd-list__item--interactive:active[aria-selected=true].is-hover,.fd-list__item--interactive:active[aria-selected=true]:hover,.fd-list__item--interractive.is-active,.fd-list__item--interractive.is-active.is-hover,.fd-list__item--interractive.is-active.is-selected,.fd-list__item--interractive.is-active.is-selected.is-hover,.fd-list__item--interractive.is-active.is-selected:hover,.fd-list__item--interractive.is-active:hover,.fd-list__item--interractive.is-active[aria-selected=true],.fd-list__item--interractive.is-active[aria-selected=true].is-hover,.fd-list__item--interractive.is-active[aria-selected=true]:hover,.fd-list__item--interractive:active,.fd-list__item--interractive:active.is-hover,.fd-list__item--interractive:active.is-selected,.fd-list__item--interractive:active.is-selected.is-hover,.fd-list__item--interractive:active.is-selected:hover,.fd-list__item--interractive:active:hover,.fd-list__item--interractive:active[aria-selected=true],.fd-list__item--interractive:active[aria-selected=true].is-hover,.fd-list__item--interractive:active[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Active_Background);--fdList_Item_Text_Color:var(--sapList_Active_TextColor);--fdList_Item_Border_Color:var(--sapList_Active_Background)}.fd-list__item--interactive.is-active,.fd-list__item--interactive.is-active .fd-list__byline,.fd-list__item--interactive.is-active .fd-list__byline-left,.fd-list__item--interactive.is-active .fd-list__byline-right,.fd-list__item--interactive.is-active .fd-list__byline-right--critical,.fd-list__item--interactive.is-active .fd-list__byline-right--informative,.fd-list__item--interactive.is-active .fd-list__byline-right--negative,.fd-list__item--interactive.is-active .fd-list__byline-right--neutral,.fd-list__item--interactive.is-active .fd-list__byline-right--positive,.fd-list__item--interactive.is-active .fd-list__content,.fd-list__item--interactive.is-active .fd-list__icon,.fd-list__item--interactive.is-active .fd-list__secondary,.fd-list__item--interactive.is-active .fd-list__thumbnail,.fd-list__item--interactive.is-active .fd-list__title,.fd-list__item--interactive.is-active.fd-list__link--navigation-indicator:after,.fd-list__item--interactive:active,.fd-list__item--interactive:active .fd-list__byline,.fd-list__item--interactive:active .fd-list__byline-left,.fd-list__item--interactive:active .fd-list__byline-right,.fd-list__item--interactive:active .fd-list__byline-right--critical,.fd-list__item--interactive:active .fd-list__byline-right--informative,.fd-list__item--interactive:active .fd-list__byline-right--negative,.fd-list__item--interactive:active .fd-list__byline-right--neutral,.fd-list__item--interactive:active .fd-list__byline-right--positive,.fd-list__item--interactive:active .fd-list__content,.fd-list__item--interactive:active .fd-list__icon,.fd-list__item--interactive:active .fd-list__secondary,.fd-list__item--interactive:active .fd-list__thumbnail,.fd-list__item--interactive:active .fd-list__title,.fd-list__item--interactive:active.fd-list__link--navigation-indicator:after,.fd-list__item--interractive.is-active,.fd-list__item--interractive.is-active .fd-list__byline,.fd-list__item--interractive.is-active .fd-list__byline-left,.fd-list__item--interractive.is-active .fd-list__byline-right,.fd-list__item--interractive.is-active .fd-list__byline-right--critical,.fd-list__item--interractive.is-active .fd-list__byline-right--informative,.fd-list__item--interractive.is-active .fd-list__byline-right--negative,.fd-list__item--interractive.is-active .fd-list__byline-right--neutral,.fd-list__item--interractive.is-active .fd-list__byline-right--positive,.fd-list__item--interractive.is-active .fd-list__content,.fd-list__item--interractive.is-active .fd-list__icon,.fd-list__item--interractive.is-active .fd-list__secondary,.fd-list__item--interractive.is-active .fd-list__thumbnail,.fd-list__item--interractive.is-active .fd-list__title,.fd-list__item--interractive.is-active.fd-list__link--navigation-indicator:after,.fd-list__item--interractive:active,.fd-list__item--interractive:active .fd-list__byline,.fd-list__item--interractive:active .fd-list__byline-left,.fd-list__item--interractive:active .fd-list__byline-right,.fd-list__item--interractive:active .fd-list__byline-right--critical,.fd-list__item--interractive:active .fd-list__byline-right--informative,.fd-list__item--interractive:active .fd-list__byline-right--negative,.fd-list__item--interractive:active .fd-list__byline-right--neutral,.fd-list__item--interractive:active .fd-list__byline-right--positive,.fd-list__item--interractive:active .fd-list__content,.fd-list__item--interractive:active .fd-list__icon,.fd-list__item--interractive:active .fd-list__secondary,.fd-list__item--interractive:active .fd-list__thumbnail,.fd-list__item--interractive:active .fd-list__title,.fd-list__item--interractive:active.fd-list__link--navigation-indicator:after{text-shadow:none}.fd-list__item--action{border-bottom:var(--fdList_Item_Action_Border);font-size:var(--sapFontSize);padding-block:0;padding-inline:0;text-align:center;--fdList_Item_Height:var(--sapElement_LineHeight)}.fd-list__item--action:first-child{border-top:var(--fdList_Item_Action_Border)}.fd-list__item--action .fd-list__title{background:transparent;border:var(--fdList_Item_Action_Button_Border);border-radius:var(--fdList_Item_Action_Button_Border_Radius);color:var(--sapButton_TextColor);cursor:pointer;height:var(--fdList_Item_Action_Button_Size);margin-block:var(--fdList_Item_Action_Button_Spacing);margin-inline:var(--fdList_Item_Action_Button_Spacing);padding-block:0;padding-inline:.9375rem;pointer-events:all;width:100%}.fd-list__item--action .fd-list__title.is-hover,.fd-list__item--action .fd-list__title:hover{--fdList_Item_Background_Color:var(--sapList_Hover_Background);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_BorderColor)}.fd-list__item--action .fd-list__title.is-selected,.fd-list__item--action .fd-list__title[aria-selected=true]{--fdList_Item_Background_Color:var(--sapList_SelectionBackgroundColor);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_SelectionBorderColor)}.fd-list__item--action .fd-list__title.is-selected .fd-list__link,.fd-list__item--action .fd-list__title[aria-selected=true] .fd-list__link{background:inherit}.fd-list__item--action .fd-list__title.is-selected .fd-list__link.is-active,.fd-list__item--action .fd-list__title.is-selected .fd-list__link:active,.fd-list__item--action .fd-list__title[aria-selected=true] .fd-list__link.is-active,.fd-list__item--action .fd-list__title[aria-selected=true] .fd-list__link:active{--fdList_Item_Background_Color:var(--sapList_Active_Background)}.fd-list__item--action .fd-list__title.is-selected.is-hover,.fd-list__item--action .fd-list__title.is-selected:hover,.fd-list__item--action .fd-list__title[aria-selected=true].is-hover,.fd-list__item--action .fd-list__title[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Hover_SelectionBackground)}.fd-list__item--action .fd-list__title.is-active,.fd-list__item--action .fd-list__title.is-active.is-hover,.fd-list__item--action .fd-list__title.is-active.is-selected,.fd-list__item--action .fd-list__title.is-active.is-selected.is-hover,.fd-list__item--action .fd-list__title.is-active.is-selected:hover,.fd-list__item--action .fd-list__title.is-active:hover,.fd-list__item--action .fd-list__title.is-active[aria-selected=true],.fd-list__item--action .fd-list__title.is-active[aria-selected=true].is-hover,.fd-list__item--action .fd-list__title.is-active[aria-selected=true]:hover,.fd-list__item--action .fd-list__title:active,.fd-list__item--action .fd-list__title:active.is-hover,.fd-list__item--action .fd-list__title:active.is-selected,.fd-list__item--action .fd-list__title:active.is-selected.is-hover,.fd-list__item--action .fd-list__title:active.is-selected:hover,.fd-list__item--action .fd-list__title:active:hover,.fd-list__item--action .fd-list__title:active[aria-selected=true],.fd-list__item--action .fd-list__title:active[aria-selected=true].is-hover,.fd-list__item--action .fd-list__title:active[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Active_Background);--fdList_Item_Text_Color:var(--sapList_Active_TextColor);--fdList_Item_Border_Color:var(--sapList_Active_Background)}.fd-list__item--action .fd-list__title.is-active,.fd-list__item--action .fd-list__title.is-active .fd-list__byline,.fd-list__item--action .fd-list__title.is-active .fd-list__byline-left,.fd-list__item--action .fd-list__title.is-active .fd-list__byline-right,.fd-list__item--action .fd-list__title.is-active .fd-list__byline-right--critical,.fd-list__item--action .fd-list__title.is-active .fd-list__byline-right--informative,.fd-list__item--action .fd-list__title.is-active .fd-list__byline-right--negative,.fd-list__item--action .fd-list__title.is-active .fd-list__byline-right--neutral,.fd-list__item--action .fd-list__title.is-active .fd-list__byline-right--positive,.fd-list__item--action .fd-list__title.is-active .fd-list__content,.fd-list__item--action .fd-list__title.is-active .fd-list__icon,.fd-list__item--action .fd-list__title.is-active .fd-list__secondary,.fd-list__item--action .fd-list__title.is-active .fd-list__thumbnail,.fd-list__item--action .fd-list__title.is-active .fd-list__title,.fd-list__item--action .fd-list__title.is-active.fd-list__link--navigation-indicator:after,.fd-list__item--action .fd-list__title:active,.fd-list__item--action .fd-list__title:active .fd-list__byline,.fd-list__item--action .fd-list__title:active .fd-list__byline-left,.fd-list__item--action .fd-list__title:active .fd-list__byline-right,.fd-list__item--action .fd-list__title:active .fd-list__byline-right--critical,.fd-list__item--action .fd-list__title:active .fd-list__byline-right--informative,.fd-list__item--action .fd-list__title:active .fd-list__byline-right--negative,.fd-list__item--action .fd-list__title:active .fd-list__byline-right--neutral,.fd-list__item--action .fd-list__title:active .fd-list__byline-right--positive,.fd-list__item--action .fd-list__title:active .fd-list__content,.fd-list__item--action .fd-list__title:active .fd-list__icon,.fd-list__item--action .fd-list__title:active .fd-list__secondary,.fd-list__item--action .fd-list__title:active .fd-list__thumbnail,.fd-list__item--action .fd-list__title:active .fd-list__title,.fd-list__item--action .fd-list__title:active.fd-list__link--navigation-indicator:after{text-shadow:none}.fd-list__item--action .fd-list__title.is-focus,.fd-list__item--action .fd-list__title:focus{outline:var(--sapContent_FocusColor) var(--sapContent_FocusStyle) var(--sapContent_FocusWidth);outline-offset:-.125rem}.fd-list__item--action .fd-list__title.is-focus.is-active,.fd-list__item--action .fd-list__title.is-focus:active,.fd-list__item--action .fd-list__title:focus.is-active,.fd-list__item--action .fd-list__title:focus:active{outline-color:var(--fdList_Active_Outline_Color)}.fd-list__item--action .fd-list__title::-moz-focus-inner{border:0}.fd-list__item--growing{--fdList_Item_Height:var(--sapElement_LineHeight)}.fd-list__item--growing .fd-list__title{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;gap:.5rem}.fd-list__item--growing .fd-list__title-text{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:inherit;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;font-weight:var(--fdList_Growing_Font_Weight);forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;overflow:hidden;padding-block:0;padding-inline:0;text-overflow:ellipsis;text-shadow:inherit;white-space:nowrap}.fd-list__item--growing .fd-list__title-text:after,.fd-list__item--growing .fd-list__title-text:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list__item--inactive,.fd-list__item--inactive .fd-list__secondary,.fd-list__item--inactive .fd-list__title{pointer-events:none}.fd-list__item.is-selected,.fd-list__item[aria-selected=true]{--fdList_Item_Background_Color:var(--sapList_SelectionBackgroundColor);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_SelectionBorderColor)}.fd-list__item.is-selected .fd-list__link,.fd-list__item[aria-selected=true] .fd-list__link{background:inherit}.fd-list__item.is-selected .fd-list__link.is-active,.fd-list__item.is-selected .fd-list__link:active,.fd-list__item[aria-selected=true] .fd-list__link.is-active,.fd-list__item[aria-selected=true] .fd-list__link:active{--fdList_Item_Background_Color:var(--sapList_Active_Background)}.fd-list__item.is-selected.is-hover,.fd-list__item.is-selected:hover,.fd-list__item[aria-selected=true].is-hover,.fd-list__item[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Hover_SelectionBackground)}.fd-list__secondary,.fd-list__title{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:var(--fdList_Item_Text_Color);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;overflow:hidden;padding-block:0;padding-inline:0;pointer-events:none;pointer-events:all;text-overflow:ellipsis;white-space:nowrap}.fd-list__secondary:after,.fd-list__secondary:before,.fd-list__title:after,.fd-list__title:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list__title{-webkit-box-flex:3;-ms-flex:3 3 10%;flex:3 3 10%;font-size:var(--fdList_Item_Text_Font_Size);min-height:var(--fdList_Item_Text_Font_Size)}.fd-list__title:last-child:first-child{-ms-flex-preferred-size:auto;flex-basis:auto}.fd-list__title .fd-list__title-text{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.fd-list__title~.fd-list__icon{-webkit-margin-end:-1rem;margin-inline-end:-1rem}.fd-list__secondary{color:var(--fdList_Status_Text_Color);font-size:var(--sapFontSize);max-width:40%;text-align:right;-webkit-padding-start:1rem;padding-inline-start:1rem}.fd-list__secondary--positive{--fdList_Status_Text_Color:var(--sapPositiveTextColor)}.fd-list__secondary--critical{--fdList_Status_Text_Color:var(--sapCriticalTextColor)}.fd-list__secondary--negative{--fdList_Status_Text_Color:var(--sapNegativeTextColor)}.fd-list__secondary--informative{--fdList_Status_Text_Color:var(--sapInformativeTextColor)}.fd-list__secondary[dir=rtl],[dir=rtl] .fd-list__secondary{text-align:left}.fd-list__icon [class*=sap-icon],.fd-list__icon[class*=sap-icon]{background:inherit;border-radius:inherit;color:inherit;display:-webkit-box;display:-ms-flexbox;display:flex;font-size:inherit;line-height:1;pointer-events:none;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;color:var(--sapContent_NonInteractiveIconColor);font-size:var(--fdList_Item_Icon_Font_Size);width:2.75rem;-webkit-box-flex:0;-ms-flex:0 0 auto;flex:0 0 auto}.fd-list__icon [class*=sap-icon]:first-child,.fd-list__icon[class*=sap-icon]:first-child{-webkit-margin-start:-1rem;margin-inline-start:-1rem}.fd-list__link{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-inline:0;-webkit-box-align:center;-ms-flex-align:center;align-items:center;--fdList_Item_Border_Color:transparent;background:var(--fdList_Item_Background_Color);border-bottom:var(--sapList_BorderWidth) solid var(--fdList_Item_Border_Color);cursor:pointer;height:100%;padding-block:0;padding-inline:.9375rem;text-decoration:none;width:100%}.fd-list__link:after,.fd-list__link:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list__link.is-focus,.fd-list__link:focus{outline:none;pointer-events:all}.fd-list__link.is-focus:before,.fd-list__link:focus:before{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);content:\"\";display:block;inset:.125rem;pointer-events:none;position:absolute}.fd-list__link.is-focus.is-active:before,.fd-list__link.is-focus:active:before,.fd-list__link:focus.is-active:before,.fd-list__link:focus:active:before{border:var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--fdList_Active_Outline_Color)}.fd-list__link.is-hover,.fd-list__link:hover{--fdList_Item_Background_Color:var(--sapList_Hover_Background);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_BorderColor)}.fd-list__link.is-selected,.fd-list__link[aria-selected=true]{--fdList_Item_Background_Color:var(--sapList_SelectionBackgroundColor);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_SelectionBorderColor)}.fd-list__link.is-selected .fd-list__link,.fd-list__link[aria-selected=true] .fd-list__link{background:inherit}.fd-list__link.is-selected .fd-list__link.is-active,.fd-list__link.is-selected .fd-list__link:active,.fd-list__link[aria-selected=true] .fd-list__link.is-active,.fd-list__link[aria-selected=true] .fd-list__link:active{--fdList_Item_Background_Color:var(--sapList_Active_Background)}.fd-list__link.is-selected.is-hover,.fd-list__link.is-selected:hover,.fd-list__link[aria-selected=true].is-hover,.fd-list__link[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Hover_SelectionBackground)}.fd-list__link.is-active,.fd-list__link.is-active.is-hover,.fd-list__link.is-active.is-selected,.fd-list__link.is-active.is-selected.is-hover,.fd-list__link.is-active.is-selected:hover,.fd-list__link.is-active:hover,.fd-list__link.is-active[aria-selected=true],.fd-list__link.is-active[aria-selected=true].is-hover,.fd-list__link.is-active[aria-selected=true]:hover,.fd-list__link:active,.fd-list__link:active.is-hover,.fd-list__link:active.is-selected,.fd-list__link:active.is-selected.is-hover,.fd-list__link:active.is-selected:hover,.fd-list__link:active:hover,.fd-list__link:active[aria-selected=true],.fd-list__link:active[aria-selected=true].is-hover,.fd-list__link:active[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Active_Background);--fdList_Item_Text_Color:var(--sapList_Active_TextColor);--fdList_Item_Border_Color:var(--sapList_Active_Background)}.fd-list__link.is-active,.fd-list__link.is-active .fd-list__byline,.fd-list__link.is-active .fd-list__byline-left,.fd-list__link.is-active .fd-list__byline-right,.fd-list__link.is-active .fd-list__byline-right--critical,.fd-list__link.is-active .fd-list__byline-right--informative,.fd-list__link.is-active .fd-list__byline-right--negative,.fd-list__link.is-active .fd-list__byline-right--neutral,.fd-list__link.is-active .fd-list__byline-right--positive,.fd-list__link.is-active .fd-list__content,.fd-list__link.is-active .fd-list__icon,.fd-list__link.is-active .fd-list__secondary,.fd-list__link.is-active .fd-list__thumbnail,.fd-list__link.is-active .fd-list__title,.fd-list__link.is-active.fd-list__link--navigation-indicator:after,.fd-list__link:active,.fd-list__link:active .fd-list__byline,.fd-list__link:active .fd-list__byline-left,.fd-list__link:active .fd-list__byline-right,.fd-list__link:active .fd-list__byline-right--critical,.fd-list__link:active .fd-list__byline-right--informative,.fd-list__link:active .fd-list__byline-right--negative,.fd-list__link:active .fd-list__byline-right--neutral,.fd-list__link:active .fd-list__byline-right--positive,.fd-list__link:active .fd-list__content,.fd-list__link:active .fd-list__icon,.fd-list__link:active .fd-list__secondary,.fd-list__link:active .fd-list__thumbnail,.fd-list__link:active .fd-list__title,.fd-list__link:active.fd-list__link--navigation-indicator:after{text-shadow:none}.fd-list__group-header{background:var(--sapList_GroupHeaderBackground);border-bottom:var(--sapList_BorderWidth) solid var(--sapList_GroupHeaderBorderColor);height:var(--sapElement_LineHeight);-webkit-box-align:end;-ms-flex-align:end;align-items:flex-end}.fd-list__group-header .fd-list__title{color:var(--sapList_TableGroupHeaderTextColor);font-family:var(--sapFontHeaderFamily);font-size:var(--sapFontHeader6Size);font-weight:700;line-height:2rem;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.fd-list__footer{background:var(--sapList_FooterBackground);color:var(--sapList_FooterTextColor);font-size:var(--sapFontSize);height:2rem}.fd-list__footer--right{-webkit-box-pack:end;-ms-flex-pack:end;justify-content:flex-end}.fd-list__footer--left{-webkit-box-pack:start;-ms-flex-pack:start;justify-content:flex-start}.fd-list__bold{font-weight:700}.fd-list__message{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;height:2rem;left:0;line-height:normal;line-height:2rem;margin-block:0;margin-inline:0;overflow:hidden;padding-block:0;padding-inline:0;position:absolute;text-overflow:ellipsis;top:0;white-space:nowrap;width:100%;-webkit-padding-start:.5rem;background-color:var(--fdList_Message_Background_Color);font-size:var(--sapFontSmallSize);padding-inline-start:.5rem}.fd-list__message:after,.fd-list__message:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list__message--error{--fdList_Message_Background_Color:var(--sapErrorBackground)}.fd-list__message--warning{--fdList_Message_Background_Color:var(--sapWarningBackground)}.fd-list__message--success{--fdList_Message_Background_Color:var(--sapSuccessBackground)}.fd-list__message--information{--fdList_Message_Background_Color:var(--sapInformationBackground)}.fd-list .fd-list__button{-webkit-margin-start:var(--fdList_Button_Spacing);margin-inline-start:var(--fdList_Button_Spacing);-ms-flex-item-align:center;align-self:center}.fd-list[class*=-compact]:not(.fd-object-list):not(.fd-list--message-view),.fd-list[class*=-condensed]:not(.fd-object-list):not(.fd-list--message-view),[class*=-compact] .fd-list:not([class*=-cozy]):not(.fd-object-list):not(.fd-list--message-view),[class*=-condensed] .fd-list:not([class*=-cozy]):not(.fd-object-list):not(.fd-list--message-view){--fdList_Item_Icon_Font_Size:1rem;--fdList_Item_Text_Font_Size:var(--sapFontSize);--fdList_Item_Height:var(--sapElement_Compact_LineHeight)}.fd-list[class*=-compact]:not(.fd-object-list):not(.fd-list--message-view) .fd-list__item--growing,.fd-list[class*=-condensed]:not(.fd-object-list):not(.fd-list--message-view) .fd-list__item--growing,[class*=-compact] .fd-list:not([class*=-cozy]):not(.fd-object-list):not(.fd-list--message-view) .fd-list__item--growing,[class*=-condensed] .fd-list:not([class*=-cozy]):not(.fd-object-list):not(.fd-list--message-view) .fd-list__item--growing{--fdList_Item_Height:var(--sapElement_LineHeight)}.fd-list[class*=-compact]:not(.fd-object-list):not(.fd-list--message-view) .fd-list__item.fd-list__item--action,.fd-list[class*=-condensed]:not(.fd-object-list):not(.fd-list--message-view) .fd-list__item.fd-list__item--action,[class*=-compact] .fd-list:not([class*=-cozy]):not(.fd-object-list):not(.fd-list--message-view) .fd-list__item.fd-list__item--action,[class*=-condensed] .fd-list:not([class*=-cozy]):not(.fd-object-list):not(.fd-list--message-view) .fd-list__item.fd-list__item--action{--fdList_Item_Height:var(--sapElement_Compact_LineHeight)}.fd-list[class*=-compact]:not(.fd-object-list):not(.fd-list--message-view) .fd-list__item.fd-list__item--action .fd-list__title,.fd-list[class*=-condensed]:not(.fd-object-list):not(.fd-list--message-view) .fd-list__item.fd-list__item--action .fd-list__title,[class*=-compact] .fd-list:not([class*=-cozy]):not(.fd-object-list):not(.fd-list--message-view) .fd-list__item.fd-list__item--action .fd-list__title,[class*=-condensed] .fd-list:not([class*=-cozy]):not(.fd-object-list):not(.fd-list--message-view) .fd-list__item.fd-list__item--action .fd-list__title{height:var(--fdList_Item_Action_Button_Size_Compact)}.fd-list--no-border .fd-list__group-header,.fd-list--no-border .fd-list__item{border-bottom:0;border-top:0}.fd-list--no-border .fd-list__item.is-selected,.fd-list--no-border .fd-list__item[aria-selected=true]{border-bottom:var(--sapList_BorderWidth) solid var(--sapList_SelectionBorderColor)}.fd-list--has-message{-webkit-padding-before:2rem;padding-block-start:2rem}.fd-list__infinite-scroll{outline:none;overflow:scroll}.fd-list--dropdown.is-focus,.fd-list--dropdown:focus{outline:none;z-index:5}.fd-list--dropdown .fd-list__item:not(.fd-list__group-header){display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center;height:auto;min-height:2.5rem;padding-block:0;padding-inline:.9375rem}.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):hover{--fdList_Item_Background_Color:var(--sapList_Hover_Background);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_BorderColor)}.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-selected,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header)[aria-selected=true]{--fdList_Item_Background_Color:var(--sapList_SelectionBackgroundColor);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_SelectionBorderColor)}.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-selected .fd-list__link,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header)[aria-selected=true] .fd-list__link{background:inherit}.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-selected .fd-list__link.is-active,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-selected .fd-list__link:active,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header)[aria-selected=true] .fd-list__link.is-active,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header)[aria-selected=true] .fd-list__link:active{--fdList_Item_Background_Color:var(--sapList_Active_Background)}.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-selected.is-hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-selected:hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header)[aria-selected=true].is-hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header)[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Hover_SelectionBackground)}.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active.is-hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active.is-selected,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active.is-selected.is-hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active.is-selected:hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active:hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active[aria-selected=true],.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active[aria-selected=true].is-hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active[aria-selected=true]:hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active.is-hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active.is-selected,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active.is-selected.is-hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active.is-selected:hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active:hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active[aria-selected=true],.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active[aria-selected=true].is-hover,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Active_Background);--fdList_Item_Text_Color:var(--sapList_Active_TextColor);--fdList_Item_Border_Color:var(--sapList_Active_Background)}.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__byline,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__byline-left,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__byline-right,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__byline-right--critical,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__byline-right--informative,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__byline-right--negative,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__byline-right--neutral,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__byline-right--positive,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__content,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__icon,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__secondary,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__thumbnail,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active .fd-list__title,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header).is-active.fd-list__link--navigation-indicator:after,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__byline,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__byline-left,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__byline-right,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__byline-right--critical,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__byline-right--informative,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__byline-right--negative,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__byline-right--neutral,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__byline-right--positive,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__content,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__icon,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__secondary,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__thumbnail,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active .fd-list__title,.fd-list--dropdown .fd-list__item:not(.fd-list__group-header):active.fd-list__link--navigation-indicator:after{text-shadow:none}.fd-list--dropdown[class*=-compact] .fd-list__item:not(.fd-list__group-header),.fd-list--dropdown[class*=-condensed] .fd-list__item:not(.fd-list__group-header),[class*=-compact] .fd-list--dropdown:not([class*=-cozy]) .fd-list__item:not(.fd-list__group-header),[class*=-condensed] .fd-list--dropdown:not([class*=-cozy]) .fd-list__item:not(.fd-list__group-header){height:auto;min-height:2rem;padding-block:0;padding-inline:.9375rem}.fd-list--multi-input .fd-list__item{height:auto;padding-block:0;padding-inline:0 1rem}.fd-list--multi-input .fd-list__item.is-hover,.fd-list--multi-input .fd-list__item:hover{--fdList_Item_Background_Color:var(--sapList_Hover_Background);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_BorderColor)}.fd-list--multi-input .fd-list__item.is-selected,.fd-list--multi-input .fd-list__item[aria-selected=true]{--fdList_Item_Background_Color:var(--sapList_SelectionBackgroundColor);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_SelectionBorderColor)}.fd-list--multi-input .fd-list__item.is-selected .fd-list__link,.fd-list--multi-input .fd-list__item[aria-selected=true] .fd-list__link{background:inherit}.fd-list--multi-input .fd-list__item.is-selected .fd-list__link.is-active,.fd-list--multi-input .fd-list__item.is-selected .fd-list__link:active,.fd-list--multi-input .fd-list__item[aria-selected=true] .fd-list__link.is-active,.fd-list--multi-input .fd-list__item[aria-selected=true] .fd-list__link:active{--fdList_Item_Background_Color:var(--sapList_Active_Background)}.fd-list--multi-input .fd-list__item.is-selected.is-hover,.fd-list--multi-input .fd-list__item.is-selected:hover,.fd-list--multi-input .fd-list__item[aria-selected=true].is-hover,.fd-list--multi-input .fd-list__item[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Hover_SelectionBackground)}.fd-list--multi-input .fd-list__item.is-active,.fd-list--multi-input .fd-list__item.is-active.is-hover,.fd-list--multi-input .fd-list__item.is-active.is-selected,.fd-list--multi-input .fd-list__item.is-active.is-selected.is-hover,.fd-list--multi-input .fd-list__item.is-active.is-selected:hover,.fd-list--multi-input .fd-list__item.is-active:hover,.fd-list--multi-input .fd-list__item.is-active[aria-selected=true],.fd-list--multi-input .fd-list__item.is-active[aria-selected=true].is-hover,.fd-list--multi-input .fd-list__item.is-active[aria-selected=true]:hover,.fd-list--multi-input .fd-list__item:active,.fd-list--multi-input .fd-list__item:active.is-hover,.fd-list--multi-input .fd-list__item:active.is-selected,.fd-list--multi-input .fd-list__item:active.is-selected.is-hover,.fd-list--multi-input .fd-list__item:active.is-selected:hover,.fd-list--multi-input .fd-list__item:active:hover,.fd-list--multi-input .fd-list__item:active[aria-selected=true],.fd-list--multi-input .fd-list__item:active[aria-selected=true].is-hover,.fd-list--multi-input .fd-list__item:active[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Active_Background);--fdList_Item_Text_Color:var(--sapList_Active_TextColor);--fdList_Item_Border_Color:var(--sapList_Active_Background)}.fd-list--multi-input .fd-list__item.is-active,.fd-list--multi-input .fd-list__item.is-active .fd-list__byline,.fd-list--multi-input .fd-list__item.is-active .fd-list__byline-left,.fd-list--multi-input .fd-list__item.is-active .fd-list__byline-right,.fd-list--multi-input .fd-list__item.is-active .fd-list__byline-right--critical,.fd-list--multi-input .fd-list__item.is-active .fd-list__byline-right--informative,.fd-list--multi-input .fd-list__item.is-active .fd-list__byline-right--negative,.fd-list--multi-input .fd-list__item.is-active .fd-list__byline-right--neutral,.fd-list--multi-input .fd-list__item.is-active .fd-list__byline-right--positive,.fd-list--multi-input .fd-list__item.is-active .fd-list__content,.fd-list--multi-input .fd-list__item.is-active .fd-list__icon,.fd-list--multi-input .fd-list__item.is-active .fd-list__secondary,.fd-list--multi-input .fd-list__item.is-active .fd-list__thumbnail,.fd-list--multi-input .fd-list__item.is-active .fd-list__title,.fd-list--multi-input .fd-list__item.is-active.fd-list__link--navigation-indicator:after,.fd-list--multi-input .fd-list__item:active,.fd-list--multi-input .fd-list__item:active .fd-list__byline,.fd-list--multi-input .fd-list__item:active .fd-list__byline-left,.fd-list--multi-input .fd-list__item:active .fd-list__byline-right,.fd-list--multi-input .fd-list__item:active .fd-list__byline-right--critical,.fd-list--multi-input .fd-list__item:active .fd-list__byline-right--informative,.fd-list--multi-input .fd-list__item:active .fd-list__byline-right--negative,.fd-list--multi-input .fd-list__item:active .fd-list__byline-right--neutral,.fd-list--multi-input .fd-list__item:active .fd-list__byline-right--positive,.fd-list--multi-input .fd-list__item:active .fd-list__content,.fd-list--multi-input .fd-list__item:active .fd-list__icon,.fd-list--multi-input .fd-list__item:active .fd-list__secondary,.fd-list--multi-input .fd-list__item:active .fd-list__thumbnail,.fd-list--multi-input .fd-list__item:active .fd-list__title,.fd-list--multi-input .fd-list__item:active.fd-list__link--navigation-indicator:after{text-shadow:none}.fd-list--multi-input .fd-list__label{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:inherit;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;overflow:hidden;padding-block:0;padding-inline:0;position:relative;text-overflow:ellipsis;width:100%;-webkit-margin-end:0;margin-inline-end:0}.fd-list--multi-input .fd-list__label:after,.fd-list--multi-input .fd-list__label:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list--multi-input .fd-list__label:before{margin-block:-.1875rem -.1875rem}.fd-list--multi-input .fd-list__input.is-focus,.fd-list--multi-input .fd-list__input:focus{-webkit-box-shadow:none;box-shadow:none;z-index:5}.fd-list--multi-input .fd-list__input.is-focus+.fd-list__label:after,.fd-list--multi-input .fd-list__input:focus+.fd-list__label:after{inset:.0625rem}.fd-list--multi-input .fd-list__footer{height:auto;text-align:right;-webkit-box-pack:end;-ms-flex-pack:end;background-color:var(--sapList_Background);justify-content:flex-end;line-height:1rem}.fd-list--multi-input .fd-list__footer,.fd-list--multi-input .fd-list__label{padding-block:.75rem;padding-inline:.9375rem}.fd-list--multi-input[class*=-compact] .fd-list__item,.fd-list--multi-input[class*=-condensed] .fd-list__item,[class*=-compact] .fd-list--multi-input:not([class*=-cozy]) .fd-list__item,[class*=-condensed] .fd-list--multi-input:not([class*=-cozy]) .fd-list__item{height:auto;padding-block:0;padding-inline:0}.fd-list--multi-input[class*=-compact] .fd-list__label:before,.fd-list--multi-input[class*=-condensed] .fd-list__label:before,[class*=-compact] .fd-list--multi-input:not([class*=-cozy]) .fd-list__label:before,[class*=-condensed] .fd-list--multi-input:not([class*=-cozy]) .fd-list__label:before{-webkit-margin-start:0;margin-inline-start:0;margin-top:0;-webkit-margin-after:0;margin-block-end:0}.fd-list--multi-input[class*=-compact] .fd-list__footer,.fd-list--multi-input[class*=-compact] .fd-list__label,.fd-list--multi-input[class*=-condensed] .fd-list__footer,.fd-list--multi-input[class*=-condensed] .fd-list__label,[class*=-compact] .fd-list--multi-input:not([class*=-cozy]) .fd-list__footer,[class*=-compact] .fd-list--multi-input:not([class*=-cozy]) .fd-list__label,[class*=-condensed] .fd-list--multi-input:not([class*=-cozy]) .fd-list__footer,[class*=-condensed] .fd-list--multi-input:not([class*=-cozy]) .fd-list__label{padding-block:.5rem;padding-inline:.9375rem}.fd-list--dropdown,.fd-list--multi-input{display:block;max-width:40rem;min-width:5rem}.fd-list--dropdown .fd-list__item,.fd-list--multi-input .fd-list__item{border:none;cursor:pointer}.fd-list--dropdown .fd-list__item.is-selected,.fd-list--dropdown .fd-list__item[aria-selected=true],.fd-list--multi-input .fd-list__item.is-selected,.fd-list--multi-input .fd-list__item[aria-selected=true]{border-bottom:var(--sapList_BorderWidth) solid var(--sapList_SelectionBorderColor)}.fd-list--dropdown .fd-list__secondary,.fd-list--dropdown .fd-list__title,.fd-list--multi-input .fd-list__secondary,.fd-list--multi-input .fd-list__title{font-size:var(--sapFontSize);white-space:normal;-webkit-box-flex:1;-ms-flex:auto;flex:auto;width:auto}.fd-list--dropdown .fd-list__secondary--no-wrap,.fd-list--dropdown .fd-list__title--no-wrap,.fd-list--multi-input .fd-list__secondary--no-wrap,.fd-list--multi-input .fd-list__title--no-wrap{white-space:nowrap}.fd-list--dropdown .fd-list__secondary,.fd-list--multi-input .fd-list__secondary{display:block}.fd-list--dropdown .fd-list__title,.fd-list--multi-input .fd-list__title{max-width:24rem}.fd-list--dropdown .fd-list__title:first-child:last-child,.fd-list--multi-input .fd-list__title:first-child:last-child{max-width:40rem}.fd-list--dropdown .fd-list__secondary,.fd-list--multi-input .fd-list__secondary{max-width:16rem}.fd-list--dropdown.fd-list--mobile,.fd-list--multi-input.fd-list--mobile{max-width:100%}.fd-list--dropdown .fd-list__icon,.fd-list--multi-input .fd-list__icon{line-height:1rem}.fd-list--dropdown.fd-list--large-dropdown,.fd-list--multi-input.fd-list--large-dropdown{max-width:62rem}.fd-list--dropdown.fd-list--large-dropdown .fd-list__title,.fd-list--multi-input.fd-list--large-dropdown .fd-list__title{max-width:37.2rem}.fd-list--dropdown.fd-list--large-dropdown .fd-list__title:first-child:last-child,.fd-list--multi-input.fd-list--large-dropdown .fd-list__title:first-child:last-child{max-width:62rem}.fd-list--dropdown.fd-list--large-dropdown .fd-list__secondary,.fd-list--multi-input.fd-list--large-dropdown .fd-list__secondary{max-width:24.8rem}.fd-list--dropdown.fd-list--large-dropdown.fd-list--mobile,.fd-list--multi-input.fd-list--large-dropdown.fd-list--mobile{max-width:100%}.fd-list--byline .fd-list__item{height:auto;min-height:5rem;padding-block:.9375rem;padding-inline:1rem;width:100%;-webkit-box-align:start;-ms-flex-align:start;align-items:flex-start}.fd-list--byline .fd-list__item.is-focus,.fd-list--byline .fd-list__item:focus{pointer-events:all;z-index:5}.fd-list--byline .fd-list__item .fd-list__item-counter{-ms-flex-align:center;-ms-flex-item-align:center;align-self:center;-webkit-box-pack:end;-ms-flex-pack:end;height:100%;justify-content:flex-end;min-width:2rem}.fd-list--byline .fd-list__item .fd-list__item-counter,.fd-list--byline .fd-list__thumbnail{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;align-items:center}.fd-list--byline .fd-list__thumbnail{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;pointer-events:none;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-ms-flex-align:center;-webkit-margin-end:.75rem;border-radius:.25rem;color:var(--sapContent_NonInteractiveIconColor);font-size:2rem;height:3rem;margin-inline-end:.75rem;max-height:3rem;max-width:3rem;min-height:3rem;min-width:3rem;width:3rem}.fd-list--byline .fd-list__thumbnail:after,.fd-list--byline .fd-list__thumbnail:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list--byline .fd-list__thumbnail [class*=sap-icon],.fd-list--byline .fd-list__thumbnail[class*=sap-icon]{background:inherit;border-radius:inherit;color:inherit;font-size:inherit;line-height:1}.fd-list--byline .fd-list__thumbnail>svg{height:100%}.fd-list--byline .fd-list__notification{color:var(--sapAccentColor6);font-size:.375rem;left:1rem;position:absolute;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%)}.fd-list--byline .fd-list__notification[dir=rtl],[dir=rtl] .fd-list--byline .fd-list__notification{left:auto;right:1rem}.fd-list--byline .fd-list__notification+.fd-list__link .fd-list__content .fd-list__title,.fd-list--byline .fd-list__notification~.fd-list__content .fd-list__title{font-weight:700}.fd-list--byline .fd-list__content{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;pointer-events:none;-ms-flex-line-pack:justify;align-content:space-between;height:100%;min-height:3rem;min-width:3rem;-webkit-box-flex:1;-ms-flex:1 1 auto;flex:1 1 auto;padding-block:.125rem;padding-inline:0}.fd-list--byline .fd-list__content:after,.fd-list--byline .fd-list__content:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list--byline .fd-list__title{font-size:var(--sapFontLargeSize);min-height:1rem;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.fd-list--byline .fd-list__byline{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:var(--sapContent_LabelColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;line-height:1rem;margin-block:0;margin-inline:0;overflow:hidden;padding-block:0;padding-inline:0;text-overflow:ellipsis;white-space:nowrap;-webkit-padding-before:.5rem;padding-block-start:.5rem}.fd-list--byline .fd-list__byline:after,.fd-list--byline .fd-list__byline:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list--byline .fd-list__byline--2-col{display:-webkit-box;display:-ms-flexbox;display:flex;pointer-events:all}.fd-list--byline .fd-list__byline--wrap .fd-list__byline-left,.fd-list--byline .fd-list__byline--wrap .fd-list__byline-right{display:inline;padding-block:0;padding-inline:0;white-space:normal;width:100%}.fd-list--byline .fd-list__byline--wrap .fd-list__byline-right{float:right;width:auto}.fd-list--byline .fd-list__link--more{pointer-events:all;text-transform:capitalize}.fd-list--byline .fd-list__byline--wrap,.fd-list--byline .fd-list__title--wrap{white-space:normal}.fd-list--byline .fd-list__byline-left{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:var(--sapContent_LabelColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;line-height:1rem;margin-block:0;margin-inline:0;overflow:hidden;padding-block:0;padding-inline:0;text-overflow:ellipsis;white-space:nowrap;width:60%;-ms-flex-item-align:end;align-self:flex-end;-webkit-padding-end:.5rem;padding-inline-end:.5rem}.fd-list--byline .fd-list__byline-left:after,.fd-list--byline .fd-list__byline-left:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list--byline .fd-list__byline-right{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);color:var(--sapContent_LabelColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;line-height:1rem;margin-block:0;margin-inline:0;overflow:hidden;padding-block:0;padding-inline:0;text-align:right;text-overflow:ellipsis;white-space:nowrap;width:40%;-ms-flex-item-align:end;align-self:flex-end;-webkit-padding-start:.5rem;padding-inline-start:.5rem}.fd-list--byline .fd-list__byline-right:after,.fd-list--byline .fd-list__byline-right:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list--byline .fd-list__byline-right[dir=rtl],[dir=rtl] .fd-list--byline .fd-list__byline-right{text-align:left}.fd-list--byline .fd-list__byline-right--neutral{color:var(--sapNeutralTextColor)}.fd-list--byline .fd-list__byline-right--positive{color:var(--sapPositiveTextColor)}.fd-list--byline .fd-list__byline-right--critical{color:var(--sapCriticalTextColor)}.fd-list--byline .fd-list__byline-right--negative{color:var(--sapNegativeTextColor)}.fd-list--byline .fd-list__byline-right--informative{color:var(--sapInformativeTextColor)}.fd-list--byline.fd-list--unread-indicator .fd-list__item{padding-inline:2.125rem 1rem}.fd-list--byline[class*=-compact] .fd-list__title,.fd-list--byline[class*=-condensed] .fd-list__title,[class*=-compact] .fd-list--byline:not([class*=-cozy]) .fd-list__title,[class*=-condensed] .fd-list--byline:not([class*=-cozy]) .fd-list__title{font-size:var(--sapFontSize)}.fd-list--byline.fd-list--no-border .fd-list__item{height:100%;max-height:5rem;padding-block:.5rem}.fd-list--byline.fd-list--no-border .fd-list__item:first-child{-webkit-padding-before:.9375rem;padding-block-start:.9375rem}.fd-list--byline.fd-list--no-border .fd-list__item:last-child{-webkit-padding-after:.9375rem;padding-block-end:.9375rem}.fd-list--message-view{--fdListNavigationIndicatorFontSize:.75rem;--fdList_Item_Height:calc(2.75rem + var(--sapList_BorderWidth));--fdList_Message_View_Item_Byline_Height:calc(3.325rem + var(--sapList_BorderWidth));--fdList_Message_View_Title_Size:var(--sapFontHeader5Size)}.fd-list--message-view .fd-list__item{height:auto}.fd-list--message-view .fd-list__item--byline{--fdList_Item_Height:var(--fdList_Message_View_Item_Byline_Height)}.fd-list--message-view .fd-list__content{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);display:-webkit-box;display:-ms-flexbox;display:flex;font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;padding-block:0;padding-inline:0;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-flex:1;-ms-flex:1 1 auto;flex:1 1 auto;overflow:hidden;pointer-events:none;text-overflow:ellipsis;white-space:nowrap}.fd-list--message-view .fd-list__content:after,.fd-list--message-view .fd-list__content:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-list--message-view .fd-list__title{color:var(--sapGroup_TitleTextColor);font-family:var(--sapFontHeaderFamily);font-size:var(--fdList_Message_View_Title_Size);font-weight:var(--sapFontHeaderWeight,normal);overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.fd-list--message-view .fd-list__subtitle{-webkit-padding-before:.25rem;color:var(--sapContent_LabelColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);overflow:hidden;padding-block-start:.25rem;text-overflow:ellipsis;white-space:nowrap}.fd-list--message-view .fd-list__link{padding-block:.5rem}.fd-list--message-view .fd-list__link.is-active .fd-list__subtitle,.fd-list--message-view .fd-list__link.is-active .fd-list__title,.fd-list--message-view .fd-list__link.is-active .fd-object-status .fd-object-status__icon,.fd-list--message-view .fd-list__link:active .fd-list__subtitle,.fd-list--message-view .fd-list__link:active .fd-list__title,.fd-list--message-view .fd-list__link:active .fd-object-status .fd-object-status__icon{color:var(--sapList_Active_TextColor)}.fd-list--message-view[class*=-compact],.fd-list--message-view[class*=-condensed],[class*=-compact] .fd-list--message-view:not([class*=-cozy]),[class*=-condensed] .fd-list--message-view:not([class*=-cozy]){--fdList_Message_View_Item_Height:calc(2rem + var(--sapList_BorderWidth));--fdList_Message_View_Title_Size:var(--sapFontHeader6Size);--fdList_Message_View_Item_Byline_Height:calc(3.25rem + var(--sapList_BorderWidth))}.fd-list--navigation .fd-list__item--link{padding-block:0;padding-inline:0}.fd-list--navigation.fd-icon-tab-bar__list .fd-list__link{gap:.25rem}.fd-list--navigation .fd-list__link{padding-inline:var(--fdList_Navigation_Link_Padding_Left,.9375rem) var(--fdList_Navigation_Link_Padding_Right,.9375rem)}.fd-list--navigation.fd-list--unread-indicator{--fdList_Navigation_Link_Padding_Left:2.125rem;--fdList_Navigation_Link_Padding_Right:.9375rem}.fd-list--navigation.fd-list--navigation-indication{--fdList_Navigation_Link_Padding_Left:.9375rem;--fdList_Navigation_Link_Padding_Right:0}.fd-list--navigation.fd-list--selection{--fdList_Navigation_Link_Padding_Left:2.75rem;--fdList_Navigation_Link_Padding_Right:.9375rem}.fd-list--navigation.fd-list--selection .fd-list__item--link{padding-block:0;padding-inline:0}.fd-list--navigation.fd-list--selection.fd-list--navigation-indication{--fdList_Navigation_Link_Padding_Left:2.75rem;--fdList_Navigation_Link_Padding_Right:0}.fd-list--navigation.fd-list--selection.fd-list--navigation-indication .fd-list__item--link{padding-block:0;padding-inline:0}.fd-list--navigation.fd-list--byline,.fd-list--navigation.fd-list--byline.fd-list--compact{--fdList_Navigation_Link_Padding_Left:1rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-list--navigation.fd-list--byline .fd-list__item--link,.fd-list--navigation.fd-list--byline.fd-list--compact .fd-list__item--link{padding-block:0;padding-inline:0}.fd-list--navigation.fd-list--byline .fd-list__link,.fd-list--navigation.fd-list--byline.fd-list--compact .fd-list__link{padding-block:.9375rem}.fd-list--navigation.fd-list--byline.fd-list--compact.fd-list--unread-indicator,.fd-list--navigation.fd-list--byline.fd-list--unread-indicator{--fdList_Navigation_Link_Padding_Left:2.125rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-list--navigation.fd-list--byline.fd-list--navigation-indication{--fdList_Navigation_Link_Padding_Left:.9375rem;--fdList_Navigation_Link_Padding_Right:0}.fd-list--navigation.fd-list--byline.fd-list--no-border .fd-list__item--link .fd-list__link{padding-block:.5rem}.fd-list--navigation.fd-list--byline.fd-list--no-border .fd-list__item--link:first-child{-webkit-padding-before:0;padding-block-start:0}.fd-list--navigation.fd-list--byline.fd-list--no-border .fd-list__item--link:first-child .fd-list__link{-webkit-padding-before:1rem;padding-block-start:1rem}.fd-list--navigation.fd-list--byline.fd-list--no-border .fd-list__item--link:first-child .fd-list__link--navigation-indicator:after{-webkit-margin-after:.5rem;margin-block-end:.5rem}.fd-list--navigation.fd-list--byline.fd-list--no-border .fd-list__item--link:last-child{-webkit-padding-after:0;padding-block-end:0}.fd-list--navigation.fd-list--byline.fd-list--no-border .fd-list__item--link:last-child .fd-list__link{-webkit-padding-after:1rem;padding-block-end:1rem}.fd-list--navigation.fd-list--byline.fd-list--no-border .fd-list__item--link:last-child .fd-list__link--navigation-indicator:after{margin-top:.25rem}.fd-list--navigation.fd-list--byline.fd-list--selection{--fdList_Navigation_Link_Padding_Left:2.75rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-list--navigation.fd-list--byline.fd-list--selection .fd-list__item--link{padding-block:0;padding-inline:0}.fd-list--navigation.fd-list--byline.fd-list--selection[class*=-compact],.fd-list--navigation.fd-list--byline.fd-list--selection[class*=-condensed],[class*=-compact] .fd-list--navigation.fd-list--byline.fd-list--selection:not([class*=-cozy]),[class*=-condensed] .fd-list--navigation.fd-list--byline.fd-list--selection:not([class*=-cozy]){--fdList_Navigation_Link_Padding_Left:2.75rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--no-border .fd-list__item--link,.fd-list--navigation.fd-list--byline.fd-list--selection[class*=-compact] .fd-list__item--link,.fd-list--navigation.fd-list--byline.fd-list--selection[class*=-condensed] .fd-list__item--link,[class*=-compact] .fd-list--navigation.fd-list--byline.fd-list--selection:not([class*=-cozy]) .fd-list__item--link,[class*=-condensed] .fd-list--navigation.fd-list--byline.fd-list--selection:not([class*=-cozy]) .fd-list__item--link{padding-block:0;padding-inline:0}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--no-border .fd-list__item--link .fd-list__link{padding-block:.5rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--no-border .fd-list__item--link:first-child{-webkit-padding-before:0;padding-block-start:0}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--no-border .fd-list__item--link:first-child .fd-list__link{-webkit-padding-before:1rem;padding-block-start:1rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--no-border .fd-list__item--link:first-child .fd-list__link--navigation-indicator:after{-webkit-margin-after:.5rem;margin-block-end:.5rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--no-border .fd-list__item--link:last-child{-webkit-padding-after:0;padding-block-end:0}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--no-border .fd-list__item--link:last-child .fd-list__link{-webkit-padding-after:1rem;padding-block-end:1rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--no-border .fd-list__item--link:last-child .fd-list__link--navigation-indicator:after{margin-top:.25rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication{--fdList_Navigation_Link_Padding_Left:2.75rem;--fdList_Navigation_Link_Padding_Right:0}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication .fd-list__item--link{padding-block:0;padding-inline:0}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication[class*=-compact],.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication[class*=-condensed],[class*=-compact] .fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication:not([class*=-cozy]),[class*=-condensed] .fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication:not([class*=-cozy]){--fdList_Navigation_Link_Padding_Left:2.75rem;--fdList_Navigation_Link_Padding_Right:0}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication.fd-list--no-border .fd-list__item--link .fd-list__link{padding-block:.5rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication.fd-list--no-border .fd-list__item--link:first-child{-webkit-padding-before:0;padding-block-start:0}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication.fd-list--no-border .fd-list__item--link:first-child .fd-list__link{-webkit-padding-before:1rem;padding-block-start:1rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication.fd-list--no-border .fd-list__item--link:first-child .fd-list__link--navigation-indicator:after{-webkit-margin-after:.5rem;margin-block-end:.5rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication.fd-list--no-border .fd-list__item--link:last-child{-webkit-padding-after:0;padding-block-end:0}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication.fd-list--no-border .fd-list__item--link:last-child .fd-list__link{-webkit-padding-after:1rem;padding-block-end:1rem}.fd-list--navigation.fd-list--byline.fd-list--selection.fd-list--navigation-indication.fd-list--no-border .fd-list__item--link:last-child .fd-list__link--navigation-indicator:after{margin-top:.25rem}.fd-list--navigation.fd-list--navigation-object{--fdList_Navigation_Link_Padding_Left:1rem;--fdList_Navigation_Link_Padding_Right:1rem}.fd-list--navigation.fd-list--navigation-object .fd-list__item--link,.fd-list--navigation.fd-list--navigation-object[dir=rtl] .fd-list__item--link,[dir=rtl] .fd-list--navigation.fd-list--navigation-object .fd-list__item--link{padding-block:0;padding-inline:0}.fd-list--navigation.fd-list--navigation-object .fd-list__link{padding-block:1rem}.fd-list--navigation-indication .fd-list:after{content:\"\";min-width:1.5rem}.fd-list--navigation-indication fd-list__item.fd-list__item--link{-webkit-padding-end:0;padding-inline-end:0}.fd-list--navigation-indication fd-list__item.fd-list__item--link:after{content:none}.fd-list--navigation-indication .fd-list__link:after{content:\"\";min-width:2.5rem}.fd-list--navigation-indication .fd-list__link--navigation-indicator:after{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;color:var(--sapContent_NonInteractiveIconColor);content:\"\\e1ed\";font-family:SAP-icons;font-size:var(--fdListNavigationIndicatorFontSize,var(--sapFontLargeSize));height:100%;text-decoration:none;text-transform:none}.fd-list--navigation-indication .fd-list__link--navigation-indicator[dir=rtl]:after,[dir=rtl] .fd-list--navigation-indication .fd-list__link--navigation-indicator:after{content:\"\\e1ee\"}.fd-list--navigation-indication .fd-list__icon:last-child,.fd-list--navigation-indication .fd-list__icon[dir=rtl]:last-child,[dir=rtl] .fd-list--navigation-indication .fd-list__icon:last-child{margin-block:0;margin-inline:0}.fd-list--navigation-indication .is-navigated{-webkit-box-shadow:inset -.1875rem 0 0 0 var(--sapList_SelectionBorderColor);box-shadow:inset -.1875rem 0 0 0 var(--sapList_SelectionBorderColor)}.fd-list--navigation-indication .is-navigated[dir=rtl],[dir=rtl] .fd-list--navigation-indication .is-navigated{-webkit-box-shadow:inset .1875rem 0 0 0 var(--sapList_SelectionBorderColor);box-shadow:inset .1875rem 0 0 0 var(--sapList_SelectionBorderColor)}.fd-list--selection .fd-list__item{cursor:pointer;padding-inline:2.75rem .9375rem}.fd-list--selection .fd-list__item.is-hover,.fd-list--selection .fd-list__item:hover{--fdList_Item_Background_Color:var(--sapList_Hover_Background)}.fd-list--selection .fd-list__item.is-selected,.fd-list--selection .fd-list__item[aria-selected=true]{--fdList_Item_Background_Color:var(--sapList_SelectionBackgroundColor);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_SelectionBorderColor)}.fd-list--selection .fd-list__item.is-selected .fd-list__link,.fd-list--selection .fd-list__item[aria-selected=true] .fd-list__link{background:inherit}.fd-list--selection .fd-list__item.is-selected .fd-list__link.is-active,.fd-list--selection .fd-list__item.is-selected .fd-list__link:active,.fd-list--selection .fd-list__item[aria-selected=true] .fd-list__link.is-active,.fd-list--selection .fd-list__item[aria-selected=true] .fd-list__link:active{--fdList_Item_Background_Color:var(--sapList_Active_Background)}.fd-list--selection .fd-list__item.is-selected.is-hover,.fd-list--selection .fd-list__item.is-selected:hover,.fd-list--selection .fd-list__item[aria-selected=true].is-hover,.fd-list--selection .fd-list__item[aria-selected=true]:hover{--fdList_Item_Background_Color:var(--sapList_Hover_SelectionBackground)}.fd-list--selection .fd-list__form-item{left:0;position:absolute;top:0;z-index:5}.fd-list--selection .fd-list__form-item[dir=rtl],[dir=rtl] .fd-list--selection .fd-list__form-item{left:auto;right:0}.fd-list--selection[class*=-compact] .fd-list__form-item,.fd-list--selection[class*=-condensed] .fd-list__form-item,[class*=-compact] .fd-list--selection:not([class*=-cozy]) .fd-list__form-item,[class*=-condensed] .fd-list--selection:not([class*=-cozy]) .fd-list__form-item{left:.375rem}.fd-list--selection[class*=-compact] .fd-list__form-item[dir=rtl],.fd-list--selection[class*=-condensed] .fd-list__form-item[dir=rtl],[class*=-compact] .fd-list--selection:not([class*=-cozy]) .fd-list__form-item[dir=rtl],[class*=-condensed] .fd-list--selection:not([class*=-cozy]) .fd-list__form-item[dir=rtl],[dir=rtl] .fd-list--selection[class*=-compact] .fd-list__form-item,[dir=rtl] .fd-list--selection[class*=-condensed] .fd-list__form-item,[dir=rtl] [class*=-compact] .fd-list--selection:not([class*=-cozy]) .fd-list__form-item,[dir=rtl] [class*=-condensed] .fd-list--selection:not([class*=-cozy]) .fd-list__form-item{left:auto;right:.375rem}.fd-list--selection.fd-list--byline .fd-list__item{padding-inline:2.75rem 1rem}.fd-list--selection.fd-list--byline .fd-list__form-item{top:.3125rem}.fd-list--selection.fd-list--byline[class*=-compact] .fd-list__item,.fd-list--selection.fd-list--byline[class*=-condensed] .fd-list__item,[class*=-compact] .fd-list--selection.fd-list--byline:not([class*=-cozy]) .fd-list__item,[class*=-condensed] .fd-list--selection.fd-list--byline:not([class*=-cozy]) .fd-list__item{padding-inline:2.75rem 1rem}.fd-list--selection.fd-list--byline[class*=-compact] .fd-list__form-item,.fd-list--selection.fd-list--byline[class*=-condensed] .fd-list__form-item,[class*=-compact] .fd-list--selection.fd-list--byline:not([class*=-cozy]) .fd-list__form-item,[class*=-condensed] .fd-list--selection.fd-list--byline:not([class*=-cozy]) .fd-list__form-item{left:.375rem;top:.5rem}.fd-list--selection.fd-list--byline[class*=-compact] .fd-list__form-item[dir=rtl],.fd-list--selection.fd-list--byline[class*=-condensed] .fd-list__form-item[dir=rtl],[class*=-compact] .fd-list--selection.fd-list--byline:not([class*=-cozy]) .fd-list__form-item[dir=rtl],[class*=-condensed] .fd-list--selection.fd-list--byline:not([class*=-cozy]) .fd-list__form-item[dir=rtl],[dir=rtl] .fd-list--selection.fd-list--byline[class*=-compact] .fd-list__form-item,[dir=rtl] .fd-list--selection.fd-list--byline[class*=-condensed] .fd-list__form-item,[dir=rtl] [class*=-compact] .fd-list--selection.fd-list--byline:not([class*=-cozy]) .fd-list__form-item,[dir=rtl] [class*=-condensed] .fd-list--selection.fd-list--byline:not([class*=-cozy]) .fd-list__form-item{left:auto;right:.375rem}.fd-list--selection.fd-list--byline.fd-list--no-border .fd-list__item{padding-block:.5rem;padding-inline:2.75rem 1rem}.fd-list--selection.fd-list--byline.fd-list--no-border .fd-list__item:first-child{-webkit-padding-before:.9375rem;padding-block-start:.9375rem}.fd-list--selection.fd-list--byline.fd-list--no-border .fd-list__item:last-child{-webkit-padding-after:.9375rem;padding-block-end:.9375rem}.fd-list--selection.fd-list--byline.fd-list--no-border .fd-list__item:not(:first-child) .fd-list__form-item,.fd-list--selection.fd-list--byline.fd-list--no-border .fd-list__item:not(:first-child) .fd-list__form-item.fd-form-item{top:0}.fd-list--selection.fd-list--byline.fd-list--no-border .fd-list__item:not(:first-child) .fd-list__form-item label,.fd-list--selection.fd-list--byline.fd-list--no-border .fd-list__item:not(:first-child) .fd-list__form-item.fd-form-item label{-webkit-padding-before:.5rem;padding-block-start:.5rem}.fd-list--selection.fd-list--byline.fd-list--no-border .fd-list__item:not(:first-child) .fd-list__form-item label:after,.fd-list--selection.fd-list--byline.fd-list--no-border .fd-list__item:not(:first-child) .fd-list__form-item.fd-form-item label:after{top:.375rem}.fd-list--selection.fd-list--selection-row .fd-list__item{-webkit-padding-start:1rem;padding-inline-start:1rem}:root{--fdList_Item_Background_Color:var(--sapList_Background);--fdList_Item_Text_Color:var(--sapList_TextColor);--fdList_Item_Border_Color:var(--sapList_BorderColor);--fdList_Item_Text_Font_Size:var(--sapFontLargeSize);--fdList_Item_Icon_Font_Size:1.125rem;--fdList_Item_Height:var(--sapElement_LineHeight);--fdList_Status_Text_Color:var(--sapNeutralTextColor);--fdList_Message_Background_Color:var(--sapNeutralBackground)}.fd-list__item-counter{border:0;-webkit-box-sizing:border-box;box-sizing:border-box;color:var(--sapTextColor);font-family:var(--sapFontFamily);font-size:var(--sapFontSize);font-weight:400;forced-color-adjust:none;line-height:normal;margin-block:0;margin-inline:0;overflow:hidden;padding-block:0;padding-inline:0;text-overflow:ellipsis;white-space:nowrap;-webkit-margin-start:1rem;color:var(--sapContent_MarkerTextColor);margin-inline-start:1rem}.fd-list__item-counter:after,.fd-list__item-counter:before{-webkit-box-sizing:inherit;box-sizing:inherit;font-size:inherit}.fd-icon-tab-bar{display:flex;flex-direction:column;height:100%}.fd-icon-tab-bar .fd-icon-tab-bar__item--hidden{display:none}.fd-icon-tab-bar__list{max-height:20rem}.fd-icon-tab-bar__list-item-counter{padding-left:.313rem}[dir=rtl] .fd-icon-tab-bar .fd-icon-tab-bar__list-item-counter,.fd-icon-tab-bar[dir=rtl] .fd-icon-tab-bar__list-item-counter{padding-left:0;padding-right:.25rem}.fd-icon-tab-bar__popover,.fd-icon-tab-bar__popover .fd-popover__control{height:100%}.cdk-drag-placeholder{opacity:.3;pointer-events:none}.screenreader-only{position:absolute!important;height:1px;width:1px;overflow:hidden;opacity:0;clip:rect(1px 1px 1px 1px);clip:rect(1px,1px,1px,1px)}\n/*! Bundled license information:\n\nfundamental-styles/dist/list.css:\n  (*!\n   * Fundamental Library Styles v0.38.0\n   * Copyright (c) 2024 SAP SE or an SAP affiliate company.\n   * Licensed under Apache License 2.0 (https://github.com/SAP/fundamental-styles/blob/main/LICENSE)\n   *)\n*/\n"] }]
        }], ctorParameters: () => [{ type: i0.ChangeDetectorRef }, { type: i1$1.ContentDensityService, decorators: [{
                    type: Optional
                }] }, { type: i1$1.RtlService, decorators: [{
                    type: Optional
                }] }], propDecorators: { stackContent: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }], _customTabs: [{
                type: HostBinding,
                args: ['class.fd-tabs-custom']
            }] } });

const components = [
    IconTabBarComponent,
    IconTabBarTextTypeComponent,
    IconTabBarIconTypeComponent,
    IconTabBarProcessTypeComponent,
    IconTabBarFilterTypeComponent,
    IconTabBarPopoverComponent,
    TextTypePopoverComponent,
    IconBarDndListDirective,
    IconBarDndItemDirective,
    IconBarDndContainerDirective,
    IconTabBarTabComponent,
    IconTabBarFreeContentDirective
];
const FDP_ICON_TAB_BAR = [IconTabBarComponent, IconTabBarTabComponent, IconTabBarFreeContentDirective];
/**
 * @deprecated
 * Import `IconTabBarComponent` directly instead.
 */
class PlatformIconTabBarModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: PlatformIconTabBarModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "19.0.7", ngImport: i0, type: PlatformIconTabBarModule, imports: [IconTabBarComponent,
            IconTabBarTextTypeComponent,
            IconTabBarIconTypeComponent,
            IconTabBarProcessTypeComponent,
            IconTabBarFilterTypeComponent,
            IconTabBarPopoverComponent,
            TextTypePopoverComponent,
            IconBarDndListDirective,
            IconBarDndItemDirective,
            IconBarDndContainerDirective,
            IconTabBarTabComponent,
            IconTabBarFreeContentDirective], exports: [IconTabBarComponent,
            IconTabBarTextTypeComponent,
            IconTabBarIconTypeComponent,
            IconTabBarProcessTypeComponent,
            IconTabBarFilterTypeComponent,
            IconTabBarPopoverComponent,
            TextTypePopoverComponent,
            IconBarDndListDirective,
            IconBarDndItemDirective,
            IconBarDndContainerDirective,
            IconTabBarTabComponent,
            IconTabBarFreeContentDirective] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: PlatformIconTabBarModule, imports: [IconTabBarComponent,
            IconTabBarTextTypeComponent,
            IconTabBarIconTypeComponent,
            IconTabBarProcessTypeComponent,
            IconTabBarFilterTypeComponent,
            IconTabBarPopoverComponent,
            TextTypePopoverComponent] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.7", ngImport: i0, type: PlatformIconTabBarModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [...components],
                    exports: [...components]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { FDP_ICON_TAB_BAR, FLIPPER_SIZE, ICON_TAB_HIDDEN_CLASS_NAME, IconBarDndContainerDirective, IconBarDndItemDirective, IconBarDndListDirective, IconTabBarBase, IconTabBarComponent, IconTabBarFilterTypeComponent, IconTabBarFreeContentDirective, IconTabBarIconTypeComponent, IconTabBarPopoverBase, IconTabBarPopoverComponent, IconTabBarProcessTypeComponent, IconTabBarTabComponent, IconTabBarTabContentDirective, IconTabBarTextTypeComponent, IconTabBarTextTypeTabItemComponent, PlatformIconTabBarModule, TextTypePopoverComponent, UNIQUE_KEY_SEPARATOR };
//# sourceMappingURL=fundamental-ngx-platform-icon-tab-bar.mjs.map
